/* Program to convert IP data file */
#define ProgName "ipbmap"
#define ProgInfo "transform image to polar coordinate system"
/*                                                         5/13/2003  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* Caution !! This program contains some bugs, that pixels far from meridional line
   is not correctly converted because pixel is no longer square.
   But I release this program, for a net correctness of the pattern is
   very important than to see the distorted image. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.04_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 12
#else
#	define MAXARG 11
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef USE_DOUBLE
typedef double my_float;
#else
typedef float my_float;
#endif

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const my_float *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr, *datPtrS;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, maxInt, readByte, flushWidth0, flushWidth,
	     intensity, xpxlOld, zpxlOld, base, x_orig, z_orig, xpxlNew, zpxlNew,
	     mr_p, mphi_p, mphi_m, pos, dataWidth, wrapVal;
	char file1[FNAMELEN], file2[FNAMELEN], confirmBuffer1[8], confirmBuffer2[8],
	     inputBuffer[20], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag, outMode,
	     outModeDoneFlag, shiftBit, strangeShiftBitFlag, changeValueTypeIndex,
	     byte, createDosFlag, verboseFlag, argNum, arg0[4], overwrapFlag,
	     drtxDoneFlag, drtzDoneFlag, xsizDoneFlag, zsizDoneFlag, replaceFlag,
	     setTimeFlag, zpxlDoneFlag, rstepDoneFlag, maxFlag, rDivF;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double intensityDbl1, intensityDbl2, xDist, zDist, rato, arg1, arg2, drtx, drtz,
	       xsiz, zsiz, sinPhi, cosPhi, maxR, minR, rstep, phistep, xsiz2, zsiz2,
	       r, r_p, r_m, d_r, phi, d_phi, phi_p, phi_m, fact_r1, fact_r2, fact_phi, r_min;
	my_float *datPtrDbl, *coefPtrDbl;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	ipDataSwapFlag = outModeDoneFlag = drtxDoneFlag = drtzDoneFlag = xsizDoneFlag
	 = zsizDoneFlag = replaceFlag = setTimeFlag = zpxlDoneFlag = rstepDoneFlag
	 = maxFlag = overwrapFlag = 0;
	readByte = 0;
	flushWidth0 = 0;
	flushWidth = 0;
	outMode = 1;
	zpxlNew = 1; /* dummy */
	rstep = 1.0; /* dummy */
	phistep = 1.0; /* dummy */
	fact_r1 = fact_r2 = -1.0; /* dummy */
	datPtrDbl = coefPtrDbl = (my_float*)NULL; /* dummy */
	wrapVal = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag++;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = createDosFlag ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
#ifdef USE_DOUBLE
		writeDat = ipDtoShortBit0;
#else
		writeDat = ipFtoShortBit0;
#endif
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
#ifdef USE_DOUBLE
				writeDat = ipDtoShort;
#else
				writeDat = ipFtoShort;
#endif
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
#ifdef USE_DOUBLE
			writeDat = ipDtoShort;
#else
			writeDat = ipFtoShort;
#endif
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " IP size is : %6ld x %6ld.\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
	}
	
/* Check the Header and Set Parameters : Interactive Mode */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z     , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
		if (ipHeader.drtx < 0.0 || ipHeader.drtx > ipHeader.xpxl || ipHeader.drtz < 0.0 || ipHeader.drtz > ipHeader.zpxl) {
			fprintf(stderr, " Direct Beam Position is not correctly set. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		
/* r-step */
		minR = MINEXT(MINEXT(ipHeader.xpxl-ipHeader.drtx, ipHeader.drtx)*ipHeader.xsiz, MINEXT(ipHeader.zpxl-ipHeader.drtz, ipHeader.drtz)*ipHeader.zsiz);
		maxR = dist(MAXEXT(ipHeader.xpxl-ipHeader.drtx, ipHeader.drtx)*ipHeader.xsiz, MAXEXT(ipHeader.zpxl-ipHeader.drtz, ipHeader.drtz)*ipHeader.zsiz);
		do {
			fprintf(stdout, "Enter step for radial direction [mm] (>%7.5f) : ", ipHeader.zsiz);
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			rstep = atof(inputBuffer);
			if (rstep < ipHeader.zsiz || rstep > minR) continue;
			rstepDoneFlag = 1;
		} while (!rstepDoneFlag);
		
/* Image Size check for X */
		do {
			fprintf(stdout, " Work for Maximum \"r\" ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') maxFlag = 1;
		xpxlNew = maxFlag ? (long)ceil(maxR/rstep) : (long)floor(minR/rstep);
		if (xpxlNew > MAXWIDTH) {
			fprintf(stderr, " Image width is too large (%ld). Exit.\n", xpxlNew);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		
/* Image Size for Z : phi-step */
		do {
			fprintf(stdout, "Enter Height of New Image [pixel].\n");
			fprintf(stdout, " This is a division number for azimuthal direction : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			zpxlNew = atol(inputBuffer);
			if (zpxlNew > MAXHEIGHT || zpxlNew <= 0) continue;
			zpxlDoneFlag = 1;
			phistep = 360.0 / zpxlNew;
		} while (!zpxlDoneFlag);
		
/* Modification Method */
		do {
			fprintf(stdout, " Use non-standard method of modification ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, " Enter Mode [1-3] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((confirmBuffer2[0] != '1') && (confirmBuffer2[0] != '1') && (confirmBuffer2[0] != '3'));
			outMode = confirmBuffer2[0];
		}
		
/* Question about Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			replaceFlag = 1;
			fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
		}
		do {
			fprintf(stdout, "Do you want to repeat the same image as the \"lowest\" quarter of converted\n");
			fprintf(stdout, " image onto the \"highest\" part of the image, in order to treat\n");
			fprintf(stdout, " peaks near \"0-deg.\" line ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			overwrapFlag = 1;
			fprintf(stdout, " Image height is automatically expanded than is already pointed.\n");
		}
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			setTimeFlag = 1;
			fprintf(stdout, " Modification time of Output File is set as Original File.\n");
		}
#endif
	}
/* Check Arguments and Set Parameters : Batch Mode */
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-mx")) maxFlag++;
				else if (!strcmp(arg0, "-wp")) overwrapFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-rs")) {
					rstep = arg1;
					if (rstep <= 0.0) break;
					rstepDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					zpxlNew = dToLong(arg1);
					if (zpxlNew <= 0 || zpxlNew > MAXHEIGHT) break;
					zpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					outMode = dToLong(arg1);
					if (outMode != 1 && outMode != 2 && outMode != 3) break;
					outModeDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((maxFlag > 1) || (rstepDoneFlag > 1) || (zpxlDoneFlag > 1)
		 || (drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (setTimeFlag > 1)  || (outModeDoneFlag > 1)
		 || (overwrapFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (ipHeader.drtx < 0.0 || ipHeader.drtx > ipHeader.xpxl || ipHeader.drtz < 0.0 || ipHeader.drtz > ipHeader.zpxl) {
			fprintf(stderr, " Direct Beam Position is not correctly set. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		minR = MINEXT(MINEXT(ipHeader.xpxl-ipHeader.drtx, ipHeader.drtx)*ipHeader.xsiz, MINEXT(ipHeader.zpxl-ipHeader.drtz, ipHeader.drtz)*ipHeader.zsiz);
		maxR = dist(MAXEXT(ipHeader.xpxl-ipHeader.drtx, ipHeader.drtx)*ipHeader.xsiz, MAXEXT(ipHeader.zpxl-ipHeader.drtz, ipHeader.drtz)*ipHeader.zsiz);
		if (!rstepDoneFlag) {
			if (verboseFlag) fprintf(stdout, " Assume r-step is %7.5f [mm].\n", ipHeader.zsiz);
			rstep = ipHeader.zsiz;
		}
		else if (rstep < ipHeader.zsiz) {
			fprintf(stderr, " r-step is too small. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (rstep > minR) {
			fprintf(stderr, " r-step is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		xpxlNew = maxFlag ? (long)ceil(maxR/rstep) : (long)floor(minR/rstep);
		if (xpxlNew > MAXWIDTH) {
			fprintf(stderr, " Image width is too large (%ld). Exit.\n", xpxlNew);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!zpxlDoneFlag) {
			phistep = 1.0;
			zpxlNew = 360;
			if (verboseFlag) fprintf(stdout, " Assume phi-step is %7.3f [deg.].\n", phistep);
		}
		else phistep = 360.0 / zpxlNew;
		if (zpxlNew > MAXHEIGHT) {
			fprintf(stderr, " Image height is too large (%ld). Exit.\n", zpxlNew);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Definition of some functions */
	minR += rstep;
	r_min = ipHeader.xsiz/2.0/tan(10.0*DEG_RAD); /* last term of 10.0*DEG_RAD is meaningless. */
	if (overwrapFlag) {
		wrapVal = zpxlNew / 4;
		zpxlNew += wrapVal;
	}
	if (verboseFlag == 2) {
		fprintf(stdout, "New image is r-phi map. r : horizontal, phi : vertical.\n");
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (!maxFlag) fprintf(stdout, " Image having imperfect \"r\" is ignored.\n");
		if (outMode==1) fprintf(stdout, " Conversion Method : Sum Mode.\n");
		else if (outMode==2) fprintf(stdout, " Conversion Method : Average Mode.\n");
		else if (outMode==3) fprintf(stdout, " Conversion Method : Simple reverse sampling Mode.\n");
		fprintf(stdout, " Output PixelSize is %7.5f [mm] x %7.3f [deg.],\n", rstep, phistep);
		fprintf(stdout, "  but recorded as %7.5f [mm] x %7.5f [mm].\n", rstep, rstep);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	xpxlOld = ipHeader.xpxl;
	zpxlOld = ipHeader.zpxl;
	drtx = ipHeader.drtx;
	drtz = ipHeader.drtz;
	xsiz = ipHeader.xsiz;
	zsiz = ipHeader.zsiz;
	xsiz2 = xsiz/2.0;
	zsiz2 = zsiz/2.0;
	ipHeader.xpxl = xpxlNew;
	ipHeader.zpxl = zpxlNew;
	ipHeader.xsiz = rstep;
	ipHeader.zsiz = ipHeader.xsiz;
	ipHeader.drtx = 0.5;
	ipHeader.drtz = (ipHeader.zpxl-wrapVal)/2.0;
	phistep *= DEG_RAD;
	dataWidth = xpxlOld*RAXDATAWIDTH;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	ipHeader.rnum -= wrapVal;
	ipHeader.zpxl = ipHeader.rnum;
	if (outMode == 3) { /* simple reverse sampling */
		flushWidth = recordLengthCreate / RAXDATAWIDTH;
		if (flushWidth0 > 0) free(datPtr0);
		flushWidth0 = 0;
		if ((datPtrS = malloc(zpxlOld*dataWidth)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		base = 0;
		for (i=0; i<zpxlOld; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrS);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtrS);
				free(datPtr);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			memcpy(datPtrS+base, datPtr, dataWidth);
			free(datPtr);
			base += xpxlOld;
		}
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtr = calloc(flushWidth, RAXDATAWIDTH)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrS);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			phi = (i+0.5)*phistep;
			cosPhi = cos(phi)/xsiz;
			sinPhi = sin(phi)/zsiz;
			for (j=0; j<ipHeader.xpxl; j++) {
				r = (j+0.5)*rstep;
				x_orig = (long)floor(r*cosPhi + drtx);
				z_orig = (long)floor(r*sinPhi + drtz);
				if (x_orig >= xpxlOld || x_orig < 0 || z_orig >= zpxlOld || z_orig < 0) continue;
				datPtr[j] = datPtrS[z_orig*xpxlOld+x_orig];
			}
			fwrite(datPtr, recordLengthCreate, 1, fp2);
			free(datPtr);
		}
		free(datPtrS);
	}
	else { /* 1: divide pixel, sum // 2: divide pixel, average */
		base = ipHeader.xpxl*ipHeader.zpxl;
		if ((datPtrDbl = calloc(base, sizeof(my_float))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		if ((coefPtrDbl = calloc(base, sizeof(my_float))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		for (i=0; i<zpxlOld; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(coefPtrDbl);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(coefPtrDbl);
				free(datPtrDbl);
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			zDist = (i+0.5-drtz) * zsiz;
			for (j=0; j<xpxlOld; j++) {
				xDist = (j+0.5-drtx) * xsiz;
				/* get parameters for pixel */
				r = dist(zDist, xDist);
				if (r < r_min) continue;
				if (!maxFlag && r > minR) continue;
				r_p = r + zsiz2;
				r_m = r - zsiz2;
				if ((phi = atan2(zDist, xDist)) < 0.0) phi += M_2PI;
				d_phi = fabs(atan(xsiz2/r));
				phi_p = phi + d_phi;
				phi_m = phi - d_phi;
				/* change to coordinates */
				r_p /= rstep;
				r_m /= rstep;
				d_r = r_p - r_m;
				phi_p /= phistep;
				phi_m /= phistep;
				d_phi *= (2.0/phistep);
				/* calc for r */
				mr_p = (long)floor(r_p);
				if (mr_p == (long)floor(r_m)) rDivF = 0;
				else {
					fact_r1 = (mr_p-r_m)/d_r;
					fact_r2 = 1.0 - fact_r1;
					rDivF = 1;
				}
				/* calc for phi */
				intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				mphi_p = (long)floor(phi_p);
				mphi_m = (long)floor(phi_m);
				if (rDivF) {
					if (mr_p < ipHeader.xpxl) {
						if (mphi_p == mphi_m) {
							intensityDbl1 = intensity * fact_r1;
							pos = mphi_p*ipHeader.xpxl + mr_p - 1;
							datPtrDbl[pos  ] += intensityDbl1;
							datPtrDbl[pos+1] += intensity - intensityDbl1;
							coefPtrDbl[pos  ] += fact_r1;
							coefPtrDbl[pos+1] += fact_r2;
						}
						else {
							intensityDbl1 = intensity * fact_r1;
							intensityDbl2 = intensity - intensityDbl1;
							if ((pos = mphi_m*ipHeader.xpxl + mr_p - 1) < 0) pos += base;
							else if (pos >= base) pos -= base;
							fact_phi = (mphi_m+1.0-phi_m)/d_phi;
							datPtrDbl[pos  ] += intensityDbl1 * fact_phi;
							datPtrDbl[pos+1] += intensityDbl2 * fact_phi;
							coefPtrDbl[pos  ] += fact_r1 * fact_phi;
							coefPtrDbl[pos+1] += fact_r2 * fact_phi;
							fact_phi = 1.0/d_phi;
							for (k=mphi_m+1; k<mphi_p; k++) {
								pos += ipHeader.xpxl;
								if (pos >= base) pos -= base;
								datPtrDbl[pos  ] += intensityDbl1 * fact_phi;
								datPtrDbl[pos+1] += intensityDbl2 * fact_phi;
								coefPtrDbl[pos  ] += fact_r1 * fact_phi;
								coefPtrDbl[pos+1] += fact_r2 * fact_phi;
							}
							pos += ipHeader.xpxl;
							if (pos >= base) pos -= base;
							fact_phi = (phi_p - mphi_p)/d_phi;
							datPtrDbl[pos  ] += intensityDbl1 * fact_phi;
							datPtrDbl[pos+1] += intensityDbl2 * fact_phi;
							coefPtrDbl[pos  ] += fact_r1 * fact_phi;
							coefPtrDbl[pos+1] += fact_r2 * fact_phi;
						}
					}
					else if (mr_p == ipHeader.xpxl) {
						if (mphi_p == mphi_m) {
							pos = mphi_p*ipHeader.xpxl + mr_p - 1;
							datPtrDbl[pos] += intensity * fact_r1;
							coefPtrDbl[pos] += fact_r1;
						}
						else {
							intensityDbl1 = intensity * fact_r1;
							if ((pos = mphi_m*ipHeader.xpxl + mr_p - 1) < 0) pos += base;
							else if (pos >= base) pos -= base;
							fact_phi = (mphi_m+1.0-phi_m)/d_phi;
							datPtrDbl[pos] += intensityDbl1 * fact_phi;
							coefPtrDbl[pos] += fact_r1 * fact_phi;
							fact_phi = 1.0/d_phi;
							for (k=mphi_m+1; k<mphi_p; k++) {
								pos += ipHeader.xpxl;
								if (pos >= base) pos -= base;
								datPtrDbl[pos] += intensityDbl1 * fact_phi;
								coefPtrDbl[pos] += fact_r1 * fact_phi;
							}
							pos += ipHeader.xpxl;
							if (pos >= base) pos -= base;
							fact_phi = (phi_p - mphi_p)/d_phi;
							datPtrDbl[pos] += intensityDbl1 * fact_phi;
							coefPtrDbl[pos] += fact_r1 * fact_phi;
						}
					}
				}
				else if (mr_p < ipHeader.xpxl) {
					if (mphi_p == mphi_m) {
						pos = mphi_p*ipHeader.xpxl + mr_p;
						datPtrDbl[pos] += intensity;
						coefPtrDbl[pos] += 1.0;
					}
					else {
						if ((pos = mphi_m*ipHeader.xpxl + mr_p) < 0) pos += base;
						else if (pos >= base) pos -= base;
						fact_phi = (mphi_m+1.0-phi_m)/d_phi;
						datPtrDbl[pos] += intensity * fact_phi;
						coefPtrDbl[pos] += fact_phi;
						fact_phi = 1.0/d_phi;
						for (k=mphi_m+1; k<mphi_p; k++) {
							pos += ipHeader.xpxl;
							if (pos >= base) pos -= base;
							datPtrDbl[pos] += intensity * fact_phi;
							coefPtrDbl[pos] += fact_phi;
						}
						pos += ipHeader.xpxl;
						if (pos >= base) pos -= base;
						fact_phi = (phi_p - mphi_p)/d_phi;
						datPtrDbl[pos] += intensity * fact_phi;
						coefPtrDbl[pos] += fact_phi;
					}
				}
			}
			free(datPtr);
		}
		fclose(fp1);
		for (i=0, base=0; i<ipHeader.rnum; i++) {
			if ((datPtr = malloc(recordLengthCreate)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(coefPtrDbl);
				if (flushWidth0 > 0) free(datPtr0);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if (outMode == 2) {
				for (j=0; j<ipHeader.xpxl; j++) {
					if (coefPtrDbl[base] != 0.0) datPtrDbl[base] /= coefPtrDbl[base];
					datPtr[j] = (*writeDat)(&(datPtrDbl[base++]), &rato, &maxInt, &ipDataSwapFlag);
				}
			}
			else {
				for (j=0; j<ipHeader.xpxl; j++)
				    datPtr[j] = (*writeDat)(&(datPtrDbl[base++]), &rato, &maxInt, &ipDataSwapFlag);
			}
			fwrite(datPtr, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtr);
		}
		free(datPtrDbl);
		free(coefPtrDbl);
	}
	if (flushWidth0 > 0) free(datPtr0);
	if (overwrapFlag) {
		fflush(fp2);
		readByte = ftell(fp2);
		base = recordLengthCreate*wrapVal;
		if ((datPtr = malloc(base)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
			fseek(fp2, 0, SEEK_SET);
			fwrite(&ipHeader, ipHeaderSize, 1, fp2);
			fprintf(stdout, " Output File has no extra part.\n");
			if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
			if (verboseFlag) {
				fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
				if (replaceFlag) {
					fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
					fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
				}
			}
			fclose(fp2);
#ifdef _HASUTIME
			if (setTimeFlag) {
				if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
			}
			if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			exit(EXIT_FAILURE);
		}
		fseek(fp2, recordLengthCreate, SEEK_SET);
		fread(datPtr, base, 1, fp2);
		fseek(fp2, readByte, SEEK_SET);
		fwrite(datPtr, base, 1, fp2);
		free(datPtr);
	}
	ipHeader.rnum += wrapVal;
	ipHeader.zpxl = ipHeader.rnum;
	
/* End */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
		if (overwrapFlag) {
			fprintf(stdout, " Top 1/5 of the Image is copied from the lowest 1/5, where the first\n");
			fprintf(stdout, "  quadrant of Original Image is shown.\n");
			fprintf(stdout, " Height of New Image is automatically extended.\n");
		}
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-rs:VALUE] [-z:VALUE] [-mx] [-xc:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rs:VALUE] [-z:VALUE] [-mx] [-xc:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-zc:VALUE] [-dx:VALUE] [-dz:VALUE] [-m:VALUE]\n");
	fprintf(fp0, "          [-wp] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program modifies R-AXIS Image to r-phi map of polar coordinates.\n");
	fprintf(fp0, "  Caution : This treat image as simple \"image\", so do not pay attension\n");
	fprintf(fp0, "            to camera type, camera length, wave length, etc.\n");
	fprintf(fp0, "            New image has \"r\" in X, \"phi\" in Z.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File. This must be set\n");
	fprintf(fp0, "                 before other arguments listed below.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	fprintf(fp0, "   -rs:VALUE ... Set radial step to \"VALUE\"[mm]. This must be greater than\n");
	fprintf(fp0, "                 Pixel size in Z of original image, and must not be so large.\n");
	fprintf(fp0, "   -z:VALUE .... Set Image Height of New Image. This also decide the number\n");
	fprintf(fp0, "                 of division for azimuthal direction.\n");
	fprintf(fp0, "   -mx ......... Do not ignore region with \"r\" of imperfect \"phi\".\n");
	fprintf(fp0, "   -m:VALUE .... Set Modification Method to \"VALUE\"(1-3).\n");
	fprintf(fp0, "                  1 : Precise, Sum.\n");
	fprintf(fp0, "                  2 : Precise, Average.\n");
	fprintf(fp0, "                  3 : Simple Reverse Sampling Method.\n");
	fprintf(fp0, "                 If absent, set to 1.\n");
	fprintf(fp0, "   -wp ......... Repeat the same image as the \"lowest\" quarter of converted\n");
	fprintf(fp0, "                 image onto the \"highest\" part of the image.\n");
	fprintf(fp0, "                 Image height is automatically expanded than is pointed.\n");
	fprintf(fp0, "                 This is useful to treat peaks near \"0-deg.\" line.\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Change Pixel Size in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change Pixel Size in Z to \"VALUE\" [mm].\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
