/* Program to convert IP data file */
#define ProgName "ipchtype"
#define ProgInfo "transform camera type between Flat and Curved"
/*                                                         5/16/2003  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* Caution !! This program contains some bugs, that pixel far from centre
   is not correctly converted because pixel is no longer square.
   But I release this program, for a net correctness of the pattern is
   very important than to see the distorted image. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.05_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 12
#else
#	define MAXARG 11
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define FLAT 0
#define CURVE_V 1
#define THETALIM 85.0

#ifdef USE_DOUBLE
typedef double my_float;
#else
typedef float my_float;
#endif

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const my_float *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr;
	long int i, j, j2, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, maxInt, readByte, flushWidth0, flushWidth,
	     intensity, xpxlOld, zpxlOld, base, xpxlNew, zpxlNew, *zpos, *datPtrL, *xpos,
	     nw_l, nw_r, nw_u, nw_d, startX, endX, zpos_low, skip, zlim_m;
	char file1[FNAMELEN], file2[FNAMELEN], confirmBuffer1[8], confirmBuffer2[8],
	     inputBuffer[20], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag, cameraTypeFlag,
	     cameraTypeDoneFlag, shiftBit, strangeShiftBitFlag, changeValueTypeIndex,
	     byte, createDosFlag, verboseFlag, argNum, arg0[4], camrDoneFlag,
	     drtxDoneFlag, drtzDoneFlag, xsizDoneFlag, zsizDoneFlag, replaceFlag,
	     setTimeFlag, xpxlDoneFlag, zpxlDoneFlag, maxFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, arg1, arg2, drtx, drtz, xsiz, zsiz, f_x, f_z;
	my_float *datPtrDbl, intensityDbl;
	float *zbuf, thetalim, *xratom, *xratop, zratop, zratom, ratoz,
	      zbuf_low, xbuf_old, xbuf_new;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	ipDataSwapFlag = cameraTypeDoneFlag = drtxDoneFlag = drtzDoneFlag = xsizDoneFlag
	 = zsizDoneFlag = replaceFlag = setTimeFlag = xpxlDoneFlag = camrDoneFlag
	 = maxFlag = zpxlDoneFlag = 0;
	cameraTypeFlag = FLAT;
	readByte = 0;
	flushWidth0 = 0;
	flushWidth = 0;
	thetalim = THETALIM*DEG_RAD;
	xpxlNew = MAXWIDTH;
	zpxlNew = MAXHEIGHT;
	startX = endX = 0; /* dummy */
	nw_l = nw_d = 0; /* dummy */
	datPtrDbl = (my_float*)NULL; /* dummy */
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag++;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = createDosFlag ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
#ifdef USE_DOUBLE
		writeDat = ipDtoShortBit0;
#else
		writeDat = ipFtoShortBit0;
#endif
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
#ifdef USE_DOUBLE
				writeDat = ipDtoShort;
#else
				writeDat = ipFtoShort;
#endif
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
#ifdef USE_DOUBLE
			writeDat = ipDtoShort;
#else
			writeDat = ipFtoShort;
#endif
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " IP size is : %6ld x %6ld.\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " CameraLength is : %7.3f mm.\n", ipHeader.camr);
	}
	
/* Check the Header and Set Parameters : Interactive Mode */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z      , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (5) Camera Length , (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					case 5:
						camrDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Camera Length [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.camr = atof(inputBuffer);
							if (ipHeader.camr <= 0.0) continue;
							camrDoneFlag = 1;
						} while (!camrDoneFlag);
						fprintf(stdout, "Camera Length is changed to %7.3f mm.\n", ipHeader.camr);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
		
/* Ask Camera Type */
		do {
			fprintf(stdout, "Camera is Flat[F] , Curved[C] , or Other[N] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'F') && (toupper(confirmBuffer1[0]) != 'C') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else if (toupper(confirmBuffer1[0]) == 'C') {
			cameraTypeFlag = CURVE_V;
			do {
				fprintf(stdout, "  Camera is Curved. Is Cylinder Axis Vertical on the Screen ? : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'N') {
				fprintf(stderr, "  Sorry, this type is not supported.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else {
			fprintf(stdout, "  Camera is Unknwon.\n");
			fprintf(stderr, "  Sorry, only Flat and Curved camera is supported.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		
/* Image Size check for X, Z */
		if (cameraTypeFlag == CURVE_V) {
			do {
				fprintf(stdout, " Enter Width of New Image [pixel] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				xpxlNew = atol(inputBuffer);
				if (xpxlNew > MAXWIDTH || xpxlNew <= 0) continue;
				xpxlDoneFlag = 1;
			} while (!xpxlDoneFlag);
			do {
				fprintf(stdout, " Enter Height of New Image [pixel] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				zpxlNew = atol(inputBuffer);
				if (zpxlNew > MAXHEIGHT || zpxlNew < ipHeader.zpxl) continue;
				zpxlDoneFlag = 1;
			} while (!zpxlDoneFlag);
		}
		
/* Image Size for Z : Max? */
		do {
			fprintf(stdout, " Do you want to generate image without meaningless region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') maxFlag = 0;
		else maxFlag = 1;
		
/* Question about Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			replaceFlag = 1;
			fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
		}
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			setTimeFlag = 1;
			fprintf(stdout, " Modification time of Output File is set as Original File.\n");
		}
#endif
	}
/* Check Arguments and Set Parameters : Batch Mode */
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-mx")) maxFlag++;
				else if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-x")) {
					xpxlNew = dToLong(arg1);
					if (xpxlNew <= 0 || xpxlNew > MAXWIDTH) break;
					xpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					zpxlNew = dToLong(arg1);
					if (zpxlNew < ipHeader.zpxl || zpxlNew > MAXHEIGHT) break;
					zpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((maxFlag > 1) || (cameraTypeDoneFlag > 1) || (xpxlDoneFlag > 1)
		 || (drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (setTimeFlag > 1) || (camrDoneFlag > 1)
		 || (zpxlDoneFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!cameraTypeDoneFlag) {
			if (ipHeader.cyld == 0) {
				if (verboseFlag >= 2) fprintf(stdout, " Assume CameraType is Flat.\n");
				cameraTypeFlag = FLAT;
			}
			else if (ipHeader.cyld == 1) {
				if (verboseFlag >= 2) fprintf(stdout, " Assume CameraType is Curved with Cylinder Axis Vertical.\n");
				cameraTypeFlag = CURVE_V;
			}
			else {
				fprintf(stderr, "Camera Type is unknown.\n");
				showUsage(stderr);
				fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		if (cameraTypeFlag == CURVE_V && !xpxlDoneFlag) {
			fprintf(stderr, "Error: Width of New Image must be specified for Curved Camera.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Set Parameters to Create */
	if (ipHeader.drtx < 0.0 || ipHeader.drtx > ipHeader.xpxl || ipHeader.drtz < 0.0 || ipHeader.drtz > ipHeader.zpxl) {
		fprintf(stderr, " Direct Beam Position is not correctly set. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.camr <= 0.0) {
		fprintf(stderr, " Camera Length is not correctly set. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	xpxlOld = ipHeader.xpxl;
	zpxlOld = ipHeader.zpxl;
	drtx = ipHeader.drtx;
	drtz = ipHeader.drtz;
	xsiz = ipHeader.xsiz;
	zsiz = ipHeader.zsiz;
	f_x = ipHeader.xsiz/ipHeader.camr;
	f_z = ipHeader.zsiz/ipHeader.camr;
	switch (cameraTypeFlag) {
		case FLAT:
			if (xpxlDoneFlag || zpxlDoneFlag) fprintf(stdout, " Options about output size is ignored with this type of camera.\n");
			nw_l = (long)ceil(ipHeader.drtx-atan(ipHeader.drtx*f_x)/f_x);
			nw_r = (long)floor(ipHeader.drtx+atan((ipHeader.xpxl-ipHeader.drtx)*f_x)/f_x);
			xpxlNew = nw_r - nw_l;
			startX = 0;
			endX = ipHeader.xpxl;
			if (maxFlag) {
				zpxlNew = ipHeader.zpxl;
				nw_d = 0;
				nw_u = ipHeader.zpxl;
			}
			else {
				if (ipHeader.xpxl-ipHeader.drtx > ipHeader.drtx) {
					nw_d = (long)ceil(ipHeader.drtz-ipHeader.drtz/dist(1.0, (ipHeader.xpxl-ipHeader.drtx)*f_x));
					nw_u = (long)floor(ipHeader.drtz+(ipHeader.zpxl-ipHeader.drtz)/dist(1.0, (ipHeader.xpxl-ipHeader.drtx)*f_x));
				}
				else {
					nw_d = (long)ceil(ipHeader.drtz-ipHeader.drtz/dist(1.0, ipHeader.drtx*f_x));
					nw_u = (long)floor(ipHeader.drtz+(ipHeader.zpxl-ipHeader.drtz)/dist(1.0, ipHeader.drtx*f_x));
				}
				zpxlNew = nw_u - nw_d;
			}
			ipHeader.cyld = 1;
			break;
		case CURVE_V:
			startX = (ipHeader.drtx*f_x > thetalim) ? (long)floor(ipHeader.drtx - thetalim/f_x) : 0; /* left edge of old image */
			endX = ((ipHeader.xpxl-ipHeader.drtx)*f_x > thetalim) ? (long)ceil(ipHeader.drtx + thetalim/f_x) : ipHeader.xpxl;
			nw_l = (long)ceil(ipHeader.drtx-tan((ipHeader.drtx-startX)*f_x)/f_x); /* left edge of new image */
			nw_r = (long)floor(ipHeader.drtx+tan((endX-ipHeader.drtx)*f_x)/f_x);
			if (nw_r - nw_l <= 0) {
				fprintf(stderr, "Error in pixel size or camera length. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (verboseFlag == 2 && (startX != 0 || endX != ipHeader.xpxl))
			    fprintf(stdout, " Pixels having large q is ignored.\n");
			if (nw_r - nw_l > xpxlNew) {
				if (nw_r - ipHeader.drtx < xpxlNew/2.0) nw_l = nw_r - xpxlNew;
				else if (ipHeader.drtx - nw_l < xpxlNew/2.0) nw_r = nw_l + xpxlNew;
				else {
					nw_l = dToLong(ipHeader.drtx - xpxlNew/2.0);
					nw_r = nw_l + xpxlNew;
				}
			}
			else {
				xpxlNew = nw_r - nw_l;
				if (verboseFlag) fprintf(stdout, " Output Image is not so wide. Shrink to %ld.\n", xpxlNew);
			}
			startX = (long)floor(ipHeader.drtx - atan((ipHeader.drtx-nw_l)*f_x)/f_x); /* left edge of old image */
			endX = (long)ceil(ipHeader.drtx + atan((nw_r-ipHeader.drtx)*f_x)/f_x);
			if (maxFlag) {
				if (endX-ipHeader.drtx > ipHeader.drtx-startX) {
					nw_d = (long)ceil(ipHeader.drtz-ipHeader.drtz/cos((endX-ipHeader.drtx)*f_x));
					nw_u = (long)floor(ipHeader.drtz+(ipHeader.zpxl-ipHeader.drtz)/cos((endX-ipHeader.drtx)*f_x));
				}
				else {
					nw_d = (long)ceil(ipHeader.drtz-ipHeader.drtz/cos((ipHeader.drtx-startX)*f_x));
					nw_u = (long)floor(ipHeader.drtz+(ipHeader.zpxl-ipHeader.drtz)/cos((ipHeader.drtx-startX)*f_x));
				}
				if (nw_u - nw_d > zpxlNew) {
					if (nw_u - ipHeader.drtz < zpxlNew/2.0) nw_d = nw_u - zpxlNew;
					else if (ipHeader.drtz - nw_d < zpxlNew/2.0) nw_u = nw_d + zpxlNew;
					else {
						nw_d = dToLong(ipHeader.drtz - zpxlNew/2.0);
						nw_u = nw_d + zpxlNew;
					}
				}
				else {
					zpxlNew = nw_u - nw_d;
					if (verboseFlag) fprintf(stdout, " Output Image is not so tall. Shrink to %ld.\n", zpxlNew);
				}
			}
			else {
				zpxlNew = ipHeader.zpxl;
				nw_d = 0;
				nw_u = ipHeader.zpxl;
			}
			ipHeader.cyld = 0;
			break;
		default:
			/* NOT REACHED */
			break;
	}
	ipHeader.xpxl = xpxlNew;
	ipHeader.zpxl = zpxlNew;
	ipHeader.drtx -= nw_l;
	ipHeader.drtz -= nw_d;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(flushWidth, ipHeaderSizeMin);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Message */
	if (verboseFlag == 2) {
		if (cameraTypeFlag==FLAT) fprintf(stdout, "Convert Flat Camera image to Curved Camera image.\n");
		else if (cameraTypeFlag==CURVE_V) fprintf(stdout, "Convert Curved Camera image to Flat Camera image.\n");
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", drtx, drtz);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", xsiz, zsiz);
		if (camrDoneFlag) fprintf(stdout, " CameraLength has changed to : %7.3f mm.\n", ipHeader.camr);
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " New Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (!maxFlag) fprintf(stdout, " Image having imperfect image is ignored.\n");
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	switch (cameraTypeFlag) {
		case FLAT:
			if (fseek(fp1, recordLength, SEEK_SET)) {
				fprintf(stderr, " Input File Read Error.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			if ((datPtrDbl = calloc(ipHeader.xpxl*ipHeader.zpxl, sizeof(my_float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((zbuf = malloc(xpxlOld*sizeof(float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((xratom = malloc(xpxlOld*sizeof(float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(zbuf);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((xpos = malloc((xpxlOld+1)*sizeof(long))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(zbuf);
				free(xratom);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((zpos = malloc(xpxlOld*sizeof(long))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(zbuf);
				free(xratom);
				free(xpos);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			zlim_m = ipHeader.zpxl-1;
			xbuf_new = atan(-drtx*f_x)/f_x + ipHeader.drtx;
			xpos[0] = (long)floor(xbuf_new);
			for (j=0, j2=1; j<xpxlOld; j++, j2++) {
				xbuf_old = xbuf_new;
				xbuf_new = atan((j2-drtx)*f_x)/f_x + ipHeader.drtx;
				xpos[j2] = (long)floor(xbuf_new);
				xratom[j] = (xpos[j2]==xpos[j]) ? 1.0 : (xpos[j2] - xbuf_old) / (xbuf_new - xbuf_old);
				zbuf[j] = -drtz/dist(1.0, (j+0.5-drtx)*f_x) + ipHeader.drtz;
				zpos[j] = (long)floor(zbuf[j]);
			}
			for (i=0; i<zpxlOld; i++) {
				if ((datPtr = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDbl);
					free(zbuf);
					free(xratom);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				fread(datPtr, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrDbl);
					free(datPtr);
					free(zbuf);
					free(xratom);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				for (j=0; j<xpxlOld; j++) {
					zbuf_low = zbuf[j];
					zpos_low = zpos[j];
					zbuf[j] = (i+1-drtz)/dist(1.0, (j+0.5-drtx)*f_x) + ipHeader.drtz;
					zpos[j] = (long)floor(zbuf[j]);
					ratoz = (zpos[j] == zpos_low) ? 1.0 : (zpos[j] - zbuf_low) / (zbuf[j] - zbuf_low);
					intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					base = zpos_low*ipHeader.xpxl + xpos[j];
					if (xratom[j] == 1.0) {
						if (zpos_low >= 0 && zpos_low < ipHeader.zpxl) datPtrDbl[base                ] += intensity * ratoz;
						if (zpos_low >= -1 && zpos_low < zlim_m) datPtrDbl[base+ipHeader.xpxl  ] += intensity * (1.0-ratoz);
					}
					else {
						if (zpos_low >= 0 && zpos_low < ipHeader.zpxl) {
							datPtrDbl[base                ] += intensity * xratom[j] * ratoz;
							datPtrDbl[base+1              ] += intensity * (1.0-xratom[j]) * ratoz;
						}
						if (zpos_low >= -1 && zpos_low < zlim_m) {
							datPtrDbl[base+ipHeader.xpxl  ] += intensity * xratom[j] * (1.0-ratoz);
							datPtrDbl[base+ipHeader.xpxl+1] += intensity * (1.0-xratom[j]) * (1.0-ratoz);
						}
					}
				}
				free(datPtr);
			}
			fclose(fp1);
			free(zbuf);
			free(xratom);
			free(xpos);
			free(zpos);
			for (i=0, base=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDbl);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (replaceFlag) {
								fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
								fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
							}
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++)
				    datPtr[j] = (*writeDat)(&(datPtrDbl[base++]), &rato, &maxInt, &ipDataSwapFlag);
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtr);
			}
			free(datPtrDbl);
			break;
		case CURVE_V:
			if (fseek(fp1, endX*RAXDATAWIDTH, SEEK_SET)) {
				fprintf(stderr, " Input File Read Error.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			xpxlOld = endX - startX;
			drtx -= startX;
			skip = recordLength - xpxlOld*RAXDATAWIDTH;
			recordLength = xpxlOld*RAXDATAWIDTH;
			if ((datPtrL = malloc(xpxlOld*zpxlOld*sizeof(long))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((zbuf = malloc(ipHeader.xpxl*sizeof(float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((xratom = malloc(ipHeader.xpxl*sizeof(float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				free(zbuf);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((xratop = malloc(ipHeader.xpxl*sizeof(float))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				free(zbuf);
				free(xratom);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((xpos = malloc((ipHeader.xpxl+1)*sizeof(long))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				free(zbuf);
				free(xratom);
				free(xratop);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			if ((zpos = malloc(ipHeader.xpxl*sizeof(long))) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDbl);
				free(zbuf);
				free(xratom);
				free(xratop);
				free(xpos);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			for (i=0, base=0; i<zpxlOld; i++) {
				if (fseek(fp1, skip, SEEK_CUR)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(datPtrL);
					free(zbuf);
					free(xratom);
					free(xratop);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrL);
					free(zbuf);
					free(xratom);
					free(xratop);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				fread(datPtr, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrL);
					free(datPtr);
					free(zbuf);
					free(xratom);
					free(xratop);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				for (j=0; j<xpxlOld; j++) datPtrL[base++] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				free(datPtr);
			}
			fclose(fp1);
			xbuf_new = atan(-ipHeader.drtx*f_x)/f_x + drtx;
			xpos[0] = (long)floor(xbuf_new);
			for (j=0, j2=1; j<ipHeader.xpxl; j++, j2++) {
				xbuf_old = xbuf_new;
				xbuf_new = atan((j2-ipHeader.drtx)*f_x)/f_x + drtx;
				xpos[j2] = (long)floor(xbuf_new);
				if (xpos[j2] == xpos[j]) xratom[j] = xbuf_new - xbuf_old;
				else {
					xratom[j] = xpos[j2] - xbuf_old;
					xratop[j] = xbuf_new - xpos[j2];
				}
				zbuf[j] = -ipHeader.drtz/dist(1.0, (j+0.5-ipHeader.drtx)*f_x) + drtz;
				zpos[j] = (long)floor(zbuf[j]);
			}
			zlim_m = zpxlOld-1;
			for (i=0; i<ipHeader.zpxl; i++) {
				if ((datPtr = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrL);
					free(zbuf);
					free(xratom);
					free(xratop);
					free(xpos);
					free(zpos);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (replaceFlag) {
								fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
								fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
							}
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					zbuf_low = zbuf[j];
					zpos_low = zpos[j];
					zbuf[j] = (i+1-ipHeader.drtz)/dist(1.0, (j+0.5-ipHeader.drtx)*f_x) + drtz;
					zpos[j] = (long)floor(zbuf[j]);
					base = zpos_low*xpxlOld + xpos[j];
					intensityDbl = 0.0;
					if (zpos[j] == zpos_low) {
						zratom = zbuf[j] - zbuf_low;
						if (xpos[j] == xpos[j+1]) {
							if (zpos_low >= 0 && zpos_low < zpxlOld) intensityDbl += datPtrL[base] * xratom[j] * zratom;
						}
						else {
							if (zpos_low >= 0 && zpos_low < zpxlOld) {
								intensityDbl += datPtrL[base] * xratom[j] * zratom;
								intensityDbl += datPtrL[base+1] * xratop[j] * zratom;
							}
						}
					}
					else {
						zratom = zpos[j] - zbuf_low;
						zratop = zbuf[j] - zpos[j];
						if (xpos[j] == xpos[j+1]) {
							if (zpos_low >= 0 && zpos_low < zpxlOld) intensityDbl += datPtrL[base] * xratom[j] * zratom;
							if (zpos_low >= -1 && zpos_low < zlim_m) intensityDbl += datPtrL[base+xpxlOld] * xratom[j] * zratop;
						}
						else {
							if (zpos_low >= 0 && zpos_low < zpxlOld) {
								intensityDbl += datPtrL[base] * xratom[j] * zratom;
								intensityDbl += datPtrL[base+1] * xratop[j] * zratom;
							}
							if (zpos_low >= -1 && zpos_low < zlim_m) {
								intensityDbl += datPtrL[base+xpxlOld] * xratom[j] * zratop;
								intensityDbl += datPtrL[base+xpxlOld+1] * xratop[j] * zratop;
							}
						}
					}
				    datPtr[j] = (*writeDat)(&intensityDbl, &rato, &maxInt, &ipDataSwapFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtr);
			}
			free(zbuf);
			free(xratop);
			free(xratom);
			free(xpos);
			free(zpos);
			free(datPtrL);
			break;
		default:
			/* NOT REACHED */
			break;
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-f|-cv] [-x:VALUE] [-z:VALUE] [-mx]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-f|-cv] [-x:VALUE] [-z:VALUE] [-mx]\n", ProgName);
#endif
	fprintf(fp0, "          [-dz:VALUE] [-xc:VALUE] [-zc:VALUE] [-dx:VALUE]\n");
	fprintf(fp0, "          [-cl:VALUE] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program changes R-AXIS Image of Flat Camera to Cylindrical Camera,\n");
	fprintf(fp0, "   and vice versa.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File. This must be set\n");
	fprintf(fp0, "                 before other arguments listed below.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	fprintf(fp0, "   -f .......... Assume CameraType is Flat.\n");
	fprintf(fp0, "   -cv ......... Assume CameraType is Cylindrical, whose axis is vertical\n");
	fprintf(fp0, "                 to the screen.\n");
	fprintf(fp0, "            If none of above exists, decide original format from file header.\n");
	fprintf(fp0, "   -x:VALUE .... Set upper limit of Width of New Image. This is necessary\n");
	fprintf(fp0, "                 for curved camera, and is ignored if camera is flat.\n");
	fprintf(fp0, "                 Value must be smaller than %d.\n", MAXWIDTH);
	fprintf(fp0, "   -z:VALUE .... Set upper limit of Height of New Image.\n");
	fprintf(fp0, "                 If camera is flat, this is ignored. Value must be larger than\n");
	fprintf(fp0, "                 the height of original image, and smaller than %d.\n", MAXHEIGHT);
	fprintf(fp0, "   -mx ......... Do not ignore region without original image.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Change Pixel Size in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change Pixel Size in Z to \"VALUE\" [mm].\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
