/* Program to convert IP data file */
#define ProgName "ipfib"
#define ProgInfo "fiber image modification"
/*                                                         4/11/2002  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* Caution !! This program contains some bugs, that pixel far from centre
   is not correctly converted because pixel is no longer square.
   But I release this program, for a net correctness of the pattern is
   very important than to see the distorted image. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.14_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 16
#else
#	define MAXARG 15
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define FLAT 0
#define CURVE_V 1
#define CURVE_H 2
#define BACK 3    /* Not used */
#define CAMERA_DEFAULT 1.0
#define LAMBDA_DEFAULT 1.0
#define CAMERA_LONG 2000
#define BIGVALUE 30000.0
#define BIG_CRIT 20000.0
#define LIM_ANGLE 30.0
#define M_AV_W 2

#ifdef USE_DOUBLE
typedef double my_float;
#else
typedef float my_float;
#endif

#ifdef _OLDHP
	char xizetaFlat();
	char xizetaCurveV();
	char xizetaCurveH();
	double sinThetaFlat();
	double sinThetaCurveV();
	double sinThetaCurveH();
	char (*xizetaf)();
	double (*sinThetaf)();
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char xizetaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz);
	char xizetaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz);
	char xizetaCurveH(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz);
	double sinThetaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz);
	double sinThetaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz);
	double sinThetaCurveH(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz);
	char (*xizetaf)(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz);
	double (*sinThetaf)(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz);
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
#ifdef USE_DOUBLE
	short int (*writeDat)(const double *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
#else
	short int (*writeDat)(const float *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
#endif
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr, signx, signz, *signx0, *signx1;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, maxInt, readByte, flushWidth,
	     xpxlOld, offX, offZ, base, meridWidth, mrMin, mrMax, mrMid,
	     mrMin0, mrMax0, mrWid0, startZ, endZ, flushWidth0;
	char file1[FNAMELEN], file2[FNAMELEN], confirmBuffer1[8], confirmBuffer2[8],
	     inputBuffer[20], specimenOrientationFlag, cameraTypeFlag,
	     cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag, outMode, outModeDoneFlag,
	     shiftBit, strangeShiftBitFlag, changeValueTypeIndex,
	     byte, createDosFlag, verboseFlag, argNum, arg0[4],
	     cameraTypeDoneFlag, camrDoneFlag, drtxDoneFlag, drtzDoneFlag,
	     xsizDoneFlag, zsizDoneFlag, replaceFlag, setTimeFlag, lambdaDoneFlag,
	     fitFlag, headFlag, meridFlag, mdone, qxDoneFlag, qzDoneFlag, q0Flag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double intensityDbl, xDist, zDist, rato, camr, arg1, arg2, drtx, drtz, q_v, q_p,
	       q_vStep, q_pStep, lambda, xsiz, zsiz, specAngle, cosPhi, sinPhi,
	       f_x, f_z, minX, minZ, maxX, maxZ, f_p, f_v,
	       newx0, newx1, newz0, newz1, ratox, ratoz, qxMin, qzMin,
	       *dBuf_x0, *dBuf_x1, *dBuf_z0, *dBuf_z1, x_c, z_c, distx, distz;
	my_float *datPtrDbl;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	ipDataSwapFlag = cameraTypeDoneFlag = camrDoneFlag = lambdaDoneFlag
	= drtxDoneFlag = drtzDoneFlag = xsizDoneFlag = zsizDoneFlag = replaceFlag
	= setTimeFlag = specimenOrientationFlag = fitFlag = headFlag
	= outModeDoneFlag = meridFlag = qxDoneFlag = qzDoneFlag = q0Flag = 0;
	cameraTypeFlag = FLAT;
	specAngle = 0.0;
	readByte = 0;
	flushWidth = 0;
	minX = minZ = 0;
	outMode = 1;
	signx0 = signx1 = (short*)NULL;
	dBuf_x0 = dBuf_x1 = dBuf_z0 = dBuf_z1 = (double*)NULL;
	meridWidth = 1;
	offX = offZ = 0; /* dummy */
	mrMin = mrMax = mrMid = 1; /* dummy */
	mdone = 0;
	qxMin = qzMin = q_pStep = q_vStep = f_p = f_v = 1.0; /* dummy */
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag++;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = createDosFlag ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
#ifdef USE_DOUBLE
		writeDat = ipDtoShortBit0;
#else
		writeDat = ipFtoShortBit0;
#endif
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
#ifdef USE_DOUBLE
				writeDat = ipDtoShort;
#else
				writeDat = ipFtoShort;
#endif
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
#ifdef USE_DOUBLE
			writeDat = ipDtoShort;
#else
			writeDat = ipFtoShort;
#endif
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " IP size is : %6ld x %6ld.\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " CameraLength is : %7.3f mm, WaveLangth is : %8.5f A.\n", ipHeader.camr, ipHeader.wlng);
	}
	
/* Check the Header and Set Parameters : Interactive Mode */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z     , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (5) Camera Length, (6) WaveLength   ,\n");
				fprintf(stdout, "            (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					case 5:
						camrDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Camera Length [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.camr = atof(inputBuffer);
							if (ipHeader.camr <= 0.0) continue;
							camrDoneFlag = 1;
						} while (!camrDoneFlag);
						fprintf(stdout, "Camera Length is changed to %7.3f mm.\n", ipHeader.camr);
						break;
					case 6:
						lambdaDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for WaveLength [A] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.wlng = atof(inputBuffer);
							if (ipHeader.wlng <= 0.0) continue;
							lambdaDoneFlag = 1;
						} while (!lambdaDoneFlag);
						fprintf(stdout, "WaveLength is changed to %7.3f A.\n", ipHeader.wlng);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
		
/* Ask Camera Type */
		do {
			fprintf(stdout, "Camera is Flat[F] , Curved[C] , or Other[N] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'F') && (toupper(confirmBuffer1[0]) != 'C')
		      && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else if (toupper(confirmBuffer1[0]) == 'C') {
			cameraTypeFlag = CURVE_V;
			do {
				fprintf(stdout, "  Camera is Curved. Is Cylinder Axis Vertical on the Screen ? : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y')
			    fprintf(stdout, "  Camera axis is along Z-direction.\n");
			else {
				fprintf(stdout, "  Camera axis is along X-direction.\n");
				cameraTypeFlag = CURVE_H;
			}
		}
		else {
			fprintf(stdout, "  Camera is Unknwon.\n");
			fprintf(stderr, "  Sorry, only Flat and Curved camera is supported.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		
/* Ask on Specimen type and Orientation */
		do {
			fprintf(stdout, "Enter Orientation of the Specimen of Uniaxially Oriented.\n");
			fprintf(stdout, "    Angle is from Z-direction of the Screen with CW [deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			specAngle = atof(inputBuffer);
			if (fabs(specAngle) > LIM_ANGLE) continue;
			specimenOrientationFlag = 1;
		} while (!specimenOrientationFlag);
		
/* Modification Method */
		do {
			fprintf(stdout, " Use non-standard method of modification ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, " Enter Mode [1-3] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((confirmBuffer2[0] != '1') && (confirmBuffer2[0] != '1')
			      && (confirmBuffer2[0] != '3'));
			outMode = confirmBuffer2[0];
		}
		if (outMode == 1 || outMode == 2) {
			do {
				fprintf(stdout, " Show Meridian Pixels on meridian line as reference ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'Y') {
				do {
					fprintf(stdout, " Enter Width : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					meridWidth = atol(inputBuffer);
					if (meridWidth < 1) continue;
					meridFlag = 1;
				} while (!meridFlag);
			}
		}
		
/* xi-zeta step */
		if (cameraTypeFlag == CURVE_V) {
			qxMin = sin(ipHeader.xsiz/ipHeader.camr/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(atan(ipHeader.zsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
		}
		else if (cameraTypeFlag == CURVE_H) {
			qxMin = sin(atan(ipHeader.xsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(ipHeader.zsiz/ipHeader.camr/2.0)*2.0/ipHeader.wlng;
		}
		else {
			qxMin = sin(atan(ipHeader.xsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(atan(ipHeader.zsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
		}
		do {
			fprintf(stdout, " Do you want to use the same q-steps in X- and Z- direction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			q0Flag = 1;
			qxMin = MAXEXT(qxMin, qzMin);
			qzMin = qxMin;
			do {
				fprintf(stdout, "Enter q-step [1/A] larger than %8.5f.\n", qxMin);
				fprintf(stdout, " Type \"D\" if use Minimum Value : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') {
					q_pStep = q_vStep = qxMin;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				q_pStep = q_vStep = atof(inputBuffer);
				if (q_pStep < qxMin) continue;
				qxDoneFlag = qzDoneFlag = 1;
			} while (!qxDoneFlag);
		}
		else {
			do {
				fprintf(stdout, "Enter q-step in X [1/A] larger than %8.5f.\n", qxMin);
				fprintf(stdout, " Type \"D\" if use Minimum Value : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') {
					q_pStep = qxMin;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				q_pStep = atof(inputBuffer);
				if (q_pStep < qxMin) continue;
				qxDoneFlag = 1;
			} while (!qxDoneFlag);
			do {
				fprintf(stdout, "Enter q-step in Z [1/A] larger than %8.5f.\n", qzMin);
				fprintf(stdout, " Type \"D\" if use Minimum Value : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') {
					q_vStep = qzMin;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				q_vStep = atof(inputBuffer);
				if (q_vStep < qzMin) continue;
				qzDoneFlag = 1;
			} while (!qzDoneFlag);
		}
		
/* Image Size */
		do {
			fprintf(stdout, "Shrink Image Ignoring Surrounding white region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') fitFlag = 1;
		
/* Header Edit */
		do {
			fprintf(stdout, "Modify Header for good handling ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') headFlag = 1;
		
/* Question about Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			replaceFlag = 1;
			fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
		}
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			setTimeFlag = 1;
			fprintf(stdout, " Modification time of Output File is set as Original File.\n");
		}
#endif
	}
/* Check Arguments and Set Parameters : Batch Mode */
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-ch")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_H;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-s")) fitFlag++;
				else if (!strcmp(arg0, "-n")) headFlag++;
				else if (!strcmp(arg0, "-qe")) q0Flag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-u")) {
					specAngle = arg1;
					if (fabs(specAngle) > LIM_ANGLE) break;
					specimenOrientationFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-lm")) {
					if (arg1 <= 0.0) break;
					ipHeader.wlng = arg1;
					lambdaDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					outMode = dToLong(arg1);
					if (outMode != 1 && outMode != 2 && outMode != 3) break;
					outModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-sm")) {
					meridWidth = dToLong(arg1);
					if (meridWidth < 1) break;
					meridFlag++;
				}
				else if (!strcmp(arg0, "-qx")) {
					if (arg1 <= 0.0) break;
					q_pStep = arg1;
					qxDoneFlag++;
				}
				else if (!strcmp(arg0, "-qz")) {
					if (arg1 <= 0.0) break;
					q_vStep = arg1;
					qzDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((cameraTypeDoneFlag > 1) || (specimenOrientationFlag > 1)
		   || (camrDoneFlag > 1) || (drtxDoneFlag > 1) || (drtzDoneFlag > 1)
		   || (xsizDoneFlag > 1) || (zsizDoneFlag > 1) || (setTimeFlag > 1)
		   || (lambdaDoneFlag > 1) || (headFlag > 1) || (fitFlag > 1)
		   || (outModeDoneFlag > 1) || (meridFlag > 1) || (qxDoneFlag > 1)
		   || (qzDoneFlag > 1) || (q0Flag > 1)
		   || (q0Flag && (qxDoneFlag || qzDoneFlag))) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!cameraTypeDoneFlag) {
			if (ipHeader.cyld == 0) {
				if (verboseFlag >= 2) fprintf(stdout, " Assume CameraType is Flat.\n");
				cameraTypeFlag = FLAT;
			}
			else if (ipHeader.cyld == 1) {
				if (verboseFlag >= 2) fprintf(stdout, " Assume CameraType is Curved with Cylinder Axis Vertical.\n");
				cameraTypeFlag = CURVE_V;
			}
			else {
				fprintf(stderr, "Camera Type is unknown.\n");
				showUsage(stderr);
				fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		if (!specimenOrientationFlag) {
			if (verboseFlag >= 2) fprintf(stdout, " Assume Specimen is Oriented along Z-Axis.\n");
			specAngle = 0.0;
		}
		if (meridFlag && outMode == 3) {
			fprintf(stdout, " Invalid Option : Impossible showing meridian pixels with this mode.\n");
			meridFlag = 0;
		}
	}
	if (cameraTypeFlag == CURVE_V
	 && ((ipHeader.drtx - ipHeader.camr*M_PI/ipHeader.xsiz >= 0.0)
	   || (ipHeader.drtx + ipHeader.camr*M_PI/ipHeader.xsiz <= ipHeader.xpxl))) {
		fprintf(stderr, " Curved Camera \"Width\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (cameraTypeFlag == CURVE_H
	 && ((ipHeader.drtz - ipHeader.camr*M_PI/ipHeader.zsiz >= 0.0)
	   || (ipHeader.drtz + ipHeader.camr*M_PI/ipHeader.zsiz <= ipHeader.zpxl))) {
		fprintf(stderr, " Curved Camera \"Height\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* Definition of some functions */
	switch (cameraTypeFlag) {
		case FLAT:
			xizetaf = xizetaFlat;
			sinThetaf = sinThetaFlat;
			break;
		case CURVE_V:
			xizetaf = xizetaCurveV;
			sinThetaf = sinThetaCurveV;
			break;
		case CURVE_H:
			xizetaf = xizetaCurveH;
			sinThetaf = sinThetaCurveH;
			break;
		default:
			break;
			/* NOT REACHED */
	}
	lambda = ipHeader.wlng;
	cosPhi = cos(radEXT(specAngle));
	sinPhi = sin(radEXT(specAngle));
	if (verboseFlag != 3) {
		if (cameraTypeFlag == CURVE_V) {
			qxMin = sin(ipHeader.xsiz/ipHeader.camr/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(atan(ipHeader.zsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
		}
		else if (cameraTypeFlag == CURVE_H) {
			qxMin = sin(atan(ipHeader.xsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(ipHeader.zsiz/ipHeader.camr/2.0)*2.0/ipHeader.wlng;
		}
		else {
			qxMin = sin(atan(ipHeader.xsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
			qzMin = sin(atan(ipHeader.zsiz/ipHeader.camr)/2.0)*2.0/ipHeader.wlng;
		}
		if (!qxDoneFlag) {
			q_pStep = qxMin;
		}
		else {
			if (q_pStep < qxMin) {
				fprintf(stderr, " Invalid q-step in X : %8.5f [1/A].\n", q_pStep);
				fprintf(stderr, "  Value must be larger than %8.5f [1/A]. Stop.\n", qxMin);
				showUsage(stderr);
				fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		if (!qzDoneFlag) {
			q_vStep = qzMin;
		}
		else {
			if (q_vStep < qzMin) {
				fprintf(stderr, " Invalid q-step in Z : %8.5f [1/A].\n", q_vStep);
				fprintf(stderr, "  Value must be larger than %8.5f [1/A]. Stop.\n", qzMin);
				showUsage(stderr);
				fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		if (q0Flag) {
			q_pStep = MAXEXT(q_pStep, q_vStep);
			q_vStep = q_pStep;
		}
	}
	if (ipHeader.drtx < 0.0 || ipHeader.drtx >= ipHeader.xpxl
	 || ipHeader.drtz < 0.0 || ipHeader.drtz >= ipHeader.zpxl) {
		fprintf(stderr, " Image Centre is out of image. Set Centre correctly. Exit.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	specAngle = dToLong(specAngle/360.0)*360.0+specAngle;
	startZ = 0;
	endZ = ipHeader.zpxl;
	if (verboseFlag == 2) {
		if (drtxDoneFlag || drtzDoneFlag)
		    fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (xsizDoneFlag || zsizDoneFlag)
		    fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (camrDoneFlag)
		    fprintf(stdout, " CameraLength has changed to : %7.3f mm.\n", ipHeader.camr);
		if (lambdaDoneFlag)
		    fprintf(stdout, " WaveLength has changed to : %8.5f [A].\n", ipHeader.wlng);
		if (cameraTypeFlag == FLAT) fprintf(stdout, " Camera is Flat.\n");
		else if (cameraTypeFlag == CURVE_V)
		    fprintf(stdout, " Camera is Cylindrical along Z-direction.\n");
		else if (cameraTypeFlag == CURVE_H) {
			fprintf(stdout, " Camera is Cylindrical along X-direction.\n");
			startZ = MAXEXT((long)floor(ipHeader.drtz-M_PI_2*ipHeader.camr/ipHeader.zsiz), 0);
			endZ = MINEXT((long)floor(ipHeader.drtz+M_PI_2*ipHeader.camr/ipHeader.zsiz), ipHeader.zpxl);
			if (startZ != 0 || endZ != ipHeader.zpxl) {
				fprintf(stdout, "  Backward diffraction is ignored.\n");
				fprintf(stdout, "  Pixels from %ld and %ld in Z-direction is used.\n", startZ, endZ);
			}
		}
		fprintf(stdout, " Specimen is Uniaxially oriented to %.2f degree direction.\n", specAngle);
		fprintf(stdout, " Output PixelSize is %10.8f [1/A] x %10.8f [1/A].\n", q_pStep, q_vStep);
		if (fitFlag) fprintf(stdout, " Image size will be shrinked.\n");
		if (meridFlag) {
			fprintf(stdout, " %ld-Width Pixels containing meridian line are dummy.\n", meridWidth);
			fprintf(stdout, "  These are reference for layer.\n");
		}
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag)
		    fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	ipHeader.cyld = 0;
	xpxlOld = ipHeader.xpxl;
	drtx = ipHeader.drtx;
	drtz = ipHeader.drtz;
	camr = ipHeader.camr;
	xsiz = ipHeader.xsiz;
	zsiz = ipHeader.zsiz;
	f_x = xsiz / camr;
	f_z = zsiz / camr;
	f_p = 1.0/lambda/q_pStep;
	f_v = 1.0/lambda/q_vStep;
	if (headFlag) {
		ipHeader.camr = CAMERA_LONG;
		ipHeader.wlng *= (camr/CAMERA_LONG);
		ipHeader.xsiz = xsiz*q_pStep/qxMin;
		ipHeader.zsiz = zsiz*q_vStep/qzMin;
		if (verboseFlag >= 2) {
			fprintf(stdout, "Caution!! : Output Image Header has some ill values. See below.\n");
			fprintf(stdout, " Dummy CameraLength has changed to : %8.1f mm.\n", ipHeader.camr);
			fprintf(stdout, " Dummy WaveLength has changed to : %10.7f [A].\n", ipHeader.wlng);
			fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
			fprintf(stdout, " q-step in X: %10.8f [1/A], Z: %10.8f [1/A].\n", q_pStep, q_vStep);
			fprintf(stdout, "== Long CameraLength makes you possible to get approximate q-value.==\n");
			fprintf(stdout, "== These Values are not correct. Treat with care. ==\n");
		}
	}
	else {
		ipHeader.camr = CAMERA_DEFAULT;
		ipHeader.wlng = LAMBDA_DEFAULT;
		ipHeader.xsiz = q_pStep;
		ipHeader.zsiz = q_vStep;
		if (verboseFlag >= 2) {
			fprintf(stdout, "Caution!! : Output Image Header has some ill values. See below.\n");
			fprintf(stdout, " CameraLength has changed to : %8.1f mm.\n", ipHeader.camr);
			fprintf(stdout, " WaveLength has changed to : %10.7f [A].\n", ipHeader.wlng);
			fprintf(stdout, " Pixelsize has changed to : %10.8f mm x %10.8f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
			fprintf(stdout, "== Here, pixelsize shows q-step size in 1/A. ==\n");
			fprintf(stdout, "== These Values are not correct. Treat with care. ==\n");
		}
	}
	if (fitFlag) {
		minX = minZ = 0.0;
		maxX = maxZ = 0.0;
		zDist = (startZ-drtz) * f_z;
		for (j=0; j<=xpxlOld; j++) {
			xDist = (j-drtx) * f_x;
			(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, &signx, &signz);
			q_p *= (signx/lambda);
			q_v *= (signz/lambda);
			if (q_v < minZ) minZ = q_v;
			else if (q_v > maxZ) maxZ = q_v;
			if (q_p < minX) minX = q_p;
			else if (q_p > maxX) maxX = q_p;
		}
		zDist = (endZ-drtz) * f_z;
		for (j=0; j<=xpxlOld; j++) {
			xDist = (j-drtx) * f_x;
			(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, &signx, &signz);
			q_p *= (signx/lambda);
			q_v *= (signz/lambda);
			if (q_v < minZ) minZ = q_v;
			else if (q_v > maxZ) maxZ = q_v;
			if (q_p < minX) minX = q_p;
			else if (q_p > maxX) maxX = q_p;
		}
		xDist = (-drtx) * f_x;
		for (i=startZ+1; i<endZ; i++) {
			zDist = (i-drtz) * f_z;
			(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, &signx, &signz);
			q_p *= (signx/lambda);
			q_v *= (signz/lambda);
			if (q_v < minZ) minZ = q_v;
			else if (q_v > maxZ) maxZ = q_v;
			if (q_p < minX) minX = q_p;
			else if (q_p > maxX) maxX = q_p;
		}
		xDist = (xpxlOld-drtx) * f_x;
		for (i=startZ+1; i<endZ; i++) {
			zDist = (i-drtz) * f_z;
			(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, &signx, &signz);
			q_p *= (signx/lambda);
			q_v *= (signz/lambda);
			if (q_v < minZ) minZ = q_v;
			else if (q_v > maxZ) maxZ = q_v;
			if (q_p < minX) minX = q_p;
			else if (q_p > maxX) maxX = q_p;
		}
		offX = (long)floor(minX/q_pStep+ipHeader.drtx)-1;
		offZ = (long)floor(minZ/q_vStep+ipHeader.drtz)-1;
		ipHeader.xpxl = (long)ceil(maxX/q_pStep+ipHeader.drtx)+1 - offX;
		ipHeader.zpxl = (long)ceil(maxZ/q_vStep+ipHeader.drtz)+1 - offZ;
		ipHeader.drtx -= offX;
		ipHeader.drtz -= offZ;
	}
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	if (meridFlag) {
		mrMid = (long)floor(ipHeader.drtx);
		mrMin = MAXEXT(mrMid - (long)floor(meridWidth/2.0), 0);
		mrMax = MINEXT(mrMid + (long)ceil(meridWidth/2.0), ipHeader.xpxl);
	}
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2)
	              : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength*(startZ+1), SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((datPtrDbl = calloc(ipHeader.xpxl*ipHeader.zpxl, sizeof(my_float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
	}
	if (outMode == 1 || outMode == 2) {
		if ((dBuf_x0 = malloc((xpxlOld+1)*sizeof(double))) == NULL) { /* lower left x-coordinate */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((dBuf_x1 = malloc((xpxlOld+1)*sizeof(double))) == NULL) { /* upper left x-coordinate */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			free(dBuf_x0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((dBuf_z0 = malloc((xpxlOld+1)*sizeof(double))) == NULL) { /* lower left z-coordinate */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			free(dBuf_x0);
			free(dBuf_x1);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((dBuf_z1 = malloc((xpxlOld+1)*sizeof(double))) == NULL) { /* upper left z-coordinate */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			free(dBuf_x0);
			free(dBuf_x1);
			free(dBuf_z0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((signx0 = malloc((xpxlOld+1)*RAXDATAWIDTH)) == NULL) { /* lower x-sign */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			free(dBuf_x0);
			free(dBuf_x1);
			free(dBuf_z0);
			free(dBuf_z1);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((signx1 = malloc((xpxlOld+1)*RAXDATAWIDTH)) == NULL) { /* upper x-sign */
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			free(dBuf_x0);
			free(dBuf_x1);
			free(dBuf_z0);
			free(dBuf_z1);
			free(signx0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		zDist = (startZ-drtz) * f_z;
		for (j=0; j<=xpxlOld; j++) {
			xDist = (j-drtx) * f_x;
			(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, signx1+j, &signz);
			dBuf_x1[j] = q_p*f_p*signx1[j] + ipHeader.drtx;
			dBuf_z1[j] = q_v*f_v*signz + ipHeader.drtz;
		}
	}
	for (i=startZ; i<endZ; i++) {
		mdone = 0;
		if ((datPtr = malloc(recordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			free(datPtrDbl);
			if (outMode == 1 || outMode == 2) {
				free(dBuf_x0);
				free(dBuf_x1);
				free(dBuf_z0);
				free(dBuf_z1);
				free(signx0);
				free(signx1);
			}
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		fread(datPtr, recordLength, 1, fp1);
		readByte += recordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			if (flushWidth0 > 0) free(datPtr0);
			if (outMode == 1 || outMode == 2) {
				free(dBuf_x0);
				free(dBuf_x1);
				free(dBuf_z0);
				free(dBuf_z1);
				free(signx0);
				free(signx1);
			}
			free(datPtrDbl);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		switch (outMode) {
			case 3: /* Use coordinate of the centre of pixel */
				zDist = (i+0.5-drtz) * f_z;
				for (j=0; j<xpxlOld; j++) {
					xDist = (j+0.5-drtx) * f_x;
					(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, &signx, &signz);
					q_p *= (signx/lambda);
					q_v *= (signz/lambda);
					datPtrDbl[ipHeader.xpxl*(long)floor(q_v/q_vStep+ipHeader.drtz)+(long)floor(q_p/q_pStep+ipHeader.drtx)]
					    += (my_float)(*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				}
				break;
			case 2: /* Use coordinate of a little smaller pixel, not square */
				zDist = (i+1-drtz) * f_z;
				for (j=0; j<=xpxlOld; j++) {
					dBuf_z0[j] = dBuf_z1[j];
					dBuf_x0[j] = dBuf_x1[j];
					signx0[j] = signx1[j];
					xDist = (j-drtx) * f_x;
					(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, signx1+j, &signz);
					dBuf_x1[j] = q_p*f_p*signx1[j] + ipHeader.drtx;
					dBuf_z1[j] = q_v*f_v*signz + ipHeader.drtz;
				}
				for (j=0; j<xpxlOld; j++) {
					/* pixels concerning meridian line are ignored for simplicity of the code */
					if (signx0[j] != signx0[j+1] || signx1[j] != signx1[j+1] || signx0[j] != signx1[j]) {
						if (meridFlag && !mdone) {
							xDist = (j+0.5-drtx) * f_x;
							q_v = (i-drtz) * f_z;
							newz0 = (*sinThetaf)(&xDist, &q_v, &cosPhi, &sinPhi, &signz);
							newz0 = newz0*f_v*signz + ipHeader.drtz;
							if (newz0 < 0 || newz0 > ipHeader.zpxl-2) continue;
							q_v = (i+1-drtz) * f_z;
							newz1 = (*sinThetaf)(&xDist, &q_v, &cosPhi, &sinPhi, &signz);
							newz1 = newz1*f_v*signz + ipHeader.drtz;
							intensityDbl = 0.0;
							mrMin0 = MAXEXT(j - M_AV_W, 0);
							mrMax0 = MINEXT(j + M_AV_W, ipHeader.xpxl);
							mrWid0 = mrMax0 - mrMin0;
							base = (long)floor(newz1)*ipHeader.xpxl+mrMid;
							for (k=mrMin0; k<mrMax0; k++)
							    intensityDbl += (*readDat)(&datPtr[k], &rato, &shiftBit, &ipDataSwapFlag);
							intensityDbl /= mrWid0;
							if ((long)floor(newz1) == (long)floor(newz0)) datPtrDbl[base] += intensityDbl;
							else {
								ratoz = (newz1 - floor(newz1)) / (newz1-newz0);
								datPtrDbl[base              ] += intensityDbl * ratoz;
								datPtrDbl[base-ipHeader.xpxl] += intensityDbl * (1.0-ratoz);
							}
							mdone = 1;
						}
						continue;
					}
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					newx0 = (dBuf_x0[  j]+dBuf_x1[  j])/2.0; /* average of left X-coordinate Up and Down */
					newx1 = (dBuf_x0[j+1]+dBuf_x1[j+1])/2.0; /* average of right X-coordinate Up and Down */
					newz0 = (dBuf_z0[  j]+dBuf_z0[j+1])/2.0; /* average of Lower Z-coordinate Left and Right */
					newz1 = (dBuf_z1[  j]+dBuf_z1[j+1])/2.0; /* average of Upper Z-coordinate Left and Right */
					base = (long)floor(newz1)*ipHeader.xpxl+(long)floor(newx1);
					if ((long)floor(newx0) == (long)floor(newx1)) ratox = 1.0;
					else ratox = (newx1 - floor(newx1)) / (newx1-newx0);
					if ((long)floor(newz0) == (long)floor(newz1)) {
						datPtrDbl[base           ] += intensityDbl * ratox;
						datPtrDbl[base-1         ] += intensityDbl * (1.0-ratox);
					}
					else {
						ratoz = (newz1 - floor(newz1)) / (newz1-newz0);
						datPtrDbl[base                ] += intensityDbl * ratox * ratoz;
						datPtrDbl[base-1              ] += intensityDbl * (1.0-ratox) * ratoz;
						datPtrDbl[base-ipHeader.xpxl  ] += intensityDbl * ratox * (1.0-ratoz);
						datPtrDbl[base-ipHeader.xpxl-1] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					}
				}
				break;
			case 1: /* Use coordinate of the rectangle pixel */
				zDist = (i+1-drtz) * f_z;
				for (j=0; j<=xpxlOld; j++) {
					dBuf_z0[j] = dBuf_z1[j];
					dBuf_x0[j] = dBuf_x1[j];
					signx0[j] = signx1[j];
					xDist = (j-drtx) * f_x;
					(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &q_p, &q_v, signx1+j, &signz);
					dBuf_x1[j] = q_p*f_p*signx1[j] + ipHeader.drtx;
					dBuf_z1[j] = q_v*f_v*signz + ipHeader.drtz;
				}
				zDist = (i+0.5-drtz) * f_z;
				for (j=0; j<xpxlOld; j++) {
					/* pixels concerning meridian line are ignored for simplicity of the code */
					if (signx0[j] != signx0[j+1] || signx1[j] != signx1[j+1] || signx0[j] != signx1[j]) {
						if (meridFlag && !mdone) {
							xDist = (j+0.5-drtx) * f_x;
							q_v = (i-drtz) * f_z;
							newz0 = (*sinThetaf)(&xDist, &q_v, &cosPhi, &sinPhi, &signz);
							newz0 = newz0*f_v*signz + ipHeader.drtz;
							if (newz0 < 0 || newz0 > ipHeader.zpxl-2) continue;
							q_v = (i+1-drtz) * f_z;
							newz1 = (*sinThetaf)(&xDist, &q_v, &cosPhi, &sinPhi, &signz);
							newz1 = newz1*f_v*signz + ipHeader.drtz;
							intensityDbl = 0.0;
							mrMin0 = MAXEXT(j - M_AV_W, 0);
							mrMax0 = MINEXT(j + M_AV_W, ipHeader.xpxl);
							mrWid0 = mrMax0 - mrMin0;
							base = (long)floor(newz1)*ipHeader.xpxl+mrMid;
							for (k=mrMin0; k<mrMax0; k++)
							    intensityDbl += (*readDat)(&datPtr[k], &rato, &shiftBit, &ipDataSwapFlag);
							intensityDbl /= mrWid0;
							if ((long)floor(newz1) == (long)floor(newz0)) datPtrDbl[base] += intensityDbl;
							else {
								ratoz = (newz1 - floor(newz1)) / (newz1-newz0);
								datPtrDbl[base              ] += intensityDbl * ratoz;
								datPtrDbl[base-ipHeader.xpxl] += intensityDbl * (1.0-ratoz);
							}
							mdone = 1;
						}
						continue;
					}
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					xDist = (j+0.5-drtx) * f_x;
					(*xizetaf)(&xDist, &zDist, &cosPhi, &sinPhi, &x_c, &z_c, &signx, &signz);
					x_c = x_c*f_p*signx + ipHeader.drtx;
					z_c = z_c*f_v*signz + ipHeader.drtz;
					distx = (dist(dBuf_x0[  j]-dBuf_x0[j+1], dBuf_z0[  j]-dBuf_z0[j+1])
					         + dist(dBuf_x1[  j]-dBuf_x1[j+1], dBuf_z1[  j]-dBuf_z1[j+1])) / 4.0;
					distz = (dist(dBuf_x0[  j]-dBuf_x1[  j], dBuf_z0[  j]-dBuf_z1[  j])
					         + dist(dBuf_x0[j+1]-dBuf_x1[j+1], dBuf_z0[j+1]-dBuf_z1[j+1])) / 4.0;
					newx0 = x_c - distx; /* average of left X-coordinate Up and Down */
					newx1 = x_c + distx; /* average of right X-coordinate Up and Down */
					newz0 = z_c - distz; /* average of Lower Z-coordinate Left and Right */
					newz1 = z_c + distz; /* average of Upper Z-coordinate Left and Right */
					base = (long)floor(newz1)*ipHeader.xpxl+(long)floor(newx1);
					if ((long)floor(newx0) == (long)floor(newx1)) ratox = 1.0;
					else ratox = (newx1 - floor(newx1)) / (newx1-newx0);
					if ((long)floor(newz0) == (long)floor(newz1)) {
						datPtrDbl[base           ] += intensityDbl * ratox;
						datPtrDbl[base-1         ] += intensityDbl * (1.0-ratox);
					}
					else {
						ratoz = (newz1 - floor(newz1)) / (newz1-newz0);
						datPtrDbl[base                ] += intensityDbl * ratox * ratoz;
						datPtrDbl[base-1              ] += intensityDbl * (1.0-ratox) * ratoz;
						datPtrDbl[base-ipHeader.xpxl  ] += intensityDbl * ratox * (1.0-ratoz);
						datPtrDbl[base-ipHeader.xpxl-1] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					}
				}
				break;
			default:
				break;
				/*NOT REACHED */
		}
		free(datPtr);
	}
	fclose(fp1);
	if (outMode == 1 || outMode == 2) {
		free(dBuf_x0);
		free(dBuf_x1);
		free(dBuf_z0);
		free(dBuf_z1);
		free(signx0);
		free(signx1);
	}
	for (i=0, base=0; i<ipHeader.rnum; i++, base+=ipHeader.xpxl) {
		if ((datPtr = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrDbl);
			if (flushWidth0 > 0) free(datPtr0);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++)
		    datPtr[j] = (*writeDat)(&(datPtrDbl[base+j]), &rato, &maxInt, &ipDataSwapFlag);
		if (meridFlag) {
			for (j=mrMin; j<mrMax; j++) {
				if (datPtr[j] == 0) datPtr[j] = datPtr[mrMid];
			}
		}
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	free(datPtrDbl);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char xizetaFlat(x, z, cosPhi, sinPhi, xi, zeta, signx, signz)
	double *x, *z, *cosPhi, *sinPhi, *xi, *zeta;
	short int *signx, *signz;
#else
char xizetaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz)
#endif
{
	double denom, denom2, denomS, xsq, zsq, x2, z2;
	/* z = z/camr , x = x/camr */
	x2 = (*cosPhi)*(*x) - (*sinPhi)*(*z);
	z2 = (*sinPhi)*(*x) + (*cosPhi)*(*z);
	xsq = x2 * x2;
	zsq = z2 * z2;
	denom = (*x)*(*x) + (*z)*(*z) + 1.0;
	denomS = sqrt(denom);
	denom2 = denom + 2.0*denomS + xsq + 1.0;
	*xi = sqrt((4.0*xsq + zsq*zsq/denom) / denom2);
	*zeta = fabs(z2) / denomS;
	(*signx) = signEXT(x2);
	(*signz) = signEXT(z2);
	return (0);
}

#ifdef _OLDHP
char xizetaCurveV(x, z, cosPhi, sinPhi, xi, zeta, signx, signz)
	double *x, *z, *cosPhi, *sinPhi, *xi, *zeta;
	short int *signx, *signz;
#else
char xizetaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz)
#endif
{
	double denom, denomS, zsqn, x2, z2, cx, sinx;
	/* z = z/camr , x = x/camr */
	sinx = sin(*x);
	x2 = (*cosPhi)*sinx - (*sinPhi)*(*z);
	z2 = (*sinPhi)*sinx + (*cosPhi)*(*z);
	denom = (*z)*(*z) + 1.0;
	denomS = sqrt(denom);
	zsqn = z2 * z2;
	if ((cx = cos(*x)) < 0.0) *xi = sqrt(2.0 - 2.0*cx/denomS - zsqn/denom);
	else *xi = sqrt((4.0*x2*x2 + zsqn*zsqn/denom) / (2.0*(denom+denomS*cx) - zsqn));
	*zeta = fabs(z2) / denomS;
	(*signx) = signEXT(x2);
	(*signz) = signEXT(z2);
	return (0);
}

#ifdef _OLDHP
char xizetaCurveH(x, z, cosPhi, sinPhi, xi, zeta, signx, signz)
	double *x, *z, *cosPhi, *sinPhi, *xi, *zeta;
	short int *signx, *signz;
#else
char xizetaCurveH(const double *x, const double *z, const double *cosPhi, const double *sinPhi, double *xi, double *zeta, short int *signx, short int *signz)
#endif
{
	double denom, denomS, zsqn, x2, z2, sinz;
	/* z = z/camr , x = x/camr */
	sinz = sin(*z);
	x2 = (*cosPhi)*(*x) - (*sinPhi)*sinz;
	z2 = (*sinPhi)*(*x) + (*cosPhi)*sinz;
	denom = (*x)*(*x) + 1.0;
	denomS = sqrt(denom);
	zsqn = z2 * z2;
	*xi = sqrt((4.0*x2*x2 + zsqn*zsqn/denom) / (2.0*(denom+denomS*cos(*z)) - zsqn));
	*zeta = fabs(z2) / denomS;
	(*signx) = signEXT(x2);
	(*signz) = signEXT(z2);
	return (0);
}

#ifdef _OLDHP
double sinThetaFlat(x, z, cosPhi, sinPhi, signz)
	double *x, *z, *cosPhi, *sinPhi;
	short int *signz;
#else
double sinThetaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz)
#endif
{
	/* these functions return 2*sin(theta) (not 2theta) */
	double denom, numer;
	/* z = z/camr , x = x/camr */
	(*signz) = signEXT((*sinPhi)*(*x) + (*cosPhi)*(*z));
	numer = (*x)*(*x) + (*z)*(*z);
	denom = numer + 1.0;
	return (sqrt(numer*2.0/(denom+sqrt(denom))));
}

#ifdef _OLDHP
double sinThetaCurveV(x, z, cosPhi, sinPhi, signz)
	double *x, *z, *cosPhi, *sinPhi;
	short int *signz;
#else
double sinThetaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz)
#endif
{
	double denom, numer, xsqn, cx;
	/* z = z/camr , x = x/camr */
	(*signz) = signEXT((*sinPhi)*sin(*x) + (*cosPhi)*(*z));
	if ((cx = cos(*x)) < 0.0) return (sqrt(2.0 - 2.0*cx/sqrt(1.0+(*z)*(*z))));
	else {
		xsqn = sin(*x);
		xsqn *= xsqn;
		numer = (*z)*(*z);
		denom = numer + 1.0;
		return (sqrt((numer+xsqn)*2.0/(denom+sqrt(denom)*cx)));
	}
}

#ifdef _OLDHP
double sinThetaCurveH(x, z, cosPhi, sinPhi, signz)
	double *x, *z, *cosPhi, *sinPhi;
	short int *signz;
#else
double sinThetaCurveH(const double *x, const double *z, const double *cosPhi, const double *sinPhi, short int *signz)
#endif
{
	double denom, numer, zsqn;
	/* z = z/camr , x = x/camr */
	(*signz) = signEXT((*sinPhi)*(*x) + (*cosPhi)*sin(*z));
	zsqn = sin(*z);
	zsqn *= zsqn;
	numer = (*x)*(*x);
	denom = numer + 1.0;
	return (sqrt((numer+zsqn)*2.0/(denom+sqrt(denom)*cos(*z))));
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-f|-cv|-ch] [-u:VALUE] [-s] [-n]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-f|-cv|-ch] [-u:VALUE] [-s] [-n]\n", ProgName);
#endif
	fprintf(fp0, "          [-lm:VALUE] [-cl:VALUE] [-xc:VALUE] [-zc:VALUE] [-sm:VALUE]\n");
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-m:VALUE] [-qx:VALUE] [-qz:VALUE]\n");
	fprintf(fp0, "          [-qe] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program modifies R-AXIS Image of Fiber Pattern into xi-zeta map.\n");
	fprintf(fp0, "  Caution : Generated image has no longer correct dimension in length,\n");
	fprintf(fp0, "            like pixel size, camera length, etc.\n");
	fprintf(fp0, "            This is useful only for visualization.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File. This must be set\n");
	fprintf(fp0, "                 before other arguments listed below.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	fprintf(fp0, "   -f .......... Assume CameraType is Flat.\n");
	fprintf(fp0, "   -cv ......... Assume CameraType is Cylindrical, whose axis is vertical\n");
	fprintf(fp0, "                 to the screen.\n");
	fprintf(fp0, "   -ch ......... Assume CameraType is Cylindrical, whose axis is horizontal\n");
	fprintf(fp0, "                 to the screen.\n");
	fprintf(fp0, "   -u:VALUE .... Set Uniaxially Oriented Direction-axis at \"VALUE\" degrees\n");
	fprintf(fp0, "                 from upper vertical direction, with CW.\n");
	fprintf(fp0, "                 Absolute of this angle must be smaller than %4.1f.\n", LIM_ANGLE);
	fprintf(fp0, "                 Use \"iprot\" to correct setting angle.\n");
	fprintf(fp0, "   -s .......... Ignore white region caused by shrinkage of the image.\n");
	fprintf(fp0, "   -n .......... Make camera length long and wave length short.\n");
	fprintf(fp0, "                 This improves image header information.\n");
	fprintf(fp0, "   -m:VALUE .... Set Modification Method to \"VALUE\"(1-3).\n");
	fprintf(fp0, "                  1 : Rectangular Approximation of area-preserve.\n");
	fprintf(fp0, "                  2 : Rectangular Approximation of shrinked area-preserve.\n");
	fprintf(fp0, "                  3 : Only Center of Pixel Approximation (Makes ill Image).\n");
	fprintf(fp0, "                 If absent, set to 1.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Change Pixel Size in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change Pixel Size in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -lm:VALUE ... Change wavelength to \"VALUE\" [A].\n");
	fprintf(fp0, "   -qx:VALUE ... Set PixelSize in X of New Image to \"VALUE\" [1/A].\n");
	fprintf(fp0, "   -qz:VALUE ... Set PixelSize in Z of New Image to \"VALUE\" [1/A].\n");
	fprintf(fp0, "                 These are correct values in calculation.\n");
	fprintf(fp0, "                 These must be larger than q-width of a pixel around centre.\n");
	fprintf(fp0, "                 If absent, use the value above.\n");
	fprintf(fp0, "   -qe ......... Try to use default q-step (that is PixelSize), but use the\n");
	fprintf(fp0, "                 same value in X- and Z- direction. Smaller one is changed to\n");
	fprintf(fp0, "                 the larger. Exclusive with the above 2 options.\n");
	fprintf(fp0, "   -sm:VALUE ... Show Meridian pixels along meridian with \"VALUE\" width.\n");
	fprintf(fp0, "                 This is merely the reference of fiber-spacing.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
