/* Program to treat IP data file */
#define ProgName "ipline"
#define ProgInfo "get line profile"
/*                                                          5/2/2002  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.04_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 14
#define MINARG 1
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define MAXLINE 100000

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1;
	long int i, ipHeaderSize, maxInt, startZn, line0, xn, zn,
	     ipHeaderSize2VER1, ipHeaderSize2VER2, readByte, num, numbyte, lines;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, arg0[4], verboseFlag, xsDoneFlag,
	     zsDoneFlag, xfDoneFlag, zfDoneFlag, stepDoneFlag, ofileDoneFlag,
	     noHeadFlag, noHeadDoneFlag, showPosFlag, showInfoFlag, timebuf[20],
	     szf, szf2, revFlag, xsizDoneFlag, zsizDoneFlag, lightMode, drtxDoneFlag,
	     drtzDoneFlag, phiDoneFlag, showDistFlag;
	double rato, step, res, startX, endX, startZ, endZ, phi, sinPhi, cosPhi,
	       ratox, x, z, arg1, arg2;
	float *outbuf, *posbuf;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	time_t timeNow;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = xsDoneFlag = zsDoneFlag = xfDoneFlag = zfDoneFlag = stepDoneFlag
	 = ofileDoneFlag = noHeadFlag = noHeadDoneFlag = showPosFlag
	 = showInfoFlag = xsizDoneFlag = zsizDoneFlag = drtxDoneFlag = drtzDoneFlag
	 = phiDoneFlag = showDistFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	numbyte = 0;
	lines = 0;
	revFlag = 0;
	szf = szf2 = 0;
	phi = 0.0;
	lightMode = 0;
	startX = startZ = endX = endZ = 0;
	posbuf = (float*)NULL;
	datPtr1 = (short*)NULL;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unabel to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
		
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	step = MAXEXT(ipHeader.xsiz, ipHeader.zsiz);
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter X-coordinate of Start Pixel to get profile : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			startX = atof(inputBuffer);
			if (startX < 0.0 || startX > ipHeader.xpxl) continue;
			xsDoneFlag = 1;
		} while (!xsDoneFlag);
		do {
			fprintf(stdout, "Enter X-coordinate of End Pixel to get profile : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			endX = atof(inputBuffer);
			if (endX < 0.0 || endX > ipHeader.xpxl) continue;
			xfDoneFlag = 1;
		} while (!xfDoneFlag);
		do {
			fprintf(stdout, "Enter Z-coordinate of Start Pixel to get profile : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			startZ = atof(inputBuffer);
			if (startZ < 0.0 || startZ > ipHeader.zpxl) continue;
			zsDoneFlag = 1;
		} while (!zsDoneFlag);
		do {
			fprintf(stdout, "Enter Z-coordinate of End Pixel to get profile : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			endZ = atof(inputBuffer);
			if (endZ < 0.0 || endZ > ipHeader.zpxl || dist(endX-startX, endZ-startZ) == 0.0) continue;
			zfDoneFlag = 1;
		} while (!zfDoneFlag);
		do {
			fprintf(stdout, " PixelSize in X is %7.5f [mm]. Do you want to change ? [Y/N] : \n", ipHeader.xsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Value for Pixel Size in X [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.xsiz = atof(inputBuffer);
				if (ipHeader.xsiz <= 0.0) continue;
				xsizDoneFlag = 1;
			} while (!xsizDoneFlag);
		}
		do {
			fprintf(stdout, " PixelSize in Z is %7.5f [mm]. Do you want to change ? [Y/N] : \n", ipHeader.zsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Value for Pixel Size in Z [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.zsiz = atof(inputBuffer);
				if (ipHeader.zsiz <= 0.0) continue;
				zsizDoneFlag = 1;
			} while (!zsizDoneFlag);
		}
		do {
			fprintf(stdout, " Enter Step of profile [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			step = atof(inputBuffer);
			if (step <= 0.0) continue;
			stepDoneFlag = 1;
		} while (!stepDoneFlag);
		do {
			fprintf(stdout, " Do you want to print coordinate for the each intensity ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') showPosFlag = 1;
		do {
			fprintf(stdout, " Do you want to print relative distance for the each intensity ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') showDistFlag = 1;
		do {
			fprintf(stdout, " Do you want to get profile with simple sampling ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') lightMode = 1;
	}
	else {
		for (i=1; i<argc; i++) {
			if (!strncmp(argv[i], "-o:", 3)) {
				strcpy(file2, argv[i]+3);
				ofileDoneFlag++;
			}
			else {
				argNum = splitArg(argv[i], arg0, &arg1, &arg2);
				if (argNum == 1) {
					if (!strcmp(arg0, "-sn")) showPosFlag++;
					else if (!strcmp(arg0, "-sh")) {
						noHeadFlag = 1;
						noHeadDoneFlag++;
					}
					else if (!strcmp(arg0, "-sH")) {
						noHeadFlag = 2;
						noHeadDoneFlag++;
					}
					else if (!strcmp(arg0, "-tm")) showInfoFlag++;
					else if (!strcmp(arg0, "-lw")) lightMode++;
					else if (!strcmp(arg0, "-sl")) showDistFlag++;
					else break;
				}
				else if (argNum == 2) {
					if (!strcmp(arg0, "-xs")) {
						startX = arg1;
						if (startX < 0.0 || startX > ipHeader.xpxl) break;
						xsDoneFlag++;
					}
					else if (!strcmp(arg0, "-zs")) {
						startZ = arg1;
						if (startZ < 0.0 || startZ > ipHeader.zpxl) break;
						zsDoneFlag++;
					}
					else if (!strcmp(arg0, "-xf")) {
						endX = arg1;
						if (endX < 0.0 || endX > ipHeader.xpxl) break;
						xfDoneFlag++;
					}
					else if (!strcmp(arg0, "-zf")) {
						endZ = arg1;
						if (endZ < 0.0 || endZ > ipHeader.zpxl) break;
						zfDoneFlag++;
					}
					else if (!strcmp(arg0, "-dx")) {
						ipHeader.xsiz = arg1;
						if (ipHeader.xsiz <= 0.0) break;
						xsizDoneFlag++;
					}
					else if (!strcmp(arg0, "-dz")) {
						ipHeader.zsiz = arg1;
						if (ipHeader.zsiz <= 0.0) break;
						zsizDoneFlag++;
					}
					else if (!strcmp(arg0, "-st")) {
						step = arg1;
						if (step <= 0) break;
						stepDoneFlag++;
					}
					else if (!strcmp(arg0, "-ph")) {
						phi = arg1;
						phiDoneFlag++;
					}
					else if (!strcmp(arg0, "-xc")) {
						ipHeader.drtx = arg1;
						if (ipHeader.drtx < 0.0 || ipHeader.drtx >= ipHeader.xpxl) break;
						drtxDoneFlag++;
					}
					else if (!strcmp(arg0, "-zc")) {
						ipHeader.drtz = arg1;
						if (ipHeader.drtz < 0.0 || ipHeader.drtz >= ipHeader.zpxl) break;
						drtzDoneFlag++;
					}
					else break;
				}
				else break;
			}
		}
		if (i != argc) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((xsDoneFlag > 1) || (zsDoneFlag > 1) || (xfDoneFlag > 1)
		 || (zfDoneFlag > 1) || (stepDoneFlag > 1)
		 || (showPosFlag > 1) || (noHeadDoneFlag > 1) || (showInfoFlag > 1)
		 || (xsizDoneFlag > 1) || (zsizDoneFlag > 1) || (lightMode > 1)
		 || (drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (phiDoneFlag > 1)
		 || (ofileDoneFlag > 1) || (showDistFlag > 1)
		 || ((xsDoneFlag || zsDoneFlag || xfDoneFlag || zfDoneFlag) && phiDoneFlag)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((!xsDoneFlag || !zsDoneFlag || !xfDoneFlag || !zfDoneFlag) && !phiDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (strcmp(file2, "-")) ofileDoneFlag = 1;
	}
	if (ofileDoneFlag) {
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	else fp2 = stdout;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Suppress Output Header ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') noHeadFlag = 1;
		if (noHeadFlag) {
			do {
				fprintf(stdout, "Suppress Output Line Title ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') noHeadFlag = 2;
		}
		else {
			do {
				fprintf(stdout, "Include \"Date\", \"Operator\", and \"Exposure Time\" in Header ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') showInfoFlag = 1;
		}
	}
	
/* Set Parameters to Create */
	if (!stepDoneFlag) {
		step = MAXEXT(ipHeader.xsiz, ipHeader.zsiz);
		if (verboseFlag >= 2) fprintf(stdout, "Step is not specified. Use %8.5f [mm]\n", step);
	}
	if (phiDoneFlag) {
		phi = phi<0.0 ? fmod(phi, 360.0)+360.0 : fmod(phi, 360.0);
		phi =radEXT(phi);
		startX = ipHeader.drtx;
		startZ = ipHeader.drtz;
		sinPhi = sin(phi);
		cosPhi = cos(phi);
		for (num=0;; num++) {
			endX = startX + step*sinPhi*num/ipHeader.xsiz;
			if (endX > ipHeader.xpxl || endX < 0.0) break;
			endZ = startZ + step*cosPhi*num/ipHeader.zsiz;
			if (endZ > ipHeader.zpxl || endZ < 0.0) break;
		}
		if (num == 0) {
			fprintf(stderr, " Image centre is out image.\n");
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
	}
	else {
		phi = atan2((endZ-startZ)*ipHeader.zsiz, (endX-startX)*ipHeader.xsiz);
		sinPhi = sin(phi);
		cosPhi = cos(phi);
		num = (long)floor(dist((startX-endX)*ipHeader.xsiz, (startZ-endZ)*ipHeader.zsiz) / step);
		res = dist((startX-endX)*ipHeader.xsiz, (startZ-endZ)*ipHeader.zsiz) - num*step;
		endX -= res*cosPhi;
		endZ -= res*sinPhi;
	}
	num++;
	if (num > MAXLINE) {
		fprintf(stderr, " Output line number exceeds limit(%d). Use more large step.\n", MAXLINE);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if (endZ < startZ) revFlag = 1;
	if (verboseFlag) {
		fprintf(stdout, "Print Profile from (%7.1f, %7.1f) to (%7.1f, %7.1f).\n", startX, startZ, endX, endZ);
		fprintf(stdout, " Step is %8.5f [mm], Number of data is %ld.\n", step, num);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " PixelSize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (showPosFlag) fprintf(stdout, " Coordinate of each position calculating intensity is shown.\n");
		if (lightMode) fprintf(stdout, " Calculate Intensity by simple sampling.\n");
		else fprintf(stdout, " Calculate Intensity by interpolation of neighbouring pixels.\n");
	}
	if (!noHeadFlag) {
		numbyte += fprintf(fp2, "# Line Intensity Profile for %s.\n", file1);
		numbyte += fprintf(fp2, "# Print Profile from (%7.1f, %7.1f) to (%7.1f, %7.1f).\n", startX, startZ, endX, endZ);
		numbyte += fprintf(fp2, "# Step is %8.5f [mm], Number of data is %ld.\n", step, num);
		if (xsizDoneFlag || zsizDoneFlag) numbyte += fprintf(fp2, "# PixelSize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (showPosFlag) numbyte += fprintf(fp2, "# Coordinate of each position calculating intensity is shown.\n");
		if (lightMode) numbyte += fprintf(fp2, "# Calculate Intensity by simple sampling.\n");
		else numbyte += fprintf(fp2, "# Calculate Intensity by interpolation of neighbouring pixels.\n");
		timeNow = time(NULL);
		strftime(timebuf, 20, "%Y-%m-%d %H:%M:%S", localtime(&timeNow));
		numbyte += fprintf(fp2, "# Converted by %s-%s on %s\n", ProgName, VERSION, timebuf);
		lines += (5 + showPosFlag + (xsizDoneFlag | zsizDoneFlag));
		if (showInfoFlag) { /* for HiPic treatment */
			numbyte += fprintf(fp2, "# Date : %s\n", ipHeader.date);
			numbyte += fprintf(fp2, "# Time : %s\n", ipHeader.opnm);
			numbyte += fprintf(fp2, "# ExposureTime : %7.3fms\n", ipHeader.fext*60000.0);
			lines += 3;
		}
	}
	if (revFlag) {
		arg1 = startX;
		startX = endX;
		endX = arg1;
		arg1 = startZ;
		startZ = endZ;
		endZ = arg1;
		sinPhi *= -1.0;
		cosPhi *= -1.0;
	}
	if (lightMode) {
		startZn = (long)floor(startZ);
	}
	else {
		startZn = dToLong(startZ)-1;
		if (startZn == -1) szf = 1;
		if (startZn == ipHeader.zpxl-1) szf2 = 1;
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((outbuf = malloc(num*sizeof(float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		else fprintf(stderr, " No data to print.\n");
		exit(EXIT_FAILURE);
	}
	if (showPosFlag && (posbuf = malloc(num*2*sizeof(float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(outbuf);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		else fprintf(stderr, " No data to print.\n");
		exit(EXIT_FAILURE);
	}
	if (fseek(fp1, ipHeader.rlng*(startZn+1+szf), SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(outbuf);
		if (showPosFlag) free(posbuf);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if ((datPtr0 = malloc(ipHeader.rlng)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(outbuf);
		if (showPosFlag) free(posbuf);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		else fprintf(stderr, " No data to print.\n");
		exit(EXIT_FAILURE);
	}
	fread(datPtr0, ipHeader.rlng, 1, fp1);
	readByte += ipHeader.rlng;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
		free(outbuf);
		if (showPosFlag) free(posbuf);
		free(datPtr0);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		else fprintf(stderr, " No data to print.\n");
		exit(EXIT_FAILURE);
	}
	line0 = startZn;
	if (lightMode) {
		for (i=0; i<num; i++) {
			x = startX + step*cosPhi*i/ipHeader.xsiz;
			z = startZ + step*sinPhi*i/ipHeader.zsiz;
			xn = (long)floor(x);
			zn = (long)floor(z);
			if (zn != line0) {
				free(datPtr0);
				if (fseek(fp1, ipHeader.rlng*(zn+1), SEEK_SET)) {
					fprintf(stderr, " Input File Read Error.\n");
					if (showPosFlag) free(posbuf);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr0 = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				fread(datPtr0, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					free(datPtr0);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				line0 = zn;
			}
			outbuf[i] = (*readDat)(&datPtr0[xn], &rato, &shiftBit, &ipDataSwapFlag);
			if (showPosFlag) {
				posbuf[i] = x;
				posbuf[i+num] = z;
			}
		}
	}
	else {
		if ((datPtr1 = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(outbuf);
			if (showPosFlag) free(posbuf);
			free(datPtr0);
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			else fprintf(stderr, " No data to print.\n");
			exit(EXIT_FAILURE);
		}
		if (szf || szf2) memcpy(datPtr1, datPtr0, ipHeader.rlng);
		else {
			fread(datPtr1, ipHeader.rlng, 1, fp1);
			readByte += ipHeader.rlng;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(outbuf);
				if (showPosFlag) free(posbuf);
				free(datPtr0);
				free(datPtr1);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				else fprintf(stderr, " No data to print.\n");
				exit(EXIT_FAILURE);
			}
		}
		for (i=0; i<num; i++) {
			x = startX + step*cosPhi*i/ipHeader.xsiz;
			z = startZ + step*sinPhi*i/ipHeader.zsiz;
			xn = dToLong(x);
			zn = dToLong(z)-1; /* lower line */
			if (zn == line0+1) { /* get 1-line */
				free(datPtr0);
				datPtr0 = datPtr1;
				if ((datPtr1 = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					free(datPtr0);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				if (zn == ipHeader.zpxl-1) memcpy(datPtr1, datPtr0, ipHeader.rlng);
				else {
					fread(datPtr1, ipHeader.rlng, 1, fp1);
					readByte += ipHeader.rlng;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(outbuf);
						if (showPosFlag) free(posbuf);
						free(datPtr0);
						free(datPtr1);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						else fprintf(stderr, " No data to print.\n");
						exit(EXIT_FAILURE);
					}
				}
				line0++;
			}
			else if (zn > line0+1) { /* get 2-lines */
				free(datPtr0);
				free(datPtr1);
				if (fseek(fp1, ipHeader.rlng*(zn+1), SEEK_SET)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr0 = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				fread(datPtr0, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					free(datPtr0);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				if ((datPtr1 = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(outbuf);
					if (showPosFlag) free(posbuf);
					free(datPtr0);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
					exit(EXIT_FAILURE);
				}
				if (zn == ipHeader.zpxl-1) memcpy(datPtr1, datPtr0, ipHeader.rlng);
				else {
					fread(datPtr1, ipHeader.rlng, 1, fp1);
					readByte += ipHeader.rlng;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(outbuf);
						if (showPosFlag) free(posbuf);
						free(datPtr0);
						free(datPtr1);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						else fprintf(stderr, " No data to print.\n");
						exit(EXIT_FAILURE);
					}
				}
				line0 = zn;
			}
			zn++;
			if (xn == 0) {
				outbuf[i] = (*readDat)(&datPtr0[xn], &rato, &shiftBit, &ipDataSwapFlag) * (zn-z+0.5)
				          + (*readDat)(&datPtr1[xn], &rato, &shiftBit, &ipDataSwapFlag) * (z-zn+0.5);
			}
			else if (xn == ipHeader.xpxl) {
				outbuf[i] = (*readDat)(&datPtr0[xn-1], &rato, &shiftBit, &ipDataSwapFlag) * (zn-z+0.5)
				          + (*readDat)(&datPtr1[xn-1], &rato, &shiftBit, &ipDataSwapFlag) * (z-zn+0.5);
			}
			else {
				ratox = xn-x+0.5;
				outbuf[i] = ((*readDat)(&datPtr0[xn-1], &rato, &shiftBit, &ipDataSwapFlag) * ratox
				            + (*readDat)(&datPtr0[xn  ], &rato, &shiftBit, &ipDataSwapFlag) * (1.0-ratox)) * (zn-z+0.5)
				          + ((*readDat)(&datPtr1[xn-1], &rato, &shiftBit, &ipDataSwapFlag) * ratox
				            + (*readDat)(&datPtr1[xn  ], &rato, &shiftBit, &ipDataSwapFlag) * (1.0-ratox)) * (z-zn+0.5);
			}
			if (showPosFlag) {
				posbuf[i] = x;
				posbuf[i+num] = z;
			}
		}
	}
	
/* End */
	free(datPtr0);
	if (!lightMode) free(datPtr1);
	fclose(fp1);
	if (showPosFlag) {
		if (showDistFlag) {
			if (revFlag) {
			if (noHeadFlag != 2) numbyte += fprintf(fp2, "# num ,  dist[mm] ,      x ,      z , Intensity\n");
				for (i=num-1; i>-1; i--) numbyte += fprintf(fp2, " %5ld, %9.5f, %7.1f, %7.1f, %9.1f\n", num-1-i, (num-1-i)*step, posbuf[i], posbuf[i+num], outbuf[i]);
			}
			else {
				for (i=0; i<num; i++) numbyte += fprintf(fp2, " %5ld, %9.5f, %7.1f, %7.1f, %9.1f\n", i, i*step, posbuf[i], posbuf[i+num], outbuf[i]);
			}
		}
		else {
			if (noHeadFlag != 2) numbyte += fprintf(fp2, "# num ,      x ,      z , Intensity\n");
			if (revFlag) {
				for (i=num-1; i>-1; i--) numbyte += fprintf(fp2, " %5ld, %7.1f, %7.1f, %9.1f\n", num-1-i, posbuf[i], posbuf[i+num], outbuf[i]);
			}
			else {
				for (i=0; i<num; i++) numbyte += fprintf(fp2, " %5ld, %7.1f, %7.1f, %9.1f\n", i, posbuf[i], posbuf[i+num], outbuf[i]);
			}
		}
	}
	else {
		if (showDistFlag) {
			if (noHeadFlag != 2) numbyte += fprintf(fp2, "# num ,  dist[mm] , Intensity\n");
			if (revFlag) {
				for (i=num-1; i>-1; i--) numbyte += fprintf(fp2, " %5ld, %9.5f, %9.1f\n", num-1-i, (num-1-i)*step, outbuf[i]);
			}
			else {
				for (i=0; i<num; i++) numbyte += fprintf(fp2, " %5ld, %9.5f, %9.1f\n", i, i*step, outbuf[i]);
			}
		}
		else {
			if (noHeadFlag != 2) numbyte += fprintf(fp2, "# num , Intensity\n");
			if (revFlag) {
				for (i=num-1; i>-1; i--) numbyte += fprintf(fp2, " %5ld, %9.1f\n", num-1-i, outbuf[i]);
			}
			else {
				for (i=0; i<num; i++) numbyte += fprintf(fp2, " %5ld, %9.1f\n", i, outbuf[i]);
			}
		}
	}
	free(outbuf);
	if (showPosFlag) free(posbuf);
	if (ofileDoneFlag) fclose(fp2);
	if (verboseFlag >= 2) ofileDoneFlag ?
	    fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, lines+num+1, numbyte) :
	    fprintf(stdout, " Output %ld lines, %ld bytes.\n", lines+num+1, numbyte);
	if (verboseFlag > 0) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file -xs:VALUE -zs:VALUE -xf:VALUE -zf:VALUE\n", ProgName);
	fprintf(fp0, "          [-st:VALUE] [-lw] [-dx:VALUE] [-dz:VALUE]\n");
	fprintf(fp0, "          [-sn] [-sh] [-sH] [-tm] [-sl] [-o:file2]\n");
	fprintf(fp0, "        %s [-q|-v] file -ph:VALUE [-xc:VALUE] [-zc:VALUE]\n", ProgName);
	fprintf(fp0, "          [-st:VALUE] [-lw] [-dx:VALUE] [-dz:VALUE]\n");
	fprintf(fp0, "          [-sn] [-sh] [-sH] [-tm] [-sl] [-o:file2]\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program show line profiles of R-AXIS Image File.\n");
	fprintf(fp0, "  Intensity is shown per pixel unit.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ........ Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -xs:VALUE ... Set X-coordinate to start point calculation to \"VALUE\".\n");
	fprintf(fp0, "   -zs:VALUE ... Set Z-coordinate to start point calculation to \"VALUE\".\n");
	fprintf(fp0, "   -xf:VALUE ... Set X-coordinate to end point calculation to \"VALUE\".\n");
	fprintf(fp0, "   -zf:VALUE ... Set Z-coordinate to end point calculation to \"VALUE\".\n");
	fprintf(fp0, "   -st:VALUE ... Set step of calculation to \"VALUE\"[mm].\n");
	fprintf(fp0, "                 If absent, larger PixelSize is used.\n");
	fprintf(fp0, "   -lw ......... Get intensity with simple sampling for the pixel.\n");
	fprintf(fp0, "                 If absent, get intensity with interpolation.\n");
	fprintf(fp0, "   -dx:VALUE ... Change PixelSize in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change PixelSize in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -sn ......... Also print the coordinate for each intensity.\n");
	fprintf(fp0, "   -sl ......... Also print distance[mm] from start for each intensity.\n");
	fprintf(fp0, "   -sh ......... Suppress Output Header. If absent, some informations about\n");
	fprintf(fp0, "                 Original File and conversion are print before profile.\n");
	fprintf(fp0, "   -sH ......... Addtion to above, suppress output of title line.\n");
	fprintf(fp0, "   -tm ......... Show about Date, Operator, and Exposure Time in Header.\n");
	fprintf(fp0, "   -ph:VALUE ... Instead of pointing both edge, calculate profile along line\n");
	fprintf(fp0, "                 from centre with \"VALUE\"[deg.] angle.\n");
	fprintf(fp0, "   -xc:VALUE ... Set coordinate in X of the centre to \"VALUE\"\n");
	fprintf(fp0, "   -zc:VALUE ... Set coordinate in Z of the centre to \"VALUE\"\n");
	fprintf(fp0, "   -o:file2 .... Output to \"file2\". If absent, print to Standard Output.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
