/* Program to convert IP data file */
#define ProgName "iplorentz"
#define ProgInfo "Lorentz and Polarization Factor Correction"
/*                                                         4/22/1999  */
/*         for ANSI C                                      5/11/1999  */
/*         Separate some functions                        11/12/1999  */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v2.77_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 16
#else
#	define MAXARG 15
#endif
#define MINARG 3
#define RadiusCrit 0.25
#define CosThetaCrit 0.1
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DIRECT_LIM_OFF
#define FLAT 0
#define CURVE_V 1
#define CURVE_H 2
#define BACK 3
#define NO_MONOCHROM 0
#define WITH_MONOCHROM 1
#define SR_HORIZONTAL 2
#define SR_VERTICAL 3
#define MAXDLAMBDA 0.1
#define DLAMBDADEFAULT 0.001

#ifdef _OLDHP
	double sinSigmaFlat();
	double sinSigmaCurveV();
	double sinSigmaCurveH();
	double cosThetaFlat();
	double cosThetaCurveV();
	double cosThetaCurveH();
	double cosThetaBack();
	double pFactor();
	double pFactorSrH();
	double pFactorSrV();
	double pNone();
	double lFactorUni();
	double lFactorUni2();
	double lFactorIso();
	double lNone();
	double absorbSpec0();
	double absorbSpec1();
	double absorbAirPlate();
	double absorbAirCyl();
	double absorbAir0();
	double (*polarf)();
	double (*lorentzf)();
	double (*cosThetaf)();
	double (*sinSigmaSqf)();
	double (*absorbf)();
	double (*absorbAirf)();
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	char checkfLp0();
	char checkfLp1();
	char (*checkfLp)();
#else
	double sinSigmaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi);
	double sinSigmaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi);
	double sinSigmaCurveH(const double *z, const double *x, const double *sinPhi, const double *cosPhi);
	double cosThetaFlat(const double *x, const double *z);
	double cosThetaCurveV(const double *x, const double *z);
	double cosThetaCurveH(const double *z, const double *x);
	double cosThetaBack(const double *x, const double *z);
	double pFactor(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM);
	double pFactorSrH(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM);
	double pFactorSrV(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM);
	double pNone(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM);
	double lFactorUni(const double *sinSigmaSq, const double *cosTheta, const double *dLambda);
	double lFactorUni2(const double *sinSigmaSq, const double *cosTheta, const double *dLambda);
	double lFactorIso(const double *sinSigmaSq, const double *cosTheta, const double *dLambda);
	double lNone(const double *sinSigmaSq, const double *cosTheta, const double *dLambda);
	double absorbSpec0(const double *mR, const double *expmR, const double *cosTheta, const double *cosThetaCrit);
	double absorbSpec1(const double *mR, const double *expmR, const double *cosTheta, const double *cosThetaCrit);
	double absorbAirPlate(const double *mR, const double *cosTheta, const double *camr, const double *z);
	double absorbAirCyl(const double *mR, const double *cosTheta, const double *camr, const double *z);
	double absorbAir0(const double *mR, const double *cosTheta, const double *camr, const double *z);
	double (*polarf)(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM);
	double (*lorentzf)(const double *sinSigmaSq, const double *cosTheta, const double *dLambda);
	double (*cosThetaf)(const double *x, const double *z);
	double (*sinSigmaSqf)(const double *x, const double *z, const double *cosPhi, const double *sinPhi);
	double (*absorbf)(const double *mR, const double *expmR, const double *cosTheta, const double *cosThetaCrit);
	double (*absorbAirf)(const double *mR, const double *cosTheta, const double *camr, const double *z);
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
	char checkfLp0(double *fLp);
	char checkfLp1(double *fLp);
	char (*checkfLp)(double *fLp);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, maxInt, readByte, flushWidth, flushWidth0;
	char file1[FNAMELEN], file2[FNAMELEN], confirmBuffer1[8], confirmBuffer2[8],
	     inputBuffer[20], specimenOrientationFlag, monochromCheckFlag,
	     cameraTypeFlag, absorbCorrectFlag,
	     cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag, absorbCorrectAirFlag,
	     shiftBit, strangeShiftBitFlag, changeValueTypeIndex,
	     byte, createDosFlag, verboseFlag, argNum, arg0[4],
	     cameraTypeDoneFlag, specimenOrientationDoneFlag, monochromCheckDoneFlag,
	     absorbCorrectDoneFlag, absorbCorrectAirDoneFlag, criticalRadiusDoneFlag,
	     camrDoneFlag, drtxDoneFlag, drtzDoneFlag, xsizDoneFlag, zsizDoneFlag,
	     replaceFlag, setTimeFlag, cosThetaCritDoneFlag,
	     lorentzFlag, polarFlag, reverseFlag, useDLFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double absorb, intensityDbl, cosTheta, cosThetaSq,
	       cosThetaSqM, xDist, zDist, radiusCrit, fLp,
	       rato, expAbsorb, camr, absorbAir, camrAir, arg1, arg2,
	       cosThetaCrit, dLambda,
	       specimenRotateAngle, monochromDirectionAngle, cosSpecPhi, sinSpecPhi,
	       cosMonoPhi, sinMonoPhi, sinSigmaSqPolar, sinSigmaSqLorentz;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	radiusCrit = RadiusCrit;
	cosThetaCrit = CosThetaCrit;
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	ipDataSwapFlag = cameraTypeDoneFlag = specimenOrientationDoneFlag = monochromCheckDoneFlag
	= absorbCorrectDoneFlag = absorbCorrectAirDoneFlag = criticalRadiusDoneFlag = camrDoneFlag
	= drtxDoneFlag = drtzDoneFlag = xsizDoneFlag = zsizDoneFlag = replaceFlag = setTimeFlag
	= cosThetaCritDoneFlag = reverseFlag = useDLFlag = 0;
	lorentzFlag = polarFlag = absorbCorrectFlag = absorbCorrectAirFlag
	= specimenOrientationFlag = cameraTypeFlag = monochromCheckFlag = 0;
	specimenRotateAngle = 0.0;
	monochromDirectionAngle = 0.0;
	cosThetaSqM = 1.0;
	readByte = 0;
	camrAir = -1.0;
	absorbAir = absorb = expAbsorb = 1.0;
	absorbAirf = absorbAir0;
	absorbf = absorbSpec0;
	dLambda = DLAMBDADEFAULT;
	flushWidth = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag++;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = createDosFlag ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " IP size is : %6ld x %6ld.\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " CameraLength is : %7.3f mm.\n", ipHeader.camr);
	}
	
/* Check the Header and Set Parameters : Interactive Mode */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z     , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (5) Camera Length, (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
#ifndef DIRECT_LIM_OFF
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
#endif
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
#ifndef DIRECT_LIM_OFF
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
#endif
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					case 5:
						camrDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Camera Length : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.camr = atof(inputBuffer);
							if (ipHeader.camr <= 0.0) continue;
							camrDoneFlag = 1;
						} while (!camrDoneFlag);
						fprintf(stdout, "Camera Length is changed to %7.3f mm.\n", ipHeader.camr);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
		
/* Ask on Correction type to execute */
		do {
			fprintf(stdout, "Execute Lorentz Correction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'N') {
			fprintf(stdout, "  No execution of Lorentz Correction.\n");
		}
		else {
			fprintf(stdout, "  Execute Lorentz Correction.\n");
			lorentzFlag = 1;
		}
		do {
			fprintf(stdout, "Execute Polarization Correction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'N') {
			fprintf(stdout, "  No execution of Polarization Correction.\n");
		}
		else {
			fprintf(stdout, "  Execute Polarization Correction.\n");
			polarFlag = 1;
		}
		
/* Ask Camera Type */
		do {
			fprintf(stdout, "Camera is Flat[F] , Curved[C] , Flat(Back)[B] or Other[N] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'F') && (toupper(confirmBuffer1[0]) != 'C')
		      && (toupper(confirmBuffer1[0]) != 'B') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else if (toupper(confirmBuffer1[0]) == 'C') {
			fprintf(stdout, "  Camera is Curved. Is Cylinder Axis\n");
			ipHeader.cyld = 1;
			do {
				fprintf(stdout, "    Horizontal[H] or Vertical[V] on the Screen ? : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'H') && (toupper(confirmBuffer2[0]) != 'V'));
			if (toupper(confirmBuffer2[0]) == 'H') {
				fprintf(stdout, "  Camera axis is in X-direction.\n");
				cameraTypeFlag = CURVE_H;
			}
			else {
				fprintf(stdout, "  Camera axis is in Z-direction.\n");
				cameraTypeFlag = CURVE_V;
			}
		}
		else if (toupper(confirmBuffer1[0]) == 'B') {
			fprintf(stdout, "  Camera is Flat, Back Type.\n");
			cameraTypeFlag = BACK;
			ipHeader.cyld = 2;
		}
		else {
			fprintf(stdout, "  Camera is Unknwon.\n");
			fprintf(stderr, "  Sorry, only Flat and Curved camera is supported.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		
/* Ask on Specimen type and Orientation */
		if (lorentzFlag) {
			do {
				fprintf(stdout, "Specimen is Isotropic[I] or Uniaxialy-Oriented[U] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'I') && (toupper(confirmBuffer1[0]) != 'U'));
			if (toupper(confirmBuffer1[0]) == 'I') {
				fprintf(stdout, "  Specimen is Isotropic.\n");
				specimenOrientationFlag = 0;
			}
			else {
				specimenOrientationFlag = 1;
				do {
					fprintf(stdout, "  Specimen is Uniaxially Oriented. Enter Stretched Direction\n");
					fprintf(stdout, "    from X-direction of the Screen [deg.] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while (!isdigitEXT(inputBuffer));
				specimenRotateAngle = atof(inputBuffer);
				do {
					fprintf(stdout, "  Use strange correction for meridian ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while (toupper(confirmBuffer2[0]) != 'Y' && toupper(confirmBuffer2[0]) != 'N');
				if (toupper(confirmBuffer2[0]) == 'Y') {
					do {
						fprintf(stdout, "  Enter spread of lambda. If use default value, enter 0 : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						dLambda = atof(inputBuffer);
						if (dLambda < 0.0 || dLambda > MAXDLAMBDA) continue;
						useDLFlag = 1;
					} while (!useDLFlag);
					if (dLambda == 0.0) dLambda = DLAMBDADEFAULT;
				}
			}
		}
		
/* Ask on Monochromator */
		if (polarFlag) {
			do {
				fprintf(stdout, "Use Monochromator ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stdout, "  No Monochromator.\n");
				monochromCheckFlag = NO_MONOCHROM;
			}
			else {
				fprintf(stdout, "  Yes, Monochromator is assumed.\n");
				monochromCheckFlag = WITH_MONOCHROM;
				do {
					fprintf(stdout, "    With Synchrotron Radiation ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'Y') {
					do {
						fprintf(stdout, "    Monochoromator reflection plane within Orbit[P] or Normal[N] to it : ");
						if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer2[0]) != 'P') && (toupper(confirmBuffer2[0]) != 'N'));
					monochromCheckFlag = toupper(confirmBuffer2[0]) == 'P' ? SR_HORIZONTAL : SR_VERTICAL;
				}
				do {
					fprintf(stdout, "  Enter Direction of Monochromator Reflection Plane\n");
					fprintf(stdout, "    from X-direction of the Screen [deg.] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while (!isdigitEXT(inputBuffer));
				monochromDirectionAngle = atof(inputBuffer);
				do {
					fprintf(stdout, "    Use Monochromator Angle 2-theta stored in file (%.4f deg.) ? [Y/N] : ", ipHeader.m2ta);
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'N') {
					do {
						fprintf(stdout, "    Enter the Monochromator angle in 2-theta(degrees) : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while (!isdigitEXT(inputBuffer));
					ipHeader.m2ta = atof(inputBuffer);
				}
			}
		}
		
/* Absorption Correction on Specimen */
		do {
			fprintf(stdout, "Execute Absorption Correction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			fprintf(stdout, "  Yes, Execute Absorption Correction with Specimen.\n");
			absorbCorrectFlag = 1;
			do {
				fprintf(stdout, "  Enter Line Absorption Coefficient of Specimen [1/mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				absorb = atof(inputBuffer);
				if (absorb <= 0.0) continue;
				absorbCorrectDoneFlag = 1;
			} while (!absorbCorrectDoneFlag);
			absorbCorrectDoneFlag = 0;
			do {
				fprintf(stdout, "  Enter Thickness of the Specimen [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				if (atof(inputBuffer) <= 0.0) continue;
				absorbCorrectDoneFlag = 1;
			} while (!absorbCorrectDoneFlag);
			absorb *= atof(inputBuffer);
			expAbsorb = exp(-absorb);
			absorbf = absorbSpec1;
			if (cameraTypeFlag == CURVE_V || cameraTypeFlag == CURVE_H) {
				do {
					fprintf(stdout, "Intensities near 2Theta = 90deg. will not be corrected.\n");
					fprintf(stdout, "  The minimum value for COS(2Theta) is %7.4f. OK? [Y/N] : ", cosThetaCrit);
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) != 'Y') {
					do {
						fprintf(stdout, "  Enter Minimum value for COS(2Theta) : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						cosThetaCrit = atof(inputBuffer);
						if (cosThetaCrit < 0.0) continue;
						cosThetaCritDoneFlag = 1;
					} while (!cosThetaCritDoneFlag);
				}
				fprintf(stdout, "  Yes, Minimum COS(2Theta) is %7.4f.\n", cosThetaCrit);
			}
		}
		else {
			fprintf(stdout, "  No Absorption Correction with Specimen.\n");
			absorbCorrectFlag = 0;
			absorbf = absorbSpec0;
		}
		
/* Absorption Correction with Air */
		do {
			fprintf(stdout, "Execute Absorption Correction with Air ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			fprintf(stdout, "  Yes, Execute Absorption Correction with Air.\n");
			absorbCorrectAirFlag = 1;
			do {
				fprintf(stdout, "  Enter Line Absorption Coefficient of Air [1/mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				absorbAir = atof(inputBuffer);
				if (absorbAir < 0.0) continue;
				absorbCorrectAirDoneFlag = 1;
			} while (!absorbCorrectAirDoneFlag);
			absorbCorrectAirDoneFlag = 0;
			do {
				fprintf(stdout, "  Enter \"Effective\" Camera Length [mm] (if \"0\", set defalut) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				camrAir = (atof(inputBuffer) == 0.0) ? ipHeader.camr : atof(inputBuffer);
				if (absorbAir <= 0.0) continue;
				absorbCorrectAirDoneFlag = 1;
			} while (!absorbCorrectAirDoneFlag);
			absorbAir *= camrAir;
			absorbAirf = (cameraTypeFlag == CURVE_V || cameraTypeFlag == CURVE_H) ? absorbAirCyl : absorbAirPlate;
		}
		else {
			fprintf(stdout, "  No Absorption Correction with Air.\n");
			absorbCorrectAirFlag = 0;
			absorbAirf = absorbAir0;
		}
		
/* Check Type of correction to Execute */
		if (!lorentzFlag && !polarFlag && !absorbCorrectFlag && !absorbCorrectAirFlag) {
			fprintf(stderr, "Error : No correction will be done !!\n");
			fprintf(stderr, " You have not choiced any correction type. Abort.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		do {
			fprintf(stdout, "Execute Reverse Correction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			fprintf(stdout, "  !! --- Reverse Correction. --- !!\n");
			reverseFlag = 1;
		}
/* Check of Critical Radius */
		do {
			fprintf(stdout, "Intensities within a certain radius around the center\n");
			fprintf(stdout, "  will not be corrected. The radius is %7.4f mm. OK? [Y/N] : ", radiusCrit);
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) != 'Y') {
			do {
				fprintf(stdout, "  Enter radius [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				radiusCrit = atof(inputBuffer);
				if (radiusCrit <= 0.0) continue;
				criticalRadiusDoneFlag = 1;
			} while (!criticalRadiusDoneFlag);
		}
		fprintf(stdout, "  Yes, Critical Radius is %7.4f mm.\n", radiusCrit);
		
/* Question about Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			replaceFlag = 1;
			fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
		}
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			setTimeFlag = 1;
			fprintf(stdout, " Modification time of Output File is set as Original File.\n");
		}
#endif
	}
/* Check Arguments and Set Parameters : Batch Mode */
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-ch")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_H;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-bf")) {
					ipHeader.cyld = 2;
					cameraTypeFlag = BACK;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-l")) lorentzFlag++;
				else if (!strcmp(arg0, "-p")) polarFlag++;
				else if (!strcmp(arg0, "-is")) {
					specimenOrientationFlag = 0;
					specimenOrientationDoneFlag++;
				}
				else if (!strcmp(arg0, "-nm")) {
					monochromCheckFlag = NO_MONOCHROM;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-rv")) reverseFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-u")) {
					specimenOrientationFlag = 1;
					specimenRotateAngle = arg1;
					specimenOrientationDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					monochromCheckFlag = WITH_MONOCHROM;
					monochromDirectionAngle = arg1;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-mr")) {
					monochromCheckFlag = SR_HORIZONTAL;
					monochromDirectionAngle = arg1;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-mR")) {
					monochromCheckFlag = SR_VERTICAL;
					monochromDirectionAngle = arg1;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-b")) {
					absorbCorrectAirFlag = 1;
					if (arg1 < 0.0) break;
					absorbAir = arg1;
					absorbCorrectAirDoneFlag++;
				}
				else if (!strcmp(arg0, "-cc")) {
					if (arg1 < 0.0) break;
					cosThetaCrit = arg1;
					cosThetaCritDoneFlag++;
				}
				else if (!strcmp(arg0, "-cr")) {
					if (arg1 <= 0.0) break;
					radiusCrit = arg1;
					criticalRadiusDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
#ifndef DIRECT_LIM_OFF
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
#endif
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
#ifndef DIRECT_LIM_OFF
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
#endif
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-U")) {
					specimenOrientationFlag = 1;
					specimenRotateAngle = arg1;
					specimenOrientationDoneFlag++;
					useDLFlag++;
				}
				else break;
			}
			else if (argNum == 3) {
				if (!strcmp(arg0, "-a")) {
					if ((arg1 <= 0.0) || (arg2 <= 0.0)) break;
					absorbCorrectFlag = 1;
					absorb = arg1 * arg2;
					expAbsorb = exp(-absorb);
					absorbf = absorbSpec1;
					absorbCorrectDoneFlag++;
				}
				else if (!strcmp(arg0, "-b")) {
					if ((arg1 < 0.0) || (arg2 <= 0.0)) break;
					absorbCorrectAirFlag = 1;
					absorbAir = arg1;
					camrAir = arg2;
					absorbCorrectAirDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					monochromCheckFlag = WITH_MONOCHROM;
					monochromDirectionAngle = arg1;
					ipHeader.m2ta = arg2;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-mr")) {
					monochromCheckFlag = SR_HORIZONTAL;
					monochromDirectionAngle = arg1;
					ipHeader.m2ta = arg2;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-mR")) {
					monochromCheckFlag = SR_VERTICAL;
					monochromDirectionAngle = arg1;
					ipHeader.m2ta = arg2;
					monochromCheckDoneFlag++;
				}
				else if (!strcmp(arg0, "-U")) {
					specimenOrientationFlag = 1;
					specimenRotateAngle = arg1;
					if ((dLambda = arg2) <= 0.0 || arg2 > MAXDLAMBDA) break;
					specimenOrientationDoneFlag++;
					useDLFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (((cameraTypeDoneFlag > 1) || (specimenOrientationDoneFlag > 1) || (monochromCheckDoneFlag > 1)
		   || (absorbCorrectDoneFlag > 1) || (absorbCorrectAirDoneFlag > 1) || (criticalRadiusDoneFlag > 1)
		   || (camrDoneFlag > 1) || (drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (xsizDoneFlag > 1)
		   || (zsizDoneFlag > 1) || (setTimeFlag > 1) || (cosThetaCritDoneFlag > 1))
		  || (lorentzFlag > 1) || (polarFlag > 1) || (reverseFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!cameraTypeDoneFlag
		   || (!lorentzFlag && !polarFlag && !absorbCorrectFlag && !absorbCorrectAirFlag)
		   || (!specimenOrientationDoneFlag && lorentzFlag)
		   || (!monochromCheckDoneFlag && polarFlag)) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!absorbCorrectDoneFlag && cosThetaCritDoneFlag && verboseFlag) fprintf(stderr, " -cc: is ignored because of no absorp correction.\n");
		if (absorbCorrectAirDoneFlag) {
			absorbAir *= (camrAir < 0.0) ? ipHeader.camr : camrAir;
			absorbAirf = (cameraTypeFlag == CURVE_H || cameraTypeFlag == CURVE_V) ? absorbAirCyl : absorbAirPlate;
		}
	}
	if (cameraTypeFlag == CURVE_V && ((ipHeader.drtx - ipHeader.camr*M_PI/ipHeader.xsiz >= 0.0) || (ipHeader.drtx + ipHeader.camr*M_PI/ipHeader.xsiz <= ipHeader.xpxl))) {
		fprintf(stderr, " Curved Camera \"Width\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (cameraTypeFlag == CURVE_H && ((ipHeader.drtz - ipHeader.camr*M_PI/ipHeader.zsiz >= 0.0) || (ipHeader.drtz + ipHeader.camr*M_PI/ipHeader.zsiz <= ipHeader.zpxl))) {
		fprintf(stderr, " Curved Camera \"Height\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (monochromCheckFlag != NO_MONOCHROM) cosThetaSqM = cos(radEXT(ipHeader.m2ta))*cos(radEXT(ipHeader.m2ta));
	if ((monochromCheckFlag == SR_HORIZONTAL || monochromCheckFlag == SR_VERTICAL) && cosThetaSqM == 0.0) {
		fprintf(stderr, " Monochromator Angle must not be pi/2 with SR.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (useDLFlag) {
	/*	dLambda *= ipHeader.wlng; */ /* this is hopeful... */
		dLambda = -sqrt(dLambda);
	}
	if (verboseFlag == 2) {
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (camrDoneFlag) fprintf(stdout, " CameraLength has changed to : %7.3f mm.\n", ipHeader.camr);
		if (polarFlag) fprintf(stdout, " Polarization Correction will be done.\n");
		if (lorentzFlag) fprintf(stdout, " Lorentz Correction will be done.\n");
		if (cameraTypeFlag == FLAT) fprintf(stdout, " Camera is Flat.\n");
		else if (cameraTypeFlag == CURVE_V) fprintf(stdout, " Camera is Cylindrical along Z-direction.\n");
		else if (cameraTypeFlag == CURVE_H) fprintf(stdout, " Camera is Cylindrical along X-direction.\n");
		else if (cameraTypeFlag == BACK) fprintf(stdout, " Camera is Flat, with BackLaue mode.\n");
		if (lorentzFlag) {
			if (!specimenOrientationFlag) fprintf(stdout, " Specimen is Isotropic.\n");
			else fprintf(stdout, " Specimen is Uniaxially oriented to %.2f degree direction.\n", specimenRotateAngle);
		}
		if (polarFlag) {
			if (monochromCheckFlag == NO_MONOCHROM) fprintf(stdout, " No Monochromator.\n");
			else {
				fprintf(stdout, " Monochromator Reflection Plane is %.2f tilt from X-direction,\n", monochromDirectionAngle);
				fprintf(stdout, "   with %.4f degrees monochromator 2-Theta.\n", ipHeader.m2ta);
				if (monochromCheckFlag == SR_HORIZONTAL) fprintf(stdout, " With Synchrotron Radiation, monochromator within orbit.\n");
				else if (monochromCheckFlag == SR_VERTICAL) fprintf(stdout, " With Synchrotron Radiation, monochromator normal to orbit.\n");
			}
		}
		if (absorbCorrectFlag) fprintf(stdout, " Absorb Correction by Specimen, with %.6f for Theta = 0 [deg.] set to 1.0.\n", expAbsorb);
		else fprintf(stdout, " No Absorb Correction by Specimen.\n");
		if (absorbCorrectAirFlag) fprintf(stdout, " Absorb Correction by Air, with %.6f for Theta = 0 [deg.] set to 1.0.\n", exp(-absorbAir));
		else fprintf(stdout, " No Absorb Correction by Air.\n");
		fprintf(stdout, " Data within a %.4f [mm] will not be corrected.\n", radiusCrit);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
		if (reverseFlag) fprintf(stdout, " Execute Reverse Correction !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}

/* Definition of Lp function */
	if (!polarFlag) polarf = pNone;
	else if (monochromCheckFlag == SR_HORIZONTAL) polarf = pFactorSrH;
	else if (monochromCheckFlag == SR_VERTICAL) polarf = pFactorSrV;
	else polarf = pFactor;
	lorentzf = lorentzFlag ? (specimenOrientationFlag ? (useDLFlag ? lFactorUni2 : lFactorUni) : lFactorIso) : lNone;
	switch (cameraTypeFlag) {
		case FLAT:
			sinSigmaSqf = sinSigmaFlat;
			cosThetaf = cosThetaFlat;
			break;
		case BACK:
			sinSigmaSqf = sinSigmaFlat;
			cosThetaf = cosThetaBack;
			break;
		case CURVE_V:
			sinSigmaSqf = sinSigmaCurveV;
			cosThetaf = cosThetaCurveV;
			break;
		case CURVE_H:
			sinSigmaSqf = sinSigmaCurveH;
			cosThetaf = cosThetaCurveH;
			break;
		default:
			break;
			/* NOT REACHED */
	}
	specimenRotateAngle -= 90.0;
	cosSpecPhi = cos(radEXT(specimenRotateAngle));
	sinSpecPhi = sin(radEXT(specimenRotateAngle));
	cosMonoPhi = cos(radEXT(monochromDirectionAngle));
	sinMonoPhi = sin(radEXT(monochromDirectionAngle));
	checkfLp = reverseFlag ? checkfLp1 : checkfLp0;
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.rlng = recordLengthCreate;
	radiusCrit = 1.0 / (radiusCrit*radiusCrit/ipHeader.camr/ipHeader.camr + 1.0);
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	camr = ipHeader.camr;
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr = malloc(recordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr, recordLength, 1, fp1);
		readByte += recordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		zDist = (i+0.5-ipHeader.drtz) * ipHeader.zsiz / camr;
		for (j=0; j<ipHeader.xpxl; j++) {
			xDist = (j+0.5-ipHeader.drtx) * ipHeader.xsiz / camr;
			cosTheta = (*cosThetaf)(&xDist, &zDist);
			if ((cosThetaSq = cosTheta * cosTheta) > radiusCrit) continue;
			sinSigmaSqPolar = (*sinSigmaSqf)(&xDist, &zDist, &cosMonoPhi, &sinMonoPhi);
			sinSigmaSqLorentz = (*sinSigmaSqf)(&xDist, &zDist, &cosSpecPhi, &sinSpecPhi);
	/* Calculation */
			fLp = (*polarf)(&sinSigmaSqPolar, &cosThetaSq, &cosThetaSqM)
			    * (*lorentzf)(&sinSigmaSqLorentz, &cosTheta, &dLambda)
			    * (*absorbf)(&absorb, &expAbsorb, &cosTheta, &cosThetaCrit)
			    * (*absorbAirf)(&absorbAir, &cosTheta, &camrAir, &zDist);
			if ((*checkfLp)(&fLp)) {
				fprintf(stderr, " Error : Coefficient fLp is %f in %ld, %ld\n", fLp, i, j);
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtr);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag) * fLp;
			datPtr[j] = (*writeDat)(&intensityDbl, &rato, &maxInt, &ipDataSwapFlag);
		}
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
double sinSigmaFlat(x, z, cosPhi, sinPhi)
	double *x, *z, *cosPhi, *sinPhi;
#else
double sinSigmaFlat(const double *x, const double *z, const double *cosPhi, const double *sinPhi)
#endif
{
	/* z = z/camr , x = x/camr */
	double buf;
	buf = ((*cosPhi)*(*z) - (*sinPhi)*(*x));
	return (buf * buf / (1.0 + (*x)*(*x) + (*z)*(*z)));
}

#ifdef _OLDHP
double sinSigmaCurveV(x, z, cosPhi, sinPhi)
	double *x, *z, *cosPhi, *sinPhi;
#else
double sinSigmaCurveV(const double *x, const double *z, const double *cosPhi, const double *sinPhi)
#endif
{
	double buf;
	buf = (*z) * (*cosPhi) - (*sinPhi) * sin(*x);
	return (buf * buf / (1.0 + (*z)*(*z)));
}

#ifdef _OLDHP
double sinSigmaCurveH(z, x, sinPhi, cosPhi)
	double *x, *z, *cosPhi, *sinPhi;
#else
double sinSigmaCurveH(const double *z, const double *x, const double *sinPhi, const double *cosPhi)
#endif
{
	double buf;
	buf = (*z) * (*cosPhi) - (*sinPhi) * sin(*x);
	return (buf * buf / (1.0 + (*z)*(*z)));
}

#ifdef _OLDHP
double cosThetaFlat(x, z)
	double *x, *z;
#else
double cosThetaFlat(const double *x, const double *z)
#endif
{
	return (1.0 / sqrt(1.0 + (*x)*(*x) + (*z)*(*z)));
}

#ifdef _OLDHP
double cosThetaCurveV(x, z)
	double *x, *z;
#else
double cosThetaCurveV(const double *x, const double *z)
#endif
{
	return (cos(*x) / sqrt(1.0 + (*z)*(*z)));
}

#ifdef _OLDHP
double cosThetaCurveH(z, x)
	double *x, *z;
#else
double cosThetaCurveH(const double *z, const double *x)
#endif
{
	return (cos(*x) / sqrt(1.0 + (*z)*(*z)));
}

#ifdef _OLDHP
double cosThetaBack(x, z)
	double *x, *z;
#else
double cosThetaBack(const double *x, const double *z)
#endif
{
	return (-1.0 / sqrt(1.0 + (*x)*(*x) + (*z)*(*z)));
}

#ifdef _OLDHP
double pFactor(sinSigmaSq, cosThetaSq, cosThetaSqM)
	double *sinSigmaSq, *cosThetaSq, *cosThetaSqM;
#else
double pFactor(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM)
#endif
{
	return (2.0 / (1.0 - (*sinSigmaSq) + (*cosThetaSqM) * ((*cosThetaSq) + (*sinSigmaSq))));
}

#ifdef _OLDHP
double pFactorSrH(sinSigmaSq, cosThetaSq, cosThetaSqM)
	double *sinSigmaSq, *cosThetaSq, *cosThetaSqM;
#else
double pFactorSrH(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM)
#endif
{
	/* Angle between polarization of incident beam and monochromator reflection plane must be the same. */
	return (1.0 / (*cosThetaSqM) / ((*cosThetaSq) + (*sinSigmaSq)));
}

#ifdef _OLDHP
double pFactorSrV(sinSigmaSq, cosThetaSq, cosThetaSqM)
	double *sinSigmaSq, *cosThetaSq, *cosThetaSqM;
#else
double pFactorSrV(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM)
#endif
{
	/* Angle between polarization of incident beam and monochromator reflection plane must be pi/2. */
	return (1.0 / (1.0 - (*sinSigmaSq)));
}

#ifdef _OLDHP
double pNone(sinSigmaSq, cosThetaSq, cosThetaSqM)
	double *sinSigmaSq, *cosThetaSq, *cosThetaSqM;
#else
double pNone(const double *sinSigmaSq, const double *cosThetaSq, const double *cosThetaSqM)
#endif
{
	return (1.0);
}

#ifdef _OLDHP
double lFactorUni(sinSigmaSq, cosTheta, dLambda)
	double *sinSigmaSq, *cosTheta, *dLambda;
#else
double lFactorUni(const double *sinSigmaSq, const double *cosTheta, const double *dLambda)
#endif
{
	return (sqrt(1.0 - (*cosTheta)*(*cosTheta) - (*sinSigmaSq)));
}

#ifdef _OLDHP
double lFactorUni2(sinSigmaSq, cosTheta, dLambda)
	double *sinSigmaSq, *cosTheta, *dLambda;
#else
double lFactorUni2(const double *sinSigmaSq, const double *cosTheta, const double *dLambda)
#endif
{
	double fact;
	
	fact = 4.0*(*dLambda)*((*dLambda) - 1.0 + (*cosTheta));
	if ((*sinSigmaSq) > 1.0 - (*cosTheta)*(*cosTheta) - fact)
	    return (sqrt(fact)*fact / ((*sinSigmaSq)-1.0+(*cosTheta)*(*cosTheta)+2.0*fact));
	return (sqrt(1.0 - (*cosTheta)*(*cosTheta) - (*sinSigmaSq)));
}

#ifdef _OLDHP
double lFactorIso(sinSigmaSq, cosTheta, dLambda)
	double *sinSigmaSq, *cosTheta, *dLambda;
#else
double lFactorIso(const double *sinSigmaSq, const double *cosTheta, const double *dLambda)
#endif
{
	return (4.0 * sqrt((1.0 - (*cosTheta)) / 2.0));
}

#ifdef _OLDHP
double lNone(sinSigmaSq, cosTheta, dLambda)
	double *sinSigmaSq, *cosTheta, *dLambda;
#else
double lNone(const double *sinSigmaSq, const double *cosTheta, const double *dLambda)
#endif
{
	return (1.0);
}

#ifdef _OLDHP
double absorbSpec0(mR, expmR, cosTheta, cosThetaCrit)
	double *mR, *expmR, *cosTheta, *cosThetaCrit;
#else
double absorbSpec0(const double *mR, const double *expmR, const double *cosTheta, const double *cosThetaCrit)
#endif
{
	return (1.0);
}

#ifdef _OLDHP
double absorbSpec1(mR, expmR, cosTheta, cosThetaCrit)
	double *mR, *expmR, *cosTheta, *cosThetaCrit;
#else
double absorbSpec1(const double *mR, const double *expmR, const double *cosTheta, const double *cosThetaCrit)
#endif
{
	double tmp, tmpcos;
	if ((*cosTheta) < 0.0) {
		tmpcos = MINEXT(-(*cosThetaCrit), (*cosTheta));
		tmp = (*mR) * (1.0 - tmpcos) / tmpcos;
		return ((*expmR) * tmp / (exp(tmp) - 1.0));
	}
	tmpcos = MAXEXT((*cosTheta), (*cosThetaCrit));
	tmp = (*mR) * (1.0 - tmpcos) / tmpcos;
	return (tmp / (1.0 - exp(-tmp)));
}

#ifdef _OLDHP
double absorbAirPlate(mR, cosTheta, camr, z)
	double *mR, *cosTheta, *camr, *z;
#else
double absorbAirPlate(const double *mR, const double *cosTheta, const double *camr, const double *z)
#endif
{
	return (exp((*mR) * (1.0 / (*cosTheta) - 1.0)));
}

#ifdef _OLDHP
double absorbAirCyl(mR, cosTheta, camr, z)
	double *mR, *cosTheta, *camr, *z;
#else
double absorbAirCyl(const double *mR, const double *cosTheta, const double *camr, const double *z)
#endif
{
	return (exp(- (*mR) * (1 - sqrt(1.0 + (*z)*(*z)/(*camr)/(*camr)))));
}

#ifdef _OLDHP
double absorbAir0(mR, cosTheta, camr, z)
	double *mR, *cosTheta, *camr, *z;
#else
double absorbAir0(const double *mR, const double *cosTheta, const double *camr, const double *z)
#endif
{
	return (1.0);
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] {-f|-cv|-ch|-bf} [-p] [-l]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 {-f|-cv|-ch|-bf} [-p] [-l]\n", ProgName);
#endif
	fprintf(fp0, "          [-nm|-m:VALUE1[:VALUE2]|-mr:VALUE1[:VALUE2]]|-mR:VALUE1[:VALUE2]]\n");
	fprintf(fp0, "          [-is|-u:VALUE|-U:VALUE1[:VALUE2]] [-a:VALUE1:VALUE2] [-cc:VALUE]\n");
	fprintf(fp0, "          [-b:VALUE1[:VALUE2]] [-cr:VALUE] [-cl:VALUE] [-xc:VALUE] [-zc:VALUE]\n");
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-rv] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Executes Lorentz, Polarization, and Absorption correction\n");
	fprintf(fp0, "  for R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File. This must be set\n");
	fprintf(fp0, "                 before other arguments listed below.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	fprintf(fp0, " Correction type :\n");
	fprintf(fp0, "   -l .......... Execute Lorentz Correction.\n");
	fprintf(fp0, "   -p .......... Execute polarization Correction.\n");
	fprintf(fp0, " Camera type :\n");
	fprintf(fp0, "   -f .......... Flat.\n");
	fprintf(fp0, "   -cv ......... Cylindrical, whose axis vertical to the screen.\n");
	fprintf(fp0, "   -ch ......... Cylindrical, whose axis horizontal to the screen.\n");
	fprintf(fp0, "   -bf ......... Flat, with \"Back Laue\" Type. \"Centre\" means incident.\n");
	fprintf(fp0, " Specimen type :\n");
	fprintf(fp0, "   -is ......... Isotropic.\n");
	fprintf(fp0, "   -u:VALUE .... Uniaxially Oriented, whose axis is settled at \"VALUE\" degrees\n");
	fprintf(fp0, "                 from horizon.\n");
	fprintf(fp0, "   -U:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Uniaxially Oriented, whose axis is settled at \"VALUE1\" degrees\n");
	fprintf(fp0, "                 from horizon, and also valid for meridian, using spread\n");
	fprintf(fp0, "                 of lambda about (100 * VALUE2) %%.\n");
	fprintf(fp0, "                 If \"VALUE2\" is absent, use %f.\n", DLAMBDADEFAULT);
	fprintf(fp0, " Monochromator :\n");
	fprintf(fp0, "   -nm ......... No Monochromator.\n");
	fprintf(fp0, "   -m:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "                 Monochromator, whose reflection plane is settled at\n");
	fprintf(fp0, "                 \"VALUE1\" degrees with horizontal direction,\n");
	fprintf(fp0, "                 and has monochromator Angle \"VALUE2\" degrees in 2-Theta.\n");
	fprintf(fp0, "                 If \"VALUE2\" is absent, angle recorded in R-AXIS File is used.\n");
	fprintf(fp0, "   -mr:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "                 Same as above, but incident x-ray is Synchrotron Radiation,\n");
	fprintf(fp0, "                 which is highly polarized. Reflection plane of the\n");
	fprintf(fp0, "                 Monochromator is within the initial polarization plane.\n");
	fprintf(fp0, "   -mR:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "                 Same as above, but incident x-ray is Synchrotron Radiation,\n");
	fprintf(fp0, "                 which is highly polarized. Reflection plane of the\n");
	fprintf(fp0, "                 Monochromator is normal to the initial polarization plane.\n");
	fprintf(fp0, " Absorption Correction with the Specimen (Assume Flat) :\n");
	fprintf(fp0, "   -a:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Execute Correction, using \"VALUE1\" as Line Absorption\n");
	fprintf(fp0, "                 Coefficient in [1/mm], and \"VALUE2\" as Thickness of the\n");
	fprintf(fp0, "                 Specimen in [mm].\n");
	fprintf(fp0, "   -cc:VALUE ... Set COS(2Theta) Lower-Limit to \"VALUE\". This is caused by\n");
	fprintf(fp0, "                 \"Flat Specimen\", and is not neccessary for Plate camera.\n");
	fprintf(fp0, "                 If absent, set to %f.\n", CosThetaCrit);
	fprintf(fp0, " Absorption Correction with the Air :\n");
	fprintf(fp0, "   -b:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Execute Correction, using \"VALUE1\" as Line Absorption\n");
	fprintf(fp0, "                 Coefficient in [1/mm], and \"VALUE2\" as Camera Length in [mm].\n");
	fprintf(fp0, "                 If \"VALUE2\" is absent, Camera Length recorded in R-AXIS File\n");
	fprintf(fp0, "                 is used.\n");
	fprintf(fp0, " Other Options :\n");
	fprintf(fp0, "   -cr:VALUE ... Set Minimum Radius for Calculation to \"VALUE\".\n");
	fprintf(fp0, "                 Default is %f[mm].\n", RadiusCrit);
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Change Pixel Size in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change Pixel Size in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -rv ......... Execute Reverse Correction.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}

#ifdef _OLDHP
char checkfLp0(fLp)
	double *fLp;
#else
char checkfLp0(double *fLp)
#endif
{
	if ((*fLp) > 0.0) return (0);
	return (2);
}

#ifdef _OLDHP
char checkfLp1(fLp)
	double *fLp;
#else
char checkfLp1(double *fLp)
#endif
{
	if ((*fLp) > 0.0) {
		(*fLp) = 1.0 / (*fLp);
		return (0);
	}
	if ((*fLp) == 0.0) {
		*fLp = 4294967296.0;
		return (0);
	}
	return (2);
}
