/* Program to treat IP data file */
#define ProgName "ippeak"
#define ProgInfo "Calculate Intensity of specified oval region"
/*                                                        11/21/2001  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.20_5/12/2003"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef MAPFILE
#	define MAXARG 15
#else
#	define MAXARG 12
#endif
#define MINARG 5
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define MAXNUM 100000
#define DLONG 4294967296.0

#ifdef _OLDHP
	long int (*readDat)();
#ifdef MAPFILE
	short int (*writeDat)();
#endif
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#ifndef _HAS_LONGLONG
	char ull_add();
	char ull_mul();
#endif
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
#ifdef MAPFILE
	short int (*writeDat)(const long int *x, const double *rato, const char *shiftBit, const long int *maxInt, const char *ipDataSwapFlag);
#endif
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#ifndef _HAS_LONGLONG
	char ull_add(unsigned long int *x, unsigned long int *base);
	char ull_mul(const unsigned long int x, const unsigned long int y, unsigned long int *base);
#endif
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, k, ipHeaderSize, maxInt, startX, endX, startXb, endXb,
	     ipHeaderSize2VER1, ipHeaderSize2VER2, minI, maxI, minIx, minIz,
	     maxIx, maxIz, readByte, maxZ, minZ, maxZb, minZb,
	     maxX, minX, maxXb, minXb, num, numb;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, arg0[4], verboseFlag, xcDoneFlag,
	     zcDoneFlag, ofileDoneFlag, long_aDoneFlag, short_bDoneFlag,
	     phiDoneFlag, phiAutoFlag, symFlag, skinDoneFlag, debug, forceFlag,
	     short_brDoneFlag, long_aPhiDoneFlag, symDoneFlag, *strPtr, simpleFlag,
	     outVerboseFlag, timebuf[20], appendFlag, retry;
	double rato, centreX, centreZ, long_a, short_b, long_ab, short_bb, phi,
	       cosPhi, sinPhi, fact1, fact2, fact2b, cosPhiSq, long_aSq, short_bSq,
	       long_abSq, short_bbSq, fact1b, fg, bg, z, minDelta, pxlSize,
	       arg1, arg2, fact3, fact3b, fact4, skin, phi0,
	       centreXt, centreZt, sinPhit, dsigma, dsigmab, dwx, dwz;
#ifdef _HAS_LONGLONG
	unsigned long long int sum, sumb, sigma, sigmab, wx, wz, ulbuf;
#else
	unsigned long int sum[2], sumb[2], ulbuf[2], sigma[2], sigmab[2], sigbuf[2],
	                  wx[2], wz[2];
	double dsum, dsumb;
#endif
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	time_t timeNow;
#ifdef MAPFILE
	short int *datPtrMap;
	long int recordLengthCreate, mapFgInt, mapBgInt, ipHeaderSizeMin, readByteMap;
	char file3[FNAMELEN], createDosFlag, mapfileDoneFlag, mapFgDoneFlag, mapBgDoneFlag;
	FILE *fp3;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = xcDoneFlag = zcDoneFlag = short_bDoneFlag = long_aDoneFlag
	 = ofileDoneFlag = phiDoneFlag = symFlag = phiAutoFlag = skinDoneFlag = debug
	 = forceFlag = short_brDoneFlag = long_aPhiDoneFlag = symDoneFlag = simpleFlag
	 = outVerboseFlag = appendFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	maxInt = 0x7fffffff;
	centreX = centreZ = long_a = short_b = phi = phi0 = centreXt = centreZt = sinPhit = 0;
	retry = 0;
#ifdef MAPFILE
	mapFgDoneFlag = mapBgDoneFlag = mapfileDoneFlag = createDosFlag = 0;
	readByteMap = 0;
	datPtrMap = NULL;
	fp3 = NULL;
	recordLengthCreate = 0;
#endif
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-qq")) {
			verboseFlag = -1;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
	}
	if (verboseFlag != -1) showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unabel to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
#ifdef MAPFILE
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
#endif
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
#ifdef MAPFILE
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
#endif
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	timeNow = time(NULL);
	
#ifdef MAPFILE
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcpy(ipHeader.memo, "Generated by ");
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, " on ");
	strcat(ipHeader.memo, ipHeader.date);
#endif
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
#ifdef MAPFILE
		writeDat = ipLtoShortBit0;
#endif
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
#ifdef MAPFILE
				writeDat = ipLtoShortBit;
#endif
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
#ifdef MAPFILE
			writeDat = ipLtoShortRato;
#endif
		}
	}
	minDelta = MAXEXT(ipHeader.xsiz, ipHeader.zsiz) * 2;
	skin = minDelta / 2.0;
	pxlSize = MAXEXT(ipHeader.xsiz, ipHeader.zsiz);
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter X-coordinate of centre of oval : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			centreX = atof(inputBuffer);
			if ((centreX < 0.0) || (centreX >= ipHeader.xpxl)) continue;
			xcDoneFlag = 1;
		} while (!xcDoneFlag);
		do {
			fprintf(stdout, "Enter Z-coordinate of centre of oval : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			centreZ = atof(inputBuffer);
			if ((centreZ < 0.0) || (centreZ >= ipHeader.zpxl)) continue;
			zcDoneFlag = 1;
		} while (!zcDoneFlag);
		do {
			fprintf(stdout, "Enter half-length of Major-axis of oval [mm].");
			fprintf(stdout, " Type \"p\" following to the value is interpreted as %6.4f pixel-unit : ", pxlSize);
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			arg1 = strtod(inputBuffer, &strPtr);
			if (strPtr == inputBuffer) continue;
			long_a = atof(inputBuffer);
			if (!strcmp(strPtr, "P") || !strcmp(strPtr, "p")) long_a /= pxlSize;
			else if (long_a == 0.0) continue;
			if (long_a <= minDelta) continue;
			long_aDoneFlag = 1;
		} while (!long_aDoneFlag);
		do {
			fprintf(stdout, "Enter half-length of Minor-axis of oval [mm] : ");
			fprintf(stdout, " Type \"p\" following to the value is interpreted as %6.4f pixel-unit.", pxlSize);
			fprintf(stdout, " Type \"r\" following to the value is interpreted as rato to Major-axis : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			arg1 = strtod(inputBuffer, &strPtr);
			if (strPtr == inputBuffer) continue;
			short_b = atof(inputBuffer);
			if (!strcmp(strPtr, "P") || !strcmp(strPtr, "p")) short_b /= pxlSize;
			else if (!strcmp(strPtr, "R") || !strcmp(strPtr, "r")) short_b *= long_a;
			else if (short_b == 0.0) continue;
			if ((short_b <= minDelta) || (short_b > long_a)) continue;
			short_bDoneFlag = 1;
		} while (!short_bDoneFlag);
		do {
			fprintf(stdout, "Enter rotation angle of long-axis from X-direction [deg.]\n");
			fprintf(stdout, " If you want to do it along tangential direction, enter \"D\". : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') {
				if (centreX == ipHeader.drtx && centreZ == ipHeader.drtz) {
					fprintf(stdout, "Centre of oval is centre of image, so enter angle manually.\n");
					continue;
				}
				phiDoneFlag = 1;
				do {
					fprintf(stdout, "Camera is Flat[F] , Curved around Z [V] or Curved around X [H] ? : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'F') && (toupper(confirmBuffer[0]) != 'V') && (toupper(confirmBuffer[0]) != 'H'));
				if (toupper(confirmBuffer[0]) == 'F') phiAutoFlag = 1;
				else if (toupper(confirmBuffer[0]) == 'V') phiAutoFlag = 2;
				else phiAutoFlag = 3;
				break;
			}
			else if (!isdigitEXT(inputBuffer)) continue;
			phi = radEXT(atof(inputBuffer));
			phiDoneFlag = 1;
		} while (!phiDoneFlag);
		do {
			fprintf(stdout, "Enter Skin-Depth taking as a background region [mm]\n");
			fprintf(stdout, " If you are OK with \"%f\"[mm], enter \"D\". : ", skin);
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') {
				skinDoneFlag = 1;
				break;
			}
			else if (!isdigitEXT(inputBuffer)) continue;
			skin = atof(inputBuffer);
			if (skin >= short_b) {
				fprintf(stdout, "Depth must be longer than length of Minor-axis.\n");
				continue;
			}
			skinDoneFlag = 1;
		} while (!skinDoneFlag);
		do {
			fprintf(stdout, "Do you want to calculate also about symmetric region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			symDoneFlag = 1;
			do {
				fprintf(stdout, "About X-Axis [X], Z-Axis [Z], X, Z-Axis [N] or Centre [R] ? : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'X') && (toupper(confirmBuffer[0]) != 'Z')
			      && (toupper(confirmBuffer[0]) != 'N') && (toupper(confirmBuffer[0]) != 'R'));
			if (toupper(confirmBuffer[0]) == 'Z') symFlag = 1;
			else if (toupper(confirmBuffer[0]) == 'X') symFlag = 2;
			else if (toupper(confirmBuffer[0]) == 'R') symFlag = 3;
			else symFlag = 4;
		}
	}
	else {
		for (i=1; i<argc; i++) {
			if (!strncmp(argv[i], "-o:", 3)) {
				strcpy(file2, argv[i]+3);
				ofileDoneFlag++;
			}
#ifdef MAPFILE
			else if (!strncmp(argv[i], "-mp:", 4)) {
				strcpy(file3, argv[i]+4);
				mapfileDoneFlag++;
			}
#endif
			else {
				argNum = splitArg(argv[i], arg0, &arg1, &arg2);
				if (argNum == 1) {
					if (!strcmp(arg0, "-sz")) {
						symFlag = 1;
						symDoneFlag++;
					}
					else if (!strcmp(arg0, "-sx")) {
						symFlag = 2;
						symDoneFlag++;
					}
					else if (!strcmp(arg0, "-sr")) {
						symFlag = 3;
						symDoneFlag++;
					}
					else if (!strcmp(arg0, "-sn")) {
						symFlag = 4;
						symDoneFlag++;
					}
					else if (!strcmp(arg0, "-f")) forceFlag++;
					else if (!strcmp(arg0, "-tf")) {
						phiAutoFlag = 1;
						phiDoneFlag++;
					}
					else if (!strcmp(arg0, "-tv")) {
						phiAutoFlag = 2;
						phiDoneFlag++;
					}
					else if (!strcmp(arg0, "-th")) {
						phiAutoFlag = 3;
						phiDoneFlag++;
					}
					else if (!strcmp(arg0, "-n")) simpleFlag++;
					else if (!strcmp(arg0, "-ov")) outVerboseFlag++;
					else if (!strcmp(arg0, "-ad")) appendFlag++;
#ifdef DEBUG
					else if (!strcmp(arg0, "-d")) debug++;
#endif
					else break;
				}
				else if (argNum == 2) {
					if (!strcmp(arg0, "-x")) {
						centreX = arg1;
						if ((centreX < 0) || (centreX >= ipHeader.xpxl)) break;
						xcDoneFlag++;
					}
					else if (!strcmp(arg0, "-z")) {
						centreZ = arg1;
						if ((centreZ < 0) || (centreZ >= ipHeader.zpxl)) break;
						zcDoneFlag++;
					}
					else if (!strcmp(arg0, "-a")) {
						long_a = arg1;
						if (long_a <= minDelta) break;
						long_aDoneFlag++;
					}
					else if (!strcmp(arg0, "-b")) {
						short_b = arg1;
						if (short_b <= minDelta) break;
						short_bDoneFlag++;
					}
					else if (!strcmp(arg0, "-ap")) {
						long_a = arg1*pxlSize;
						if (long_a <= minDelta) break;
						long_aDoneFlag++;
					}
					else if (!strcmp(arg0, "-bp")) {
						short_b = arg1*pxlSize;
						if (short_b <= minDelta) break;
						short_bDoneFlag++;
					}
					else if (!strcmp(arg0, "-rb")) {
						short_b = arg1;
						if (short_b <= 0.0 || short_b > 1.0) break;
						short_bDoneFlag++;
						short_brDoneFlag++;
					}
					else if (!strcmp(arg0, "-g")) {
						skin = arg1;
						if (skin < 0.0) break;
						skinDoneFlag++;
					}
					else if (!strcmp(arg0, "-t")) {
						phi = radEXT(arg1);
						phiDoneFlag++;
					}
#ifdef MAPFILE
					else if (!strcmp(arg0, "-fm")) {
						mapFgInt = dToLong(arg1);
						if (mapFgInt < 0) break;
						mapFgDoneFlag++;
					}
					else if (!strcmp(arg0, "-bm")) {
						mapBgInt = dToLong(arg1);
						if (mapBgInt < 0) break;
						mapBgDoneFlag++;
					}
#endif
					else break;
				}
				else if (argNum == 3) {
					if (!strcmp(arg0, "-xa")) {
						phi0 = arg1;
						phi = arg2;
						phiDoneFlag++;
						long_aDoneFlag++;
						long_aPhiDoneFlag++;
					}
					else break;
				}
				else break;
			}
		}
		if (i != argc) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((xcDoneFlag > 1) || (zcDoneFlag > 1) || (symDoneFlag > 1)
		 || (long_aDoneFlag > 1) || (short_bDoneFlag > 1)
		 || (phiDoneFlag > 1) || (skinDoneFlag > 1) || (forceFlag > 1)
		 || (short_brDoneFlag > 1) || (simpleFlag > 1) || (ofileDoneFlag > 1)
		 || (outVerboseFlag > 1)
#ifdef MAPFILE
		 || (mapfileDoneFlag > 1) || (mapFgDoneFlag > 1) || (mapBgDoneFlag > 1)
#endif
		 ) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!xcDoneFlag || !zcDoneFlag || !long_aDoneFlag
		 || !short_bDoneFlag || !phiDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
#ifdef MAPFILE
		if (!mapfileDoneFlag && (mapFgDoneFlag || mapBgDoneFlag)) {
			fprintf(stderr, " Map Filename is not specified. Ignored.\n");
		}
		if (mapfileDoneFlag && !mapFgDoneFlag) mapFgInt = maxInt;
		if (mapfileDoneFlag && !mapBgDoneFlag) mapBgInt = maxInt/2;
#endif
		if (!ofileDoneFlag && appendFlag) appendFlag = 0;
		if (long_aPhiDoneFlag) {
			if (centreX-phi0 == 0.0 && centreZ-phi == 0.0) {
				centreX = ipHeader.drtx;
				centreZ = ipHeader.drtz;
				phiAutoFlag++; /* Cause error in the Next routine */
			}
			else {
				long_a = dist((centreX-phi0)*ipHeader.xsiz, (centreZ-phi)*ipHeader.zsiz);
				phi = atan2((centreZ-phi)*ipHeader.zsiz, (centreX-phi0)*ipHeader.xsiz);
			}
		}
		if (short_brDoneFlag) short_b *= long_a;
		if (long_a < short_b || (phiAutoFlag && centreX == ipHeader.drtx && centreZ == ipHeader.drtz)
		 || (skinDoneFlag && skin >= short_b)) {
			fprintf(stderr, " Invalid Value.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (outVerboseFlag && simpleFlag) outVerboseFlag = 0;
	if (phiAutoFlag == 1) phi = fmod(atan2(centreZ-ipHeader.drtz, centreX-ipHeader.drtx)+M_PI, M_PI) - M_PI_2;
	else if (phiAutoFlag == 2) {
		z = (centreZ-ipHeader.drtz)*ipHeader.zsiz/ipHeader.camr;
		if (z == 0.0 || fabs((centreX-ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr) == M_PI_2) phi = M_PI_2;
		else phi = (atan(- (z + 1.0/z) * tan((centreX-ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr)) + M_PI_2);
	}
	else if (phiAutoFlag == 3) {
		z = (centreX-ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr;
		if (z == 0.0 || fabs((centreZ-ipHeader.drtz)*ipHeader.zsiz/ipHeader.camr) == M_PI_2) phi = M_PI_2;
		else phi = (atan(- (z + 1.0/z) * tan((centreZ-ipHeader.drtz)*ipHeader.zsiz/ipHeader.camr)) + M_PI_2);
	}
	if ((cosPhi = cos(phi)) < 0.0) {
		phi -= M_PI;
		cosPhi *= -1.0;
	}
	sinPhi = sin(phi);
#if 0
	fact1 = long_a * long_a * sinPhi * sinPhi + short_b * short_b * cosPhi * cosPhi;
	long_ab = long_a - skin;
	short_bb = short_b - skin;
	fact1b = long_ab * long_ab * sinPhi * sinPhi + short_bb * short_bb * cosPhi * cosPhi;
#else
	cosPhiSq = cosPhi * cosPhi;
	long_aSq = long_a * long_a;
	short_bSq = short_b * short_b;
	fact1 = long_aSq + (short_bSq - long_aSq) * cosPhiSq;
	long_ab = long_a - skin;
	short_bb = short_b - skin;
	long_abSq = long_ab * long_ab;
	short_bbSq = short_bb * short_bb;
	fact1b = long_abSq + (short_bbSq - long_abSq) * cosPhiSq;
#endif
	if (verboseFlag >= 2 && symDoneFlag) {
		fprintf(stdout, " Direct beam is ( %6.1f x %6.1f ).\n", ipHeader.drtx, ipHeader.drtz);
	}
	if (outVerboseFlag) {
		strftime(timebuf, 20, "%Y-%m-%d %H:%M:%S", localtime(&timeNow));
	}
	
/* Set Parameters to Create */
	if (M_PI*long_a*short_b/ipHeader.xsiz/ipHeader.zsiz > MAXNUM) {
		if (verboseFlag == 3) {
			do {
				fprintf(stdout, " Calculation Area Exceeds %d. Do you want to continue ? [Y/N] : ", MAXNUM);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else if (!forceFlag) {
			fprintf(stderr, " Calculation Area Exceeds %d. Unable to continue. Stop.\n", MAXNUM);
			fprintf(stderr, " Check arguments, or use \"-f\" to force with this value.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!strcmp(file2, "-")) ofileDoneFlag = 1;
	}
	
	while (createFileEXT(file2, &fp2)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
	if (ofileDoneFlag) {
		for (retry=0;;) {
			if (retry) {
				retry = 0;
				if (verboseFlag == 3) {
					appendFlag = 0;
					fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
					if (fscanf(stdin, "%s", file2) == EOF) {
						fprintf(stderr, "\nError in getting filename. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!strcmp(file2, "-")) {
						ofileDoneFlag = 1;
						break;
					}
				}
				else {
					fprintf(stdout, " Enter Another Filename : ");
					if (fscanf(stdin, "%s", file2) == EOF) {
						fprintf(stderr, "\nError in getting filename. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				}
			}
			if (createFileEXT(file2, &fp2) == 0) {
				appendFlag = 0;
				break;
			}
			if ((fp2 = fopen(file2, "rb")) == NULL) {
				if (!verboseFlag) {
					fprintf(stderr, " Output File Creation Error. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				retry = 1;
				fprintf(stdout, " Failed to Create Output File.");
				continue;
			}
			if (verboseFlag == 3) {
				fprintf(stderr, " Output File Already Exists.\n");
				do {
					fprintf(stdout, "  Do you want append to the existing file ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						fclose(fp2);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
				if (toupper(confirmBuffer[0]) == 'Y') appendFlag = 1;
			}
			fclose(fp2);
			if (!appendFlag) {
				if (!verboseFlag) {
					fprintf(stderr, " Output File Creation Error. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				retry = 1;
				fprintf(stdout, " Output File Already Exists.");
				continue;
			}
			if ((fp2 = fopen(file2, "a+")) != NULL) break;
			if (!verboseFlag) {
				fprintf(stderr, " Cannot Append to Output File. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			retry = 1;
			fprintf(stdout, " Cannot Append to Output File.");
		}
	}
	else fp2 = stdout;
	
#ifdef MAPFILE
/* makes map file */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to generate map file ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					if (!appendFlag) {
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
				}
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') mapfileDoneFlag = 1;
	}
	if (mapfileDoneFlag) {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Map  Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					if (!appendFlag) {
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
				}
				exit(EXIT_FAILURE);
			}
			do {
				fprintf(stdout, " Enter Map intensity of foreground : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						if (!appendFlag) {
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
					}
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				mapFgInt = atol(inputBuffer);
				if (mapFgInt < 0) continue;
				mapFgDoneFlag = 1;
			} while (!mapFgDoneFlag);
			do {
				fprintf(stdout, " Enter Map intensity of background : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						if (!appendFlag) {
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
					}
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				mapBgInt = atol(inputBuffer);
				if (mapBgInt < 0) continue;
				mapBgDoneFlag = 1;
			} while (!mapBgDoneFlag);
		}
		while (createFileEXT(file3, &fp3)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					if (!appendFlag) {
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
				}
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					if (!appendFlag) {
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
				}
				exit(EXIT_FAILURE);
			}
		}
		recordLengthCreate = MAXEXT(ipHeaderSizeMin, ipHeader.xpxl*RAXDATAWIDTH);
		if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
		if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
		else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
		if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
		else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
		ipHeader.rlng = recordLengthCreate;
		ipHeader.rnum = ipHeader.zpxl;
		if (ipHeaderSwapFlag) {
			swapIpHeader(&ipHeader);
			createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
		}
		if ((datPtrMap = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				if (!appendFlag) {
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
			}
			fclose(fp3);
			remove(file3);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		memset(datPtrMap, 0, recordLengthCreate);
		fwrite(&ipHeader, ipHeaderSize, 1, fp3);
		createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp3) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp3);
		fwrite(datPtrMap, recordLengthCreate-ipHeaderSizeMin, 1, fp3);
		if (ipHeaderSwapFlag) {
			swapIpHeader(&ipHeader);
			createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
		}
		for (i=0; i<ipHeader.rnum; i++) fwrite(datPtrMap, recordLengthCreate, 1, fp3);
		free(datPtrMap);
		mapBgInt = (*writeDat)(&mapBgInt, &rato, &shiftBit, &maxInt, &ipDataSwapFlag);
		mapFgInt = (*writeDat)(&mapFgInt, &rato, &shiftBit, &maxInt, &ipDataSwapFlag);
	}
#endif
	
/* Start Calculation */
	if (verboseFlag > 0) fprintf(stdout, " Now, in progress...\n");
	/* start routine ... */
	/* some variables for calculation */
	if (outVerboseFlag) {
		fprintf(fp2, "------------------------------------------------------------------------\n");
		fprintf(fp2, "Work About %s.\n", file1);
		fprintf(fp2, "Calculated by %s-%s on %s\n", ProgName, VERSION, timebuf);
	}
	for (k=0; k<4; k++) {
		if (k == 0) {
			centreXt = centreX;
			centreZt = centreZ;
			sinPhit = sinPhi;
		}
		else if (k == 1) {
			if (symFlag != 1 && symFlag != 4) continue;
			centreXt = 2.0*ipHeader.drtx - centreX;
			centreZt = centreZ;
			sinPhit = -sinPhi;
		}
		else if (k == 2) {
			if (symFlag != 2 && symFlag != 4) continue;
			centreXt = centreX;
			centreZt = 2.0*ipHeader.drtz - centreZ;
			sinPhit = -sinPhi;
		}
		else if (k == 3) {
			if (symFlag != 3 && symFlag != 4) continue;
			centreXt = 2.0*ipHeader.drtx - centreX;
			centreZt = 2.0*ipHeader.drtz - centreZ;
			sinPhit = sinPhi;
		}
		if (!simpleFlag) fprintf(fp2, "------------------------------------------------------------------------\n");
		if (centreXt < 0 || centreXt >= ipHeader.xpxl || centreZt < 0 || centreZt >= ipHeader.zpxl) {
			if (verboseFlag > 0) fprintf(fp2, "[%1ld]Center is out of image. Skip.\n", k+1);
			continue;
		}
#if 0
		/* these are range of rectangle which just fit to the outline of oval. */
		minX = dToLong(centreXt - sqrt(long_a*long_a+short_b*short_b-fact1) / ipHeader.xsiz);
		maxX = dToLong(centreXt + sqrt(long_a*long_a+short_b*short_b-fact1) / ipHeader.xsiz);
		minZ = dToLong(centreZt - sqrt(fact1) / ipHeader.zsiz);
		maxZ = dToLong(centreZt + sqrt(fact1) / ipHeader.zsiz);
		/* background region */
		minXb = dToLong(centreXt - sqrt(long_ab*long_ab+short_bb*short_bb-fact1b) / ipHeader.xsiz);
		maxXb = dToLong(centreXt + sqrt(long_ab*long_ab+short_bb*short_bb-fact1b) / ipHeader.xsiz);
		minZb = dToLong(centreZt - sqrt(fact1b) / ipHeader.zsiz);
		maxZb = dToLong(centreZt + sqrt(fact1b) / ipHeader.zsiz);
#else
		/* these are range of rectangle which just fit to the outline of oval. */
		minX = dToLong(centreXt - sqrt(long_aSq+short_bSq-fact1) / ipHeader.xsiz);
		maxX = dToLong(centreXt + sqrt(long_aSq+short_bSq-fact1) / ipHeader.xsiz);
		minZ = dToLong(centreZt - sqrt(fact1) / ipHeader.zsiz);
		maxZ = dToLong(centreZt + sqrt(fact1) / ipHeader.zsiz);
		/* background region */
		minXb = dToLong(centreXt - sqrt(long_abSq+short_bbSq-fact1b) / ipHeader.xsiz);
		maxXb = dToLong(centreXt + sqrt(long_abSq+short_bbSq-fact1b) / ipHeader.xsiz);
		minZb = dToLong(centreZt - sqrt(fact1b) / ipHeader.zsiz);
		maxZb = dToLong(centreZt + sqrt(fact1b) / ipHeader.zsiz);
#endif
		if (verboseFlag > 0 && (minX < 0 || maxX >= ipHeader.xpxl || minZ < 0 || maxZ >= ipHeader.zpxl)) {
			fprintf(stderr, "*** Warning : Some of the data are out of screen. Ignored. ***\n");
			if (ofileDoneFlag) fprintf(fp2, "*** Warning : Some of the data are out of screen. Ignored. ***\n");
		}
#ifdef DEBUG
		if (debug) fprintf(stderr, "  minZ, minZb,  maxZ, maxZb,  minX, minXb,  maxX, maxXb\n%6ld, %6ld, %6ld, %6ld, %6ld, %6ld, %6ld, %6ld\n", minZ, minZb, maxZ, maxZb, minX, minXb, maxX, maxXb);
#endif
		minZ = MAXEXT(0, minZ);
		minZb = MAXEXT(0, minZb);
		maxZ = MINEXT(ipHeader.zpxl, maxZ);
		maxZb = MINEXT(ipHeader.zpxl, maxZb);
		minX = MAXEXT(0, minX);
		minXb = MAXEXT(0, minXb);
		maxX = MINEXT(ipHeader.xpxl, maxX);
		maxXb = MINEXT(ipHeader.xpxl, maxXb);
		fact2 = (long_aSq-short_bSq)*sinPhit*cosPhi/fact1/ipHeader.xsiz;
		fact2b = (long_abSq-short_bbSq)*sinPhit*cosPhi/fact1b/ipHeader.xsiz;
		fact3 = long_a*short_b/fact1/ipHeader.xsiz;
		fact3b = long_ab*short_bb/fact1b/ipHeader.xsiz;
		/* start working */
		if (fseek(fp1, ipHeader.rlng*(minZ+1), SEEK_SET)) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				if (k && verboseFlag > 0) fprintf(stdout, " Output File is saved using available data.\n");
				else if (!k && !appendFlag) {
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
			}
			exit(EXIT_FAILURE);
		}
		readByte = ftell(fp1);
#ifdef MAPFILE
		if (mapfileDoneFlag) {
			if (fseek(fp3, ipHeader.rlng*(minZ+1), SEEK_SET)) {
				fprintf(stderr, " Map File Seek Error.\n");
				fclose(fp1);
				fclose(fp3);
				fprintf(stderr, " Output File is not created or removed.\n");
				if (k || appendFlag) {
					if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
					if (ofileDoneFlag) fclose(fp2);
				}
				else {
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
				}
				if (k) {
					if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
				}
				else {
					remove(file3);
					fprintf(stderr, " Map File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			readByteMap = ftell(fp3);
		}
#endif
#ifdef _HAS_LONGLONG
		sum = sumb = 0L;
		sigma = sigmab = 0L;
		wx = wz = 0L;
#else
		sum[0] = sum[1] = sumb[0] = sumb[1] = 0L;
		ulbuf[0] = 0L;
		sigma[0] = sigma[1] = sigmab[0] = sigmab[1] = 0L;
		wx[0] = wx[1] = wz[0] = wz[1] = 0L;
#endif
		num = numb = 0;
		maxI = 0;
		minI = maxInt;
		minIx = minIz = maxIx = maxIz = 0;
		for (i=minZ; i<maxZ; i++) {
			if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp1);
				if (k || appendFlag) {
					if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
					if (ofileDoneFlag) fclose(fp2);
				}
				else {
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
				}
#ifdef MAPFILE
				if (mapfileDoneFlag) {
					fclose(fp3);
					if (k) {
						if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
					}
					else {
						remove(file3);
						fprintf(stderr, " Map File is not created or removed.\n");
					}
				}
#endif
				exit(EXIT_FAILURE);
			}
			fread(datPtr, ipHeader.rlng, 1, fp1);
			readByte += ipHeader.rlng;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				fclose(fp1);
				if (k || appendFlag) {
					if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
					if (ofileDoneFlag) fclose(fp2);
				}
				else {
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					else fprintf(stderr, " No data to print.\n");
				}
#ifdef MAPFILE
				if (mapfileDoneFlag) {
					fclose(fp3);
					if (k) {
						if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
					}
					else {
						remove(file3);
						fprintf(stderr, " Map File is not created or removed.\n");
					}
				}
#endif
				exit(EXIT_FAILURE);
			}
#ifdef MAPFILE
			if (mapfileDoneFlag) {
				if ((datPtrMap = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (k || appendFlag) {
						if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
						if (ofileDoneFlag) fclose(fp2);
					}
					else {
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						else fprintf(stderr, " No data to print.\n");
					}
					fclose(fp3);
					if (k) {
						if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
					}
					else {
						remove(file3);
						fprintf(stderr, " Map File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrMap, ipHeader.rlng, 1, fp3);
				readByteMap += ipHeader.rlng;
				if (ftell(fp3) != readByteMap) {
					fprintf(stderr, " Map File Read Error : Map File is Too Short.\n");
					free(datPtr);
					free(datPtrMap);
					fclose(fp1);
					if (k || appendFlag) {
						if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
						if (ofileDoneFlag) fclose(fp2);
					}
					else {
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						else fprintf(stderr, " No data to print.\n");
					}
					fclose(fp3);
					if (k) {
						if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
					}
					else {
						remove(file3);
						fprintf(stderr, " Map File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
			}
#endif
			z = (i + 0.5 - centreZt) * ipHeader.zsiz;
			fact4 = fact3 * sqrt(fact1 - z*z);
			startX = MAXEXT(dToLong(centreXt + (z * fact2 - fact4)), 0);
			endX   = MINEXT(dToLong(centreXt + (z * fact2 + fact4)), ipHeader.xpxl);
			if (i >= minZb && i < maxZb) {
				fact4 = fact3b * sqrt(fact1b - z*z);
				startXb = MAXEXT(dToLong(centreXt + (z * fact2b - fact4)), 0);
				endXb   = MINEXT(dToLong(centreXt + (z * fact2b + fact4)), ipHeader.xpxl);
			}
			else startXb = endXb = endX;
#ifdef DEBUG
			if (debug) fprintf(stderr, "i=%5ld : startX, startXb, endXb, endX : %ld, %ld, %ld, %ld\n", i, startX, startXb, endXb, endX);
#endif
			for (j=startX; j<startXb; j++) {
#ifdef _HAS_LONGLONG
				ulbuf = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				sumb += ulbuf;
				sigmab += (ulbuf*ulbuf);
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %lld, sumb = %llu\n", j, i, ulbuf, sumb);
#endif
#else
				ulbuf[1] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				ull_add(ulbuf, sumb);
				ull_mul(ulbuf[1], ulbuf[1], sigbuf);
				ull_add(sigbuf, sigmab);
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %ld, sumb[1] = %lu, sumb[0] = %lu\n", j, i, ulbuf[1], sumb[1], sumb[0]);
#endif
#endif
				numb++;
#ifdef MAPFILE
				if (mapfileDoneFlag) datPtrMap[j] = mapBgInt;
#endif
			}
			for (j=startXb; j<endXb; j++) {
#ifdef _HAS_LONGLONG
				ulbuf = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				sum += ulbuf;
				sigma += (ulbuf*ulbuf);
				if (ulbuf >= maxI) {
					maxI = ulbuf;
					maxIx = j;
					maxIz = i;
				}
				if (ulbuf <= minI) {
					minI = ulbuf;
					minIx = j;
					minIz = i;
				}
				wx += ulbuf * j;
				wz += ulbuf * i;
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %lld, sum = %llu\n", j, i, ulbuf, sum);
#endif
#else
				ulbuf[1] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				ull_add(ulbuf, sum);
				ull_mul(ulbuf[1], ulbuf[1], sigbuf);
				ull_add(sigbuf, sigma);
				if (ulbuf[1] >= maxI) {
					maxI = ulbuf[1];
					maxIx = j;
					maxIz = i;
				}
				if (ulbuf[1] <= minI) {
					minI = ulbuf[1];
					minIx = j;
					minIz = i;
				}
				ull_mul(ulbuf[1], j, sigbuf);
				ull_add(sigbuf, wx);
				ull_mul(ulbuf[1], i, sigbuf);
				ull_add(sigbuf, wz);
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %ld, sum[1] = %lu, sum[0] = %lu\n", j, i, ulbuf[1], sum[1], sum[0]);
#endif
#endif
				num++;
#ifdef MAPFILE
				if (mapfileDoneFlag) datPtrMap[j] = mapFgInt;
#endif
			}
			for (j=endXb; j<endX; j++) {
#ifdef _HAS_LONGLONG
				ulbuf = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				sumb += ulbuf;
				sigmab += (ulbuf*ulbuf);
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %lld, sumb = %llu\n", j, i, ulbuf, sumb);
#endif
#else
				ulbuf[1] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				ull_add(ulbuf, sumb);
				ull_mul(ulbuf[1], ulbuf[1], sigbuf);
				ull_add(sigbuf, sigmab);
#ifdef DEBUG
				if (debug) fprintf(stderr, "I[%5ld, %5ld] = %ld, sumb[1] = %lu, sumb[0] = %lu\n", j, i, ulbuf[1], sumb[1], sumb[0]);
#endif
#endif
				numb++;
#ifdef MAPFILE
				if (mapfileDoneFlag) datPtrMap[j] = mapBgInt;
#endif
			}
			free(datPtr);
#ifdef MAPFILE
			if (mapfileDoneFlag) {
				if (fseek(fp3, -ipHeader.rlng, SEEK_CUR)) {
					fprintf(stderr, " Map File Seek Error.\n");
					fclose(fp1);
					fclose(fp3);
					fprintf(stderr, " Output File is not created or removed.\n");
					if (k || appendFlag) {
						if (verboseFlag > 0 && ofileDoneFlag) fprintf(stdout, " Output File is saved using available data.\n");
						if (ofileDoneFlag) fclose(fp2);
					}
					else {
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						else fprintf(stderr, " No data to print.\n");
					}
					if (k) {
						if (verboseFlag > 0) fprintf(stdout, " Map File is saved using available data.\n");
					}
					else {
						remove(file3);
						fprintf(stderr, " Map File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fwrite(datPtrMap, recordLengthCreate, 1, fp3);
				readByteMap = ftell(fp3);
				free(datPtrMap);
			}
#endif
		}
		if (num == 0) {
			fprintf(stderr, "Error : No effective region.\n");
			continue;
		}
#ifdef _HAS_LONGLONG
		bg = numb == 0 ? 0.0 : sumb / (double)numb;
		fg = sum / (double)num;
		dwx = wx / (double)sum + 0.5;
		dwz = wz / (double)sum + 0.5;
		dsigma  = sqrt(sigma /(double)num  - fg*fg);
		dsigmab = sqrt(sigmab/(double)numb - bg*bg);
		phi = degEXT(asin(sinPhit));
		if (simpleFlag) {
			fprintf(fp2, "%.1f, %.1f, ", centreXt, centreZt);
			fprintf(fp2, "%.3f, %.1f, %.3f, ", long_a*2.0, phi, short_b*2.0);
			fprintf(fp2, "%ld, %ld, %ld, ", num+numb, num, numb);
			fprintf(fp2, "%lld, %.0f, ", sum+sumb, (fg-bg)*num);
			fprintf(fp2, "%11.4f, %11.4f, ", fg-bg, dsigma);
			fprintf(fp2, "%11.4f, %11.4f, ", bg, dsigmab);
			fprintf(fp2, "%ld, %ld, %ld, %ld, %ld, %ld, ", maxI, maxIx, maxIz, minI, minIx, minIz);
			fprintf(fp2, "%.2f, %.2f\n", dwx, dwz);
		}
		else {
			if (symFlag) fprintf(fp2, "Area Information[%1ld] / Centre ( %.1f, %.1f )\n", k+1, centreXt, centreZt);
			else fprintf(fp2, "Area Information / Centre ( %.1f, %.1f )\n", centreXt, centreZt);
			fprintf(fp2, "     MajorAxis : %.3f[mm] with angle %.1f[deg.] / MinorAxis : %.3f[mm]\n", long_a*2.0, phi, short_b*2.0);
			fprintf(fp2, " Work about %ld pixels : %ld is effective, %ld is surrounding regions.\n", num+numb, num, numb);
			fprintf(fp2, "  Total Intensities : %lld,  Effective Total Intensities : %.0f\n", sum+sumb, (fg-bg)*num);
			fprintf(fp2, "  Average            : %11.4f,  Sigma : %11.4f\n", fg-bg, dsigma);
			fprintf(fp2, "  Average Background : %11.4f,  Sigma : %11.4f\n", bg, dsigmab);
			fprintf(fp2, "  Maximum : %ld ( %ld, %ld ), Minimum : %ld ( %ld, %ld )\n", maxI, maxIx, maxIz, minI, minIx, minIz);
			fprintf(fp2, "  Centre of Mass : ( %.2f, %.2f )\n", dwx, dwz);
		}
#else
		dsum  =  sum[0]*DLONG+ sum[1];
		dsumb = sumb[0]*DLONG+sumb[1];
		dsigma  =  sigma[0]*DLONG+ sigma[1];
		dsigmab = sigmab[0]*DLONG+sigmab[1];
		dwx = wx[0]*DLONG+wx[1];
		dwz = wz[0]*DLONG+wz[1];
		bg = numb == 0 ? 0.0 : dsumb / numb;
		fg = dsum / num;
		dwx = dwx / dsum + 0.5;
		dwz = dwz / dsum + 0.5;
		dsigma  = sqrt(dsigma /num  - fg*fg);
		dsigmab = sqrt(dsigmab/numb - bg*bg);
		phi = degEXT(asin(sinPhit));
		if (simpleFlag) {
			fprintf(fp2, "%.1f, %.1f, ", centreXt, centreZt);
			fprintf(fp2, "%.3f, %.1f, %.3f, ", long_a*2.0, phi, short_b*2.0);
			fprintf(fp2, "%ld, %ld, %ld, ", num+numb, num, numb);
			fprintf(fp2, "%.0f, %.0f, ", dsum+dsumb, (fg-bg)*num);
			fprintf(fp2, "%11.4f, %11.4f, ", fg-bg, dsigma);
			fprintf(fp2, "%11.4f, %11.4f, ", bg, dsigmab);
			fprintf(fp2, "%ld, %ld, %ld, %ld, %ld, %ld, ", maxI, maxIx, maxIz, minI, minIx, minIz);
			fprintf(fp2, "%.2f, %.2f\n", dwx, dwz);
		}
		else {
			if (symFlag) fprintf(fp2, "Area Information[%1ld] / Centre ( %.1f, %.1f )\n", k+1, centreXt, centreZt);
			else fprintf(fp2, "Area Information / Centre ( %.1f, %.1f )\n", centreXt, centreZt);
			fprintf(fp2, "     MajorAxis : %.3f[mm] with angle %.1f[deg.] / MinorAxis : %.3f[mm]\n", long_a*2.0, phi, short_b*2.0);
			fprintf(fp2, " Work about %ld pixels : %ld is effective, %ld is surrounding regions.\n", num+numb, num, numb);
			fprintf(fp2, "  Total Intensities : %.0f,  Effective Total Intensities : %.0f\n", dsum+dsumb, (fg-bg)*num);
			fprintf(fp2, "  Average            : %11.4f,  Sigma : %11.4f\n", fg-bg, dsigma);
			fprintf(fp2, "  Average Background : %11.4f,  Sigma : %11.4f\n", bg, dsigmab);
			fprintf(fp2, "  Maximum : %ld ( %ld, %ld ), Minimum : %ld ( %ld, %ld )\n", maxI, maxIx, maxIz, minI, minIx, minIz);
			fprintf(fp2, "  Centre of Mass : ( %.2f, %.2f )\n", dwx, dwz);
		}
#endif
	}
	if (!simpleFlag) fprintf(fp2, "------------------------------------------------------------------------\n");
	
/* End */
	fclose(fp1);
	if (ofileDoneFlag) fclose(fp2);
#ifdef MAPFILE
	if (mapfileDoneFlag) {
		fclose(fp3);
		if (verboseFlag >= 2) {
			fprintf(stdout, " Output Map File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
			fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		}
	}
#endif
	if (verboseFlag >= 2) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-qq|-v] file [-sx|-sz|-sn|-sr] [-f] -x:VALUE -z:VALUE\n", ProgName);
	fprintf(fp0, "          {-a:VALUE|-ap:VALUE} {-b:VALUE|-br:VALUE|-bp:VALUE} [-n|-ov]\n");
	fprintf(fp0, "          [-t:VALUE|-tf|-tv|-th] [-g:VALUE] [-xa:VALUE1:VALUE2]\n");
#ifdef MAPFILE
	fprintf(fp0, "          [-ad] [-o:file2] [-fm:VALUE] [-bm:VALUE] [-mp:file3]\n");
#else
	fprintf(fp0, "          [-ad] [-o:file2]\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Calculate Intensity of specified oval region\n");
	fprintf(fp0, "  of R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -qq ......... Very Quiet Mode. Only a few messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ........ Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -sx ......... Also calculate on symmetric region about X-axis.\n");
	fprintf(fp0, "   -sz ......... Also calculate on symmetric region about Z-axis.\n");
	fprintf(fp0, "   -sn ......... Also calculate on symmetric region about X- and Z-axis.\n");
	fprintf(fp0, "   -sr ......... Also calculate on symmetric region about centre.\n");
	fprintf(fp0, "   -x:VALUE .... Set X-coordinate of the center for calculation to \"VALUE\".\n");
	fprintf(fp0, "   -z:VALUE .... Set Z-coordinate of the center for calculation to \"VALUE\".\n");
	fprintf(fp0, "   -a:VALUE .... Set half length of Major-axis of oval to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -b:VALUE .... Set half length of Minor-axis of oval to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -ap:VALUE ... Set half length of Major-axis of oval to \"VALUE\" [pixel].\n");
	fprintf(fp0, "                 PixelSize used is the larger between X and Z.\n");
	fprintf(fp0, "   -bp:VALUE ... Set half length of Minor-axis of oval to \"VALUE\" [pixel].\n");
	fprintf(fp0, "                 PixelSize used is the larger between X and Z.\n");
	fprintf(fp0, "   -rb:VALUE ... Set relative length of Minor-axis of oval to Major-axis\n");
	fprintf(fp0, "                 to \"VALUE\".\n");
	fprintf(fp0, "   -t:VALUE .... Set Rotation angle of oval to \"VALUE\"[deg.].\n");
	fprintf(fp0, "                 Angle is taken as the rotation of long-axis from X-direction.\n");
	fprintf(fp0, "   -tf ......... Set rotation angle automatically along tangential direction.\n");
	fprintf(fp0, "                 For Flat Camera.\n");
	fprintf(fp0, "   -tv ......... Set rotation angle automatically along tangential direction.\n");
	fprintf(fp0, "                 For Curved Camera, with axis along Z.\n");
	fprintf(fp0, "   -th ......... Set rotation angle automatically along tangential direction.\n");
	fprintf(fp0, "                 For Curved Camera, with axis along X.\n");
	fprintf(fp0, "   -xa:VALUE ... Set length of Major-axis and rotation angle of oval using\n");
	fprintf(fp0, "                 tip coordinate (\"VALUE1\", \"VALUE2\") of Major-axis.\n");
	fprintf(fp0, "   -g:VALUE .... Set skin-depth for background to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to the 1-pixel size.\n");
	fprintf(fp0, "   -f .......... Execute calculation even if number of area exceeds %d.\n", MAXNUM);
	fprintf(fp0, "   -ov ......... Output Verbose (filename, program version, and date).\n");
	fprintf(fp0, "                 Ignored when used with \"-n\".\n");
	fprintf(fp0, "   -o:file2 .... Output to \"file2\". If absent, print to Standard Output.\n");
	fprintf(fp0, "   -ad ......... When output to file, append to existing file.\n");
#ifdef MAPFILE
	fprintf(fp0, "   -mp:file3 ... Makes map file named \"file3\". Selected Region is indicated\n");
	fprintf(fp0, "                 as R-AXIS File Format. Background and Foreground regions are\n");
	fprintf(fp0, "                 shown using the intensity set by another option.\n");
	fprintf(fp0, "   -fm:VALUE ... Set foreground intensity of map file to \"VALUE\".\n");
	fprintf(fp0, "   -bm:VALUE ... Set background intensity of map file to \"VALUE\".\n");
#endif
	fprintf(fp0, "   -n .......... Output by 1 line, without any messages.\n");
	fprintf(fp0, "                 This should be used with other command like awk.\n");
	fprintf(fp0, "                 Output means from n1 to n22 shown below.\n");
	fprintf(fp0, "        Area Information / Centre ( n1, n2 )\n");
	fprintf(fp0, "             MajorAxis : n3[mm] with angle n4 / MinorAxis : n5[mm]\n");
	fprintf(fp0, "         Work about n6 pixels : n7 is effective, n8 is surrounding regions.\n");
	fprintf(fp0, "          Total Intensities : n9,  Effective Total Intensities : n10\n");
	fprintf(fp0, "          Average            :   n11,  Sigma :  n12\n");
	fprintf(fp0, "          Average Background :   n13,  Sigma :  n14\n");
	fprintf(fp0, "          Maximum : n15 ( n16, n17 ), Minimum : n18 ( n19, n20 )\n");
	fprintf(fp0, "          Centre of Mass : ( n21, n22 )\n");
	return (0);
}

#ifndef _HAS_LONGLONG
#ifdef _OLDHP
char ull_add(x, base)
	unsigned long int *x, *base;
#else
char ull_add(unsigned long int *x, unsigned long int *base)
#endif
{
	unsigned long int buf;
	/* base[0] is high, base[1] is low */
	buf = base[1];
	if (buf > (base[1] += x[1])) base[0]++;
	base[0] += x[0];
	return (0);
}
#endif

#ifndef _HAS_LONGLONG
#ifdef _OLDHP
char ull_mul(x, y, base)
	unsigned long int x, y, *base;
#else
char ull_mul(const unsigned long int x, const unsigned long int y, unsigned long int *base)
#endif
{
	unsigned long int buf[2], xbuf[2] , ybuf[2], nbuf[2];
	/* base[0] is high, base[1] is low */
	/* init */
	xbuf[0] = x >> 16;
	xbuf[1] = x & 0xffff;
	ybuf[0] = y >> 16;
	ybuf[1] = y & 0xffff;
	base[1] = xbuf[1]*ybuf[1];
	base[0] = xbuf[0]*ybuf[0];
	/* 2^16 */
	buf[0] = 0;
	buf[1] = xbuf[0]*ybuf[1];
	nbuf[0] = 0;
	nbuf[1] = xbuf[1]*ybuf[0];
	ull_add(nbuf, buf);
	/* add to 2^32 and 1^0 */
	nbuf[1] = buf[1] << 16;
	ull_add(nbuf, base);
	base[0] += ((buf[1] >> 16) + (buf[0] << 16));
	return (0);
}
#endif

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
