/* Program to convert IP data file */
#define ProgName "iprax2png"
#define ProgInfo "Convert R-AXIS Data File to Portable Network Graphic (PNG) File."
/*                                                         6/28/2000  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* This Program depends on "zlib" library.
   You should already have installed zlib shared library and
   header file, or compile and link it together.
   Compilng with HP-UX 9.0x Standard cc compiler might cause ill result,
   for zlib cannot be compiled correctly with the compiler.*/

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.12_5/20/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASZLIB
#	include <zlib.h>
#endif
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "libpng.h"

#ifdef _HASUTIME
#	define MAXARG 11
#else
#	define MAXARG 10
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define WRITEBUFSIZE 16384

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	unsigned short int *datPtr, *shortBuf;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2,
	     ipHeaderSizeMin, maxInt, highLimit, lowLimit, maxRange, window,
	     arg1, readByte, pngRecordLength, writeByte, crc, idatStartPos,
	     chunkLen, zStatus, zFlush, depth, filterCheck[5], zLevel, width;
	unsigned long int xSize, zSize;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, removeFlag, setTimeFlag,
	     arg0[4], verboseFlag, lowLimitDoneFlag, highLimitDoneFlag,
	     reverseFlag, unit, mink, sizeFlag, charBuf[8], noHeadFlag,
	     chunkType[4], smallFlag, filterMode, filterModeDoneFlag,
	     levelDoneFlag;
	unsigned char *readBuf, *writeBuf[5], *outputBuf, *outputBufPrev;
	double rato, intensityDbl;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	struct pngIHDR pngHeader;
	FILE *fp1, *fp2;
#ifdef _HASZLIB
	z_stream z;
#endif
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	depth = 16;
	readByte = writeByte = 0;
	ipDataSwapFlag = lowLimitDoneFlag = highLimitDoneFlag = reverseFlag
	 = removeFlag = setTimeFlag = noHeadFlag = smallFlag = filterModeDoneFlag
	 = levelDoneFlag = 0;
	filterMode = 0xff;
	maxInt = 0x7fff;
	highLimit = lowLimit = 0;
	zStatus = 0;
#ifdef _HASZLIB
	zLevel = Z_DEFAULT_COMPRESSION;
#endif
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check zlib */
#ifndef _HASZLIB
	showMessage(stdout, cpuIntelFlag, 2);
	showUsage(stderr);
	fprintf(stderr, "This program does not work without \"zlib\".\n");
	fprintf(stderr, " Re-compile after install zlib and define \"_HASZLIB\".\n");
	fprintf(stderr, " On HP-UX 9.0x, standard \"cc\" compiler cannot make zlib correctly!!\n");
	exit(EXIT_SUCCESS);
#else
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unabel to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get information about Output File */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to use 16-bits/pixel Colour Depth ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			fprintf(stdout, " Use 8-bits/pixel Colour Depth.\n");
			depth = 8;
		}
		maxRange = (0x00000001L << depth) - 1;
		do {
			fprintf(stdout, "Do you want to change range of 0 - %5ld ? [Y/N] : ", maxRange);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Lower Intensity to set to 0 : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lowLimit = atol(inputBuffer);
				if ((lowLimit < 0) || (lowLimit > maxInt)) continue;
				lowLimitDoneFlag = 1;
			} while (!lowLimitDoneFlag);
			do {
				fprintf(stdout, "  Enter Higher Intensity to set to %5ld : ", maxRange);
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				highLimit = atol(inputBuffer);
				if ((highLimit <= lowLimit) || (highLimit > maxInt)) continue;
				highLimitDoneFlag = 1;
			} while (!highLimitDoneFlag);
		}
		else {
			lowLimit = 0;
			highLimit = maxRange;
		}
		do {
			fprintf(stdout, "Reverse Intensity (i.e. \"0\" corresponds to WHITE) ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseFlag = 1;
		do {
			fprintf(stdout, "Delete Original R-AXIS File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
		do {
			fprintf(stdout, "Do you want to skip saving Original R-AXIS File Information ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') noHeadFlag = 1;
		do {
			fprintf(stdout, "Do you want to decide a PNG Filter Type by yourself ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Filter Type to use [0-4] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				filterMode = atol(inputBuffer);
				if ((filterMode < 0) || (filterMode >= 5)) continue;
				filterModeDoneFlag = 1;
			} while (!filterModeDoneFlag);
		}
		do {
			fprintf(stdout, "Do you want to change compression level by yourself ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Level [%d-%d] : ", Z_NO_COMPRESSION, Z_BEST_COMPRESSION);
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				zLevel = atol(inputBuffer);
				if ((zLevel < Z_NO_COMPRESSION) || (zLevel > Z_BEST_COMPRESSION)) continue;
				levelDoneFlag = 1;
			} while (!levelDoneFlag);
		}
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-rm")) removeFlag++;
				else if (!strcmp(arg0, "-r")) reverseFlag++;
				else if (!strcmp(arg0, "-nu")) noHeadFlag++;
				else if (!strcmp(arg0, "-8")) {
					depth = 8;
					smallFlag++;
				}
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-rl")) {
					lowLimit = arg1;
					if ((lowLimit < 0) || (lowLimit > maxInt)) break;
					lowLimitDoneFlag++;
				}
				else if (!strcmp(arg0, "-rh")) {
					highLimit = arg1;
					if ((highLimit <= 0) || (highLimit > maxInt)) break;
					highLimitDoneFlag++;
				}
				else if (!strcmp(arg0, "-fm")) {
					filterMode = arg1;
					if ((filterMode < 0) || (filterMode >= 5)) break;
					filterModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					zLevel = arg1;
					if ((zLevel < Z_NO_COMPRESSION) || (zLevel > Z_BEST_COMPRESSION)) break;
					levelDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((reverseFlag > 1) || (lowLimitDoneFlag > 1) || (highLimitDoneFlag > 1)
		 || (removeFlag > 1) || (setTimeFlag > 1) || (noHeadFlag > 1)
		 || (smallFlag > 1) || (filterModeDoneFlag > 1) || (levelDoneFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		maxRange = (0x00000001L << depth) - 1;
		if (!lowLimitDoneFlag) lowLimit = 0;
		if (!highLimitDoneFlag) highLimit = maxRange;
		if (highLimit <= lowLimit) {
			fprintf(stderr, " Arguments out of range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		if (reverseFlag) fprintf(stdout, " Convert %ld - %ld in Original R-AXIS File to Black - White.\n", lowLimit, highLimit);
		else fprintf(stdout, " Convert %ld - %ld in Original R-AXIS File to White - Black.\n", lowLimit, highLimit);
		if (removeFlag) fprintf(stdout, " Original R-AXIS Image File will be removed !\n");
		if (noHeadFlag) fprintf(stdout, " Original R-AXIS File Information is not stored !!\n");
		if (filterModeDoneFlag) fprintf(stdout, " Use PNG Filter of %1d for all lines.\n", filterMode);
		if (levelDoneFlag) fprintf(stdout, " Compression Level is %ld.\n", zLevel);
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters */
	pngHeader.width = cpuIntelFlag ? swapLong(ipHeader.xpxl) : ipHeader.xpxl;
	pngHeader.height = cpuIntelFlag ? swapLong(ipHeader.zpxl) : ipHeader.zpxl;
	pngHeader.bitDepth = depth;
	pngHeader.colorType = 0;
	pngHeader.compressMethod = 0;
	pngHeader.filterMethod = 0;
	pngHeader.interlaceMethod = 0;
	unit = (depth == 16) ? 2 : 1;
	pngRecordLength = (long)ceil(ipHeader.xpxl * depth / 8.0);
	window = highLimit - lowLimit;
	width = ipHeader.xpxl * sizeof(short);
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Create and Write PNG Headers */
	/* Signature */
	pngSignatureGenerate(charBuf);
	fwrite(charBuf, 8, 1, fp2);
	writeByte = ftell(fp2);
	/* IHDR Chunk */
	crc = 0x00000000L;
	chunkLen = cpuIntelFlag ? swapLong(13L) : 13;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "IHDR", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite(&pngHeader, 13, 1, fp2);
	crc = crc32(crc, (char*)(&pngHeader), 13);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + 13 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+13+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* pHYs Chunk */
	crc = 0x00000000L;
	chunkLen = cpuIntelFlag ? swapLong(9L) : 9;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "pHYs", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	xSize = cpuIntelFlag ? swapLong(dToLong(1000.0/ipHeader.xsiz)) : dToLong(1000.0/ipHeader.xsiz);
	fwrite(&xSize, 4, 1, fp2);
	crc = crc32(crc, (char*)(&xSize), 4);
	zSize = cpuIntelFlag ? swapLong(dToLong(1000.0/ipHeader.zsiz)) : dToLong(1000.0/ipHeader.zsiz);
	fwrite(&zSize, 4, 1, fp2);
	crc = crc32(crc, (char*)(&zSize), 4);
	sizeFlag = 1;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + 9 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		exit(EXIT_FAILURE);
		fprintf(stderr, " Output File is not created or removed.\n");
	}
	fseek(fp2, writeByte+4+4+9+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tIME Chunk -- Not Currently Supported. */
	/* tEXt Chunk - Title */
	crc = 0x00000000L;
	for (i=0; i<20; i++) if (ipHeader.crnm[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+6) : i+6;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Title", 5, 1, fp2);
	crc = crc32(crc, "Title", 5);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(ipHeader.crnm, i, 1, fp2);
	crc = crc32(crc, ipHeader.crnm, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 6 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+6+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tEXt Chunk - Author */
	crc = 0x00000000L;
	for (i=0; i<20; i++) if (ipHeader.opnm[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+7) : i+7;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Author", 6, 1, fp2);
	crc = crc32(crc, "Author", 6);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(ipHeader.opnm, i, 1, fp2);
	crc = crc32(crc, ipHeader.opnm, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 7 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+7+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tEXt Chunk - Creation Time */
	crc = 0x00000000L;
	for (i=0; i<12; i++) if (ipHeader.date[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+14) : i+14;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Creation Time", 13, 1, fp2);
	crc = crc32(crc, "Creation Time", 13);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(ipHeader.date, i, 1, fp2);
	crc = crc32(crc, ipHeader.date, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 14 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+14+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tEXt Chunk - Software */
	crc = 0x00000000L;
	for (i=0; i<12; i++) if (ProgName[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+9) : i+9;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Software", 8, 1, fp2);
	crc = crc32(crc, "Software", 8);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(ProgName, i, 1, fp2);
	crc = crc32(crc, ProgName, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 9 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+9+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tEXt Chunk - Source */
	crc = 0x00000000L;
	for (i=0; i<256; i++) if (file1[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+7) : i+7;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Source", 6, 1, fp2);
	crc = crc32(crc, "Source", 6);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(file1, i, 1, fp2);
	crc = crc32(crc, file1, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 7 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+7+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* tEXt Chunk - Comment */
	crc = 0x00000000L;
	for (i=0; i<80; i++) if (ipHeader.memo[i] == 0) break;
	chunkLen = cpuIntelFlag ? swapLong(i+8) : i+8;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "tEXt", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	fwrite("Comment", 7, 1, fp2);
	crc = crc32(crc, "Comment", 7);
	sizeFlag = 0;
	fwrite(&sizeFlag, 1, 1, fp2);
	crc = crc32(crc, (char*)(&sizeFlag), 1);
	fwrite(ipHeader.memo, i, 1, fp2);
	crc = crc32(crc, ipHeader.memo, i);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + i + 8 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fseek(fp2, writeByte+4+4+i+8+4, SEEK_SET);
	writeByte = ftell(fp2);
	/* zTXt Chunk */
	if (!noHeadFlag) {
		for (;;) {
			if ((writeBuf[0] = malloc((long)ceil(ipHeaderSize*4.0/3))) == NULL) {
				fprintf(stderr, " Failed to allocate memory. IP Header is not stored.\n");
				break;
			}
			if ((readBuf = malloc(ipHeaderSize*2)) == NULL) {
				fprintf(stderr, " Failed to allocate memory. IP Header is not stored.\n");
				free(writeBuf[0]);
				break;
			}
			if (cpuIntelFlag) swapIpHeader(&ipHeader);
			base64en((char*)(&ipHeader), writeBuf[0], ipHeaderSize);
			if (cpuIntelFlag) swapIpHeader(&ipHeader);
			z.zalloc = Z_NULL;
			z.zfree = Z_NULL;
			z.opaque = Z_NULL;
			if (deflateInit(&z, zLevel) != Z_OK) {
				fprintf(stderr, " DeflateInit Error : %s\n", (z.msg) ? z.msg : "unknown");
				fprintf(stderr, " IP Header is not stored.\n");
				free(readBuf);
				free(writeBuf[0]);
				break;
			}
			z.next_in = writeBuf[0];
			z.avail_in = (long)ceil(ipHeaderSize*4.0/3);
			z.next_out = readBuf;
			z.avail_out = ipHeaderSize * 2;
			sizeFlag = 0;
			do {
				zStatus = deflate(&z, Z_FINISH);
				if (zStatus == Z_STREAM_END) break;
				if (z.avail_in == 0) {
					if (sizeFlag ++ > 10) break;
				}
			} while (zStatus == Z_OK);
			free(writeBuf[0]);
			if (zStatus != Z_STREAM_END) {
				fprintf(stderr, " Deflate Error. IP Header is not stored.\n");
				free(readBuf);
				break;
			}
			chunkLen = ipHeaderSize * 2 - z.avail_out + 12;
			if (deflateEnd(&z) != Z_OK) {
				fprintf(stderr, " DeflateEnd Error : %s\n", (z.msg) ? z.msg : "unknown");
				free(readBuf);
				break;
			}
			crc = 0x00000000L;
			if (cpuIntelFlag) chunkLen =  swapLong(chunkLen);
			fwrite(&chunkLen, 4, 1, fp2);
			if (cpuIntelFlag) chunkLen =  swapLong(chunkLen);
			strncpy(chunkType, "zTXt", 4);
			fwrite(chunkType, 4, 1, fp2);
			crc = crc32(crc, chunkType, 4);
			fwrite("xRAXISHEAD", 10, 1, fp2);
			crc = crc32(crc, "xRAXISHEAD", 10);
			sizeFlag = 0;
			fwrite(&sizeFlag, 1, 1, fp2);
			fwrite(&sizeFlag, 1, 1, fp2);
			crc = crc32(crc, (char*)(&sizeFlag), 1);
			crc = crc32(crc, (char*)(&sizeFlag), 1);
			fwrite(readBuf, chunkLen-12, 1, fp2);
			crc = crc32(crc, readBuf, chunkLen-12);
			free(readBuf);
			if (cpuIntelFlag) crc = swapLong(crc);
			fwrite(&crc, 4, 1, fp2);
			if (ftell(fp2) < writeByte + 4 + 4 + chunkLen + 4) {
				fprintf(stderr, " File Write Error. Exit.\n");
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fseek(fp2, writeByte+4+4+chunkLen+4, SEEK_SET);
			writeByte = ftell(fp2);
			break;
		}
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp1, ipHeader.rlng*ipHeader.rnum, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	crc = 0x00000000L;
	idatStartPos = ftell(fp2);
	chunkLen = 0;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "IDAT", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	writeByte += 8;
	/* Deflate */
	z.zalloc = Z_NULL;
	z.zfree = Z_NULL;
	z.opaque = Z_NULL;
	if ((datPtr = malloc(width)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((shortBuf = malloc(ipHeader.xpxl*sizeof(short))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((outputBufPrev = malloc(pngRecordLength)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr);
		free(shortBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((outputBuf = malloc(pngRecordLength)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr);
		free(shortBuf);
		free(outputBufPrev);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((readBuf = malloc(WRITEBUFSIZE)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr);
		free(shortBuf);
		free(outputBufPrev);
		free(outputBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	for (k=0; k<5; k++) {
		if ((writeBuf[k] = malloc(pngRecordLength+1)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr);
			free(shortBuf);
			free(outputBufPrev);
			free(outputBuf);
			free(readBuf);
			for (j=0; j<k; j++) free(writeBuf[j]);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
	}
	memset(outputBufPrev, 0, pngRecordLength);
	if (deflateInit(&z, zLevel) != Z_OK) {
		fprintf(stderr, " DeflateInit Error. : %s\n", (z.msg) ? z.msg : "unknown");
		free(readBuf);
		for (j=0; j<5; j++) free(writeBuf[j]);
		free(outputBuf);
		free(outputBufPrev);
		free(shortBuf);
		free(datPtr);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	z.avail_in = 0;
	z.next_out = readBuf;
	z.avail_out = WRITEBUFSIZE;
	zFlush = Z_NO_FLUSH;
	for (i=0; i<ipHeader.rnum; i++) {
		sizeFlag = 0;
		fread(datPtr, width, 1, fp1);
		fseek(fp1, -ipHeader.rlng-width, SEEK_CUR);
		for (j=0; j<ipHeader.xpxl; j++) {
			intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			if (intensityDbl >= highLimit) intensityDbl = maxRange;
			else if (intensityDbl <= lowLimit) intensityDbl = 0;
			else intensityDbl = (intensityDbl - lowLimit) * maxRange / window;
			if (reverseFlag) intensityDbl = maxRange - intensityDbl;
			shortBuf[j] = dToShort(intensityDbl);
		}
		pngMakeScanline(shortBuf, outputBuf, depth, ipHeader.xpxl, cpuIntelFlag);
		if (filterMode >= 0 && filterMode < 5) {
			pngFilter(outputBuf, outputBufPrev, writeBuf[(int)filterMode], unit, filterMode, pngRecordLength);
			mink = filterMode;
		}
		else {
			for (k=0; k<5; k++) {
				pngFilter(outputBuf, outputBufPrev, writeBuf[k], unit, k, pngRecordLength);
				filterCheck[k] = 0;
				mink = 0;
				for (j=0; j<pngRecordLength+1; j++) filterCheck[k] += writeBuf[k][j] < 128 ? writeBuf[k][j] : 256 - writeBuf[k][j];
				if (k && (filterCheck[k] < filterCheck[(int)mink])) mink = k;
			}
		}
		free(outputBufPrev);
		if (i < ipHeader.rnum -1) {
			outputBufPrev = outputBuf;
			if ((outputBuf = malloc(pngRecordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(readBuf);
				for (j=0; j<5; j++) free(writeBuf[j]);
				free(outputBufPrev);
				free(shortBuf);
				free(datPtr);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
		}
		else zFlush = Z_FINISH;
		z.next_in = writeBuf[(int)mink];
		z.avail_in = pngRecordLength + 1;
		for (;;) {
			zStatus = deflate(&z, zFlush);
			if (zStatus == Z_STREAM_END) {
				if (i == ipHeader.rnum - 1) break;
				else z.msg = "Unexpected End.";
			}
			if (zStatus != Z_OK) {
				fprintf(stderr, "Deflate Error : %s\n", (z.msg) ? z.msg : "unknown");
				free(readBuf);
				for (j=0; j<5; j++) free(writeBuf[j]);
				if (i < ipHeader.rnum - 1) free(outputBufPrev);
				free(shortBuf);
				free(datPtr);
				free(outputBuf);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			if (z.avail_out == 0) {
				fwrite(readBuf, WRITEBUFSIZE, 1, fp2);
				crc = crc32(crc, readBuf, WRITEBUFSIZE);
				z.next_out = readBuf;
				z.avail_out = WRITEBUFSIZE;
				writeByte += WRITEBUFSIZE;
			}
			if (z.avail_in == 0) {
				if (i < ipHeader.rnum - 1) break;
				if (sizeFlag++ > 10) break;
			}
		}
	}
	for (j=0; j<5; j++) free(writeBuf[j]);
	free(shortBuf);
	free(datPtr);
	if (i < ipHeader.rnum - 1) free(outputBufPrev);
	free(outputBuf);
	fclose(fp1);
	if (zStatus != Z_STREAM_END) {
		fprintf(stderr, "Deflate Error. Exit.\n");
		free(readBuf);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fwrite(readBuf, WRITEBUFSIZE-z.avail_out, 1, fp2);
	if (deflateEnd(&z) != Z_OK) {
		fprintf(stderr, "DeflateEnd Error : %s\n", (z.msg) ? z.msg : "unknown");
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	crc = crc32(crc, readBuf, WRITEBUFSIZE-z.avail_out);
	free(readBuf);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	writeByte += WRITEBUFSIZE - z.avail_out + 4;
	if (writeByte != ftell(fp2)) {
		fprintf(stderr, " Deflation Error : Output Length does not match. Exit.\n");
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	chunkLen = writeByte - idatStartPos - 12;
	fseek(fp2, idatStartPos, SEEK_SET);
	if (cpuIntelFlag) chunkLen = swapLong(chunkLen);
	fwrite(&chunkLen, 4, 1, fp2);
	fseek(fp2, writeByte, SEEK_SET);
	
/* End */
	/* IEND Chunk */
	crc = 0x00000000L;
	chunkLen = 0;
	fwrite(&chunkLen, 4, 1, fp2);
	strncpy(chunkType, "IEND", 4);
	fwrite(chunkType, 4, 1, fp2);
	crc = crc32(crc, chunkType, 4);
	if (cpuIntelFlag) crc = swapLong(crc);
	fwrite(&crc, 4, 1, fp2);
	if (ftell(fp2) < writeByte + 4 + 4 + 4) {
		fprintf(stderr, " File Write Error. Exit.\n");
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	writeByte = ftell(fp2);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels, each pixel has %ld bits,\n", ipHeader.xpxl, ipHeader.zpxl, depth);
		fprintf(stdout, "  total %ld bytes.\n", writeByte);
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original R-AXIS Image File.\n");
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
#endif /* _HASZLIB */
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-t] [-r] [-nu] [-8]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-r] [-nu] [-8]\n", ProgName);
#endif
	fprintf(fp0, "          [-rl:VALUE] [-rh:VALUE2] [-fm:VALUE] [-z:VALUE] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
#ifdef _HASZLIB
	fprintf(fp0, " This Program Convert R-AXIS Image File to 16 or 8-bits gray-scale PNG File.\n");
	fprintf(fp0, "  Note that R-AXIS File has about 18bits-depth, which is much greater than\n");
	fprintf(fp0, "  PNG File can hold. You must specify the way.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -8 .......... Generate 8bits/pixel PNG File. If absent, 16bits-depth.\n");
	fprintf(fp0, "   -rl:VALUE ... Set Lower Critical Intensity for convertion to 0 to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, set to 0.\n");
	fprintf(fp0, "   -rh:VALUE ... Set Higher Critical Intensity for convertion to 255\n");
	fprintf(fp0, "                 to \"VALUE\". If absent, set to 255.\n");
	fprintf(fp0, "   -fm:VALUE ... Use Filter \"VALUE\" without optimization.\n");
	fprintf(fp0, "                 This makes the faster work, but the file might be larger.\n");
	fprintf(fp0, "                 Only 0 to 4 values are valid. See PNG Documents.\n");
	fprintf(fp0, "   -z:VALUE .... Set Compression Level to \"VALUE\".\n");
	fprintf(fp0, "                 \"VALUE\" must be integer between %d and %d.\n", Z_NO_COMPRESSION, Z_BEST_COMPRESSION);
	fprintf(fp0, "                 If absent, set to default value in zlib.\n");
	fprintf(fp0, "   -nu ......... Do not store R-AXIS File Information to PNG File.\n");
	fprintf(fp0, "                 This is not convenient when re-convert to R-AXIS File,\n");
	fprintf(fp0, "                 for conversion to PNG File losses many original R-AXIS File\n");
	fprintf(fp0, "                 information. Original Header is stored after Base64 coding\n");
	fprintf(fp0, "                 preceding to LZ77 deflation.\n");
	fprintf(fp0, "   -rm ......... Remove Original R-AXIS Image File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of R-AXIS Image Data File.\n");
	fprintf(fp0, "   file2 ....... Output Bitmap Filename. This must be the last argument.\n");
#endif /* _HASZLIB */
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
