/* Program to convert IP data file */
#define ProgName "ipsmooth"
#define ProgInfo "Image smoothing"
/*                                                         5/15/2000  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.19_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 6
#else
#	define MAXARG 5
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define MAXNUMBER 19
#define SMALLSUM 0.001
#define SIGMADEFAULT 2.0
#define CRITDEFAULT 1.2

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	double (*myFunc)();
	double nullFunc();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
	double (*myFunc)(double val);
	double nullFunc(double val);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr, **datPtrPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, maxInt, intensity,
	     readByte, centrex, centrez, k, ii, jj, iiTmp, outNum,
	     pointx, pointz, longWidth, flushWidth, padInt, **longPtrPtr,
	     *lBuf, lBufLen, il, ik, lBufLen2, pointzm, pointxm, i2, flushWidth0;
	char file1[FNAMELEN], file2[FNAMELEN], ffile[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, createDosFlag,
	     arg0[4], verboseFlag, modeDoneFlag, replaceFlag, setTimeFlag, modeFlag,
	     padDoneFlag, absFlag, bufCheck, useWeight, critDoneFlag,
	     sigmaDoneFlag, uselBuf;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, sum, *weight, sizePtr[2], dblBuf, s_out, s_out2,
	     arg1, sigmaRatio, critRatio, critVal;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2, *fp0;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	ipDataSwapFlag = modeDoneFlag = replaceFlag = setTimeFlag
	 = padDoneFlag = absFlag = bufCheck = useWeight = sigmaDoneFlag
	 = critDoneFlag = uselBuf = 0;
	modeFlag = 1;
	padInt = 0;
	flushWidth = 0;
	outNum = 0;
	sigmaRatio = SIGMADEFAULT;
	critRatio = CRITDEFAULT;
	pointx = pointz = 3;
	centrex = centrez = 1;
	sum = 0.0;
	fp0 = (FILE*)NULL;
	weight = (double*)NULL;
	lBuf = (long*)NULL;
	lBufLen = lBufLen2 = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* Get Constant to Add */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter Smoothing Type : \n");
			fprintf(stdout, "  (1) Simple Average ,     (2) Weighted Average ,\n");
			fprintf(stdout, "  (3) Polynomial Fitting , (4) MAX value norm   ,\n");
			fprintf(stdout, "  (5) Clean Peaks ,        (6) Median norm      ,\n");
			fprintf(stdout, "  (7) Prewitt Mask ,       (0) Incorporate from file : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			modeFlag = atol(inputBuffer);
			if (modeFlag < 0 || modeFlag > 7) continue;
			modeDoneFlag = 1;
		} while (!modeDoneFlag);
		if (modeFlag == 3) {
			do {
				pointx = pointz = 5;
				fprintf(stdout, "  Smooth with 5x5 Square. OK ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, "  Sorry, only 5x5 Square is supported in current version.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else if (modeFlag == 7) pointx = pointz = 3;
		else if (modeFlag == 0) {
			fprintf(stdout, "Enter Filter Filename : ");
			if (fscanf(stdin, "%s", ffile) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if ((fp0 = fopen(ffile, "r")) == NULL) {
				fprintf(stderr, " Unable to Open File.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else {
			modeDoneFlag = 0;
			do {
				fprintf(stdout, "  Enter Number of Pixel of 1-side of Square for Smoothing : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				pointx = pointz = atol(inputBuffer);
				if (pointx < 3 || pointx > MAXNUMBER || pointx % 2 == 0) continue;
				modeDoneFlag = 1;
			} while (!modeDoneFlag);
		}
		if (modeFlag != 4 && modeFlag != 5 && modeFlag != 6) useWeight = 1;
	/* Get Constant to Add */
		if (useWeight && modeFlag != 7) {
			do {
				fprintf(stdout, "Enter Constant Value to add(pad) before processing : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					if (modeFlag == 0) fclose(fp0);
					exit(EXIT_FAILURE);
				}
			} while (!isdigitEXT(inputBuffer));
			padInt = atol(inputBuffer);
		}
	/* absolute check */
		if (useWeight && modeFlag != 7) {
			do {
				fprintf(stdout, "Use absolute value instead of minus ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					if (modeFlag == 0) fclose(fp0);
					exit(EXIT_FAILURE);
				}
			} while (toupper(confirmBuffer[0]) != 'Y' && toupper(confirmBuffer[0]) != 'N');
			if (toupper(confirmBuffer[0]) == 'Y') absFlag++;
		}
	/* parameter for cleaning */
		if (modeFlag == 5) {
			do {
				fprintf(stdout, "Enter Maximum Ratio of variance to average : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				sigmaRatio = atof(inputBuffer);
				if (sigmaRatio < 1.0) continue;
				sigmaDoneFlag = 1;
			} while (!sigmaDoneFlag);
			do {
				fprintf(stdout, "Enter Minimum Ratio for cleaning : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				critRatio = atof(inputBuffer);
				if (critRatio < 1.0) continue;
				critDoneFlag = 1;
			} while (!critDoneFlag);
		}
	/* Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				if (modeFlag == 0) fclose(fp0);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				if (modeFlag == 0) fclose(fp0);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-ab")) absFlag++;
				else if (!strcmp(arg0, "-P")) {
					pointx = pointz = 3;
					modeFlag = 7;
					modeDoneFlag++;
				}
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
				else break;
#else
				break;
#endif
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-n")) {
					pointx = pointz = dToLong(arg1);
					if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER)) break;
					modeFlag = 1;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-w")) {
					pointx = pointz = dToLong(arg1);
					if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER)) break;
					modeFlag = 2;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-s")) {
					pointx = pointz = dToLong(arg1);
					if (pointx != 5) break;
					modeFlag = 3;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					pointx = pointz = dToLong(arg1);
					if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER)) break;
					modeFlag = 4;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-x")) {
					pointx = pointz = dToLong(arg1);
					if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER)) break;
					modeFlag = 5;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-e")) {
					pointx = pointz = dToLong(arg1);
					if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER)) break;
					modeFlag = 6;
					modeDoneFlag++;
				}
				else if (!strcmp(arg0, "-p")) {
					padInt = dToLong(arg1);
					padDoneFlag++;
				}
				else if (!strcmp(arg0, "-sv")) {
					sigmaRatio = arg1;
					if (sigmaRatio < 1.0) break;
					sigmaDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					critRatio = arg1;
					if (critRatio < 1.0) break;
					critDoneFlag++;
				}
				else break;
			}
			else if (argNum == 10) {
				if (!strcmp(arg0, "-f")) {
					strcpy(ffile, argv[i]+3);
					modeDoneFlag++;
					modeFlag = 0;
					if ((fp0 = fopen(ffile, "r")) == NULL) {
						fprintf(stderr, " Unable to Open File.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			if (modeFlag == 0) fclose(fp0);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((modeDoneFlag > 1) || (setTimeFlag > 1) || (absFlag > 1)
		 || (padDoneFlag > 1) || (sigmaDoneFlag > 1) || (critDoneFlag > 1)
		 || ((modeFlag == 4 || modeFlag == 5 || modeFlag == 6 || modeFlag == 7) && (absFlag || padDoneFlag))
		 || (modeFlag != 5 && (sigmaDoneFlag || critDoneFlag))) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			if (modeFlag == 0) fclose(fp0);
			exit(EXIT_FAILURE);
		}
		if (modeFlag != 4 && modeFlag != 5 && modeFlag != 6) useWeight = 1;
	}
	
/* Create Transfer Matrix */
	if (modeFlag != 0 && modeFlag != 7 && useWeight) {
		centrex = (pointx - 1) / 2;
		centrez = (pointz - 1) / 2;
		if ((weight = malloc(sizeof(double)*pointx*pointz)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			fprintf(stderr, " Output File is not created.\n");
			exit(EXIT_FAILURE);
		}
	}
	switch (modeFlag) {
		case 1:
			for (i=0; i<pointx; i++) {
				for (j=0; j<pointz; j++) weight[i*pointx+j] = 1.0;
			}
			sum = pointx * pointz;
			break;
		case 2:
			sum = 0.0;
			for (i=0; i<pointx; i++) {
				for (j=0; j<pointz; j++) {
					weight[i*pointx+j] = MAXEXT(centrex, centrez) + 1 - MAXEXT(abs(i-centrex), abs(j-centrez));
					sum += weight[i*pointx+j];
				}
			}
			break;
		case 3:
			weight[0*pointx+0] = weight[0*pointx+4] = weight[4*pointx+0] = weight[4*pointx+4] = 0;
			weight[0*pointx+1] = weight[0*pointx+3] = weight[1*pointx+0] = weight[1*pointx+4] = weight[3*pointx+0] = weight[3*pointx+4] = weight[4*pointx+1] = weight[4*pointx+3] = -14;
			weight[0*pointx+2] = weight[2*pointx+0] = weight[2*pointx+4] = weight[4*pointx+2] = 3;
			weight[1*pointx+1] = weight[1*pointx+3] = weight[3*pointx+1] = weight[3*pointx+3] = 37;
			weight[1*pointx+2] = weight[2*pointx+1] = weight[2*pointx+3] = weight[3*pointx+2] = 54;
			weight[2*pointx+2] = 71;
			sum = 335;
			break;
		case 0:
			if (getValueFromStream(fp0, sizePtr, 2, 1000, 1) < 2) {
				fprintf(stderr, " Filtering Matrix File Read Error. Exit.\n");
				fclose(fp0);
				fclose(fp1);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
			pointz = (long)sizePtr[0];
			pointx = (long)sizePtr[1];
			if ((pointx % 2 == 0) || (pointx < 3) || (pointx > MAXNUMBER) || (pointz % 2 == 0) || (pointz < 3) || (pointz > MAXNUMBER)) {
				fprintf(stderr, " Filtering Matrix File Size Error. Exit.\n");
				fclose(fp0);
				fclose(fp1);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
			centrex = (pointx - 1) / 2;
			centrez = (pointz - 1) / 2;
			if ((weight = malloc(sizeof(double)*pointx*pointz)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp0);
				fclose(fp1);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
			if (verboseFlag >= 2) fprintf(stdout, "Reading Matrix of %ld x %ld...\n", pointx, pointz);
			i = getValueFromStream(fp0, weight, pointx*pointz, 1000, 0);
			fclose(fp0);
			if (i < pointx*pointz) {
				fprintf(stderr, " Filtering Matrix File Read Error. Exit.\n");
				fclose(fp1);
				free(weight);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
			if (i > pointx*pointz && verboseFlag >= 2)
			    fprintf(stderr, " Filtering Matrix File is too Long... But ignore it.\n");
			for (i=0; i<centrez; i++) {
				for (j=0; j<pointx; j++) {
					dblBuf = weight[i*pointx+j];
					weight[i*pointx+j] = weight[(pointz-i-1)*pointx+j];
					weight[(pointz-i-1)*pointx+j] = dblBuf;
				}
			}
			sum = 1.0;
			break;
		case 4:
		case 6:
			sum = 1.0;
			break;
		case 5:
			sum = (pointx + pointz - 2) * 2;
			break;
		case 7:
			sum = 1.0;
			centrex = centrez = 1;
			if ((weight = malloc(sizeof(double)*pointx*pointz*8)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp1);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
			for (i=0; i<8; i++) {
				for (j=0; j<9; j++) weight[i*9+j] = 1.0;
				weight[i*9+4] = -2.0;
			}
			weight[0] = weight[1] = weight[2] = weight[10] = weight[11] = weight[14]
			 = weight[20] = weight[23] = weight[26] = weight[32] = weight[34] = weight[35]
			 = weight[42] = weight[43] = weight[44] = weight[48] = weight[51] = weight[52]
			 = weight[54] = weight[57] = weight[60] = weight[63] = weight[64] = weight[66] = -1.0;
			break;
		default:
			fprintf(stderr, " Error in average type definition. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
	}
	
	if ((2 * pointx - 1 > ipHeader.xpxl) || (2 * pointz - 1 > ipHeader.zpxl)) {
		fprintf(stderr, "Data Area is too small for %ldx%ld-point smoothing. Exit.\n", pointx, pointz);
		fclose(fp1);
		if (useWeight) free(weight);
		exit(EXIT_FAILURE);
	}
	sum = fabs(sum);
	if ((sum < SMALLSUM) && (verboseFlag <= 1)) {
		fprintf(stderr, " Too Small Sum Value of the elements. Use 1.0 instead.");
		sum = 1.0;
	}
	if (verboseFlag >= 2) {
		if (modeFlag == 1) fprintf(stdout, " %ldx%ld-points Smoothing with simple average.\n", pointx, pointz);
		else if (modeFlag == 2) fprintf(stdout, " %ldx%ld-points Smoothing with simple weighted average.\n", pointx, pointz);
		else if (modeFlag == 3) fprintf(stdout, " %ldx%ld-points Smoothing with polynomial fitting.\n", pointx, pointz);
		else if (modeFlag == 4) fprintf(stdout, " %ldx%ld-points Maximum Filtering.\n", pointx, pointz);
		else if (modeFlag == 5) fprintf(stdout, " %ldx%ld-points Peak Filtering (cleaning).\n", pointx, pointz);
		else if (modeFlag == 6) fprintf(stdout, " %ldx%ld-points Median Filtering.\n", pointx, pointz);
		else if (modeFlag == 7) fprintf(stdout, " %ldx%ld-points Prewitt Filtering.\n", pointx, pointz);
		else if (modeFlag == 0) {
			fprintf(stdout, " %ldx%ld-points Filtering with the following Matrix :\n", pointx, pointz);
			for (i=pointz-1; i>=0; i--) {
				for (j=0; j<pointx; j++) fprintf(stdout, " %5.1f", weight[i*pointx+j]);
				fprintf(stdout, "\n");
			}
		}
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	ipHeader.rlng = recordLengthCreate;
	longWidth = recordLengthCreate*sizeof(long)/RAXDATAWIDTH;
	flushWidth0 = recordLengthCreate - flushWidth;
	myFunc = absFlag ? fabs : nullFunc;
	pointzm = pointz - 1;
	pointxm = pointx - 1; /* 2*centrex */
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				if (useWeight) free(weight);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				if (useWeight) free(weight);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				if (useWeight) free(weight);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		if (useWeight) free(weight);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((longPtrPtr = malloc(pointz * sizeof(long*))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		if (useWeight) free(weight);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	for (k=0; k<pointzm; k++) {
		if ((longPtrPtr[k] = malloc(longWidth)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			if (useWeight) free(weight);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			for (i=0; i<k; i++) free(longPtrPtr[i]);
			free(longPtrPtr);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
	}
	if (modeFlag == 5) {
		if ((datPtrPtr = malloc(pointz * sizeof(short*))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			for (i=0; i<pointzm; i++) free(longPtrPtr[i]);
			free(longPtrPtr);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		for (k=0; k<pointzm; k++) {
			if ((datPtrPtr[k] = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				for (i=0; i<pointzm; i++) free(longPtrPtr[i]);
				free(longPtrPtr);
				for (i=0; i<k; i++) free(datPtrPtr[i]);
				free(datPtrPtr);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
		}
		for (i=0; i<pointzm; i++) { /* for first few columns */
			fread(datPtrPtr[i], recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				if (flushWidth0 > 0) free(datPtr0);
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				for (k=0; k<pointz; k++) free(datPtrPtr[k]);
				free(longPtrPtr);
				free(datPtrPtr);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			for (j=0; j<ipHeader.xpxl; j++)
			    longPtrPtr[i][j] = (*readDat)(&datPtrPtr[i][j], &rato, &shiftBit, &ipDataSwapFlag);
		}
		for (i=pointzm; i<ipHeader.rnum; i++) {
			bufCheck = (longPtrPtr[pointzm] = malloc(longWidth)) == NULL ? 0 : 1;
			if ((datPtrPtr[pointzm] = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp1);
				if (flushWidth0 > 0) free(datPtr0);
				if (i != pointzm) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				for (k=0; k<pointzm; k++) free(datPtrPtr[k]);
				if (bufCheck) free(longPtrPtr[pointzm]);
				free(longPtrPtr);
				free(datPtrPtr);
				exit(EXIT_FAILURE);
			}
			fread(datPtrPtr[pointzm], recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i != pointzm) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				for (k=0; k<pointz; k++) free(longPtrPtr[k]);
				for (k=0; k<pointz; k++) free(datPtrPtr[k]);
				free(longPtrPtr);
				free(datPtrPtr);
				exit(EXIT_FAILURE);
			}
			for (j=0; j<ipHeader.xpxl; j++) {
				longPtrPtr[pointzm][j] = (*readDat)(&datPtrPtr[pointzm][j], &rato, &shiftBit, &ipDataSwapFlag);
			}
			for (j=pointx; j<ipHeader.xpxl; j++) {
				s_out = s_out2 = 0.0;
				for (jj=j-pointx; jj<j; jj++) {
					s_out += longPtrPtr[0][jj] + longPtrPtr[pointzm][jj];
					s_out2 += longPtrPtr[0][jj]*longPtrPtr[0][jj]
					        + longPtrPtr[pointzm][jj]*longPtrPtr[pointzm][jj];
				}
				for (ii=1; ii<pointzm; ii++) {
					s_out += longPtrPtr[ii][j-pointx] + longPtrPtr[ii][j-1];
					s_out2 += longPtrPtr[ii][j-pointx]*longPtrPtr[ii][j-pointx]
					        + longPtrPtr[ii][j-1]*longPtrPtr[ii][j-1];
				}
				if ((s_out /= sum) == 0.0) continue;
				if ((s_out2 = s_out2/sum/s_out - s_out) > sigmaRatio) continue;
				critVal = s_out * critRatio;
				for (ii=1; ii<pointzm; ii++) {
					for (jj=j-pointx+1; jj<j-1; jj++) {
						if (longPtrPtr[ii][jj] > critVal) datPtrPtr[ii][jj] = (*writeDat)(&s_out, &rato, &maxInt, &ipDataSwapFlag);
					}
				}
				j += pointx-2;
				outNum++;
			}
			fwrite(datPtrPtr[0], flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrPtr[0]);
			free(longPtrPtr[0]);
			for (k=1; k<pointz; k++) {
				longPtrPtr[k-1] = longPtrPtr[k];
				datPtrPtr[k-1] = datPtrPtr[k];
			}
		}
		for (i=0; i<pointzm; i++) {
			fwrite(datPtrPtr[i], flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrPtr[i]);
			free(longPtrPtr[i]);
		}
		free(datPtrPtr);
	}
	else {
		for (i=0; i<pointzm; i++) { /* for first few columns */
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				if (useWeight) free(weight);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				free(longPtrPtr);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				if (useWeight) free(weight);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				free(longPtrPtr);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			for (j=0; j<ipHeader.xpxl; j++)
			    longPtrPtr[i][j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			if ((i2 = i-centrez) >= 0) {
				for (j=0; j<ipHeader.xpxl; j++) {
					dblBuf = (*myFunc)((double)(longPtrPtr[i2][j] + padInt));
					datPtr[j] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			}
			free(datPtr);
		}
		if (modeFlag == 6) {
			uselBuf = 1;
			lBufLen = (long)ceil(pointx*pointz/2.0);
			lBufLen2 = lBufLen * sizeof(long);
			if ((lBuf = malloc(lBufLen2)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				free(longPtrPtr);
				fprintf(stderr, " Output File is not created.\n");
				exit(EXIT_FAILURE);
			}
		}
		for (i=pointzm; i<ipHeader.rnum; i++) {
			bufCheck = (longPtrPtr[pointzm] = malloc(longWidth)) == NULL ? 0 : 1;
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp1);
				if (flushWidth0 > 0) free(datPtr0);
				if (useWeight) free(weight);
				if (uselBuf) free(lBuf);
				if (i != pointzm) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				if (bufCheck) free(longPtrPtr[pointzm]);
				free(longPtrPtr);
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				if (useWeight) free(weight);
				if (uselBuf) free(lBuf);
				fclose(fp1);
				if (i != pointzm) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				for (k=0; k<pointz; k++) free(longPtrPtr[k]);
				free(longPtrPtr);
				exit(EXIT_FAILURE);
			}
			for (j=0; j<ipHeader.xpxl; j++) {
				longPtrPtr[pointzm][j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			}
			for (j=0; j<centrex; j++) {
				dblBuf = (*myFunc)((double)(longPtrPtr[centrex][j] + padInt));
				datPtr[j] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
			}
			if (modeFlag == 4) {
				for (j=pointxm; j<ipHeader.xpxl; j++) {
					intensity = 0;
					for (ii=0; ii<pointz; ii++) {
						for (jj=j-pointxm; jj<=j; jj++) {
							if (longPtrPtr[ii][jj] > intensity) intensity = longPtrPtr[ii][jj];
						}
					}
					dblBuf = (double)intensity;
					datPtr[j-centrex] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
				}
			}
			else if (modeFlag == 6) {
				for (j=pointxm; j<ipHeader.xpxl; j++) {
					memset(lBuf, 0, lBufLen2);
					for (ii=0; ii<pointz; ii++) {
						for (jj=j-pointxm; jj<=j; jj++) {
							intensity = longPtrPtr[ii][jj];
							for (il=0; il<lBufLen; il++) {
								if (intensity > lBuf[il]) {
									for (ik=lBufLen-1; ik>il; ik--) lBuf[ik] = lBuf[ik-1];
									lBuf[il] = intensity;
									break;
								}
							}
						}
					}
					dblBuf = (double)lBuf[lBufLen-1];
					datPtr[j-centrex] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
				}
			}
			else if (modeFlag == 7) {
				for (j=pointxm; j<ipHeader.xpxl; j++) {
					s_out = 0.0;
					for (k=0; k<8; k++) {
						dblBuf = 0.0;
						ik = k*9;
						for (ii=0; ii<pointz; ii++) {
							iiTmp = ii * pointx - j + pointxm + ik;
							for (jj=j-pointxm; jj<=j; jj++) {
								dblBuf += longPtrPtr[ii][jj] * weight[iiTmp + jj];
							}
						}
						if (dblBuf > s_out) s_out = dblBuf;
					}
					datPtr[j-centrex] = (*writeDat)(&s_out, &rato, &maxInt, &ipDataSwapFlag);
				}
			}
			else {
				for (j=pointxm; j<ipHeader.xpxl; j++) {
					dblBuf = 0.0;
					for (ii=0; ii<pointz; ii++) {
						iiTmp = ii * pointx - j + pointxm;
						for (jj=j-pointxm; jj<=j; jj++) {
							dblBuf += longPtrPtr[ii][jj] * weight[iiTmp + jj];
						}
					}
					dblBuf = (*myFunc)(dblBuf / sum + padInt);
					datPtr[j-centrex] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
				}
			}
			for (j=ipHeader.xpxl-centrex; j<ipHeader.xpxl; j++) {
				dblBuf = (*myFunc)((double)(longPtrPtr[centrex][j] + padInt));
				datPtr[j] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
			}
			fwrite(datPtr, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtr);
			free(longPtrPtr[0]);
			for (k=1; k<pointz; k++) longPtrPtr[k-1] = longPtrPtr[k];
		}
		if (uselBuf) free(lBuf);
		if (useWeight) free(weight);
		for (i=ipHeader.rnum-centrez; i<ipHeader.rnum; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fclose(fp1);
				if (flushWidth0 > 0) free(datPtr0);
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				for (k=0; k<pointzm; k++) free(longPtrPtr[k]);
				free(longPtrPtr);
				exit(EXIT_FAILURE);
			}
			for (j=0; j<ipHeader.xpxl; j++) {
				dblBuf = (*myFunc)((double)(longPtrPtr[0][j] + padInt));
				datPtr[j] = (*writeDat)(&dblBuf, &rato, &maxInt, &ipDataSwapFlag);
			}
			fwrite(datPtr, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtr);
			free(longPtrPtr[0]);
			for (k=1; k<ipHeader.rnum-i; k++) longPtrPtr[k-1] = longPtrPtr[k];
		}
	}
	free(longPtrPtr);
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
		if (modeFlag == 5) fprintf(stdout, "  %ld peaks are cleared.\n", outNum);
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-ab] [-p:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-ab] [-p:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          {-n:VALUE|-w:VALUE|-s:VALUE|-f:filter|-m:VALUE|-x:VALUE|-e:VALUE|-P}\n");
	fprintf(fp0, "          [-cv:VALUE] [-sv:VALUE] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program do 2-D Smoothing for R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -n:VALUE .... Do \"VALUE\"x\"VALUE\"-points smoothing by simple average.\n");
	fprintf(fp0, "                 \"VALUE\" must be an odd number between 3 and 19.\n");
	fprintf(fp0, "   -w:VALUE .... Do \"VALUE\"x\"VALUE\"-points smoothing by weighted average.\n");
	fprintf(fp0, "                 Weight is \"j\" for \"j\"-th square and normalized.\n");
	fprintf(fp0, "                 \"VALUE\" must be an odd number between 3 and %d.\n", MAXNUMBER);
	fprintf(fp0, "   -s:VALUE .... Do \"VALUE\"x\"VALUE\"-points smoothing by another average.\n");
	fprintf(fp0, "                 Weight is according to polynomial fitting.\n");
	fprintf(fp0, "                 \"VALUE\" must be 5 with current version.\n");
	fprintf(fp0, "   -f:filter ... Read \"filter\" file and use it as a filtering matrix.\n");
	fprintf(fp0, "                 The file must begins with the dimensions of the matrix,\n");
	fprintf(fp0, "                 and followed by the matrix elements. For example :\n");
	fprintf(fp0, "                 3, 5\n");
	fprintf(fp0, "                 1.0, 2.0, 4, 5.0, -1.0\n");
	fprintf(fp0, "                 3, 6, 8, 10 -2\n");
	fprintf(fp0, "                 -1, -2, -4  5.0, 1.0\n");
	fprintf(fp0, "                 Dimensions must be an odd number between 3 and 19.\n");
	fprintf(fp0, "                 In each line, right hand of \"#\" is ignored.\n");
	fprintf(fp0, "   -x:VALUE .... Clean spike-noise like peaks smaller than \"VALUE\" pixels.\n");
	fprintf(fp0, "                 This may take a lot of times, and might clean correct peaks.\n");
	fprintf(fp0, "                 \"VALUE\" must be an odd number between 3 and 19.\n");
	fprintf(fp0, "   -m:VALUE .... Use MAXIMUM intensity arround \"VALUE\"x\"VALUE\"-region.\n");
	fprintf(fp0, "                 This is exclusive with above methods, and \"-p\" and \"-ab\".\n");
	fprintf(fp0, "   -e:VALUE .... Use Median intensity arround \"VALUE\"x\"VALUE\"-region.\n");
	fprintf(fp0, "                 This is exclusive with above methods, and \"-p\" and \"-ab\".\n");
	fprintf(fp0, "   -P .......... Use Prewitt Filter, to extract edge of the image.\n");
	fprintf(fp0, "                 This is exclusive with above methods, and \"-p\" and \"-ab\".\n");
	fprintf(fp0, "   -p:VALUE .... Set the padding intensity to \"VALUE\" after calculation.\n");
	fprintf(fp0, "                 If absent, set to 0.\n");
	fprintf(fp0, "   -ab ......... Use absolute value instead of minus.\n");
	fprintf(fp0, "                 These 2 options are only valid with true \"smoothing\" options.\n");
	fprintf(fp0, "   -sv:VALUE ... Set Maximum Ratio of variance to average in surrounding region\n");
	fprintf(fp0, "                 to \"VALUE\". This is used for checking whether the region is\n");
	fprintf(fp0, "                 flat and suitable for correction. VALUE must be larger than 1.\n");
	fprintf(fp0, "                 If absent, set to %4.1f.\n", SIGMADEFAULT);
	fprintf(fp0, "   -cv:VALUE ... Set Minimum intensity ratio to \"VALUE\". If intensity is larger\n");
	fprintf(fp0, "                 than \"this value x average intensity\", reset to average.\n");
	fprintf(fp0, "                 VALUE must be larger than 1. If absent, set to %4.1f.\n", CRITDEFAULT);
	fprintf(fp0, "                 These 2 options are valid only with \"-x\" option.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!strcmp(arg0, "-f")) {
		if (strlen(argV) < 4) return (0);
		return (10);
	}
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}

#ifdef _OLDHP
double nullFunc(val)
	double val;
#else
double nullFunc(double val)
#endif
{
	return (val);
}
