/* Program to convert IP data file */
#define ProgName "ipsub"
#define ProgInfo "Data Subtract after Shift Image"
/*                                                        12/22/1997  */
/*         for ANSI C                                      4/25/1999  */
/*         for command-line argument                      11/19/1999  */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.58_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 12
#else
#	define MAXARG 11
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#ifdef RAXMODE
#undef ProgName
#define ProgName "ipsubrax"
#endif

#ifdef _OLDHP
	long int (*readDat1)();
	long int (*readDat2)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	double (*myFunc)();
	double logFunc();
	double absFunc();
	double logabsFunc();
	double nullFunc();
#else
	long int (*readDat1)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	long int (*readDat2)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
	double (*myFunc)(double val);
	double logFunc(double val);
	double absFunc(double val);
	double logabsFunc(double val);
	double nullFunc(double val);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1, *datPtr2;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     ipHeaderSizeMin, offsetX, offsetZ, maxInt1, maxInt2, padInt, readByte1,
	     readByte2, recordLengthCreate, startZ, endZ, startX, endX, flushWidth, flushWidth0;
	char file1[FNAMELEN], file2[FNAMELEN], file3[FNAMELEN], shiftBit1,
	     shiftBit2, cpuIntelFlag, ipHeaderSwapFlag1, ipHeaderSwapFlag2,
	     strangeShiftBitFlag, ipDataSwapFlag1, ipDataSwapFlag2, byte1, byte2,
	     argNum, arg0[4], offsetXDoneFlag, offsetZDoneFlag, createDosFlag,
	     verboseFlag, autoDoneFlag, file1DoneFlag, file2DoneFlag,
	     mag1DoneFlag, mag2DoneFlag, padDoneFlag, inputBuffer[20], confirmBuffer[8],
	     replaceFlag, setTimeFlag, logFlag, absFlag, forceFlag, overwrapFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double mag1, mag2, rato1, rato2, arg1, intensityDbl;
	struct ipHead ipHeaderF1, ipHeaderF2;
	struct ipHead2VER1 ipHeader2VER1F1, ipHeader2VER1F2;
	struct ipHead2VER2 ipHeader2VER2F1, ipHeader2VER2F2;
	FILE *fp1, *fp2, *fp3, *fp4;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	readByte1 = readByte2 = 0;
	verboseFlag = 1;
	mag1 = mag2 = 1.0;
	offsetX = offsetZ = 0;
	padInt = 0;
	ipDataSwapFlag1 = ipDataSwapFlag2 = offsetXDoneFlag = offsetZDoneFlag = autoDoneFlag =
	file1DoneFlag = file2DoneFlag = mag1DoneFlag = mag2DoneFlag = padDoneFlag =
	replaceFlag = setTimeFlag = logFlag = absFlag = forceFlag = overwrapFlag = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
#ifdef RAXMODE
	if (argc == 11 && !strcmp(argv[1], "-rax")) { /* raxwish mode */
		argv += 2;
		argc -= 2;
		verboseFlag = 0;
		forceFlag = 1;
		if ((fp4 = fopen(argv[8], "a+")) == NULL) {
			fprintf(stderr, " Unable to Open ErrorLog File.\n");
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[2]);
		strcpy(file3, argv[7]);
		fprintf(fp4, "%s on %s and %s.\n", ProgName, file1, file2); /* Logging */
	}
	else {
		fprintf(stderr, " Invalid Arguments.\n");
		exit(EXIT_FAILURE);
	}
#else
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	fp4 = stderr;
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-f1:", 4)) {
				strcpy(file1, argv[i]+4);
				file1DoneFlag++;
			}
			else if (!strncmp(argv[i], "-f2:", 4)) {
				strcpy(file2, argv[i]+4);
				file2DoneFlag++;
			}
		}
		if (!file1DoneFlag && !file2DoneFlag) {
			strcpy(file1, argv[0]);
			strcpy(file2, argv[1]);
			argv += 2;
			argc -= 2;
			file1DoneFlag = file2DoneFlag = 1;
		}
		if (!file1DoneFlag || !file2DoneFlag) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Lack of Argument (File Name).\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if ((file1DoneFlag > 1) || (file2DoneFlag > 1)) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Arguments Duplicated (File Name).\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file3, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
#endif
	
/* File Read 1 */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename 1 : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(fp4, " Unable to Open File 1.\n");
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeaderF1, ipHeaderSize, 1, fp1);
	readByte1 += ipHeaderSize;
	if (ftell(fp1) != readByte1) {
		fprintf(fp4, " Input File 1 Read Error.\n");
		fclose(fp1);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	
/* File Format Check 1 */
	ipHeaderSwapFlag1 = (ipHeaderF1.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag1)) {
		if (verboseFlag) {
			fprintf(stdout, " File 1 is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1F1, ipHeaderSize2VER1, 1, fp1);
		readByte1 += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte1) {
			fprintf(fp4, " Input File 1 Read Error.\n");
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2F1, ipHeaderSize2VER2, 1, fp1);
		readByte1 += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte1) {
			fprintf(fp4, " Input File 1 Read Error.\n");
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header 1 */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1F1);
		else swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	byte1 = createDosFlag ? ipHeader2VER1F1.byte : ipHeader2VER2F1.byte;
	
/* Get ipHeader Information and Check 1 */
	if (XOREXT(byte1, ipHeaderSwapFlag1)) ipDataSwapFlag1 = 1;
	rato1 = ipHeaderF1.rato;
	if (ipHeaderF1.zpxl != ipHeaderF1.rnum) {
		ipHeaderF1.zpxl = ipHeaderF1.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in File 1 has modified.\n");
	}
	recordLength = ipHeaderF1.rlng;
	if (ipHeaderF1.xpxl*RAXDATAWIDTH > ipHeaderF1.rlng) {
		ipHeaderF1.xpxl = ipHeaderF1.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in File 1 has modified.\n");
	}
	if ((ipHeaderF1.xpxl <= 0) || (ipHeaderF1.xpxl > MAXWIDTH) || (ipHeaderF1.zpxl <= 0) || (ipHeaderF1.zpxl > MAXHEIGHT)) {
		fprintf(fp4, " Format of file1 is Incorrect.\n");
		fclose(fp1);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	if (ipHeaderF1.xsiz <= 0.0) {
		ipHeaderF1.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X in File 1 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF1.zsiz <= 0.0) {
		ipHeaderF1.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z in File 1 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF1.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeaderF1.cyld);
		ipHeaderF1.cyld = 0;
	}
	
/* Check bits to shift to gain dynamic-range 1 */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato1 < MINRATO) || (rato1 > MAXRATO)) {
		fprintf(fp4, " Input File 1 Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	else if (rato1 == 1.0) {
		maxInt1 = 0x7fff;
		readDat1 = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit1 = 0;
	}
	else {
		for (shiftBit1=1; shiftBit1<16; shiftBit1++) {
			if ((double)(0x0001 << shiftBit1) == rato1) {
				strangeShiftBitFlag = 0;
				maxInt1 = ((long)(0x7fff)) << shiftBit1;
				readDat1 = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt1 = dToLong((0x7fff) * rato1);
			readDat1 = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* File Read 2 */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Original Filename 2 : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
	}
	if ((fp2 = fopen(file2, "rb")) == NULL) {
		fprintf(fp4, " Unable to Open File 2.\n");
		fclose(fp1);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	fread(&ipHeaderF2, ipHeaderSize, 1, fp2);
	readByte2 += ipHeaderSize;
	if (ftell(fp2) != readByte2) {
		fprintf(fp4, " Input File 2 Read Error.\n");
		fclose(fp2);
		fclose(fp1);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	
/* File Format Check 2 */
	ipHeaderSwapFlag2 = (ipHeaderF2.rlng & 0x0000ffff) ? 0 : 1;
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag2)) {
		if (verboseFlag) {
			fprintf(stdout, " File 2 is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					fclose(fp2);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		}
		fread(&ipHeader2VER1F2, ipHeaderSize2VER1, 1, fp2);
		readByte2 += ipHeaderSize2VER1;
		if (ftell(fp2) != readByte2) {
			fprintf(fp4, " Input File 2 Read Error.\n");
			fclose(fp2);
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2F2, ipHeaderSize2VER2, 1, fp2);
		readByte2 += ipHeaderSize2VER2;
		if (ftell(fp2) != readByte2) {
			fprintf(fp4, " Input File 2 Read Error.\n");
			fclose(fp2);
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header 2 */
	if (ipHeaderSwapFlag2) {
		swapIpHeader(&ipHeaderF2);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1F2);
		else swapIpHeader2VER2(&ipHeader2VER2F2);
	}
	byte2 = XOREXT(cpuIntelFlag, ipHeaderSwapFlag2) ? ipHeader2VER1F2.byte : ipHeader2VER2F2.byte;
	
/* Get ipHeader Information and Check 2 */
	if (XOREXT(byte2, ipHeaderSwapFlag2)) ipDataSwapFlag2 = 1;
	rato2 = ipHeaderF2.rato;
	if (ipHeaderF2.zpxl != ipHeaderF2.rnum) {
		ipHeaderF2.zpxl = ipHeaderF2.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in File 2 has modified.\n");
	}
	if (ipHeaderF2.xpxl*RAXDATAWIDTH > ipHeaderF2.rlng) {
		ipHeaderF2.xpxl = ipHeaderF2.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in File 2 has modified.\n");
	}
	if ((ipHeaderF2.xpxl <= 0) || (ipHeaderF2.xpxl > MAXWIDTH) || (ipHeaderF2.zpxl <= 0) || (ipHeaderF2.zpxl > MAXHEIGHT)) {
		fprintf(fp4, " Format of file 2 is Incorrect.\n");
		fclose(fp1);
		fclose(fp2);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	if (ipHeaderF2.xsiz <= 0.0) {
		ipHeaderF2.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X in File 2 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF2.zsiz <= 0.0) {
		ipHeaderF2.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z in File 2 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF2.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeaderF2.cyld);
		ipHeaderF2.cyld = 0;
	}
	
/* Check bits to shift to gain dynamic-range 2 */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato2 < MINRATO) || (rato2 > MAXRATO)) {
		fprintf(fp4, " Input File 2 Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		fclose(fp2);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	else if (rato2 == 1.0) {
		maxInt2 = 0x7fff;
		readDat2 = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit2 = 0;
	}
	else {
		for (shiftBit2=1; shiftBit2<16; shiftBit2++) {
			if ((double)(0x0001 << shiftBit2) == rato2) {
				strangeShiftBitFlag = 0;
				maxInt2 = ((long)(0x7fff)) << shiftBit2;
				readDat2 = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt2 = dToLong((0x7fff) * rato2);
			readDat2 = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* File Size Check 1 */
	if (ipHeaderF1.xsiz != ipHeaderF2.xsiz
	 || ipHeaderF1.zsiz != ipHeaderF2.zsiz
	 || ipHeaderF1.camr != ipHeaderF2.camr) {
		fprintf(fp4, " Pixel size or camera length does not match. Exit.\n");
		fclose(fp1);
		fclose(fp2);
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	if ((ipHeaderSwapFlag1 != ipHeaderSwapFlag2) || (rato1 != rato2)) {
		fprintf(stdout, " File Format is set according to the 1st File.\n");
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeaderF1);
	cleanIpHeader(&ipHeaderF2);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1F1) : cleanIpHeader2VER2(&ipHeader2VER2F1);
	XOREXT(cpuIntelFlag, ipHeaderSwapFlag2) ? cleanIpHeader2VER1(&ipHeader2VER1F2) : cleanIpHeader2VER2(&ipHeader2VER2F2);
	
/* Logging */
	if (logWrite(&ipHeaderF1, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Get Parameters or Check Arguments */
	if (verboseFlag == 3) {
		fprintf(stdout, " Input File 1 has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ipHeaderF1.rlng*(ipHeaderF1.rnum+1));
	/* Check Method */
		do {
			fprintf(stdout, "Overwrap Image instead of addition ? [Y/N] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		} while (toupper(inputBuffer[0]) != 'Y' && toupper(inputBuffer[0]) != 'N');
		if (toupper(inputBuffer[0]) == 'Y') overwrapFlag++;
	/* File Size Check */
		if ((ipHeaderF1.xpxl != ipHeaderF2.xpxl) || (ipHeaderF1.zpxl != ipHeaderF2.zpxl)) {
			fprintf(stdout, " Input File 2 has ( %ld x %ld ) pixels,\n", ipHeaderF2.xpxl, ipHeaderF2.zpxl);
			fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF2.rlng, ipHeaderF2.rnum, ipHeaderF2.rlng*(ipHeaderF2.rnum+1));
			if (!overwrapFlag) {
				do {
					fprintf(stdout, " File size of two images are different. Do you like to proceed ? [Y/N] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp2);
						fclose(fp1);
#ifdef RAXMODE
						fclose(fp4);
#endif
						exit(EXIT_FAILURE);
					}
				} while (toupper(inputBuffer[0]) != 'Y' && toupper(inputBuffer[0]) != 'N');
				if (toupper(inputBuffer[0]) == 'Y') forceFlag++;
				else {
					fprintf(stderr, " Data size does not match. Exit.\n");
					fclose(fp1);
					fclose(fp2);
					exit(EXIT_FAILURE);
				}
			}
		}
	/* Get coefficients for input files */
		do {
			fprintf(stdout, "Enter Coefficient for 1st File : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		} while (!isdigitEXT(inputBuffer));
		mag1 = atof(inputBuffer);
		if (overwrapFlag) fprintf(stdout, "With overrwap mode, enter plus value at next question.\n");
		else fprintf(stdout, "If you want to subtract File 2 from 1, enter minus value at next question.\n");
		do {
			fprintf(stdout, "Enter Coefficient for 2nd File : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		} while (!isdigitEXT(inputBuffer));
		mag2 = atof(inputBuffer);
		
	/* Get Offset Size */
		if ((ipHeaderF1.drtx >= ipHeaderF1.xpxl) || (ipHeaderF1.drtx < 0.0) ||
		    (ipHeaderF1.drtz >= ipHeaderF1.zpxl) || (ipHeaderF1.drtz < 0.0) ||
		    (ipHeaderF2.drtx >= ipHeaderF2.xpxl) || (ipHeaderF2.drtx < 0.0) ||
		    (ipHeaderF2.drtz >= ipHeaderF2.zpxl) || (ipHeaderF2.drtz < 0.0)) {
			fprintf(stdout, " Centre Position is not correctly recorded.\n");
			autoDoneFlag = 0;
		}
		else if (!overwrapFlag) {
			do {
				fprintf(stdout, "Do you want to set OFFSET Values using Center Positions of Two Files? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) != 'Y') autoDoneFlag = 1;
		}
		if (autoDoneFlag) {
			offsetX = dToLong(ipHeaderF2.drtx - ipHeaderF1.drtx);
			offsetZ = dToLong(ipHeaderF2.drtz - ipHeaderF1.drtz);
			fprintf(stdout, " OFFSET Value (X, Z) is %6ld , %6ld\n", offsetX, offsetZ);
		}
		else {
			fprintf(stdout, " OFFSET Values means Coordinate of File 2 corresponding to File 1(0, 0).\n");
			do {
				fprintf(stdout, "Enter OFFSET for X (pixel) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				offsetX = atol(inputBuffer);
				if ((offsetX > 0 && ipHeaderF2.xpxl - offsetX <= 0)
				 || (offsetX < 0 && -offsetX >= ipHeaderF1.xpxl)) continue;
				offsetXDoneFlag = 1;
			} while (!offsetXDoneFlag);
			do {
				fprintf(stdout, "Enter OFFSET for Z (pixel) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				offsetZ = atol(inputBuffer);
				if ((offsetZ > 0 && ipHeaderF2.zpxl - offsetZ <= 0)
				 || (offsetZ < 0 && -offsetZ >= ipHeaderF1.zpxl)) continue;
				offsetZDoneFlag = 1;
			} while (!offsetZDoneFlag);
		}
	/* Get Constant to Add */
		if (!overwrapFlag) {
			do {
				fprintf(stdout, "Enter Constant Value to add(pad) before processing : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
			} while (!isdigitEXT(inputBuffer));
			padInt = atol(inputBuffer);
		}
	/* check log-conv. */
		if (!overwrapFlag) {
			do {
				fprintf(stdout, "Log Conversion ? [Y/N] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
			} while (toupper(inputBuffer[0]) != 'Y' && toupper(inputBuffer[0]) != 'N');
			if (toupper(inputBuffer[0]) == 'Y') logFlag++;
		}
	/* absolute check */
		if (!overwrapFlag) {
			do {
				fprintf(stdout, "Use absolute value instead of minus ? [Y/N] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
#ifdef RAXMODE
					fclose(fp4);
#endif
					exit(EXIT_FAILURE);
				}
			} while (toupper(inputBuffer[0]) != 'Y' && toupper(inputBuffer[0]) != 'N');
			if (toupper(inputBuffer[0]) == 'Y') absFlag++;
		}
	/* Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File 1 ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File 1 ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
#ifdef RAXMODE
	else {
		i = argc-1;
		if (!isdigitEXT(argv[1]) || !isdigitEXT(argv[3]) || !isdigitEXT(argv[6])) i = 0;
		if (!strcmp(argv[4], "auto") && !strcmp(argv[5], "auto")) autoDoneFlag = 1;
		else if (!isdigitEXT(argv[4]) || !isdigitEXT(argv[5])) i = 0;
		else if (offsetX >= ipHeaderF2.xpxl || -offsetX < ipHeaderF1.xpxl
		      || offsetZ >= ipHeaderF2.zpxl || -offsetZ < ipHeaderF1.zpxl) i = 0;
		if (!i) {
			fprintf(fp4, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp2);
			fclose(fp1);
			fclose(fp4);
			exit(EXIT_FAILURE);
		}
		if (autoDoneFlag &&
		      ((ipHeaderF1.drtx >= ipHeaderF1.xpxl) || (ipHeaderF1.drtx < 0.0) ||
			    (ipHeaderF1.drtz >= ipHeaderF1.zpxl) || (ipHeaderF1.drtz < 0.0) ||
			    (ipHeaderF2.drtx >= ipHeaderF2.xpxl) || (ipHeaderF2.drtx < 0.0) ||
			    (ipHeaderF2.drtz >= ipHeaderF2.zpxl) || (ipHeaderF2.drtz < 0.0))) {
			fprintf(fp4, " Centre Position is not correctly recorded.\n");
			fclose(fp1);
			fclose(fp2);
			fclose(fp4);
			exit(EXIT_FAILURE);
		}
		if (autoDoneFlag) {
			offsetX = dToLong(ipHeaderF2.drtx - ipHeaderF1.drtx);
			offsetZ = dToLong(ipHeaderF2.drtz - ipHeaderF1.drtz);
		}
		else {
			offsetX = atol(argv[4]);
			offsetZ = atol(argv[5]);
		}
		mag1 = atof(argv[1]);
		mag2 = atof(argv[3]);
		padInt = atol(argv[6]);
	}
#else
	else {
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-f1:", 4) || !strncmp(argv[i], "-f2:", 4)) continue;
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-a")) autoDoneFlag++;
				else if (!strcmp(arg0, "-l")) logFlag++;
				else if (!strcmp(arg0, "-ab")) absFlag++;
				else if (!strcmp(arg0, "-f")) forceFlag++;
				else if (!strcmp(arg0, "-j")) overwrapFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-m1")) {
					mag1 = arg1;
					mag1DoneFlag++;
				}
				else if (!strcmp(arg0, "-m2")) {
					mag2 = arg1;
					mag2DoneFlag++;
				}
				else if (!strcmp(arg0, "-x")) {
					offsetX = dToLong(arg1);
					if ((offsetX > 0 && ipHeaderF2.xpxl - offsetX <= 0)
					 || (offsetX < 0 && -offsetX >= ipHeaderF1.xpxl)) break;
					offsetXDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					offsetZ = dToLong(arg1);
					if ((offsetZ > 0 && ipHeaderF2.zpxl - offsetZ <= 0)
					 || (offsetZ < 0 && -offsetZ >= ipHeaderF1.zpxl)) break;
					offsetZDoneFlag++;
				}
				else if (!strcmp(arg0, "-p")) {
					padInt = dToLong(arg1);
					padDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(fp4, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((autoDoneFlag > 1) || (mag1DoneFlag > 1) || (mag2DoneFlag > 1)
		   || (offsetXDoneFlag > 1) || (offsetZDoneFlag > 1)|| (padDoneFlag > 1)
		   || (setTimeFlag > 1) || (absFlag > 1) || (logFlag > 1)
		   || (forceFlag > 1) || (overwrapFlag > 1)
		   || ((offsetXDoneFlag || offsetZDoneFlag) && autoDoneFlag)) {
			fprintf(fp4, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (overwrapFlag && (padDoneFlag || logFlag || absFlag || forceFlag)) {
			logFlag = absFlag = padDoneFlag = 0;
			padInt = 0;
			fprintf(stderr, " Conflicting options with overwrap mode are ignored.\n");
		}
		if (overwrapFlag) forceFlag = 1;
		if (verboseFlag >= 2) {
			fprintf(stdout, " Input File 1 has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
			fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ipHeaderF1.rlng*(ipHeaderF1.rnum+1));
		}
		if (ipHeaderF1.xpxl != ipHeaderF2.xpxl || ipHeaderF1.zpxl != ipHeaderF2.zpxl) {
			if (forceFlag) {
				fprintf(stdout, " Input File 2 has ( %ld x %ld ) pixels,\n", ipHeaderF2.xpxl, ipHeaderF2.zpxl);
				fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF2.rlng, ipHeaderF2.rnum, ipHeaderF2.rlng*(ipHeaderF2.rnum+1));
				if (!overwrapFlag) fprintf(stderr, " Warning : Data size does not match.\n");
			}
			else {
				fprintf(fp4, " Data size does not match. Exit.\n");
				fclose(fp1);
				fclose(fp2);
				exit(EXIT_FAILURE);
			}
		}
		if (autoDoneFlag) {
			if ((ipHeaderF1.drtx >= ipHeaderF1.xpxl) || (ipHeaderF1.drtx < 0.0) ||
			    (ipHeaderF1.drtz >= ipHeaderF1.zpxl) || (ipHeaderF1.drtz < 0.0) ||
			    (ipHeaderF2.drtx >= ipHeaderF2.xpxl) || (ipHeaderF2.drtx < 0.0) ||
			    (ipHeaderF2.drtz >= ipHeaderF2.zpxl) || (ipHeaderF2.drtz < 0.0)) {
				fprintf(fp4, " Centre Position is not correctly recorded.\n");
				fclose(fp1);
				fclose(fp2);
				exit(EXIT_FAILURE);
			}
			offsetX = dToLong(ipHeaderF2.drtx - ipHeaderF1.drtx);
			offsetZ = dToLong(ipHeaderF2.drtz - ipHeaderF1.drtz);
		}
	}
#endif
	
/* Set Parameters to Create */
	flushWidth = ipHeaderF1.xpxl * RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeaderF1.drtx >= ipHeaderF1.xpxl) ipHeaderF1.drtx = ipHeaderF1.xpxl - 1.0;
	else if (ipHeaderF1.drtx < 0.0) ipHeaderF1.drtx = 0.0;
	if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
	else if (ipHeaderF1.drtz < 0.0) ipHeaderF1.drtz = 0.0;
	ipHeaderF1.rlng = recordLengthCreate;
	if (absFlag && logFlag) myFunc = logabsFunc;
	else if (absFlag) myFunc = absFunc;
	else if (logFlag) myFunc = logFunc;
	else myFunc = nullFunc;
	startZ = MAXEXT(-offsetZ, 0);
	endZ = MINEXT(ipHeaderF1.rnum, ipHeaderF2.rnum-offsetZ);
	startX = MAXEXT(-offsetX, 0);
	endX = MINEXT(ipHeaderF1.xpxl, ipHeaderF2.xpxl-offsetX);
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file3, &fp3)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp2);
			fclose(fp1);
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file3, &fp3)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp2);
				fclose(fp1);
#ifdef RAXMODE
				fclose(fp4);
#endif
				exit(EXIT_FAILURE);
			}
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " New File of %s = \n", file3);
		fprintf(stdout, "   %s(     x,     z) x (%6.3f)\n", file1, mag1);
		fprintf(stdout, " + %s(x%+5ld, z%+5ld) x (%6.3f) %+7ld .\n", file2, offsetX, offsetZ, mag2, padInt);
		if (overwrapFlag) fprintf(stdout, " Original Image is overwrapped by 2nd Image.\n");
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File 1 will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File 1.\n");
#endif
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1F1) : swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(fp4, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(fp4, " Output File is not created or removed.\n");
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
	createDosFlag ? fwrite(&ipHeader2VER1F1, ipHeaderSize2VER1, 1, fp3) : fwrite(&ipHeader2VER2F1, ipHeaderSize2VER2, 1, fp3);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp3);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(fp4, " Input File 1 Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(fp4, " Output File is not created or removed.\n");
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	readByte1 = ftell(fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1F1) : swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp2, ipHeaderF2.rlng*(MAXEXT(offsetZ, 0)+1), SEEK_SET)) {
		fprintf(fp4, " Input File 2 Read Error.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(fp4, " Output File is not created or removed.\n");
#ifdef RAXMODE
		fclose(fp4);
#endif
		exit(EXIT_FAILURE);
	}
	readByte2 = ftell(fp2);
	for (i=0; i<startZ; i++) {
		if ((datPtr1 = malloc(ipHeaderF1.rlng)) == NULL) {
			fprintf(fp4, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, ipHeaderF1.rlng, 1, fp1);
		readByte1 += ipHeaderF1.rlng;
		if (ftell(fp1) != readByte1) {
			fprintf(fp4, " Input File 1 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeaderF1.xpxl; j++) {
			intensityDbl = (*myFunc)((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) * mag1  + padInt);
			datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
		}
		fwrite(datPtr1, flushWidth, 1, fp3);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp3);
		free(datPtr1);
	}
	for (i=startZ; i<endZ; i++) {
		if ((datPtr1 = malloc(ipHeaderF1.rlng)) == NULL) {
			fprintf(fp4, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		if ((datPtr2 = malloc(ipHeaderF2.rlng)) == NULL) {
			fprintf(fp4, " Failed to allocate memory.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, ipHeaderF1.rlng, 1, fp1);
		readByte1 += ipHeaderF1.rlng;
		if (ftell(fp1) != readByte1) {
			fprintf(fp4, " Input File 1 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			free(datPtr2);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		fread(datPtr2, ipHeaderF2.rlng, 1, fp2);
		readByte2 += ipHeaderF2.rlng;
		if (ftell(fp2) != readByte2) {
			fprintf(fp4, " Input File 2 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			free(datPtr2);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		for (j=0; j<startX; j++) {
			intensityDbl = (*myFunc)((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) * mag1 + padInt);
			datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
		}
		if (overwrapFlag) {
			for (j=startX; j<endX; j++) {
				intensityDbl = (*myFunc)((*readDat2)(&datPtr2[j+offsetX], &rato2, &shiftBit2, &ipDataSwapFlag2) * mag2);
				datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
			}
		}
		else {
			for (j=startX; j<endX; j++) {
				intensityDbl = (*myFunc)((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) * mag1
				              + (*readDat2)(&datPtr2[j+offsetX], &rato2, &shiftBit2, &ipDataSwapFlag2) * mag2 + padInt);
				datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
			}
		}
		for (j=endX; j<ipHeaderF1.xpxl; j++) {
			intensityDbl = (*myFunc)((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) * mag1 + padInt);
			datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
		}
		fwrite(datPtr1, flushWidth, 1, fp3);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp3);
		free(datPtr1);
		free(datPtr2);
	}
	for (i=endZ; i<ipHeaderF1.rnum; i++) {
		if ((datPtr1 = malloc(ipHeaderF1.rlng)) == NULL) {
			fprintf(fp4, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, ipHeaderF1.rlng, 1, fp1);
		readByte1 += ipHeaderF1.rlng;
		if (ftell(fp1) != readByte1) {
			fprintf(fp4, " Input File 1 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(fp4, " Output File is not created or removed.\n");
			}
#ifdef RAXMODE
			fclose(fp4);
#endif
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeaderF1.xpxl; j++) {
			intensityDbl = (*myFunc)((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) * mag1  + padInt);
			datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
		}
		fwrite(datPtr1, flushWidth, 1, fp3);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp3);
		free(datPtr1);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	fclose(fp2);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
	}
	fclose(fp3);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file3, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file3);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
		}
		else if (rename(file3, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File 1 has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
#ifdef RAXMODE
	fprintf(fp4, "     ... Finished Successfully.\n");
	fclose(fp4);
#endif
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] [-t] -f1:file1 -f2:file2 [-m1:VALUE] [-m2:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] -f1:file1 -f2:file2 [-m1:VALUE] [-m2:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [{-x:VALUE -z:VALUE|-a}] [-p:VALUE] [-ab] [-l] [-f] {file3|-replace}\n");
	/*                 " %s -rax file1 mag1 file2 mag2 offsetX offsetZ padInt file3 errfile "
	                   is also valid for raxwish version patch. */
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] [-t] -f1:file1 -f2:file2 [-m1:VALUE] [-m2:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] -f1:file1 -f2:file2 -j [-m1:VALUE] [-m2:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-x:VALUE] [-z:VALUE] {file3|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Add Two R-AXIS Image after shift 2nd file. That is:\n");
	fprintf(fp0, "            file1(x, z)       * coefficient1\n");
	fprintf(fp0, "          + file2(x+dx, z+dz) * coefficient2  + padding\n");
	fprintf(fp0, "          = file3(x, z)\n");
	fprintf(fp0, "  -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "  -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "  -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "  -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "  -f1:file1 ... Set Filename of Original R-AXIS Image File 1 to file1.\n");
	fprintf(fp0, "  -f2:file2 ... Set Filename of Original R-AXIS Image File 2 to file2.\n");
	fprintf(fp0, "                If none of above two options is present, 1st and 2nd\n");
	fprintf(fp0, "                arguments are used as 1st and 2nd filenames respectively.\n");
	fprintf(fp0, "  -m1:VALUE ... Set Coefficient to the 1st file to \"VALUE\".\n");
	fprintf(fp0, "                If absent, set to 1.0.\n");
	fprintf(fp0, "  -m2:VALUE ... Set Coefficient to the 2st file to \"VALUE\".\n");
	fprintf(fp0, "                If absent, set to 1.0.\n");
	fprintf(fp0, "  -x:VALUE .... Set X-coordinate of 2nd file corresponding to the origin\n");
	fprintf(fp0, "                of 1st file to \"VALUE\". If absent, set to 0.\n");
	fprintf(fp0, "  -z:VALUE .... Set Z-coordinate of 2nd file corresponding to the origin\n");
	fprintf(fp0, "                of 1st file to \"VALUE\". If absent, set to 0.\n");
	fprintf(fp0, "  -a .......... Set X- and Z-coordinate of 2nd file automatically, using\n");
	fprintf(fp0, "                direct-beam position data stored in 1st and 2nd file.\n");
	fprintf(fp0, "  -p:VALUE .... Set the padding intensity to \"VALUE\" for the 1st file\n");
	fprintf(fp0, "                before calculation. If absent, set to 0.\n");
	fprintf(fp0, "  -l .......... Log-conversion after addition.\n");
	fprintf(fp0, "  -ab ......... Use absolute value instead of minus.\n");
	fprintf(fp0, "  -f .......... Work between images with different size.\n");
	fprintf(fp0, "  -j .......... Just put the 2nd image on the 1st image, without adding.\n");
	fprintf(fp0, "                With this option, \"-p\", \"-l\" and \"-ab\" options are ignored, ");
	fprintf(fp0, "                and \"-f\" options is not necessary.\n");
#ifdef _HASUTIME
	fprintf(fp0, "  -t .......... Set Modification time of Output File as Original File 1.\n");
#endif
	fprintf(fp0, "  file3 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                If last argument is \"-replace\", Original R-AXIS Image File 1 is\n");
	fprintf(fp0, "                replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0); /* Not argument */
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (10); /* Without ':', and first string is too long */
		strcpy(arg0, argV);
		return (1); /* Without ':', and first string is good */
	}
	if (chPtr1-argV > 3) return (10); /* With ':', and first string is too long */
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (20); /* With ':', and 2nd part is not numeric. */
	*arg1 = atof(chPtr1+1);
	return (2);
}

#ifdef _OLDHP
double logFunc(val)
	double val;
#else
double logFunc(double val)
#endif
{
	return (log(MAXEXT(val, 0)+1));
}

#ifdef _OLDHP
double absFunc(val)
	double val;
#else
double absFunc(double val)
#endif
{
	return (fabs(val));
}

#ifdef _OLDHP
double logabsFunc(val)
	double val;
#else
double logabsFunc(double val)
#endif
{
	return (log(fabs(val)+1));
}

#ifdef _OLDHP
double nullFunc(val)
	double val;
#else
double nullFunc(double val)
#endif
{
	return (val);
}
