/* Program to convert IP data file */
#define ProgName "iptrim"
#define ProgInfo "Resize (or trim) Image"
/*                                                        12/14/1999  */
/*         for command-line argument                      12/14/1999  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.29_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#ifdef _HASUTIME
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 7
#else
#	define MAXARG 6
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1;
	long int i, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     ipHeaderSizeMin, startX, startZ, endX, endZ, newWidth, newHeight,
	     recordLengthCreate, startXmax, startZmax, endXmin, endZmin,
	     arg1, readByte, oldWidth, oldHeight, flushWidthA, flushWidthB, flushWidthC;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, ipHeaderSwapFlag,
	     createDosFlag, confirmBuffer[8], inputBuffer[20], argNum, verboseFlag,
	     startXDoneFlag, startZDoneFlag, newWidthFlag, newHeightFlag,
	     arg0[4], replaceFlag, setTimeFlag, endXDoneFlag, endZDoneFlag, *strPtr,
	     cxFlag, czFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	startXDoneFlag = startZDoneFlag = newWidthFlag = newHeightFlag = replaceFlag =
	setTimeFlag = endXDoneFlag = endZDoneFlag = cxFlag = czFlag = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Get ipHeader Information */
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	oldWidth = ipHeader.xpxl;
	oldHeight = ipHeader.zpxl;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Get Offset Size */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels, \n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	startX = startZ = 0;
	endX = newWidth = ipHeader.xpxl;
	endZ = newHeight = ipHeader.zpxl;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter the Smallest Coordinate of X in pixel. Type \"D\" if keep unchanged : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') break;
			if (!isdigitEXT(inputBuffer)) continue;
			startX = atol(inputBuffer);
			if (startX >= ipHeader.xpxl) continue;
			startXDoneFlag = 1;
		} while (!startXDoneFlag);
		do {
			fprintf(stdout, "Enter the Smallest Coordinate of Z in pixel. Type \"D\" if keep unchanged : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') break;
			if (!isdigitEXT(inputBuffer)) continue;
			startZ = atol(inputBuffer);
			if (startZ >= ipHeader.zpxl) continue;
			startZDoneFlag = 1;
		} while (!startZDoneFlag);
		do {
			fprintf(stdout, "Enter the Largest Coordinate of X in pixel.\n");
			fprintf(stdout, " Type \"D\" to use the largest coordinate of the Original Image.\n");
			fprintf(stdout, " Type \"W\" following to the value is interpreted as Width : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') break;
			if (!isdigitEXT(inputBuffer)) continue;
			arg1 = strtol(inputBuffer, &strPtr, 0);
			if (strPtr == inputBuffer) continue;
			if (strPtr == 0) {
				endX = arg1;
				newWidth = endX - startX;
			}
			else if (!strcmp(strPtr, "W") || !strcmp(strPtr, "w")) {
				newWidth = arg1;
				endX = startX + newWidth;
			}
			else continue;
			if ((endX <= 0) || (newWidth > MAXWIDTH) || (newWidth <= 0)) continue;
			endXDoneFlag = 1;
		} while (!endXDoneFlag);
		if (!endXDoneFlag) newWidth = ipHeader.xpxl - startX;
		do {
			fprintf(stdout, "Enter the Largest Coordinate of Z in pixel.\n");
			fprintf(stdout, " Type \"D\" to use the largest coordinate of the Original Image.\n");
			fprintf(stdout, " Type \"H\" following to the value is interpreted as Height : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') break;
			if (!isdigitEXT(inputBuffer)) continue;
			arg1 = strtol(inputBuffer, &strPtr, 0);
			if (strPtr == inputBuffer) continue;
			if (strPtr == 0) {
				endZ = arg1;
				newHeight = endZ - startZ;
			}
			else if (!strcmp(strPtr, "H") || !strcmp(strPtr, "h")) {
				newHeight = arg1;
				endZ = startZ + newHeight;
			}
			else continue;
			if ((endZ <= 0) || (newHeight > MAXHEIGHT) || (newHeight <= 0)) continue;
			endZDoneFlag = 1;
		} while (!endZDoneFlag);
		if (!endZDoneFlag) newHeight = ipHeader.zpxl - startZ;
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-cx")) {
					startXDoneFlag++;
					cxFlag++;
				}
				else if (!strcmp(arg0, "-cz")) {
					startZDoneFlag++;
					czFlag++;
				}
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-x") || !strcmp(arg0, "-xs")) {
					startX = arg1;
					if (startX >= ipHeader.xpxl) break;
					startXDoneFlag++;
				}
				else if (!strcmp(arg0, "-z") || !strcmp(arg0, "-zs")) {
					startZ = arg1;
					if (startZ >= ipHeader.zpxl) break;
					startZDoneFlag++;
				}
				else if (!strcmp(arg0, "-X")) {
					newWidth = arg1;
					endXDoneFlag++;
					newWidthFlag++;
				}
				else if (!strcmp(arg0, "-Z")) {
					newHeight = arg1;
					endZDoneFlag++;
					newHeightFlag++;
				}
				else if (!strcmp(arg0, "-xf")) {
					endX = arg1;
					endXDoneFlag++;
				}
				else if (!strcmp(arg0, "-zf")) {
					endZ = arg1;
					endZDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!startXDoneFlag && !startZDoneFlag && !endXDoneFlag && !endZDoneFlag) {
			fprintf(stderr, " No Effective Argument.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((cxFlag && !newWidthFlag) || (czFlag && !newHeightFlag)) {
			fprintf(stderr, " Lack of Argument.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((startXDoneFlag > 1) || (startZDoneFlag > 1) || (endXDoneFlag > 1)
		 || (endZDoneFlag > 1) || (setTimeFlag > 1)
		 || (cxFlag > 1) || (czFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (cxFlag) {
			startX = dToLong(ipHeader.drtx - newWidth/2.0);
			endX = startX + newWidth;
			if (endX < 0 || startX > ipHeader.xpxl) {
				fprintf(stderr, "Irregal centre of image. Try after setting it correctly.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (verboseFlag >= 2) fprintf(stdout, " Center of Width is decided automatically.\n");
			newWidthFlag = 0;
		}
		if (czFlag) {
			startZ = dToLong(ipHeader.drtz - newHeight/2.0);
			endZ = startZ + newHeight;
			if (endZ < 0 || startZ > ipHeader.zpxl) {
				fprintf(stderr, "Irregal centre of image. Try after setting it correctly.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (verboseFlag >= 2) fprintf(stdout, " Center of Height is decided automatically.\n");
			newHeightFlag = 0;
		}
		if (!startXDoneFlag) {
			startX = 0;
			if (verboseFlag >= 2) fprintf(stdout, " Start X-coodinate omitted. Use 0 instead.\n");
		}
		if (!startZDoneFlag) {
			startZ = 0;
			if (verboseFlag >= 2) fprintf(stdout, " Start Z-coodinate omitted. Use 0 instead.\n");
		}
		if (!endXDoneFlag) {
			endX = ipHeader.xpxl;
			newWidth = endX - startX;
			if (verboseFlag >= 2) fprintf(stdout, " New Width omitted. Use %ld instead.\n", newWidth);
		}
		else if (newWidthFlag) endX = startX + newWidth;
		else newWidth = endX - startX;
		if (!endZDoneFlag) {
			endZ = ipHeader.zpxl;
			newHeight = endZ - startZ;
			if (verboseFlag >= 2) fprintf(stdout, " New Height omitted. Use %ld instead.\n", newHeight);
		}
		else if (newHeightFlag) endZ = startZ + newHeight;
		else newHeight = endZ - startZ;
		if (newWidth <= 0 || newHeight <= 0) {
			fprintf(stderr, "Invalid Arguments. Argument out of range.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (newWidth > MAXWIDTH || newHeight > MAXHEIGHT) {
			fprintf(stderr, "New Image is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (endX <= 0 || endZ <= 0) {
			fprintf(stderr, "New Image contains no original data. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " New Image is (%6ld, %6ld ) - (%6ld, %6ld )\n", startX, startZ, endX, endZ);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	ipHeader.xpxl = newWidth;
	ipHeader.zpxl = newHeight;
	recordLengthCreate = MAXEXT(ipHeader.xpxl*RAXDATAWIDTH, ipHeaderSizeMin);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.drtx = ipHeader.drtx - startX;
	ipHeader.drtz = ipHeader.drtz - startZ;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	startXmax = MAXEXT(startX, 0);
	startZmax = MAXEXT(startZ, 0);
	endXmin = MINEXT(endX, oldWidth);
	endZmin = MINEXT(endZ, oldHeight);
	ipHeader.rnum = ipHeader.zpxl;
	ipHeader.rlng = recordLengthCreate;
	flushWidthA = -startX*RAXDATAWIDTH;
	flushWidthB = (endXmin-startXmax)*RAXDATAWIDTH;
	flushWidthC = recordLengthCreate-(endXmin-startX)*RAXDATAWIDTH;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	for (i=startZ; i<0; i++) fwrite(datPtr0, recordLengthCreate, 1, fp2);
	if (startZ > 0) {
		if (fseek(fp1, recordLength*startZ, SEEK_CUR)) {
			fprintf(stderr, " Input File Read Error.\n");
			free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		readByte = ftell(fp1);
	}
	for (i=startZmax; i<endZmin; i++) {
		if ((datPtr1 = malloc(recordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr0);
			fclose(fp1);
			if (i != startZmax) {
				ipHeader.zpxl = ipHeader.rnum = i - startZ;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels, \n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, recordLength, 1, fp1);
		readByte += recordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr1);
			free(datPtr0);
			fclose(fp1);
			if (i != startZmax) {
				ipHeader.zpxl = ipHeader.rnum = i - startZ;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels, \n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		if (startX < 0) fwrite(datPtr0, flushWidthA, 1, fp2);
		fwrite(datPtr1+startXmax, flushWidthB, 1, fp2);
		fwrite(datPtr0, flushWidthC, 1, fp2);
		free(datPtr1);
	}
	for (i=oldHeight; i<endZ; i++) fwrite(datPtr0, recordLengthCreate, 1, fp2);
	
/* End */
	free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels, \n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", file2);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-x:VALUE] [-z:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-x:VALUE] [-z:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-X:VALUE|-xf:VALUE] [-Z:VALUE|-zf:VALUE] {file2|-replace}\n");
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-cx] [-cz] [-X:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-cx] [-cz] [-X:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-Z:VALUE] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Resize (or \"trim\") Image for R-AXIS Image File.\n");
	fprintf(fp0, "  New Image must contain at least one pixel of Original Image.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -x:VALUE .... Set Smallest Coodinate of New Image in X-direction\n");
	fprintf(fp0, "                 to \"VALUE\". VALUE must be integer. If absent, use \"0\".\n");
	fprintf(fp0, "   -z:VALUE .... Set Smallest Coodinate of New Image in Z-direction\n");
	fprintf(fp0, "                 to \"VALUE\". VALUE must be integer. If absent, use \"0\".\n");
	fprintf(fp0, "   -xs:VALUE, -zs:VALUE is the same as above.\n");
	fprintf(fp0, "   -X:VALUE .... Set Width of New Image to \"VALUE\". VALUE must be integer.\n");
	fprintf(fp0, "   -xf:VALUE ... Set Largest X-Coordinate of Original Image to use for New Image\n");
	fprintf(fp0, "                 to \"VALUE\". VALUE must be integer.\n");
	fprintf(fp0, "                 If absent, use the right edge of original Image.\n");
	fprintf(fp0, "                 Only one of above 2 options is valid.\n");
	fprintf(fp0, "   -Z:VALUE .... Set Height of New Image to \"VALUE\". VALUE must be integer.\n");
	fprintf(fp0, "   -zf:VALUE ... Set Largest Z-Coordinate of Original Image to use for New Image\n");
	fprintf(fp0, "                 to \"VALUE\". VALUE must be integer.\n");
	fprintf(fp0, "                 If absent, use the upper edge of original Image.\n");
	fprintf(fp0, "                 Only one of above 2 options is valid.\n");
	fprintf(fp0, "   -cx ......... Instead of pointing starting coordinate in X, use direct-beam\n");
	fprintf(fp0, "                 position in X as a centre of new image in X.\n");
	fprintf(fp0, "   -cz ......... Instead of pointing starting coordinate in Z, use direct-beam\n");
	fprintf(fp0, "                 position in Z as a centre of new image in Z.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
