/* Libraries to treat PNG file                             6/22/2000  */
/*         for ANSI C                                                 */
/*                              v1.04c                     3/31/2005  */
/*                                               (C) Tsurutani Naoki  */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#ifdef _HASZLIB
#	include <zlib.h>
#endif
#include "libm2.h"
#include "libpng.h"

#ifdef _OLDHP
char pngSignatureGenerate(charBuf)
	unsigned char *charBuf;
#else
char pngSignatureGenerate(unsigned char *charBuf)
#endif
{
	charBuf[0] = 0x89;
	charBuf[1] = 0x50;
	charBuf[2] = 0x4e;
	charBuf[3] = 0x47;
	charBuf[4] = 0x0d;
	charBuf[5] = 0x0a;
	charBuf[6] = 0x1a;
	charBuf[7] = 0x0a;
	return (0);
}

#ifdef _OLDHP
char pngSignatureCheck(charBuf)
	unsigned char *charBuf;
#else
char pngSignatureCheck(unsigned char *charBuf)
#endif
{
	unsigned char refBuf[8];
	
	pngSignatureGenerate(refBuf);
	if (strncmp(charBuf, refBuf, 8)) return (1);
	return (0);
}

#ifdef _OLDHP
char pngReadChunk1(type, len, fp, flag)
	char *type, flag;
	long int *len;
	FILE *fp;
#else
char pngReadChunk1(char *type, long int *len, FILE *fp, char flag)
#endif
{
	/* Check chunk Length and Type */
	unsigned long int pos;
	
	pos = ftell(fp);
	fread(len, 4, 1, fp);
	if (ftell(fp) != (pos + 4)) return (11);
	if (flag) *len = swapLong(*len);
	if (*len < 0) return (10);
	fread(type, 4, 1, fp);
	if (ftell(fp) != (pos + 8)) return (11);
	return (0);
}

#ifdef _OLDHP
char pngMakeScanline(input, output, bpp, num, flag)
	unsigned char *output, bpp;
	unsigned short int *input;
	long int num;
	char flag;
#else
char pngMakeScanline(unsigned short int *input, unsigned char *output, unsigned char bpp, long int num, char flag)
#endif
{
	long int len, i;
	
	if (num <= 0) return (1);
	switch (bpp) {
		case 1:
			len = (long)ceil(num / 8.0);
			memset(output, 0, len);
			for (i=0; i<num; i++) output[i>>3] += input[i] << (7 - i % 8);
			break;
		case 2:
			len = (long)ceil(num / 4.0);
			memset(output, 0, len);
			for (i=0; i<num; i++) output[i>>2] += input[i] << (6 - ((i % 4)<<1));
			break;
		case 4:
			len = (num+1) >> 1;
			memset(output, 0, len);
			for (i=0; i<num; i++) output[i>>1] += input[i] << (4 - ((i % 2)<<2));
			break;
		case 8:
			for (i=0; i<num; i++) output[i] = input[i];
			break;
		case 16:
			if (flag) for (i=0; i<num; i++) ((short*)output)[i] = swapShort(input[i]);
			else memcpy(output, input, num * 2);
			break;
		default:
			return (1);
	}
	return (0);
}

#ifdef _OLDHP
char pngResolveScanline(input, output, bpp, num, flag)
	unsigned char *input, bpp;
	unsigned short int *output;
	long int num;
	char flag;
#else
char pngResolveScanline(unsigned char *input, unsigned short int *output, unsigned char bpp, long int num, char flag)
#endif
{
	long int i;
	
	if (num <= 0) return (1);
	switch (bpp) {
		case 1:
			for (i=0; i<num; i++) output[i] = (input[i>>3] << (i % 8)) >> (7 - i % 8);
			break;
		case 2:
			for (i=0; i<num; i++) output[i] = (input[i>>2] << ((i % 4)<<1)) >> (6 - ((i % 4)<<1));
			break;
		case 4:
			for (i=0; i<num; i++) output[i] = (input[i>>1] << ((i % 2)<<2)) >> (4 - ((i % 2)<<2));
			break;
		case 8:
			for (i=0; i<num; i++) output[i] = input[i];
			break;
		case 16:
			if (flag) for (i=0; i<num; i++) output[i] = swapShort(((short*)input)[i]);
			else memcpy(output, input, num * 2);
			break;
		default:
			return (1);
	}
	return (0);
}

#ifdef _OLDHP
char pngFilter(input, inputP, output, unit, type, length)
	unsigned char *input, *inputP, *output, unit, type;
	long int length;
#else
char pngFilter(unsigned char *input, unsigned char *inputP, unsigned char *output, unsigned char unit, char type, long int length)
#endif
{
	long int i;
	short int tmp, pa, pb, pc;
	
	output[0] = type;
	switch (type) {
		case 0:
			memcpy(output+1, input, length);
			break;
		case 1:
			for (i=0; i<unit; i++) output[i+1] = input[i];
			for (i=unit; i<length; i++) output[i+1] = input[i] - input[i-unit];
			break;
		case 2:
			for (i=0; i<length; i++) output[i+1] = input[i] - inputP[i];
			break;
		case 3:
			for (i=0; i<unit; i++) output[i+1] = input[i] - (inputP[i] >> 1);
			for (i=unit; i<length; i++) {
				tmp = ((short)(input[i-unit] + inputP[i])) >> 1;
				output[i+1] = input[i] - (unsigned char)tmp;
			}
			break;
		case 4:
			for (i=0; i<unit; i++) output[i+1] = inputP[i] ? input[i] - inputP[i] : input[i];
			for (i=unit; i<length; i++) {
				tmp = (short)input[i-unit] + inputP[i] - inputP[i-unit];
				pa = abs(tmp - input[i-unit]);
				pb = abs(tmp - inputP[i]);
				pc = abs(tmp - inputP[i-unit]);
				if (pa <= pb && pa <= pc) output[i+1] = input[i] - input[i-unit];
				else if (pb <= pc) output[i+1] = input[i] - inputP[i];
				else output[i+1] = input[i] - inputP[i-unit];
			}
			break;
		default:
			return (1);
	}
	return (0);
}

#ifdef _OLDHP
char pngDefilter(input, output, outputP, unit, length)
	unsigned char *input, *output, *outputP, unit;
	long int length;
#else
char pngDefilter(unsigned char *input, unsigned char *output, unsigned char *outputP, unsigned char unit, long int length)
#endif
{
	long int i;
	short int tmp, pa, pb, pc;
	
	switch (input[0]) {
		case 0:
			memcpy(output, input+1, length);
			break;
		case 1:
			for (i=0; i<unit; i++) output[i] = input[i+1];
			for (i=unit; i<length; i++) output[i] = input[i+1] + output[i-unit];
			break;
		case 2:
			for (i=0; i<length; i++) output[i] = input[i+1] + outputP[i];
			break;
		case 3:
			for (i=0; i<unit; i++) output[i] = input[i+1] + (outputP[i] >> 1);
			for (i=unit; i<length; i++) {
				tmp = ((short)(output[i-unit] + outputP[i])) >> 1;
				output[i] = input[i+1] + (unsigned char)tmp;
			}
			break;
		case 4:
			for (i=0; i<unit; i++) output[i] = outputP[i] ? input[i+1] + outputP[i] : input[i+1];
			for (i=unit; i<length; i++) {
				tmp = (short)output[i-unit] + outputP[i] - outputP[i-unit];
				pa = abs(tmp - output[i-unit]);
				pb = abs(tmp - outputP[i]);
				pc = abs(tmp - outputP[i-unit]);
				if (pa <= pb && pa <= pc) output[i] = input[i+1] + output[i-unit];
				else if (pb <= pc) output[i] = input[i+1] + outputP[i];
				else output[i] = input[i+1] + outputP[i-unit];
			}
			break;
		default:
			return (1);
	}
	return (0);
}

