/* Libraries to treat xray intensities                    11/19/2001  */
/*         for ANSI C                                                 */
/*                              v1.04b                     3/29/2005  */
/*                                               (C) Tsurutani Naoki  */

/* All data are taken from International Tables for Crystallography
                                                         19 Nov. 2001 */
/* Anomalous Scattering data are taken from the next:
     Any reports or publications of these data will acknowledge its use
     by an appropriate citation.  This would refer to the following publication:
       S.Sasaki (1989) Numerical Tables of Anomalous Scattering Factors
       Calculated by the Cromer and Liberman's Method,
       KEK Report, 88-14, 1-136
   Fitting functions are not taken from the above reports.
                                                         16 Dec. 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include "libm2.h"
#include "libxray.h"

static struct coefficient coef[ATOMTABLESIZE];
static int copyright;
static double xray_qlim_normal1;
static double xray_qlim_compton1;

#ifdef _OLDHP
char initXrayAtom()
#else
char initXrayAtom(void)
#endif
{
	copyright = 0;
	/* definition of parameters */
	strcpy(coef[0].atom, "H");
	coef[0].val[0] =   0.489918;
	coef[0].val[1] =  20.6593;
	coef[0].val[2] =   0.262003;
	coef[0].val[3] =   7.74039;
	coef[0].val[4] =   0.196767;
	coef[0].val[5] =  49.5519;
	coef[0].val[6] =   0.049879;
	coef[0].val[7] =   2.20159;
	coef[0].val[8] =   0.001305;
	coef[0].eleNum = 1;
	coef[0].cvl[0] =   0.64141;
	coef[0].cvl[1] =  14.3399;
	coef[0].cvl[2] =   0.28219;
	coef[0].cvl[3] =  45.4078;
	coef[0].cvl[4] =   0.07973;
	coef[0].cvl[5] =   0.99161;
	coef[0].cvl[6] =  -0.00332;
	strcpy(coef[1].atom, "C");
	coef[1].val[0] =   2.31000;
	coef[1].val[1] =  20.8439;
	coef[1].val[2] =   1.02000;
	coef[1].val[3] =  10.2075;
	coef[1].val[4] =   1.58860;
	coef[1].val[5] =   0.568700;
	coef[1].val[6] =   0.865000;
	coef[1].val[7] =  51.6512;
	coef[1].val[8] =   0.215600;
	coef[1].eleNum = 6;
	coef[1].cvl[0] =   0.4831;
	coef[1].cvl[1] =  14.121;
	coef[1].cvl[2] =   0.57026;
	coef[1].cvl[3] =  40.4826;
	coef[1].cvl[4] =   1.23428;
	coef[1].cvl[5] =   0.64475;
	coef[1].cvl[6] =   0.16138;
	strcpy(coef[2].atom, "N");
	coef[2].val[0] =  12.2126;
	coef[2].val[1] =   0.005700;
	coef[2].val[2] =   3.13220;
	coef[2].val[3] =   9.89330;
	coef[2].val[4] =   2.01250;
	coef[2].val[5] =  28.9975;
	coef[2].val[6] =   1.16630;
	coef[2].val[7] =   0.582600;
	coef[2].val[8] = -11.529;
	coef[2].eleNum = 7;
	coef[2].cvl[0] =   0.78731;
	coef[2].cvl[1] =  10.1943;
	coef[2].cvl[2] =   0.43169;
	coef[2].cvl[3] =  34.975;
	coef[2].cvl[4] =   1.24402;
	coef[2].cvl[5] =   0.48727;
	coef[2].cvl[6] =   0.18209;
	strcpy(coef[3].atom, "S");
	coef[3].val[0] =   6.90530;
	coef[3].val[1] =   1.46790;
	coef[3].val[2] =   5.20340;
	coef[3].val[3] =  22.2151;
	coef[3].val[4] =   1.43790;
	coef[3].val[5] =   0.253600;
	coef[3].val[6] =   1.58630;
	coef[3].val[7] =  56.1720;
	coef[3].val[8] =   0.866900;
	coef[3].eleNum = 16;
	coef[3].cvl[0] =   1.9033;
	coef[3].cvl[1] =   1.12988;
	coef[3].cvl[2] =   0.85122;
	coef[3].cvl[3] =  31.0599;
	coef[3].cvl[4] =  -0.00314;
	coef[3].cvl[5] =  -0.71992;
	coef[3].cvl[6] =   1.24163;
	strcpy(coef[4].atom, "O");
	coef[4].val[0] =   3.04850;
	coef[4].val[1] =  13.2771;
	coef[4].val[2] =   2.28680;
	coef[4].val[3] =   5.70110;
	coef[4].val[4] =   1.54630;
	coef[4].val[5] =   0.323900;
	coef[4].val[6] =   0.867000;
	coef[4].val[7] =  32.9089;
	coef[4].val[8] =   0.250800;
	coef[4].eleNum = 8;
	coef[4].cvl[0] =   0.99923;
	coef[4].cvl[1] =   7.91809;
	coef[4].cvl[2] =   0.38057;
	coef[4].cvl[3] =  28.6684;
	coef[4].cvl[4] =   1.25313;
	coef[4].cvl[5] =   0.37761;
	coef[4].cvl[6] =   0.19491;
	
	xray_qlim_normal1 = 2.0 * 2;
	xray_qlim_normal1 *= xray_qlim_normal1;
	xray_qlim_compton1 = 3.0 * 2;
	xray_qlim_compton1 *= xray_qlim_compton1;
	
	return (0);
}

#ifdef _OLDHP
long int checkXrayAtom(atom)
	char *atom;
#else
long int checkXrayAtom(char *atom)
#endif
{
	long int i;
	for (i=0; i<ATOMTABLESIZE; i++) {
		if (!strcmp(coef[i].atom, atom)) return (i);
	}
	return (-1);
}

#ifdef _OLDHP
char calcXrayFactor(qval2, type, ret)
	double *qval2, *ret;
	long int *type;
#else
char calcXrayFactor(double *qval2, long int *type, double *ret)
#endif
{
	/* Return Scattering Factor for the Atom */
	char i;
	
	if ((*qval2) > xray_qlim_normal1) return (XRAY_OUT_OF_QRANGE_NORMAL);
	
	(*ret) = coef[*type].val[8];
	
	for (i=0; i<4; i++) {
		(*ret) += coef[*type].val[i*2] * exp(- coef[*type].val[i*2+1] * (*qval2) / 4.0);
	}
	return (0);
}

#ifdef _OLDHP
char calcXrayAnFactor(fp, lambda, type)
	long int *type;
	double *fp, *lambda;
#else
char calcXrayAnFactor(double *fp, double *lambda, long int *type)
#endif
{
	if (!copyright) {
		fprintf(stderr, "********** Information about Anomalous Scattering Factor **********************\n");
		fprintf(stderr, "* Anomalous Scattering data are taken from the next:\n");
		fprintf(stderr, "*   Any reports or publications of these data will acknowledge its use\n");
		fprintf(stderr, "*   by an appropriate citation.  This would refer to the following publication:\n");
		fprintf(stderr, "*     S.Sasaki (1989) Numerical Tables of Anomalous Scattering Factors\n");
		fprintf(stderr, "*     Calculated by the Cromer and Liberman's Method,\n");
		fprintf(stderr, "*     KEK Report, 88-14, 1-136\n");
		fprintf(stderr, "*******************************************************************************\n");
		copyright++;
	}
	/* definition of parameters */
	switch (*type) {
		case 0:	/* H */
			if ((*lambda) < 0.1 || (*lambda) > 2.89) return (XRAY_OUT_OF_QRANGE_ANOMALOUS);
			fp[0] = 0.0;
			fp[1] = 0.0;
			break;
		case 1:	/* C */
			if ((*lambda) < 0.1 || (*lambda) > 2.89) return (XRAY_OUT_OF_QRANGE_ANOMALOUS);
			fp[0] = ((( 1.47623e-4 * (*lambda)
			                      - 1.80966e-3) * (*lambda)
			                                  + 1.02712e-2) * (*lambda)
			                                              + 8.85369e-4) * (*lambda) - 3.07818e-3;
			fp[1] = (((- 7.24040e-5 * (*lambda)
			                      + 4.51054e-4) * (*lambda)
			                                  + 3.66184e-3) * (*lambda)
			                                              - 6.16914e-4) * (*lambda) + 5.88224e-5;
			break;
		case 2:	/* N */
			if ((*lambda) < 0.1 || (*lambda) > 2.89) return (XRAY_OUT_OF_QRANGE_ANOMALOUS);
			fp[0] = ((( 3.65232e-4 * (*lambda)
			                      - 3.82261e-3) * (*lambda)
			                                  + 1.80650e-2) * (*lambda)
			                                              + 2.18102e-3) * (*lambda) - 5.23607e-3;
			fp[1] = (((- 8.15203e-5 * (*lambda)
			                      + 3.78820e-4) * (*lambda)
			                                  + 8.13045e-3) * (*lambda)
			                                              - 1.54494e-3) * (*lambda) + 1.72974e-4;
			break;
		case 3:	/* S */
			if ((*lambda) < 0.1 || (*lambda) > 2.89) return (XRAY_OUT_OF_QRANGE_ANOMALOUS);
			fp[0] = ((( 6.67370e-3 * (*lambda)
			                      - 8.67422e-2) * (*lambda)
			                                  + 1.85244e-1) * (*lambda)
			                                              + 1.35819e-1) * (*lambda) - 4.95547e-2;
			fp[1] = ((( 2.08456e-3 * (*lambda)
			                      - 3.64523e-2) * (*lambda)
			                                  + 2.99997e-1) * (*lambda)
			                                              - 2.30544e-2) * (*lambda) + 1.50970e-3;
			break;
		case 4:	/* O */
			if ((*lambda) < 0.1 || (*lambda) > 2.89) return (XRAY_OUT_OF_QRANGE_ANOMALOUS);
			fp[0] = ((( 6.80139e-4 * (*lambda)
			                      - 6.86788e-3) * (*lambda)
			                                  + 2.87998e-2) * (*lambda)
			                                              + 4.46920e-3) * (*lambda) - 7.51100e-3;
			fp[1] = (((- 8.96531e-5 * (*lambda)
			                      + 1.44632e-4) * (*lambda)
			                                  + 1.52232e-2) * (*lambda)
			                                              - 2.73862e-3) * (*lambda) + 2.92293e-4;
			break;
		default:
			return (2);
	}
	return (0);
}

#ifdef _OLDHP
char calcXrayCompFactor(qval2, type, ret)
	double *qval2, *ret;
	long int *type;
#else
char calcXrayCompFactor(double *qval2, long int *type, double *ret)
#endif
{
	/* Return (Compton Scattering Factor)^2 for the Atom */
	char i;
	
	if ((*qval2) > xray_qlim_compton1) return (XRAY_OUT_OF_QRANGE_COMPTON);
	
	(*ret) = coef[*type].cvl[6];
	
	for (i=0; i<3; i++) {
		(*ret) += coef[*type].cvl[i*2] * exp(- coef[*type].cvl[i*2+1] * (*qval2) / 4.0);
	}
	
	(*ret) = coef[*type].eleNum - (*ret)*(*ret);
	return (0);
}


#ifdef _OLDHP
long int getEleNum(type)
	long int *type;
#else
long int getEleNum(long int *type)
#endif
{
	/* Return Number of Electron */
	return (coef[*type].eleNum);
}
