/* Program to treat IP data file */
#define ProgName "ipprofile"
#define ProgInfo "R-AXIS Image 2theta-I conversion"
/*                                                         1/14/2000  */
/*         for ANSI C                                                 */

/* Option for iiccd is removed, for it is not correct.
   Use ipiifix for the future.                     v1.61,   1/5/2004  */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.63_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef OMMODE
#define MAXARG 25
#else
#define MAXARG 24
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define SMALLVALUE 0.00001
#define COS_MINIMUM 0.00000001
#define CV_DIRECTX_DIFF 0.001
#define LIM_NEWTON 0.00000001
#define MAXLOOP 100000000
#define SQRT_IGNORE (1.0e-10)
#define ASIN_IGNORE (1.0e-10)
#define ACOS_IGNORE (1.0e-10)

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	double dArea();
	char pixel2Theta();
	char pixel2ThetaV();
	char pixel2ThetaH();
	char pixel2ThetaC();
	char pixel2ThetaVC();
	char pixel2ThetaHC();
	char pixel2ThetaCB();
	char pixel2ThetaHCB();
	long int (*getfuncF)();
	long int (*getfuncC)();
	long int (*getfuncCB)();
	long int getThetaF();
	long int getQF();
	long int getThetaC();
	long int getQC();
	long int getThetaCB();
	long int getQCB();
	long int getThetaFHiPic();
	long int getQFHiPic();
	double (*getcosT)();
	double getcosTt();
	double getcosTq();
	double newton();
	double xSqrt();
	double xAsin();
	double xAcos();
#ifdef OMMODE
 	char omitFlags();
#endif
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
	double dArea(double r, double delta);
	char pixel2Theta(long int jInt);
	char pixel2ThetaV(long int jInt);
	char pixel2ThetaH(long int jInt);
	char pixel2ThetaC(long int jInt);
	char pixel2ThetaVC(long int jInt);
	char pixel2ThetaHC(long int jInt);
	char pixel2ThetaCB(long int jInt);
	char pixel2ThetaHCB(long int jInt);
	long int (*getfuncF)(double x, double z);
	long int (*getfuncC)(double x, double z);
	long int (*getfuncCB)(double x, double z);
	long int getThetaF(double x, double z);
	long int getQF(double x, double z);
	long int getThetaC(double x, double z);
	long int getQC(double x, double z);
	long int getThetaCB(double x, double z);
	long int getQCB(double x, double z);
	long int getThetaFHiPic(double x, double z);
	long int getQFHiPic(double x, double z);
	double (*getcosT)(double x);
	double getcosTt(double x);
	double getcosTq(double x);
	double newton(double r, double r0, double delta);
	double xSqrt(double r);
	double xAsin(double r);
	double xAcos(double r);
#ifdef OMMODE
	char omitFlags(long int j);
#endif
#endif

char shiftBit, ipDataSwapFlag;
long int mThetaDL, mThetaUL, mThetaDR, mThetaUR, mThetaUC, mThetaDC,
     mThetaCL, mThetaCR, *datPtrPxlNum, stepNumPi;
#ifdef OMMODE
long int omX0, omX1, omZ0, omZ1;
#endif
double *datPtrDblInt, *datPtrDblArea, xsiz, zsiz, camr, step, rato, area,
       zUP, zDN, xRT, xLT, *datPtrDblR, areaFactor, lambda, dstep, lam2, calib;
short int *datPtr;
#if (defined(DEBUG) || defined(OMMODE))
long int iLines;
#define USELINE 1
#else
#define USELINE 0
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2,
	     maxInt, numbyte, readByte, lines, xpxlMin, zpxlMin,
	     *datPtrThetaM, jL0, jL1, jR0, jR1, mMax, xStartUP, xStartDN, xEndUP, xEndDN,
	     xStart, xEnd, edgemInt, edgepInt, edgemIntLoop, edgepIntLoop,
	     startLeft, startRight, tmpBuf, tmpBufC, mMaxPerf, startX, startZ, endX, endZ,
	     xpxlOriginal, zpxlOriginal, xWidth, zWidth, fWidth, fWidth0, drtxN, drtzN;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     changeValueTypeIndex, confirmBuffer1[8], confirmBuffer2[8],
	     cpuIntelFlag, ipHeaderSwapFlag, strangeShiftBitFlag, byte,
	     arg0[4], verboseFlag, ofileDoneFlag, useMaxFlag,
	     showNumEffFlag, showNumIntFlag, noHeadFlag, xsizDoneFlag,
	     zsizDoneFlag, drtxDoneFlag, drtzDoneFlag, camrDoneFlag, stepDoneFlag,
	     drtxFFlag, drtzFFlag, timebuf[20], drtxSwapFlag, drtzSwapFlag,
	     phiDoneFlag, ignoreUP, ignoreDN, cameraTypeFlag, cameraTypeDoneFlag,
	     outputMode, outputModeDoneFlag, showNumEffDoneFlag, startXDoneFlag,
	     startZDoneFlag, endXDoneFlag, endZDoneFlag, useSymRegionFlag,
	     noHeadDoneFlag, xWidthDoneFlag, zWidthDoneFlag, midxFlag, qFlag,
#ifdef OMMODE
	     omXDoneFlag, omZDoneFlag,
#endif
	     qDoneFlag, wlngDoneFlag, *colstr, *ucolstr, iiccdFlag, calibDoneFlag,
	     showInfoFlag, lorentzFlag;
	double arg1, arg2, minStep, rMax, tmpArea0, xD, xU, zL, zR, r1, drtx, drtz,
	       phi0, phi1, tan0, tan1, phiC, tMax, numer, cosTheta, cosX, cosTheta2,
	       cosX2, cosMax, drtxm, drtxp, edgem, edgep, stepD,
	       tMaxPerf, r2, dsFactor, phiC2, phi0tmp, phi1tmp, dr;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	time_t timeNow;
#ifdef DEBUG
	iLines = -1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	xsizDoneFlag = zsizDoneFlag = drtxDoneFlag = drtzDoneFlag = ofileDoneFlag = phiDoneFlag
	 = useMaxFlag = showNumEffFlag = showNumIntFlag = noHeadFlag = camrDoneFlag
	 = stepDoneFlag = ignoreUP = ignoreDN = cameraTypeFlag = cameraTypeDoneFlag
	 = outputModeDoneFlag = showNumEffDoneFlag = startXDoneFlag
	 = startZDoneFlag = endXDoneFlag = endZDoneFlag = useSymRegionFlag = noHeadDoneFlag
	 = xWidthDoneFlag = zWidthDoneFlag = midxFlag = qFlag = qDoneFlag = wlngDoneFlag
	 = iiccdFlag = calibDoneFlag = showInfoFlag = lorentzFlag = 0;
	outputMode = 5;
	lines = 1;
	numbyte = readByte = 0;
	drtxN = drtzN = 0;
#ifdef OMMODE
	omX0 = omX1 = omZ0 = omZ1 = omXDoneFlag = omZDoneFlag = 0;
#endif
	/* dummy init */
	jL0 = edgemIntLoop = edgepIntLoop = startX = startZ = endX = endZ =
	startLeft = startRight = tmpBuf = xWidth = zWidth = 0;
	phi0 = phi1 = tan0 = tan1 = drtxm = drtxp = stepD = phi0tmp = phi1tmp = 0.0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-qq")) {
			verboseFlag = -1;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		if (strcmp(argv[argc-1], "-")) {
			strcpy(file2, argv[argc-1]);
			ofileDoneFlag++;
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unabel to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag > 0) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	ipDataSwapFlag = XOREXT(byte, ipHeaderSwapFlag) ? 1 : 0;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag > 0) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag > 0) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag > 0) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " CameraLength is : %7.3f mm, WaveLangth is : %8.5f A.\n", ipHeader.camr, ipHeader.wlng);
	}
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z     , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (5) Camera Length, (6) WaveLength   ,\n");
				fprintf(stdout, "            (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					case 5:
						camrDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Camera Length [mm] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.camr = atof(inputBuffer);
							if (ipHeader.camr <= 0.0) continue;
							camrDoneFlag = 1;
						} while (!camrDoneFlag);
						fprintf(stdout, "Camera Length is changed to %7.3f mm.\n", ipHeader.camr);
						break;
					case 6:
						wlngDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for WaveLength [A] : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.wlng = atof(inputBuffer);
							if (ipHeader.wlng <= 0.0) continue;
							wlngDoneFlag = 1;
						} while (!wlngDoneFlag);
						fprintf(stdout, "WaveLength is changed to %7.3f A.\n", ipHeader.wlng);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
		do {
			fprintf(stdout, "Camera is Curved ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'N') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = 0;
#ifdef USE_IICCD
			do {
				fprintf(stdout, "Use iiccd mode ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y') {
				iiccdFlag = 1;
				calib = (256.0*exp(-0.003*ipHeader.camr)+82.0)*1.0e-7;
				fprintf(stdout, " Image is corrected using equation of \"r = 1.0 - factor * r * r\".\n");
				do {
					fprintf(stdout, " Do you want to fix correction factor of %10.5E ? [Y/N] : ", calib);
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'Y') {
					do {
						fprintf(stdout, "  Enter pre-factor (caution!! keep it small !!) : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						calib = atof(inputBuffer);
						if (calib <=0.0) continue;
						calibDoneFlag = 1;
					} while (!calibDoneFlag);
				}
			}
#endif
		}
		else {
			cameraTypeFlag = 1;
			do {
				fprintf(stdout, "  Camera is Curved. Cylinder Axis is vertical to the screen. OK ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'N') {
				fprintf(stdout, "  Curved Camera only with the axis vertical to the screen is supported.\n");
				fprintf(stdout, "  Camera Type Error. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		do {
			fprintf(stdout, "Output by 2-Theta[T] or q[Q] ? [T/Q] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'T') && (toupper(confirmBuffer1[0]) != 'Q'));
		minStep = cameraTypeFlag ? MAXEXT(ipHeader.xsiz/ipHeader.camr, atan(ipHeader.zsiz/ipHeader.camr)) : atan(MAXEXT(ipHeader.xsiz, ipHeader.zsiz) / ipHeader.camr);
		if (toupper(confirmBuffer1[0]) == 'T') {
			colstr = "2theta";
			ucolstr = "[deg.]";
		}
		else {
			qFlag = 1;
			if (ipHeader.wlng <= 0.0) {
				fprintf(stderr, "You must specify correct WaveLength.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			minStep = 2.0 * sin(minStep/2.0) / ipHeader.wlng;
			colstr = "q";
			ucolstr = "[1/A]";
		}
		do {
			if (qFlag) fprintf(stdout, "Enter %s step of 1-D Profile %s larger than %8.5f.\n", colstr, ucolstr, minStep);
			else fprintf(stdout, "Enter %s step of 1-D Profile %s larger than %8.5f.\n", colstr, ucolstr, degEXT(minStep));
			fprintf(stdout, " Type \"D\" if use Minimum Value : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (toupper(inputBuffer[0]) == 'D') {
				step = minStep;
				stepD = qFlag ? step :degEXT(step);
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			stepD = atof(inputBuffer);
			step = qFlag ? stepD : radEXT(stepD);
			if (step < minStep) continue;
			stepDoneFlag = 1;
		} while (!stepDoneFlag);
		startX = startZ = 0;
		endX = ipHeader.xpxl;
		endZ = ipHeader.zpxl;
		do {
			fprintf(stdout, "Do you want to output the angle using the centre of the step ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') midxFlag = 1;
		do {
			fprintf(stdout, "Do you want to abort some surrounding regions ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			fprintf(stdout, " Type \"D\" if keep unchanged, type \"W\" to specify \"Width\" instead.\n");
			do {
				fprintf(stdout, " Enter the Smallest Coordinate of X in pixel : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') break;
				if (toupper(inputBuffer[0]) == 'W') {
					do {
						fprintf(stdout, "  Enter the Width of X-direction in pixel : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						if ((xWidth = atol(inputBuffer)) <= 0) continue;
						startX = dToLong(ipHeader.drtx - xWidth/2.0);
						endX = startX + xWidth;
						if (endX > ipHeader.xpxl || startX < 0) continue;
						xWidthDoneFlag = 1;
					} while (!xWidthDoneFlag);
					startXDoneFlag = endXDoneFlag = 1;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				startX = atol(inputBuffer);
				if (startX > ipHeader.drtx || startX < 0) continue;
				startXDoneFlag = 1;
			} while (!startXDoneFlag);
			do {
				fprintf(stdout, " Enter the Smallest Coordinate of Z in pixel : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') break;
				if (toupper(inputBuffer[0]) == 'W') {
					do {
						fprintf(stdout, "  Enter the Width of Z-direction in pixel : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						if ((zWidth = atol(inputBuffer)) <= 0) continue;
						startZ = dToLong(ipHeader.drtz - zWidth/2.0);
						endZ = startZ + zWidth;
						if (endZ > ipHeader.zpxl || startZ < 0) continue;
						zWidthDoneFlag = 1;
					} while (!zWidthDoneFlag);
					startZDoneFlag = endZDoneFlag = 1;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				startZ = atol(inputBuffer);
				if (startZ > ipHeader.drtz || startZ < 0) continue;
				startZDoneFlag = 1;
			} while (!startZDoneFlag);
			if (!xWidthDoneFlag) {
				do {
					fprintf(stdout, " Enter the Largest Coordinate of X in pixel : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (toupper(inputBuffer[0]) == 'D') break;
					if (!isdigitEXT(inputBuffer)) continue;
					endX = atol(inputBuffer);
					if (endX < ipHeader.drtx || endX > ipHeader.xpxl || endX - startX <= 0) continue;
					endXDoneFlag = 1;
				} while (!endXDoneFlag);
			}
			if (!zWidthDoneFlag) {
				do {
					fprintf(stdout, " Enter the Largest Coordinate of Z in pixel : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (toupper(inputBuffer[0]) == 'D') break;
					if (!isdigitEXT(inputBuffer)) continue;
					endZ = atol(inputBuffer);
					if (endZ < ipHeader.drtz || endZ > ipHeader.zpxl || endZ - startZ <= 0) continue;
					endZDoneFlag = 1;
				} while (!endZDoneFlag);
			}
		}
		do {
			fprintf(stdout, "Do you want to calculate even for imperfect Debye-ring ?\n");
			fprintf(stdout, "  This might also generate statistically meaningless data. [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') useMaxFlag = 1;
		if (!cameraTypeFlag) {
			do {
				fprintf(stdout, "Do you want to calculate only for some fan-shaped region ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'Y') {
				do {
					fprintf(stdout, "  Enter start angle from X-direction [deg.] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					phi0 = atof(inputBuffer);
					if ((phi0 <= -360.0) || (phi0 >= 360.0)) continue;
					phiDoneFlag = 1;
				} while (!phiDoneFlag);
				phiDoneFlag = 0;
				do {
					fprintf(stdout, "  Enter end angle from X-direction [deg.] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					phi1 = atof(inputBuffer);
					if ((phi1 < phi0) || (phi1 > 360.0) || (phi1-phi0 > 360.0)) continue;
					phiDoneFlag = 1;
				} while (!phiDoneFlag);
				if (phi1 - phi0 < 180.0) {
					do {
						fprintf(stdout, " Do you want to include the region symmetrical to the centre ? [Y/N] : ");
						if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
					if (confirmBuffer2[0] == 'Y') useSymRegionFlag = 1;
				}
			}
		}
#ifdef OMMODE
		do {
			fprintf(stdout, "Do you want to ignore some rectangular region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter start coodinate to ignore in X-direction.\n");
				fprintf(stdout, "    If only in Z-direction, type \"D\" : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') break;
				if (!isdigitEXT(inputBuffer)) continue;
				omX0 = atol(inputBuffer);
				if ((omX0 < 0) || (omX0 > ipHeader.xpxl)) continue;
				omXDoneFlag = 1;
			} while (!omXDoneFlag);
			omXDoneFlag = 0;
			if (toupper(inputBuffer[0]) != 'D') {
				do {
					fprintf(stdout, "  Enter end coodinate to ignore in X-direction : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					omX1 = atol(inputBuffer);
					if ((omX1 < omX0) || (omX1 > ipHeader.xpxl)) continue;
					omXDoneFlag = 1;
				} while (!omXDoneFlag);
			}
			do {
				fprintf(stdout, "  Enter start coodinate to ignore in Z-direction.\n");
				fprintf(stdout, "    If only in X-direction, type \"D\" : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'D') break;
				if (!isdigitEXT(inputBuffer)) continue;
				omZ0 = atol(inputBuffer);
				if ((omZ0 < 0) || (omZ0 > ipHeader.zpxl)) continue;
				omZDoneFlag = 1;
			} while (!omZDoneFlag);
			if (toupper(inputBuffer[0]) != 'D') {
				do {
					fprintf(stdout, "  Enter end coodinate to ignore in Z-direction : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					omZ1 = atol(inputBuffer);
					if ((omZ1 < omZ0) || (omZ1 > ipHeader.zpxl)) continue;
					omZDoneFlag = 1;
				} while (!omZDoneFlag);
			}
		}
#endif
		do {
			fprintf(stdout, "Print Area of Pixel contributing to each angle in PixelSize Unit? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			showNumEffFlag = 1;
			if (cameraTypeFlag) {
				do {
					fprintf(stdout, "  Area is reduced to spherical surface area. OK ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'N') showNumEffFlag = 2;
			}
		}
		do {
			fprintf(stdout, "Print Number of Pixel contributing to each angle ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') showNumIntFlag = 1;
		do {
			fprintf(stdout, "Print 1-D Profile data without normalizing (or averaging) ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			outputMode = 1;
			if (useMaxFlag) {
				do {
					fprintf(stdout, "  Calculate for imperfect Debye-Ring region using average data ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'Y') outputMode = 4;
			}
		}
		else {
			do {
				fprintf(stdout, "  Average for pixel[P], d(%s)[S] or Solid Angle[R] ? [P/S/R] : ", colstr);
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'P') && (toupper(confirmBuffer2[0]) != 'S') && (toupper(confirmBuffer2[0]) != 'R'));
			if (toupper(confirmBuffer2[0]) == 'S') outputMode = 5;
			else if (toupper(confirmBuffer2[0]) == 'R') outputMode = 6;
			else if (cameraTypeFlag) {
				do {
					fprintf(stdout, "  Use Area reduced to spherical surface ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
				if (toupper(confirmBuffer2[0]) == 'N') outputMode = 2;
			}
		}
		do {
			fprintf(stdout, "Execute Lorentz correction (for isotropic specimen type) ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') lorentzFlag = 1;
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-mx")) useMaxFlag++;
				else if (!strcmp(arg0, "-xx")) midxFlag++;
				else if (!strcmp(arg0, "-a")) { /* average */
					outputMode = 0;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-s")) {/* sum */
					outputMode = 1;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-an")) {/* average, wrong for curved */
					outputMode = 2;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-sl")) {/* sum, corrected */
					outputMode = 4;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-av")) {/* correct average, default */
					outputMode = 5;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-sv")) { /* average about solid angle */
					outputMode = 6;
					outputModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-sn")) {
					showNumEffFlag = 1;
					showNumEffDoneFlag++;
				}
				else if (!strcmp(arg0, "-sr")) {
					showNumEffFlag = 2;
					showNumEffDoneFlag++;
				}
				else if (!strcmp(arg0, "-sN")) showNumIntFlag++;
				else if (!strcmp(arg0, "-sh")) {
					noHeadFlag = 1;
					noHeadDoneFlag++;
				}
				else if (!strcmp(arg0, "-sH")) {
					noHeadFlag = 2;
					noHeadDoneFlag++;
				}
				else if (!strcmp(arg0, "-tm")) showInfoFlag++;
				else if (!strcmp(arg0, "-f")) {
					cameraTypeDoneFlag++;
					cameraTypeFlag = 0;
				}
#ifdef USE_IICCD
				else if (!strcmp(arg0, "-fh")) {
					cameraTypeDoneFlag++;
					cameraTypeFlag = 0;
					iiccdFlag++;
				}
#endif
				else if (!strcmp(arg0, "-cv")) {
					cameraTypeDoneFlag++;
					cameraTypeFlag = 1;
				}
				else if (!strcmp(arg0, "-rs")) useSymRegionFlag++;
				else if (!strcmp(arg0, "-dt")) qDoneFlag++;
				else if (!strcmp(arg0, "-dq")) {
					qDoneFlag++;
					qFlag = 1;
				}
				else if (!strcmp(arg0, "-lc")) lorentzFlag++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0.0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0.0) break;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-dt")) {
					stepD = arg1;
					if (stepD <= 0.0) break;
					stepDoneFlag++;
					qDoneFlag++;
				}
				else if (!strcmp(arg0, "-dq")) {
					stepD = arg1;
					if (stepD <= 0.0) break;
					stepDoneFlag++;
					qDoneFlag++;
					qFlag = 1;
				}
				else if (!strcmp(arg0, "-lm")) {
					ipHeader.wlng = arg1;
					if (ipHeader.wlng <= 0.0) break;
					wlngDoneFlag++;
				}
				else if (!strcmp(arg0, "-xs")) {
					startX = dToLong(arg1);
					startXDoneFlag++;
				}
				else if (!strcmp(arg0, "-zs")) {
					startZ = dToLong(arg1);
					startZDoneFlag++;
				}
				else if (!strcmp(arg0, "-xf")) {
					endX = dToLong(arg1);
					endXDoneFlag++;
				}
				else if (!strcmp(arg0, "-zf")) {
					endZ = dToLong(arg1);
					endZDoneFlag++;
				}
				else if (!strcmp(arg0, "-xw")) {
					xWidth = dToLong(arg1);
					if (xWidth <= 0) break;
					startXDoneFlag++;
					endXDoneFlag++;
					xWidthDoneFlag++;
				}
				else if (!strcmp(arg0, "-zw")) {
					zWidth = dToLong(arg1);
					if (zWidth <= 0) break;
					startZDoneFlag++;
					endZDoneFlag++;
					zWidthDoneFlag++;
				}
#ifdef USE_IICCD
				else if (!strcmp(arg0, "-fh")) {
					calib = arg1;
					if (calib <= 0.0) break;
					cameraTypeDoneFlag++;
					cameraTypeFlag = 0;
					calibDoneFlag++;
					iiccdFlag++;
				}
#endif
				else break;
			}
			else if (argNum == 3) {
				if (!strcmp(arg0, "-rg")) {
					phi0 = arg1;
					phi1 = arg2;
					if (phi0 > phi1) {
						phi1 = arg1;
						phi0 = arg2;
					}
					if ((phi0 <= -360.0) || (phi1 > 360.0) || (phi1-phi0 > 360.0)) break;
					phiDoneFlag++;
				}
#ifdef OMMODE
				else if (!strcmp(arg0, "-xo")) {
					omX0 = dToLong(arg1);
					omX1 = dToLong(arg2);
					if (omX0 > omX1) {
						omX1 = dToLong(arg1);
						omX0 = dToLong(arg2);
					}
					if (omX0 > omX1 || omX0 < 0 || omX1 > ipHeader.xpxl) break;
					omXDoneFlag++;
				}
				else if (!strcmp(arg0, "-zo")) {
					omZ0 = dToLong(arg1);
					omZ1 = dToLong(arg2);
					if (omZ0 > omZ1) {
						omZ1 = dToLong(arg1);
						omZ0 = dToLong(arg2);
					}
					if (omZ0 > omZ1 || omZ0 < 0 || omZ1 > ipHeader.zpxl) break;
					omZDoneFlag++;
				}
#endif
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!qDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (camrDoneFlag > 1)
		 || (stepDoneFlag > 1) || (outputModeDoneFlag > 1) || (ofileDoneFlag > 1)
		 || (useMaxFlag > 1) || (xsizDoneFlag > 1) || (zsizDoneFlag > 1)
		 || (showNumIntFlag > 1) || (showNumEffDoneFlag > 1) || (noHeadDoneFlag > 1)
		 || (phiDoneFlag > 1) || (cameraTypeDoneFlag > 1) || (startXDoneFlag > 1)
		 || (startZDoneFlag > 1) || (endXDoneFlag > 1) || (endZDoneFlag > 1)
		 || (wlngDoneFlag > 1) || (lorentzFlag > 1)
#ifdef OMMODE
		 || (omXDoneFlag > 1) || (omZDoneFlag > 1)
#endif
		 || (qDoneFlag > 1) || (showInfoFlag > 1)
		 || (cameraTypeFlag && phiDoneFlag) || (useSymRegionFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((!cameraTypeFlag && (showNumEffFlag == 2 || outputMode == 2))
		 || (!useMaxFlag && outputMode == 4) || (useSymRegionFlag && !phiDoneFlag)
		 || (showInfoFlag && noHeadDoneFlag)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!startXDoneFlag) startX = 0;
		if (!startZDoneFlag) startZ = 0;
		if (!endXDoneFlag) endX = ipHeader.xpxl;
		if (!endZDoneFlag) endZ = ipHeader.zpxl;
		if (xWidthDoneFlag) {
			startX = dToLong(ipHeader.drtx - xWidth/2.0);
			endX = startX + xWidth;
		}
		if (zWidthDoneFlag) {
			startZ = dToLong(ipHeader.drtz - zWidth/2.0);
			endZ = startZ + zWidth;
		}
		if (startX < 0 || startX > ipHeader.drtx || startZ < 0 || startZ > ipHeader.drtz
		 || endX > ipHeader.xpxl || endX < ipHeader.drtx || endZ > ipHeader.zpxl || endZ < ipHeader.drtz
		 || endX - startX <= 0 || endZ - startZ <= 0) {
			fprintf(stderr, " Input Region is out of range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		minStep = cameraTypeFlag ? MAXEXT(ipHeader.xsiz/ipHeader.camr, atan(ipHeader.zsiz/ipHeader.camr)) : atan(MAXEXT(ipHeader.xsiz, ipHeader.zsiz) / ipHeader.camr);
		if (qFlag) {
			if (ipHeader.wlng <= 0.0) {
				fprintf(stderr, "You must specify correct WaveLength.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			minStep = 2.0 * sin(minStep/2.0) / ipHeader.wlng;
			colstr = "q";
			ucolstr = "[1/A]";
		}
		else {
			colstr = "2Theta";
			ucolstr = "[deg.]";
		}
		if (!stepDoneFlag) {
			step = minStep;
			stepD = qFlag ? step : degEXT(step);
		}
		else {
			step = qFlag ? stepD : radEXT(stepD);
			if (step < minStep) {
				fprintf(stderr, " Invalid %s step : %8.5f %s. Stop.\n", colstr, stepD, ucolstr);
				showUsage(stderr);
				fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		if (iiccdFlag && !calibDoneFlag) calib = (256.0*exp(-0.003*ipHeader.camr)+82.0)*1.0e-7;
	}
	if (cameraTypeFlag && ((ipHeader.drtx - ipHeader.camr*M_PI/ipHeader.xsiz >= 0.0) || (ipHeader.drtx + ipHeader.camr*M_PI/ipHeader.xsiz <= ipHeader.xpxl))) {
		fprintf(stderr, " Curved Camera \" Width\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (phiDoneFlag) {
		if (useSymRegionFlag && (phi1 - phi0 >= 180.0)) {
			fprintf(stderr, " Region of %6.1f - %6.1f is larger than expected. Exit.\n", phi0, phi1);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (phi1 - phi0 == 360.0) phiDoneFlag = 0; /* No Phi */
		else {
			if (phi0 < -180.0) phi0 += 360.0;
			else if (phi0 >= 180.0) phi0 += -360.0;
			if (phi1 <= -180.0) phi1 += 360.0;
			else if (phi1 > 180.0) phi1 += -360.0;
		}
		if (((phi0 >= 0.0) || (phi0 == -180.0)) && (phi1 >= 0.0)) ignoreDN++;
		if ((phi0 <= 0.0) && ((phi1 <= 0.0) || (phi1 == 180.0))) ignoreUP++;
	}
	if (!phiDoneFlag) {
		phi0 = -180.0;
		phi1 = 180.0;
		ignoreDN++;
		ignoreUP++;
	}
	if (verboseFlag >= 2) {
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " PixelSize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (camrDoneFlag) fprintf(stdout, " CameraLength has changed to : %7.3f mm.\n", ipHeader.camr);
		if (qFlag && wlngDoneFlag) fprintf(stdout, " WaveLength has changed to : %8.5f [A].\n", ipHeader.wlng);
	}
	if (qFlag) {
		getfuncF = iiccdFlag ? getQFHiPic : getQF;
		getfuncC = getQC;
		getfuncCB = getQCB;
		getcosT = getcosTq;
	}
	else {
		getfuncF = iiccdFlag ? getThetaFHiPic : getThetaF;
		getfuncC = getThetaC;
		getfuncCB = getThetaCB;
		getcosT = getcosTt;
	}
	
/* Set Parameters to Create */
	camr = ipHeader.camr;
	xsiz = ipHeader.xsiz;
	zsiz = ipHeader.zsiz;
	area = xsiz * zsiz;
	lambda = ipHeader.wlng;
	lam2 = lambda * lambda / 2.0;
	drtxFFlag = (long)ceil(ipHeader.drtx) - (long)floor(ipHeader.drtx);
	if (!drtxFFlag && cameraTypeFlag) {
		ipHeader.drtx -= CV_DIRECTX_DIFF;
		drtxFFlag++;
	} /* ad-hoc modification for curved camera */
	drtzFFlag = (long)ceil(ipHeader.drtz) - (long)floor(ipHeader.drtz);
	drtxSwapFlag = (ipHeader.drtx-floor(ipHeader.drtx) > 0.5) ? 1 : 0;
	drtzSwapFlag = (ipHeader.drtz-floor(ipHeader.drtz) > 0.5) ? 1 : 0;
	if ((phi0 != -180.0) && (phi0 != 0.0) && (phi0 != 180.0)) tan0 = tan(M_PI*(0.5-phi0/180.0))*zsiz/xsiz;
	if ((phi1 != -180.0) && (phi1 != 0.0) && (phi1 != 180.0)) tan1 = tan(M_PI*(0.5-phi1/180.0))*zsiz/xsiz;
	xpxlOriginal = ipHeader.xpxl;
	zpxlOriginal = ipHeader.zpxl;
	ipHeader.xpxl = endX - startX;
	ipHeader.zpxl = endZ - startZ;
	ipHeader.drtx -= startX;
	ipHeader.drtz -= startZ;
#ifdef OMMODE
	if (omXDoneFlag) {
		omX0 -= startX;
		omX1 -= startX;
		if (!omZDoneFlag) {
			omZ0 = 0;
			omZ1 = ipHeader.zpxl;
		}
	}
	if (omZDoneFlag) {
		omZ0 -= startZ;
		omZ1 -= startZ;
		if (!omXDoneFlag) {
			omX0 = 0;
			omX1 = ipHeader.xpxl;
		}
	}
#endif
	if (cameraTypeFlag) {
		tMaxPerf = MINEXT(atan(MINEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz)*zsiz/camr), MINEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz/camr);
		if (qFlag) tMaxPerf = 2.0 * sin(tMaxPerf/2.0) / lambda;
		mMaxPerf = (long)floor(tMaxPerf / step);
		if (useMaxFlag) {
			if (MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz/camr <= M_PI_2)
			    tMax = acos(cos(MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz/camr)
			                 / dist(1.0, MAXEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz)*zsiz/camr));
			else tMax = MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz/camr;
			if (qFlag) tMax = 2.0 * sin(tMax/2.0) / lambda;
			mMax = (long)ceil(tMax / step);
		}
		else {
			tMax = tMaxPerf;
			mMax = mMaxPerf;
		}
		drtxm = ipHeader.drtx - M_PI*camr/xsiz;
		edgem = ipHeader.drtx - M_PI_2*camr/xsiz;
		edgep = ipHeader.drtx + M_PI_2*camr/xsiz;
		drtxp = ipHeader.drtx + M_PI*camr/xsiz;
		if (qFlag) {
			stepNumPi = (long)floor(2.0/lambda/step);
			dstep = 2.0/lambda - step * stepNumPi;
			r1 = 2.0*asin((floor(sqrt(2.0)/lambda/step)+1.0)*step*lambda/2.0);
		}
		else {
			stepNumPi = (long)floor(M_PI / step);
			dstep = M_PI - step * stepNumPi;
			r1 = (floor(M_PI_2/step)+1.0)*step;
		}
		edgemInt = (long)floor(edgem);
		edgemIntLoop = ((ipHeader.drtx - edgemInt)*xsiz/camr > r1) ? edgemInt + 1 : edgemInt;
		startLeft = MAXEXT(edgemIntLoop, 0);
		edgepInt = (long)ceil(edgep);
		edgepIntLoop = ((edgepInt - ipHeader.drtx)*xsiz/camr > r1) ? edgepInt - 1 : edgepInt;
		startRight = MINEXT(edgepIntLoop, ipHeader.xpxl);
	}
	else {
		tMaxPerf = atan(MINEXT(MINEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz, MINEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz)*zsiz) / camr);
		if (iiccdFlag) tMaxPerf = atan((1.0-calib*camr*tan(tMaxPerf)*camr*tan(tMaxPerf)) * tan(tMaxPerf));
		if (qFlag) tMaxPerf = 2.0 * sin(tMaxPerf/2.0) / lambda;
		mMaxPerf = (long)floor(tMaxPerf / step);
		if (useMaxFlag) {
			tMax = atan(dist(MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)*xsiz, MAXEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz)*zsiz) / camr);
			if (iiccdFlag) tMax = atan((1.0-calib*camr*tan(tMax)*camr*tan(tMax)) * tan(tMax));
			if (qFlag) tMax = 2.0 * sin(tMax/2.0) / lambda;
			mMax = (long)ceil(tMax / step);
		}
		else {
			tMax = tMaxPerf;
			mMax = mMaxPerf;
		}
	}
	if (mMax < 1) {
		fprintf(stderr, " Output Data is too short. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	tMax = mMax * step;
	if (qFlag) {
		if (iiccdFlag) {
			rMax = newton(tMax*lambda*sqrt(4.0-tMax*tMax*lambda*lambda)/(2.0-tMax*tMax*lambda*lambda) * camr,
			               tMax*lambda*sqrt(4.0-tMax*tMax*lambda*lambda)/(2.0-tMax*tMax*lambda*lambda) * camr/2.0, LIM_NEWTON);
			if (rMax < 0) {
				fprintf(stderr, " Calculation of rMax failed. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else rMax = tMax*lambda*sqrt(4.0-tMax*tMax*lambda*lambda)/(2.0-tMax*tMax*lambda*lambda) * camr;
		cosMax = 1.0 - tMax*tMax*lam2;
	}
	else {
		if (iiccdFlag) {
			rMax = newton(tan(tMax)*camr, tan(tMax)*camr/2.0, LIM_NEWTON);
			if (rMax < 0) {
				fprintf(stderr, " Calculation of rMax failed. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else rMax = tan(tMax) * camr; /* max radius (vertical if curved) */
		cosMax = cos(tMax); /* max horizontal radius for curved */
	}
	if (verboseFlag == 2) {
		fprintf(stdout, " Profile is calculated with %8.5f %s step,\n", stepD, ucolstr);
		fprintf(stdout, "   from 0.0 to %9.5f %s.\n", mMax*stepD, ucolstr);
		useMaxFlag ? fprintf(stdout, "  : Calculate profile for angles of all data.\n") : fprintf(stdout, "  : Calculate profile for angles having sufficient data.\n");
		if (useMaxFlag) fprintf(stdout, "   Intensity for angles larger than %9.5f %s is imperfect.\n", mMaxPerf*stepD, ucolstr);
		fprintf(stdout, " Calculation is done in region : Phi in %6.1f - %6.1f [deg.].\n", phi0, phi1);
		if (useSymRegionFlag) fprintf(stdout, "                Also in region : Phi in %6.1f - %6.1f [deg.].\n", phi0<0.0?phi0+180.0:phi0-180.0, phi0<0.0?phi1+180.0:(phi1<0.0?phi1+180.0:phi1-180.0));
		if (startXDoneFlag || endXDoneFlag || startZDoneFlag || endZDoneFlag)
		    fprintf(stdout, " Region outside of [%ld, %ld] - [%ld, %ld] are ignored.\n", startX, startZ, endX, endZ);
#ifdef OMMODE
		if (omXDoneFlag || omZDoneFlag) fprintf(stdout, " Region of [%ld, %ld] - [%ld, %ld] are ignored.\n", omX0+startX, omZ0+startZ, omX1+startX, omZ1+startZ);
#endif
		switch (outputMode) {
			case 1:
				fprintf(stdout, " Intensity is not averaged by area.\n");
				break;
			case 2:
				cameraTypeFlag ? fprintf(stdout, " Intensity is averaged by area without Curved camera type correction.\n") : fprintf(stdout, " Intensity is not averaged by area.\n");
				break;
			case 4:
				fprintf(stdout, " Intensity is averaged by area and modified to Integrated Intensity.\n");
				break;
			case 0:
				cameraTypeFlag ? fprintf(stdout, " Intensity is averaged by pixel with Curved camera type correction.\n") : fprintf(stdout, " Intensity is averaged by pixel.\n");
				break;
			case 6:
				fprintf(stdout, " Intensity is averaged by solid angle.\n");
				break;
			case 5:
			default:
				cameraTypeFlag ? fprintf(stdout, " Intensity is averaged along azimuthal direction\n  with Curved camera type correction.\n")
				               : fprintf(stdout, " Intensity is averaged along azimuthal direction.\n");
		}
		if (lorentzFlag) fprintf(stdout, " Intensity is lorentz-corrected.\n");
		if (iiccdFlag) {
			fprintf(stdout, " Image is corrected using r = 1.0 - %10.5E * r * r.\n", calib);
			if (calibDoneFlag) fprintf(stdout, " Caution!! Factor for correction is changed by user.\n");
		}
		if (showNumEffFlag == 1 && cameraTypeFlag) fprintf(stdout, " Area contributing to each angle is also output (reduced to sphere).\n");
		else if (showNumEffFlag == 2 && cameraTypeFlag) fprintf(stdout, " Area contributing to each angle is also output (not reduced to sphere).\n");
		else if (showNumEffFlag) fprintf(stdout, " Area contributing to each angle is also output.\n");
		if (showNumIntFlag) fprintf(stdout, " Number of Pixel for each angle is also output.\n");
	}
	dr = 0.5*midxFlag;
	fWidth = ipHeader.xpxl*RAXDATAWIDTH;
	fWidth0 = ipHeader.rlng - fWidth;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (strcmp(file2, "-")) ofileDoneFlag = 1;
	}
	if (ofileDoneFlag) {
		while (createFileEXT(file2, &fp2)) {
			if (verboseFlag <= 0) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	else fp2 = stdout;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Suppress Output Header ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			noHeadFlag = 1;
			do {
				fprintf(stdout, "Suppress Output Line Title ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y') noHeadFlag = 2;
		}
		else {
			do {
				fprintf(stdout, "Include \"Date\", \"Operator\", and \"Exposure Time\" in Header ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y') showInfoFlag = 1;
		}
	}
	if (verboseFlag >= 2) {
		if (noHeadFlag == 1) fprintf(stdout, " Output Information is suppressed.\n");
		else if (noHeadFlag == 2) fprintf(stdout, " Output Information and Title Line is suppressed.\n");
	}
	if (!noHeadFlag) {
		numbyte += fprintf(fp2, "# 1-D Intensity Profile for %s.\n", file1);
		numbyte += fprintf(fp2, "# Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl+startX, ipHeader.zpxl+startZ);
		numbyte += fprintf(fp2, "#  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		numbyte += fprintf(fp2, "# Centre is : %6.1f, %6.1f.\n", ipHeader.drtx+startX, ipHeader.drtz+startZ);
		numbyte += fprintf(fp2, "# PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		numbyte += qFlag ? fprintf(fp2, "# CameraLength is : %7.3f mm, Wavelength is : %9.6f A.\n", ipHeader.camr, lambda)
		                 : fprintf(fp2, "# CameraLength is : %7.3f mm.\n", ipHeader.camr);
		numbyte += fprintf(fp2, "# Calculation is done in region : Phi in %6.1f - %6.1f [deg.].\n", phi0, phi1<phi0?phi1+360.0:phi1);
		if (useSymRegionFlag)
		    numbyte += fprintf(fp2, "#                Also in region : Phi in %6.1f - %6.1f [deg.].\n",
		                       phi0<0.0?phi0+180.0:phi0-180.0, phi0<0.0?phi1+180.0:(phi1<0.0?phi1+180.0:phi1-180.0));
		if (useMaxFlag) numbyte += fprintf(fp2, "# Intensity for angles larger than %9.5f %s is imperfect.\n", mMaxPerf*stepD, ucolstr);
		numbyte += fprintf(fp2, "# Profile is calculated with %8.5f %s step, from 0.0 to %9.5f %s.\n", stepD, ucolstr, mMax*stepD, ucolstr);
		if (startXDoneFlag || endXDoneFlag || startZDoneFlag || endZDoneFlag)
		    numbyte += fprintf(fp2, "# Region outside of [%ld, %ld] - [%ld, %ld] are ignored.\n", startX, startZ, endX, endZ);
#ifdef OMMODE
		if (omXDoneFlag || omZDoneFlag) numbyte += fprintf(fp2, "# Region of [%ld, %ld] - [%ld, %ld] are ignored.\n", omX0+startX, omZ0+startZ, omX1+startX, omZ1+startZ);
#endif
		switch (outputMode) {
			case 1:
				numbyte += fprintf(fp2, "# Intensity is not averaged by area.\n");
				break;
			case 2:
				numbyte += cameraTypeFlag ? fprintf(fp2, "# Intensity is averaged by area without Curved camera type correction.\n") : fprintf(fp2, "# Intensity is not averaged by area.\n");
				break;
			case 4:
				numbyte += fprintf(fp2, "# Intensity is averaged by area and modified to Integrated Intensity.\n");
				break;
			case 0:
				numbyte += cameraTypeFlag ? fprintf(fp2, "# Intensity is averaged by pixel with Curved camera type correction.\n") : fprintf(fp2, "# Intensity is averaged by pixel.\n");
				break;
			case 6:
				numbyte += fprintf(fp2, "# Intensity is averaged by solid angle.\n");
				break;
			case 5:
			default:
				numbyte += cameraTypeFlag ? fprintf(fp2, "# Intensity is averaged along azimuthal direction with Curved camera type correction.\n")
				                          : fprintf(fp2, "# Intensity is averaged along azimuthal direction.\n");
		}
		if (lorentzFlag) numbyte += fprintf(fp2, "# Intensity is lorentz-corrected.\n");
		timeNow = time(NULL);
		strftime(timebuf, 20, "%Y-%m-%d %H:%M:%S", localtime(&timeNow));
		if (iiccdFlag) {
			fprintf(fp2, "# Image is corrected using r = 1.0 - %10.5E * r * r.\n", calib);
			if (calibDoneFlag) fprintf(fp2, "#  Factor for correction is changed by user.\n");
		}
		numbyte += fprintf(fp2, "# Converted by %s-%s on %s\n", ProgName, VERSION, timebuf);
		lines += (10 + useMaxFlag + useSymRegionFlag + iiccdFlag + calibDoneFlag
		          + lorentzFlag + (startXDoneFlag | endXDoneFlag | startZDoneFlag | endZDoneFlag));
#ifdef OMMODE
		lines += (omXDoneFlag | omZDoneFlag);
#endif
		if (showInfoFlag) { /* for HiPic treatment, Feb. 23, 2002 */
			numbyte += fprintf(fp2, "# Date : %s\n", ipHeader.date);
			numbyte += fprintf(fp2, "# Time : %s\n", ipHeader.opnm);
			numbyte += fprintf(fp2, "# ExposureTime : %7.3fms\n", ipHeader.fext*60000.0);
			lines += 3;
		}
	}
	if (!useMaxFlag) {
		r1 = cameraTypeFlag ? acos(cosMax)*camr : rMax;
		ipHeader.xpxl = xpxlOriginal;
		ipHeader.zpxl = zpxlOriginal;
		ipHeader.drtx += startX;
		ipHeader.drtz += startZ;
		xpxlMin = (long)floor(ipHeader.drtx - r1/xsiz);
		zpxlMin = (long)floor(ipHeader.drtz - rMax/zsiz);
		ipHeader.xpxl = (long)ceil(ipHeader.drtx + r1/xsiz) - xpxlMin;
		ipHeader.zpxl = (long)ceil(ipHeader.drtz + rMax/zsiz) - zpxlMin;
		ipHeader.drtx -= xpxlMin;
		ipHeader.drtz -= zpxlMin;
		if (cameraTypeFlag) {
			edgem -= xpxlMin;
			edgep -= xpxlMin;
			edgemInt -= xpxlMin;
			edgemIntLoop -= xpxlMin;
			startLeft = MAXEXT(edgemIntLoop, 0);
			edgepInt -= xpxlMin;
			edgepIntLoop -= xpxlMin;
			startRight = MINEXT(edgepIntLoop, ipHeader.xpxl);
		}
	}
	else {
		xpxlMin = startX;
		zpxlMin = startZ;
	}
	
/* Start Calculation */
	if (verboseFlag > 0) fprintf(stdout, " Now, in progress...\n");
	/* Initialize for Theta */
	if ((datPtrDblInt = malloc((mMax+2)*sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if ((datPtrDblArea = malloc((mMax+2)*sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtrDblInt);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if ((datPtrPxlNum = malloc((mMax+2)*sizeof(long))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtrDblInt);
		free(datPtrDblArea);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if ((datPtrDblR = malloc((mMax+2)*sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtrDblInt);
		free(datPtrDblArea);
		free(datPtrPxlNum);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	for (i=0; i<mMax+2; i++) {
		datPtrDblInt[i] = 0.0;
		datPtrDblArea[i] = 0.0;
		datPtrPxlNum[i] = 0;
		if (iiccdFlag) {
			datPtrDblR[i] = qFlag ? newton(camr*tan(asin(step*i*lambda/2.0)*2.0), camr*tan(asin(step*i*lambda/2.0)*2.0)/2.0, LIM_NEWTON)
		                          : newton(tan(i*step)*camr, tan(i*step)*camr/2, LIM_NEWTON);
			if (datPtrDblR[i] < 0.0) {
				fprintf(stderr, " Calculation of datPtrDblR[%ld] failed. Exit.\n", i);
				fclose(fp1);
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrDblR);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
		}
		else datPtrDblR[i] = cameraTypeFlag ? 0.0 : (qFlag ? camr * tan(asin(step*i*lambda/2.0) * 2.0) : camr * tan(i * step));
	}
	/* Initialize for File */
	if (fseek(fp1, ipHeader.rlng*zpxlMin+(xpxlMin+ipHeader.xpxl)*RAXDATAWIDTH, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtrDblInt);
		free(datPtrDblArea);
		free(datPtrPxlNum);
		free(datPtrDblR);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if ((datPtrThetaM = malloc((ipHeader.xpxl+1)*sizeof(long))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtrDblInt);
		free(datPtrDblArea);
		free(datPtrPxlNum);
		free(datPtrDblR);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if (!cameraTypeFlag) { /*----FLAT CAMERA----*/
	/* Start bottom Loop */
		if (ignoreDN && (phi0 < phi1) && (phi0 >= 0.0) && !useSymRegionFlag) { /* SKIP ME !! */
			for (j=0; j<=ipHeader.xpxl; j++) datPtrThetaM[j] = (*getfuncF)((ipHeader.drtx-j)*xsiz, (ipHeader.drtz-floor(ipHeader.drtz))*zsiz);
			mThetaUC = (*getfuncF)(0.0, (ipHeader.drtz-floor(ipHeader.drtz))*zsiz);
			if (fseek(fp1, ipHeader.rlng*(long)floor(ipHeader.drtz), SEEK_CUR)) {
				fprintf(stderr, " Input File Read Error.\n");
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			i = (long)floor(ipHeader.drtz);
		}
		else {
			if (useSymRegionFlag) {
				phi0tmp = phi0;
				if (phi0 >= 0.0) phi0 -= 180.0;
				phi1tmp = phi1;
				if (phi1 > 0.0) phi1 -= 180.0;
			}
			drtz = ipHeader.drtz;
			drtzN = (long)floor(drtz);
			mThetaUC = (*getfuncF)(0.0, drtz*zsiz);
			if (useMaxFlag) {
				for (j=0; j<=ipHeader.xpxl; j++) datPtrThetaM[j] = (*getfuncF)((ipHeader.drtx-j)*xsiz, drtz*zsiz);
				for (i=0; i<drtxN; i++) {
#if USELINE
					iLines++;
#endif
					if (fseek(fp1, fWidth0, SEEK_CUR)) {
						fprintf(stderr, " Input File Read Error.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					readByte = ftell(fp1);
					if ((datPtr = malloc(fWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtr, fWidth, 1, fp1);
					readByte += fWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtr);
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					zDN = (drtz-i)*zsiz;
					zUP = (drtz-i-1)*zsiz;
					if (phi0 == -180.0) xStartDN = 0;
					else if (phi0 >= 0.0) xStartDN = ipHeader.xpxl;
					else xStartDN = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5-(drtz-i-0.5)*tan0), 0), ipHeader.xpxl);
					xEndDN = phi1 >= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5-(drtz-i-0.5)*tan1)+1, 0), ipHeader.xpxl);
				/* Left, Bottom */
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					xStart = MINEXT(xStartDN, drtxN);
					xEnd = MINEXT(xEndDN, drtxN);
					mThetaDR = datPtrThetaM[0];
					mThetaUR = (*getfuncF)(xRT, zUP);
					if (phi0 < phi1) {
						for (j=0; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xStart; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
						for (j=xEnd; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
					}
					else {
						for (j=0; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
						for (j=xEnd; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xStart; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
					}
				/* Centre, Bottom */
					if (drtxFFlag) {
						datPtrThetaM[j] = mThetaUR;
						if (drtxSwapFlag) {
							drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
							xLT = (drtx-j)*xsiz;
							mThetaUL = (*getfuncF)(xLT, zUP);
							mThetaDL = datPtrThetaM[j+1];
						}
						else {
							drtx = ipHeader.drtx;
							xLT = xRT;
							xRT = (j+1-drtx)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						mThetaDC = mThetaUC;
						mThetaUC = (*getfuncF)(0.0, zUP);
						if (((phi0 < phi1) && (xStartDN <= j) && (xEndDN > j)) || ((phi0 > phi1) && ((xStartDN <= j) || (xEndDN > j)))) pixel2ThetaV(j);
					}
				/* Right, Bottom */
					drtx = ipHeader.xpxl - ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					xStart = MINEXT(ipHeader.xpxl-xStartDN, drtxN);
					xEnd = MINEXT(ipHeader.xpxl-xEndDN, drtxN);
					mThetaDR = datPtrThetaM[ipHeader.xpxl];
					mThetaUR = (*getfuncF)(xRT, zUP);
					if (phi0 < phi1) {
						for (j=0; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xEnd; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xStart; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
					}
					else {
						for (j=0; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xStart; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xEnd; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
					}
					datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaUR;
					free(datPtr);
				}
			}
			else {
				jL1 = (long)floor(ipHeader.drtx);
				jR1 = ipHeader.xpxl-(long)ceil(ipHeader.drtx);
				for (j=jL1-1; j<=jL1+1; j++) datPtrThetaM[j] = (*getfuncF)((ipHeader.drtx-j)*xsiz, ipHeader.drtz*zsiz);
				for (i=0; i<drtzN; i++) {
#if USELINE
					iLines++;
#endif
					if (fseek(fp1, fWidth0, SEEK_CUR)) {
						fprintf(stderr, " Input File Read Error.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					readByte = ftell(fp1);
					if ((datPtr = malloc(fWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtr, fWidth, 1, fp1);
					readByte += fWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtr);
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					zDN = (drtz-i)*zsiz;
					zUP = (drtz-i-1)*zsiz;
					if (phi0 == -180.0) xStartDN = 0;
					else if (phi0 >= 0.0) xStartDN = ipHeader.xpxl;
					else xStartDN = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5-(drtz-i-0.5)*tan0), 0), ipHeader.xpxl);
					xEndDN = phi1 >= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5-(drtz-i-0.5)*tan1)+1, 0), ipHeader.xpxl);
				/* Left, Bottom */
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					jL0 = jL1;
					jL1 = (long)floor(drtx - sqrt(rMax*rMax - zUP*zUP) / xsiz);
					xRT = (drtx-jL1)*xsiz;
					xStart = MAXEXT(MINEXT(xStartDN, drtxN), jL1);
					xEnd = MAXEXT(MINEXT(xEndDN, drtxN), jL1);
					mThetaDR = (jL0-jL1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[jL1];
					mThetaUR = (*getfuncF)(xRT, zUP);
					if (phi0 < phi1) {
						for (j=jL1; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xStart; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
						for (j=xEnd; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
					}
					else {
						for (j=jL1; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
						for (j=xEnd; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xStart; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[j] = mThetaUR;
							mThetaDR = (j<jL0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(j);
						}
					}
				/* Centre, Bottom */
					if (drtxFFlag) {
						datPtrThetaM[j] = mThetaUR;
						if (drtxSwapFlag) {
							drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
							xLT = (drtx-j)*xsiz;
							mThetaUL = (*getfuncF)(xLT, zUP);
							mThetaDL = datPtrThetaM[j+1];
						}
						else {
							drtx = ipHeader.drtx;
							xLT = xRT;
							xRT = (j+1-drtx)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = datPtrThetaM[j+1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						mThetaDC = mThetaUC;
						mThetaUC = (*getfuncF)(0.0, zUP);
						if (((phi0 < phi1) && (xStartDN <= j) && (xEndDN > j)) || ((phi0 > phi1) && ((xStartDN <= j) || (xEndDN > j)))) pixel2ThetaV(j);
					}
				/* Right, Bottom */
					drtx = ipHeader.xpxl - ipHeader.drtx;
					drtxN = (long)floor(drtx);
					jR0 = jR1;
					jR1 = (long)floor(drtx - sqrt(rMax*rMax - zUP*zUP) / xsiz);
					xRT = (drtx-jR1)*xsiz;
					xStart = MAXEXT(MINEXT(ipHeader.xpxl-xStartDN, drtxN), jR1);
					xEnd = MAXEXT(MINEXT(ipHeader.xpxl-xEndDN, drtxN), jR1);
					mThetaDR = (jR0-jR1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-jR1];
					mThetaUR = (*getfuncF)(xRT, zUP);
					if (phi0 < phi1) {
						for (j=jR1; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xEnd; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xStart; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
					}
					else {
						for (j=jR1; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xStart; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
						}
						for (j=xEnd; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = mThetaDR;
							mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
							mThetaDR = (j<jR0-1) ? (*getfuncF)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
							mThetaUR = (*getfuncF)(xRT, zUP);
							pixel2Theta(ipHeader.xpxl-j-1);
						}
					}
					datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaUR;
					free(datPtr);
				}
			}
		}
		/* Start Centre Line */
#if USELINE
		iLines++;
#endif
		if (drtzFFlag) {
			if (fseek(fp1, fWidth0, SEEK_CUR)) {
				fprintf(stderr, " Input File Read Error.\n");
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			if ((datPtr = malloc(fWidth)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtr, fWidth, 1, fp1);
			readByte += fWidth;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			drtz = drtzSwapFlag ? 2.0*floor(ipHeader.drtz) + 1.0 - ipHeader.drtz : ipHeader.drtz;
			zDN = (drtz-i)*zsiz;
			zUP = (i+1-drtz)*zsiz;
		/* Left, Centre */
			if (useSymRegionFlag) {
				if (phi0tmp > -90.0 && phi0tmp < 0.0) phi0 = phi0tmp + 180.0;
				else if (phi0tmp >= 0.0 && phi0tmp < 90.0) phi0 = phi0tmp - 180.0;
				else phi0 = phi0tmp;
				if (phi1tmp > -90.0 && phi1tmp <= 0.0) phi1 = phi1tmp + 180.0;
				else if (phi1tmp > 0.0 && phi1tmp < 90.0) phi1 = phi1tmp - 180.0;
				else phi1 = phi1tmp;
			}
			drtx = ipHeader.drtx;
			drtxN = (long)floor(drtx);
			xRT = drtx*xsiz;
			mThetaDR = (*getfuncF)(xRT, zDN);
			mThetaUR = (*getfuncF)(xRT, zUP);
			mThetaCR = (*getfuncF)(xRT, 0.0);
			if (drtzSwapFlag) {
				if (phi0 == -180.0) xStartDN = 0;
				else if (phi0 >= 0.0) xStartDN = ipHeader.xpxl;
				else xStartDN = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5+(ipHeader.drtz-i-0.5)*tan0), 0), ipHeader.xpxl);
				xEndDN = phi1 >= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5+(ipHeader.drtz-i-0.5)*tan1)+1, 0), ipHeader.xpxl);
				xStart = MINEXT(xStartDN, drtxN);
				xEnd = MINEXT(xEndDN, drtxN);
				if (phi0 < phi1) {
					for (j=0; j<xStart; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xStart; j<xEnd; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
					for (j=xEnd; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<xEnd; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
					for (j=xEnd; j<xStart; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xStart; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
				}
			}
			else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
				if ((phi0 < phi1) && (phi0 != -180.0)) {
					for (j=0; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[j+1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
				}
			}
			else {
				xStartUP = phi0 <= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5-(ipHeader.drtz-i-0.5)*tan0)+1, 0), ipHeader.xpxl);
				if (phi1 == 180.0) xEndUP = 0;
				else if (phi1 <= 0.0) xEndUP = ipHeader.xpxl;
				else xEndUP = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5-(ipHeader.drtz-i-0.5)*tan1), 0), ipHeader.xpxl);
				xStart = MINEXT(xStartUP, drtxN);
				xEnd = MINEXT(xEndUP, drtxN);
				if (phi0 < phi1) {
					for (j=0; j<xEnd; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xEnd; j<xStart; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
					for (j=xStart; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<xStart; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
					for (j=xStart; j<xEnd; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xEnd; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(j);
					}
				}
			}
		/* Centre, Centre(DIRECT BEAM) */
			if (drtxFFlag) {
				datPtrThetaM[j] = drtzSwapFlag ? mThetaDR : mThetaUR;
				phiC = (((ipHeader.drtx-floor(ipHeader.drtx) != 0.5) || (ipHeader.drtz-floor(ipHeader.drtz) != 0.5))) ?
				    atan2((i+0.5-ipHeader.drtz)*zsiz, (j+0.5-ipHeader.drtx)*xsiz)*RAD_DEG : -360.0;
				phiC2 = phiC > 0 ? phiC - 180.0 : phiC + 180.0;
#ifdef OMMODE
				if (!omitFlags((long)floor(ipHeader.drtx))) {
#endif
				if ((phiC < -200.0)
				 || ((phi0 > phi1) && ((phiC <= phi1) || (phiC >= phi0))) || ((phi0 < phi1) && (phiC <= phi1) && (phiC >= phi0))
				 || ((phi0 > phi1) && ((phiC2 <= phi1) || (phiC2 >= phi0))) || ((phi0 < phi1) && (phiC2 <= phi1) && (phiC2 >= phi0))) {
					drtx = drtxSwapFlag ? (2.0*floor(ipHeader.drtx)+1.0-ipHeader.drtx) : ipHeader.drtx;
					xRT = (j+1-drtx)*xsiz;
					xLT = (drtx-j)*xsiz;
					if ((*getfuncF)(xRT, zUP)) {
						r1 = datPtrDblR[1];
						xU = xRT - xSqrt(r1*r1 - zUP*zUP);
						zR = zUP - xSqrt(r1*r1 - xRT*xRT);
						tmpArea0 = 1.0 - (xU*zR/2.0 - dArea(r1, dist(xU, zR))) / area;
						if ((*getfuncF)(xLT, zUP)) {
							xD = xRT - xSqrt(r1*r1 - zDN*zDN);
							zR += 2.0 * zDN - zsiz;
							tmpArea0 -= (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
						}
						else if ((*getfuncF)(xRT, zDN)) {
							xU += 2.0 * xLT - xsiz;
							zL = zUP - xSqrt(r1*r1 - xLT*xLT);
							tmpArea0 -= (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
						}
						datPtrDblInt[0] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
						datPtrDblArea[0] += tmpArea0;
						datPtrPxlNum[0]++;
						datPtrDblInt[1] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
						datPtrDblArea[1] += 1.0 - tmpArea0;
						datPtrPxlNum[1]++;
					}
					else {
						datPtrDblInt[0] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
						datPtrDblArea[0] += 1.0;
						datPtrPxlNum[0]++;
					}
				}
#ifdef OMMODE
				}
#endif
			}
		/* Right, Centre */
			if (useSymRegionFlag) {
				if (phi0tmp < -90.0) phi0 = phi0tmp + 180.0;
				else if (phi0tmp > 90.0) phi0 = phi0tmp - 180.0;
				else phi0 = phi0tmp;
				if (phi1tmp < -90.0) phi1 = phi1tmp + 180.0;
				else if (phi1tmp > 90.0) phi1 = phi1tmp - 180.0;
				else phi1 = phi1tmp;
			}
			drtx = ipHeader.xpxl - ipHeader.drtx;
			drtxN = (long)floor(drtx);
			xRT = drtx*xsiz;
			mThetaDR = (*getfuncF)(xRT, zDN);
			mThetaUR = (*getfuncF)(xRT, zUP);
			mThetaCR = (*getfuncF)(xRT, 0.0);
			if (drtzSwapFlag) {
				if (phi0 == -180.0) xStartDN = 0;
				else if (phi0 >= 0.0) xStartDN = ipHeader.xpxl;
				else xStartDN = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5+(ipHeader.drtz-i-0.5)*tan0), 0), ipHeader.xpxl);
				xEndDN = phi1 >= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5+(ipHeader.drtz-i-0.5)*tan1)+1, 0), ipHeader.xpxl);
				xStart = MINEXT(ipHeader.xpxl-xStartDN, drtxN);
				xEnd = MINEXT(ipHeader.xpxl-xEndDN, drtxN);
				if (phi0 < phi1) {
					for (j=0; j<xEnd; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xEnd; j<xStart; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
					for (j=xStart; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<xStart; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
					for (j=xStart; j<xEnd; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xEnd; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncF)(xRT, zDN);
						mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
				}
			}
			else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
				if (((phi0 < phi1) && (phi0*phi1 > 0.0)) || ((phi0 > phi1) && (phi0*phi1 < 0.0))) {
					for (j=0; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
				}
			}
			else {
				xStartUP = phi0 <= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5-(ipHeader.drtz-i-0.5)*tan0)+1, 0), ipHeader.xpxl);
				if (phi1 == 180.0) xEndUP = 0;
				else if (phi1 <= 0.0) xEndUP = ipHeader.xpxl;
				else xEndUP = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5-(ipHeader.drtz-i-0.5)*tan1), 0), ipHeader.xpxl);
				xStart = MINEXT(ipHeader.xpxl-xStartUP, drtxN);
				xEnd = MINEXT(ipHeader.xpxl-xEndUP, drtxN);
				if (phi0 < phi1) {
					for (j=0; j<xStart; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xStart; j<xEnd; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
					for (j=xEnd; j<drtxN; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
				}
				else {
					for (j=0; j<xEnd; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
					for (j=xEnd; j<xStart; j++) {
						xRT = (drtx-j-1)*xsiz;
						datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCR = (*getfuncF)(xRT, 0.0);
					}
					for (j=xStart; j<drtxN; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
						mThetaDR = datPtrThetaM[ipHeader.xpxl-j-1];
						mThetaUR = (*getfuncF)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncF)(xRT, 0.0);
						pixel2ThetaH(ipHeader.xpxl-j-1);
					}
				}
			}
			datPtrThetaM[(long)ceil(ipHeader.drtx)] = drtzSwapFlag ? mThetaDR : mThetaUR;
			free(datPtr);
		}
		/* Start top Loop */
		if (!ignoreUP || (phi0 >= phi1) || (phi1 > 0.0) || useSymRegionFlag) {
			if (useSymRegionFlag) {
				phi0 = phi0tmp < 0.0 ? phi0tmp + 180.0 : phi0tmp;
				phi1 = phi1tmp <= 0.0 ? phi1tmp + 180.0 : phi1tmp;
			}
			drtz = ipHeader.zpxl - ipHeader.drtz;
			drtzN = (long)floor(drtz);
			mThetaDC = 0;
			if (useMaxFlag) {
				for (i=drtzN-1; i>=0; i--) {
#if USELINE
					iLines++;
#endif
					if (fseek(fp1, fWidth0, SEEK_CUR)) {
						fprintf(stderr, " Input File Read Error.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					readByte = ftell(fp1);
					if ((datPtr = malloc(fWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtr, fWidth, 1, fp1);
					readByte += fWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtr);
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					zDN = (drtz-i)*zsiz;
					zUP = (drtz-i-1)*zsiz;
					xStartUP = phi0 <= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5+(drtz-i-0.5)*tan0)+1, 0), ipHeader.xpxl);
					if (phi1 == 180.0) xEndUP = 0;
					else if (phi1 <= 0.0) xEndUP = ipHeader.xpxl;
					else xEndUP = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5+(drtz-i-0.5)*tan1), 0), ipHeader.xpxl);
				/* Left, Top */
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					xStart = MINEXT(xStartUP, drtxN);
					xEnd = MINEXT(xEndUP, drtxN);
					mThetaDR = (*getfuncF)(xRT, zDN);
					mThetaUR = datPtrThetaM[0];
					if (phi0 < phi1) {
						for (j=0; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
						for (j=xEnd; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
						for (j=xStart; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
					}
					else {
						for (j=0; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
						for (j=xStart; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
						for (j=xEnd; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
					}
				/* Centre, Top */
					if (drtxFFlag) {
						datPtrThetaM[j] = mThetaDR;
						if (drtxSwapFlag) {
							drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
							xLT = (drtx-j)*xsiz;
							mThetaUL = datPtrThetaM[j+1];
							mThetaDL = (*getfuncF)(xLT, zDN);
						}
						else {
							drtx = ipHeader.drtx;
							xLT = xRT;
							xRT = (j+1-drtx)*xsiz;
							mThetaDL = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUL = mThetaUR;
							mThetaUR = datPtrThetaM[j+1];
						}
						mThetaUC = mThetaDC;
						mThetaDC = (*getfuncF)(0.0, zDN);
						if (((phi0 < phi1) && (xEndUP <= j) && (xStartUP > j)) || ((phi0 > phi1) && ((xEndUP <= j) || (xStartUP > j)))) pixel2ThetaV(j);
					}
				/* Right, Top */
					drtx = ipHeader.xpxl - ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					xStart = MINEXT(ipHeader.xpxl-xStartUP, drtxN);
					xEnd = MINEXT(ipHeader.xpxl-xEndUP, drtxN);
					mThetaDR = (*getfuncF)(xRT, zDN);
					mThetaUR = datPtrThetaM[ipHeader.xpxl];
					if (phi0 < phi1) {
						for (j=0; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
						for (j=xStart; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xEnd; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
					}
					else {
						for (j=0; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xEnd; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
						for (j=xStart; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
					}
					datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaDR;
					free(datPtr);
				}
			}
			else {
				for (i=drtzN-1; i>=0; i--) {
#if USELINE
					iLines++;
#endif
					if (fseek(fp1, fWidth0, SEEK_CUR)) {
						fprintf(stderr, " Input File Read Error.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					readByte = ftell(fp1);
					if ((datPtr = malloc(fWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtr, fWidth, 1, fp1);
					readByte += fWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtr);
						free(datPtrDblInt);
						free(datPtrDblArea);
						free(datPtrPxlNum);
						free(datPtrThetaM);
						free(datPtrDblR);
						fclose(fp1);
						if (ofileDoneFlag) {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					zDN = (drtz-i)*zsiz;
					zUP = (drtz-i-1)*zsiz;
					xStartUP = phi0 <= 0.0 ? ipHeader.xpxl : MINEXT(MAXEXT((long)floor(ipHeader.drtx-0.5+(drtz-i-0.5)*tan0)+1, 0), ipHeader.xpxl);
					if (phi1 == 180.0) xEndUP = 0;
					else if (phi1 <= 0.0) xEndUP = ipHeader.xpxl;
					else xEndUP = MINEXT(MAXEXT((long)ceil(ipHeader.drtx-0.5+(drtz-i-0.5)*tan1), 0), ipHeader.xpxl);
				/* Left, Top */
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					jL1 = (long)floor(drtx - sqrt(rMax*rMax - zUP*zUP) / xsiz);
					xRT = (drtx-jL1)*xsiz;
					xStart = MAXEXT(MINEXT(xStartUP, drtxN), jL1);
					xEnd = MAXEXT(MINEXT(xEndUP, drtxN), jL1);
					mThetaDR = (*getfuncF)(xRT, zDN);
					mThetaUR = datPtrThetaM[jL1];
					if (phi0 < phi1) {
						for (j=jL1; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
						for (j=xEnd; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
						for (j=xStart; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
					}
					else {
						for (j=jL1; j<xStart; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
						for (j=xStart; j<xEnd; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
						}
						for (j=xEnd; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[j+1];
							pixel2Theta(j);
						}
					}
				/* Centre, Top */
					if (drtxFFlag) {
						datPtrThetaM[j] = mThetaDR;
						if (drtxSwapFlag) {
							drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
							xLT = (drtx-j)*xsiz;
							mThetaUL = datPtrThetaM[j+1];
							mThetaDL = (*getfuncF)(xLT, zDN);
						}
						else {
							drtx = ipHeader.drtx;
							xLT = xRT;
							xRT = (j+1-drtx)*xsiz;
							mThetaDL = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUL = mThetaUR;
							mThetaUR = datPtrThetaM[j+1];
						}
						mThetaUC = mThetaDC;
						mThetaDC = (*getfuncF)(0.0, zDN);
						if (((phi0 < phi1) && (xEndUP <= j) && (xStartUP > j)) || ((phi0 > phi1) && ((xEndUP <= j) || (xStartUP > j)))) pixel2ThetaV(j);
					}
				/* Right, Top */
					drtx = ipHeader.xpxl - ipHeader.drtx;
					drtxN = (long)floor(drtx);
					jR1 = (long)floor(drtx - sqrt(rMax*rMax - zUP*zUP) / xsiz);
					xRT = (drtx-jR1)*xsiz;
					xStart = MAXEXT(MINEXT(ipHeader.xpxl-xStartUP, drtxN), jR1);
					xEnd = MAXEXT(MINEXT(ipHeader.xpxl-xEndUP, drtxN), jR1);
					mThetaDR = (*getfuncF)(xRT, zDN);
					mThetaUR = datPtrThetaM[ipHeader.xpxl-jR1];
					if (phi0 < phi1) {
						for (j=jR1; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
						for (j=xStart; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xEnd; j<drtxN; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
					}
					else {
						for (j=jR1; j<xEnd; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
						for (j=xEnd; j<xStart; j++) {
							xRT = (drtx-j-1)*xsiz;
							datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
						}
						for (j=xStart; j<drtxN; j++) {
							xLT = xRT;
							xRT = (drtx-j-1)*xsiz;
							mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
							mThetaUL = mThetaUR;
							mThetaDR = (*getfuncF)(xRT, zDN);
							mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
							pixel2Theta(ipHeader.xpxl-j-1);
						}
					}
					datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaDR;
					free(datPtr);
				}
			}
		}
		free(datPtrThetaM);
	}
	else { /*----CURVED CAMERA----*/
	/* Start bottom Loop */
		drtz = ipHeader.drtz;
		drtzN = (long)floor(drtz);
		mThetaUC = (*getfuncF)(0.0, drtz*zsiz);
		if (useMaxFlag) {
			for (j=0; j<=ipHeader.xpxl; j++) datPtrThetaM[j] = (*getfuncC)((ipHeader.drtx-j)*xsiz, drtz*zsiz);
			for (i=0; i<(long)floor(drtz); i++) {
#if USELINE
				iLines++;
#endif
				if (fseek(fp1, fWidth0, SEEK_CUR)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr = malloc(fWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, fWidth, 1, fp1);
				readByte += fWidth;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				zDN = (drtz-i)*zsiz;
				zUP = (drtz-i-1)*zsiz;
				areaFactor = sqrt(1.0 + zDN*zUP/camr/camr) * (1.0 + zDN*zUP/camr/camr);
			/* -Back, Bottom */
				if (edgemIntLoop > 0) {
					drtx = edgemIntLoop - drtxm;
					xRT = drtx*xsiz;
					mThetaDR = tmpBuf = datPtrThetaM[edgemIntLoop];
					mThetaUR = (*getfuncCB)(xRT, zUP);
					for (j=0; j<edgemIntLoop; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[edgemIntLoop-j] = mThetaUR;
						mThetaDR = datPtrThetaM[edgemIntLoop-j-1];
						mThetaUR = (*getfuncCB)(xRT, zUP);
						pixel2ThetaCB(edgemIntLoop-j-1);
					}
					datPtrThetaM[0] = mThetaUR;
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = (drtx - edgemIntLoop)*xsiz;
					mThetaDR = tmpBuf;
					mThetaUR = datPtrThetaM[edgemIntLoop];
				}
				else {
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					mThetaDR = datPtrThetaM[0];
					mThetaUR = (*getfuncC)(xRT, zUP);
				}
			/* Left, Bottom */
				for (j=startLeft; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[j] = mThetaUR;
					mThetaDR = datPtrThetaM[j+1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					pixel2ThetaC(j);
				}
			/* Centre, Bottom */
				if (drtxFFlag) {
					datPtrThetaM[j] = mThetaUR;
					if (drtxSwapFlag) {
						drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
						xLT = (drtx-j)*xsiz;
						mThetaUL = (*getfuncC)(xLT, zUP);
						mThetaDL = datPtrThetaM[j+1];
					}
					else {
						drtx = ipHeader.drtx;
						xLT = xRT;
						xRT = (j+1-drtx)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncC)(xRT, zUP);
					}
					mThetaDC = mThetaUC;
					mThetaUC = (*getfuncF)(0.0, zUP);
					pixel2ThetaVC(j);
				}
			/* Right, Bottom */
				drtx = startRight - ipHeader.drtx;
				drtxN = (long)floor(drtx);
				xRT = drtx*xsiz;
				mThetaDR = tmpBuf = datPtrThetaM[startRight];
				mThetaUR = (*getfuncC)(xRT, zUP);
				for (j=0; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[startRight-j] = mThetaUR;
					mThetaDR = datPtrThetaM[startRight-j-1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					pixel2ThetaC(startRight-j-1);
				}
				datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaUR;
			/* +Back, Bottom */
				if (edgepIntLoop <= ipHeader.xpxl-1) {
					drtx = drtxp;
					xRT = (drtx - edgepIntLoop)*xsiz;
					mThetaDR = tmpBuf;
					mThetaUR = datPtrThetaM[edgepIntLoop];
					for (j=edgepIntLoop; j<ipHeader.xpxl; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncCB)(xRT, zUP);
						pixel2ThetaCB(j);
					}
					datPtrThetaM[ipHeader.xpxl] = mThetaUR;
				}
				free(datPtr);
			}
		}
		else {
			jL1 = (long)floor(ipHeader.drtx);
			jR1 = ipHeader.xpxl-(long)ceil(ipHeader.drtx);
			for (j=jL1-1; j<=jL1+1; j++) datPtrThetaM[j] = (*getfuncC)((ipHeader.drtx-j)*xsiz, ipHeader.drtz*zsiz);
			for (i=0; i<drtzN; i++) {
#if USELINE
				iLines++;
#endif
				if (fseek(fp1, fWidth0, SEEK_CUR)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr = malloc(fWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, fWidth, 1, fp1);
				readByte += fWidth;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				zDN = (drtz-i)*zsiz;
				zUP = (drtz-i-1)*zsiz;
				areaFactor = sqrt(1.0 + zDN*zUP/camr/camr) * (1.0 + zDN*zUP/camr/camr);
			/* Left, Bottom */
				drtx = ipHeader.drtx;
				drtxN = (long)floor(drtx);
				jL0 = jL1;
				jL1 = (long)floor(drtx - camr * acos(dist(1.0, zUP/camr) * cosMax) / xsiz);
				xRT = (drtx-jL1)*xsiz;
				mThetaDR = (jL0-jL1) ? (*getfuncC)(xRT, zDN) : datPtrThetaM[jL1];
				mThetaUR = (*getfuncC)(xRT, zUP);
				for (j=jL1; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[j] = mThetaUR;
					mThetaDR = (j<jL0-1) ? (*getfuncC)(xRT, zDN) : datPtrThetaM[j+1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					pixel2ThetaC(j);
				}
			/* Centre, Bottom */
				if (drtxFFlag) {
					datPtrThetaM[j] = mThetaUR;
					if (drtxSwapFlag) {
						drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
						xLT = (drtx-j)*xsiz;
						mThetaUL = (*getfuncC)(xLT, zUP);
						mThetaDL = datPtrThetaM[j+1];
					}
					else {
						drtx = ipHeader.drtx;
						xLT = xRT;
						xRT = (j+1-drtx)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncC)(xRT, zUP);
					}
					mThetaDC = mThetaUC;
					mThetaUC = (*getfuncF)(0.0, zUP);
					pixel2ThetaVC(j);
				}
			/* Right, Bottom */
				drtx = ipHeader.xpxl - ipHeader.drtx;
				drtxN = (long)floor(drtx);
				jR0 = jR1;
				jR1 = (long)floor(drtx - camr * acos(dist(1.0, zUP/camr) * cosMax) / xsiz);
				xRT = (drtx-jR1)*xsiz;
				mThetaDR = (jR0-jR1) ? (*getfuncC)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-jR1];
				mThetaUR = (*getfuncC)(xRT, zUP);
				for (j=jR1; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[ipHeader.xpxl-j] = mThetaUR;
					mThetaDR = (j<jR0-1) ? (*getfuncC)(xRT, zDN) : datPtrThetaM[ipHeader.xpxl-j-1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					pixel2ThetaC(ipHeader.xpxl-j-1);
				}
				datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaUR;
				free(datPtr);
			}
		}
		/* Start Centre Line */
		if (drtzFFlag) {
#if USELINE
		iLines++;
#endif
			if (fseek(fp1, fWidth0, SEEK_CUR)) {
				fprintf(stderr, " Input File Read Error.\n");
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			if ((datPtr = malloc(fWidth)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtr, fWidth, 1, fp1);
			readByte += fWidth;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				free(datPtrDblInt);
				free(datPtrDblArea);
				free(datPtrPxlNum);
				free(datPtrThetaM);
				free(datPtrDblR);
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			drtz = drtzSwapFlag ? 2.0*floor(ipHeader.drtz)+1.0-ipHeader.drtz : ipHeader.drtz;
			zDN = (drtz-i)*zsiz;
			zUP = (i+1-drtz)*zsiz;
			areaFactor = 1.0;
		/* -Back, Centre */
			if (edgemIntLoop > 0) {
				drtx = edgemIntLoop - drtxm;
				drtxN = (long)floor(drtx);
				xRT = drtx*xsiz;
				mThetaCR = tmpBufC = (*getfuncCB)(xRT, 0.0);
				if (drtzSwapFlag) {
					mThetaUR = tmpBuf = datPtrThetaM[edgemIntLoop];
					mThetaDR = (*getfuncCB)(xRT, zDN);
					for (j=0; j<edgemIntLoop; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[edgemIntLoop-j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncCB)(xRT, zDN);
						mThetaUR = datPtrThetaM[edgemIntLoop-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(edgemIntLoop-j-1);
					}
					datPtrThetaM[0] = mThetaDR;
					mThetaDR = tmpBuf;
					mThetaUR = datPtrThetaM[edgemIntLoop];
				}
				else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
					mThetaDR = mThetaUR = datPtrThetaM[edgemIntLoop];
					for (j=0; j<edgemIntLoop; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaUL = datPtrThetaM[edgemIntLoop-j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[edgemIntLoop-j-1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(edgemIntLoop-j-1);
					}
					datPtrThetaM[0] = mThetaUR;
					mThetaDR = mThetaUR = datPtrThetaM[edgemIntLoop];
				}
				else {
					mThetaDR = tmpBuf = datPtrThetaM[edgemIntLoop];
					mThetaUR = (*getfuncCB)(xRT, zUP);
					for (j=0; j<edgemIntLoop; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[edgemIntLoop-j] = mThetaUR;
						mThetaDR = datPtrThetaM[edgemIntLoop-j-1];
						mThetaUR = (*getfuncCB)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(edgemIntLoop-j-1);
					}
					datPtrThetaM[0] = mThetaUR;
					mThetaDR = tmpBuf;
					mThetaUR = datPtrThetaM[edgemIntLoop];
				}
				drtx = ipHeader.drtx;
				xRT = (drtx - edgemIntLoop)*xsiz;
				mThetaCR = tmpBufC;
			}
			else {
				drtx = ipHeader.drtx;
				drtxN = (long)floor(drtx);
				xRT = drtx*xsiz;
				if (drtzSwapFlag) {
					mThetaUR = datPtrThetaM[0];
					mThetaDR = (*getfuncC)(xRT, zDN);
				}
				else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
					mThetaUR = mThetaDR = (*getfuncC)(xRT, zDN);
				}
				else {
					mThetaDR = datPtrThetaM[0];
					mThetaUR = (*getfuncC)(xRT, zUP);
				}
				mThetaCR = (*getfuncC)(xRT, 0.0);
			}
		/* Left, Centre */
			if (drtzSwapFlag) {
				for (j=startLeft; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = datPtrThetaM[j] = mThetaDR;
					mThetaUL = mThetaUR;
					mThetaDR = (*getfuncC)(xRT, zDN);
					mThetaUR = datPtrThetaM[j+1];
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(j);
				}
			}
			else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
				for (j=startLeft; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaUL = datPtrThetaM[j] = mThetaUR;
					mThetaDR = mThetaUR = datPtrThetaM[j+1];
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(j);
				}
			}
			else {
				for (j=startLeft; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[j] = mThetaUR;
					mThetaDR = datPtrThetaM[j+1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(j);
				}
			}
		/* Centre, Centre(DIRECT BEAM) */
			if (drtxFFlag) {
				datPtrThetaM[j] = drtzSwapFlag ? mThetaDR : mThetaUR;
				drtx = drtxSwapFlag ? (2.0*floor(ipHeader.drtx)+1.0-ipHeader.drtx) : ipHeader.drtx;
				xRT = (j+1-drtx)*xsiz;
				xLT = (drtx-j)*xsiz;
#ifdef OMMODE
				if (!omitFlags((long)floor(ipHeader.drtx))) {
#endif
				if ((*getfuncC)(xRT, zUP)) {
					cosTheta = qFlag ? 1.0-step*step*lam2 : cos(step);
					cosTheta2 = cosTheta * cosTheta;
					cosX = cos(xRT / camr);
					xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) - xRT;
					zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
					numer = cos((xRT + xU / 2.0) / camr); /* Approximation for curvature */
					cosX2 = cosX * cosX;
					numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
					r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 - 2.0 * cosX2 * cosTheta2 + cosTheta2);
					tmpArea0 = 1.0 - (xU*zR/2.0 - dArea(r1, dist(xU, zR))) / area;
					if ((*getfuncC)(xLT, zUP)) {
						xD = camr * acos(dist(1.0, zDN/camr) * cosTheta) - xRT;
						zR += 2.0 * zDN - zsiz;
						numer = cos((xRT + xD / 2.0) / camr); /* Approximation for curvature */
						cosX2 = cosX * cosX;
						numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
						r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 - 2.0 * cosX2 * cosTheta2 + cosTheta2); /* IF cosTheta==0.0, FPE might be caused !! */
						tmpArea0 -= (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
					}
					else if ((*getfuncC)(xRT, zDN)) {
						cosX = cos(xLT / camr);
						xU += 2.0 * xLT - xsiz;
						zL = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
						numer = cos((xLT + xU / 2.0) / camr); /* Approximation for curvature */
						cosX2 = cosX * cosX;
						numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
						r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 - 2.0 * cosX2 * cosTheta2 + cosTheta2); /* IF cosTheta==0.0, FPE might be caused !! */
						tmpArea0 -= (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
					}
					datPtrDblInt[0] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
					datPtrDblArea[0] += tmpArea0;
					datPtrDblR[0] += tmpArea0;
					datPtrPxlNum[0]++;
					datPtrDblInt[1] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
					datPtrDblArea[1] += 1.0 - tmpArea0;
					datPtrDblR[1] += 1.0 - tmpArea0;
					datPtrPxlNum[1]++;
				}
				else {
					datPtrDblInt[0] += (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					datPtrDblArea[0] += 1.0;
					datPtrDblR[0] += 1.0;
					datPtrPxlNum[0]++;
				}
#ifdef OMMODE
				}
#endif
			}
		/* Right, Centre */
			drtx = startRight - ipHeader.drtx;
			drtxN = (long)floor(drtx);
			xRT = drtx*xsiz;
			mThetaCR = tmpBufC = (*getfuncC)(xRT, 0.0);
			if (drtzSwapFlag) {
				mThetaUR = tmpBuf = datPtrThetaM[startRight];
				mThetaDR = (*getfuncC)(xRT, zDN);
				for (j=0; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = datPtrThetaM[startRight-j] = mThetaDR;
					mThetaUL = mThetaUR;
					mThetaDR = (*getfuncC)(xRT, zDN);
					mThetaUR = datPtrThetaM[startRight-j-1];
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(startRight-j-1);
				}
			}
			else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
				mThetaDR = mThetaUR = datPtrThetaM[startRight];
				for (j=0; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaUL = datPtrThetaM[startRight-j] = mThetaUR;
					mThetaDR = mThetaUR = datPtrThetaM[startRight-j-1];
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(startRight-j-1);
				}
			}
			else {
				mThetaDR = tmpBuf = datPtrThetaM[startRight];
				mThetaUR = (*getfuncC)(xRT, zUP);
				for (j=0; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = mThetaDR;
					mThetaUL = datPtrThetaM[startRight-j] = mThetaUR;
					mThetaDR = datPtrThetaM[startRight-j-1];
					mThetaUR = (*getfuncC)(xRT, zUP);
					mThetaCL = mThetaCR;
					mThetaCR = (*getfuncC)(xRT, 0.0);
					pixel2ThetaHC(startRight-j-1);
				}
			}
			datPtrThetaM[(long)ceil(ipHeader.drtx)] = drtzSwapFlag ? mThetaDR : mThetaUR;
		/* +Back, Centre */
			if (edgepIntLoop <= ipHeader.xpxl-1) {
				drtx = drtxp;
				xRT = (drtx - edgepIntLoop)*xsiz;
				mThetaCR = tmpBufC;
				if (drtzSwapFlag) {
					mThetaUR = tmpBuf;
					mThetaDR = datPtrThetaM[edgepIntLoop];
					for (j=edgepIntLoop; j<ipHeader.xpxl; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = datPtrThetaM[j] = mThetaDR;
						mThetaUL = mThetaUR;
						mThetaDR = (*getfuncCB)(xRT, zDN);
						mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(j);
					}
					datPtrThetaM[ipHeader.xpxl] = mThetaDR;
				}
				else if (ipHeader.drtz-floor(ipHeader.drtz) == 0.5) {
					mThetaUR = mThetaDR = datPtrThetaM[edgepIntLoop];
					for (j=edgepIntLoop; j<ipHeader.xpxl; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = mThetaUR = datPtrThetaM[j+1];
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(j);
					}
					datPtrThetaM[ipHeader.xpxl] = mThetaUR;
				}
				else {
					mThetaDR = tmpBuf;
					mThetaUR = datPtrThetaM[edgepIntLoop];
					for (j=edgepIntLoop; j<ipHeader.xpxl; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaDL = mThetaDR;
						mThetaUL = datPtrThetaM[j] = mThetaUR;
						mThetaDR = datPtrThetaM[j+1];
						mThetaUR = (*getfuncCB)(xRT, zUP);
						mThetaCL = mThetaCR;
						mThetaCR = (*getfuncCB)(xRT, 0.0);
						pixel2ThetaHCB(j);
					}
					datPtrThetaM[ipHeader.xpxl] = mThetaUR;
				}
			}
			free(datPtr);
		}
		/* Start top Loop */
		drtz = ipHeader.zpxl - ipHeader.drtz;
		drtzN = (long)floor(drtz);
		mThetaDC = 0;
		if (useMaxFlag) {
			for (i=drtzN-1; i>=0; i--) {
#if USELINE
				iLines++;
#endif
				if (fseek(fp1, fWidth0, SEEK_CUR)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr = malloc(fWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, fWidth, 1, fp1);
				readByte += fWidth;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				zDN = (drtz-i)*zsiz;
				zUP = (drtz-i-1)*zsiz;
				areaFactor = sqrt(1.0 + zDN*zUP/camr/camr) * (1.0 + zDN*zUP/camr/camr);
			/* -Back, Top */
				if (edgemIntLoop > 0) {
					drtx = edgemIntLoop - drtxm;
					xRT = drtx*xsiz;
					mThetaDR = (*getfuncCB)(xRT, zDN);
					mThetaUR = tmpBuf = datPtrThetaM[edgemIntLoop];
					for (j=0; j<edgemIntLoop; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaUL = mThetaUR;
						mThetaDL = datPtrThetaM[edgemIntLoop-j] = mThetaDR;
						mThetaUR = datPtrThetaM[edgemIntLoop-j-1];
						mThetaDR = (*getfuncCB)(xRT, zDN);
						pixel2ThetaCB(edgemIntLoop-j-1);
					}
					datPtrThetaM[0] = mThetaDR;
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = (drtx - edgemIntLoop)*xsiz;
					mThetaUR = tmpBuf;
					mThetaDR = datPtrThetaM[edgemIntLoop];
				}
				else {
					drtx = ipHeader.drtx;
					drtxN = (long)floor(drtx);
					xRT = drtx*xsiz;
					mThetaUR = datPtrThetaM[0];
					mThetaDR = (*getfuncC)(xRT, zDN);
				}
			/* Left, Top */
				for (j=startLeft; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = datPtrThetaM[j] = mThetaDR;
					mThetaUL = mThetaUR;
					mThetaDR = (*getfuncC)(xRT, zDN);
					mThetaUR = datPtrThetaM[j+1];
					pixel2ThetaC(j);
				}
			/* Centre, Top */
				if (drtxFFlag) {
					datPtrThetaM[j] = mThetaDR;
					if (drtxSwapFlag) {
						drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
						xLT = (drtx-j)*xsiz;
						mThetaUL = datPtrThetaM[j+1];
						mThetaDL = (*getfuncC)(xLT, zDN);
					}
					else {
						drtx = ipHeader.drtx;
						xLT = xRT;
						xRT = (j+1-drtx)*xsiz;
						mThetaDL = mThetaDR;
						mThetaDR = (*getfuncC)(xRT, zDN);
						mThetaUL = mThetaUR;
						mThetaUR = datPtrThetaM[j+1];
					}
					mThetaUC = mThetaDC;
					mThetaDC = (*getfuncF)(0.0, zDN); /* also valid for curved camera*/
					pixel2ThetaVC(j);
				}
			/* Right, Top */
				drtx = startRight - ipHeader.drtx;
				drtxN = (long)floor(drtx);
				xRT = drtx*xsiz;
				mThetaDR = (*getfuncC)(xRT, zDN);
				mThetaUR = tmpBuf = datPtrThetaM[startRight];
				for (j=0; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaUL = mThetaUR;
					mThetaDL = datPtrThetaM[startRight-j] = mThetaDR;
					mThetaUR = datPtrThetaM[startRight-j-1];
					mThetaDR = (*getfuncC)(xRT, zDN);
					pixel2ThetaC(startRight-j-1);
				}
				datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaDR;
			/* +Back, Top */
				if (edgepIntLoop <= ipHeader.xpxl-1) {
					drtx = drtxp;
					xRT = (drtx - edgepIntLoop)*xsiz;
					mThetaUR = tmpBuf;
					mThetaDR = datPtrThetaM[edgepIntLoop];
					for (j=edgepIntLoop; j<ipHeader.xpxl; j++) {
						xLT = xRT;
						xRT = (drtx-j-1)*xsiz;
						mThetaUL = mThetaUR;
						mThetaDL = datPtrThetaM[j] = mThetaDR;
						mThetaUR = datPtrThetaM[j+1];
						mThetaDR = (*getfuncCB)(xRT, zDN);
						pixel2ThetaCB(j);
					}
					datPtrThetaM[ipHeader.xpxl] = mThetaDR;
				}
				free(datPtr);
			}
		}
		else {
			for (i=drtzN-1; i>=0; i--) {
#if USELINE
				iLines++;
#endif
				if (fseek(fp1, fWidth0, SEEK_CUR)) {
					fprintf(stderr, " Input File Read Error.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				readByte = ftell(fp1);
				if ((datPtr = malloc(fWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, fWidth, 1, fp1);
				readByte += fWidth;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrDblInt);
					free(datPtrDblArea);
					free(datPtrPxlNum);
					free(datPtrThetaM);
					free(datPtrDblR);
					fclose(fp1);
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				zDN = (drtz-i)*zsiz;
				zUP = (drtz-i-1)*zsiz;
				areaFactor = sqrt(1.0 + zDN*zUP/camr/camr) * (1.0 + zDN*zUP/camr/camr);
			/* Left, Top */
				drtx = ipHeader.drtx;
				drtxN = (long)floor(drtx);
				jL1 = (long)floor(drtx - camr * acos(dist(1.0, zUP/camr) * cosMax) / xsiz);
				xRT = (drtx-jL1)*xsiz;
				mThetaDR = (jL0-jL1) ? (*getfuncC)(xRT, zDN) : datPtrThetaM[jL1];
				mThetaUR = datPtrThetaM[jL1];
				for (j=jL1; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = datPtrThetaM[j] = mThetaDR;
					mThetaUL = mThetaUR;
					mThetaDR = (*getfuncC)(xRT, zDN);
					mThetaUR = datPtrThetaM[j+1];
					pixel2ThetaC(j);
				}
			/* Centre, Top */
				if (drtxFFlag) {
					datPtrThetaM[j] = mThetaDR;
					if (drtxSwapFlag) {
						drtx = 2.0*floor(ipHeader.drtx) + 1.0 - ipHeader.drtx;
						xLT = (drtx-j)*xsiz;
						mThetaUL = datPtrThetaM[j+1];
						mThetaDL = (*getfuncC)(xLT, zDN);
					}
					else {
						drtx = ipHeader.drtx;
						xLT = xRT;
						xRT = (j+1-drtx)*xsiz;
						mThetaDL = mThetaDR;
						mThetaDR = (*getfuncC)(xRT, zDN);
						mThetaUL = mThetaUR;
						mThetaUR = datPtrThetaM[j+1];
					}
					mThetaUC = mThetaDC;
					mThetaDC = (*getfuncF)(0.0, zDN); /* also valid for curved camera*/
					pixel2ThetaVC(j);
				}
			/* Right, Top */
				drtx = ipHeader.xpxl - ipHeader.drtx;
				drtxN = (long)floor(drtx);
				jR1 = (long)floor(drtx - camr * acos(dist(1.0, zUP/camr) * cosMax) / xsiz);
				xRT = (drtx-jR1)*xsiz;
				mThetaDR = (*getfuncC)(xRT, zDN);
				mThetaUR = datPtrThetaM[ipHeader.xpxl-jR1];
				for (j=jR1; j<drtxN; j++) {
					xLT = xRT;
					xRT = (drtx-j-1)*xsiz;
					mThetaDL = datPtrThetaM[ipHeader.xpxl-j] = mThetaDR;
					mThetaUL = mThetaUR;
					mThetaDR = (*getfuncC)(xRT, zDN);
					mThetaUR = datPtrThetaM[ipHeader.xpxl-j-1];
					pixel2ThetaC(ipHeader.xpxl-j-1);
				}
				datPtrThetaM[(long)ceil(ipHeader.drtx)] = mThetaDR;
				free(datPtr);
			}
		}
		free(datPtrThetaM);
	}
	fclose(fp1);
	
/* Output */
	switch (outputMode) {
		case 1: /* -s */
			break;
		case 2: /* -an */
			for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblArea[i] == 0.0 ? 0.0 : datPtrDblInt[i] / datPtrDblArea[i];
			break;
		case 4: /* -sl */
			if (cameraTypeFlag) {
				if (qFlag) {
					dsFactor = 4.0*camr*camr*step*lam2*M_PI/xsiz/zsiz;
					for (i=mMaxPerf; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : dsFactor * (i+0.5) * step * datPtrDblInt[i] / datPtrDblR[i];
				}
				else {
					dsFactor = 2.0*camr*camr*step*M_PI/xsiz/zsiz;
					for (i=mMaxPerf; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : dsFactor * sin((i+0.5) * step) * datPtrDblInt[i] / datPtrDblR[i];
				}
			}
			else {
				if (useSymRegionFlag) {
					phi0 = phi0tmp;
					phi1 = phi1tmp;
				}
				r1 = phi1-phi0 > 0.0 ? phi1 - phi0 : phi1 - phi0 + 360.0;
				if (useSymRegionFlag) r1 *= 2.0;
				dsFactor = camr*camr*M_PI*r1/360.0/xsiz/zsiz;
				r1 = 0.0;
				for (i=0; i<mMax; i++) {
					r2 = r1;
					if (iiccdFlag) {
						r1 = qFlag ? newton(camr*tan(asin(step*(i+1)*lambda/2.0)*2.0), camr*tan(asin(step*(i+1)*lambda/2.0)*2.0)/2.0, LIM_NEWTON)/camr
						           : newton(tan((i+1)*step)*camr, tan((i+1)*step)*camr/2.0, LIM_NEWTON)/camr;
						if (r1 < 0.0) {
							fprintf(stderr, " Calculation of r1 failed. Exit.\n");
							free(datPtrDblInt);
							free(datPtrDblArea);
							free(datPtrPxlNum);
							free(datPtrDblR);
							if (ofileDoneFlag) {
								fclose(fp2);
								remove(file2);
								fprintf(stderr, " Output File is not created or removed.\n");
							}
							exit(EXIT_FAILURE);
						}
					}
					else r1 = qFlag ? tan(asin(step*(i+1)*lambda/2.0) * 2.0) : tan((i+1) * step);
					r1 *= r1;
					datPtrDblInt[i] = datPtrDblArea[i] == 0.0 ? 0.0 : dsFactor * (r1 - r2) * datPtrDblInt[i] / datPtrDblArea[i];
				}
			}
			break;
		case 0: /* -a */
			if (cameraTypeFlag) for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : datPtrDblInt[i] / datPtrDblR[i];
			else for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblArea[i] == 0.0 ? 0.0 : datPtrDblInt[i] / datPtrDblArea[i];
			break;
		case 6: /* -sv */
			if (cameraTypeFlag) {
				dsFactor = camr*camr/xsiz/zsiz;
				for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : dsFactor * datPtrDblInt[i] / datPtrDblR[i];
			}
			else {
				dsFactor = camr*camr/xsiz/zsiz/2.0;
				r1 = 1.0;
				for (i=0; i<mMax; i++) {
					r2 = r1;
					if (iiccdFlag) {
						r1 = qFlag ? newton(camr*tan(asin(step*(i+1)*lambda/2.0)*2.0), camr*tan(asin(step*(i+1)*lambda/2.0)*2.0)/2.0, LIM_NEWTON)/camr
						           : newton(tan((i+1)*step)*camr, tan((i+1)*step)*camr/2.0, LIM_NEWTON)/camr;
						if (r1 < 0.0) {
							fprintf(stderr, " Calculation of r1 failed. Exit.\n");
							free(datPtrDblInt);
							free(datPtrDblArea);
							free(datPtrPxlNum);
							free(datPtrDblR);
							if (ofileDoneFlag) {
								fclose(fp2);
								remove(file2);
								fprintf(stderr, " Output File is not created or removed.\n");
							}
							exit(EXIT_FAILURE);
						}
						r1 = 1.0/sqrt(1.0+r1*r1);
					}
					else r1 = qFlag ? 1.0-(step*(i+1)*lambda)*(step*(i+1)*lambda)/2.0 : cos((i+1) * step);
					datPtrDblInt[i] = datPtrDblArea[i] == 0.0 ? 0.0 : dsFactor * ((r1+r2)/(r1*r1*r2*r2)) * datPtrDblInt[i] / datPtrDblArea[i];
				}
			}
			break;
		case 5: /* -av */
		default:
			if (cameraTypeFlag) {
				if (qFlag) {
					dsFactor = 2.0*camr*step*lambda/xsiz/zsiz;
					for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : dsFactor / sqrt(4.0 - (i+0.5)*(i+0.5)*step*step*lambda*lambda) * datPtrDblInt[i] / datPtrDblR[i];
				}
				else {
					dsFactor = camr*step/xsiz/zsiz;
					for (i=0; i<mMax; i++) datPtrDblInt[i] = datPtrDblR[i] == 0.0 ? 0.0 : dsFactor * datPtrDblInt[i] / datPtrDblR[i];
				}
			}
			else {
				dsFactor = camr/xsiz/zsiz;
				r1 = 0.0;
				for (i=0; i<mMax; i++) {
					r2 = r1;
					if (iiccdFlag) {
						r1 = qFlag ? newton(camr*tan(asin(step*(i+1)*lambda/2.0)*2.0), camr*tan(asin(step*(i+1)*lambda/2.0)*2.0)/2.0, LIM_NEWTON)/camr
						           : newton(tan((i+1)*step)*camr, tan((i+1)*step)*camr/2.0, LIM_NEWTON)/camr;
						if (r1 < 0.0) {
							fprintf(stderr, " Calculation of r1 failed. Exit.\n");
							free(datPtrDblInt);
							free(datPtrDblArea);
							free(datPtrPxlNum);
							free(datPtrDblR);
							if (ofileDoneFlag) {
								fclose(fp2);
								remove(file2);
								fprintf(stderr, " Output File is not created or removed.\n");
							}
							exit(EXIT_FAILURE);
						}
					}
					else r1 = qFlag ? tan(asin(step*(i+1)*lambda/2.0) * 2.0) : tan((i+1) * step);
					datPtrDblInt[i] = datPtrDblArea[i] == 0.0 ? 0.0 : dsFactor * (r1 - r2) * datPtrDblInt[i] / datPtrDblArea[i];
				}
			}
			break;
	}
	if (lorentzFlag) {
		if (qFlag) for (i=0; i<mMax; i++) datPtrDblInt[i] *= (2.0 * lambda * (i+0.5) * step);
		else for (i=0; i<mMax; i++) datPtrDblInt[i] *= (4.0 * sin((i+0.5) * step / 2.0));
	}
	if (showNumEffFlag == 1 && cameraTypeFlag) {
		free(datPtrDblArea);
		datPtrDblArea = datPtrDblR;
	}
	else free(datPtrDblR);
	if (showNumEffFlag && showNumIntFlag) {
		if (noHeadFlag != 2) numbyte += fprintf(fp2, "# %s%s , Intensity , Effective Area , Num. of Pixel\n", colstr, ucolstr);
		for (i=0; i<mMax; i++) numbyte += fprintf(fp2, " %9.6f , %15.4f , %10.5f , %7ld\n", stepD*(i+dr), datPtrDblInt[i], datPtrDblArea[i], datPtrPxlNum[i]);
	}
	else if (showNumEffFlag && !showNumIntFlag) {
		if (noHeadFlag != 2) numbyte += fprintf(fp2, "# %s%s , Intensity , Effective Area\n", colstr, ucolstr);
		for (i=0; i<mMax; i++) numbyte += fprintf(fp2, " %9.6f , %15.4f , %10.5f\n", stepD*(i+dr), datPtrDblInt[i], datPtrDblArea[i]);
	}
	else if (!showNumEffFlag && showNumIntFlag) {
		if (noHeadFlag != 2) numbyte += fprintf(fp2, "# %s%s , Intensity , Num. of Pixel\n", colstr, ucolstr);
		for (i=0; i<mMax; i++) numbyte += fprintf(fp2, " %9.6f , %15.4f , %7ld\n", stepD*(i+dr), datPtrDblInt[i], datPtrPxlNum[i]);
	}
	else {
		if (noHeadFlag != 2) numbyte += fprintf(fp2, "# %s%s , Intensity\n", colstr, ucolstr);
		for (i=0; i<mMax; i++) numbyte += fprintf(fp2, " %9.6f , %15.4f\n", stepD*(i+dr), datPtrDblInt[i]);
	}
	free(datPtrDblInt);
	free(datPtrDblArea);
	free(datPtrPxlNum);
	
/* End */
	if (ofileDoneFlag) fclose(fp2);
	if (verboseFlag >= 2) ofileDoneFlag ?
	    fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, lines+mMax+1, numbyte) :
	    fprintf(stdout, " Output %ld lines, %ld bytes.\n", lines+mMax+1, numbyte);
	if (verboseFlag > 0) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	if (verboseFlag == -1) return (0);
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef USE_IICCD
	fprintf(fp0, "        %s [-q|-v] file [-f|-cv|-fh[:VALUE]] [-dx:VALUE] [-dz:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file [-f|-cv] [-dx:VALUE] [-dz:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-cl:VALUE] [-xc:VALUE] [-zc:VALUE] {-dt[:VALUE]|-dq[:VALUE]} [-mx]\n");
	fprintf(fp0, "          [-sn|-sr] [-sN] [-sh] [-sH] [-tm] [-s|-a|-an|-sl|-av|-sv] [-lc]\n");
	fprintf(fp0, "          [-xx] [-xs:VALUE] [-zs:VALUE] [-xf:VALUE] [-zf:VALUE] [-lm:VALUE]\n");
#ifdef OMMODE
	fprintf(fp0, "          [-xo:VALUE1:VALUE2] [-zo:VALUE1:VALUE2]\n");
#endif
	fprintf(fp0, "          [-rg:VALUE1:VALUE2] [-xw:VALUE] [-zw:VALUE] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " Generate 1-Dimensional 2theta-I (or q-I) Profile from R-AXIS Image File.\n");
	fprintf(fp0, "  Output Format is \" 2theta[deg.](or q[1/A]), I (...) \" per line.\n");
	fprintf(fp0, "  Calculation is more carefully carried out than RIGAKU R-AXIS software.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -qq ......... Very Quiet Mode. Only error messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ........ Filename of R-AXIS Image File.\n");
	fprintf(fp0, " Camera and Optics :\n");
	fprintf(fp0, "   -f .......... Camera type is Flat. This is default setting.\n");
#ifdef USE_IICCD
	fprintf(fp0, "   -fh[:VALUE] ...\n");
	fprintf(fp0, "                 Camera type is Flat, with iiccd correction.\n");
	fprintf(fp0, "                 With this option, centre of II is assumed to be\n");
	fprintf(fp0, "                 the same as the position of direct beam.\n");
	fprintf(fp0, "                 If \"VALUE\" is also set, it is used as a Pre-Factor of x^2.\n");
	fprintf(fp0, "                 This value is normally calculated using camera length\n");
	fprintf(fp0, "                 via [(256*exp(-0.003*cameraLength[mm])+82)*10^(-7)],\n");
	fprintf(fp0, "                 but ignore this and decide this value directly.\n");
	fprintf(fp0, "                 Caution : Do not use large value !!\n");
#endif
	fprintf(fp0, "   -cv ......... Camera type is Cylindrical, with axis vertical to the screen.\n");
	fprintf(fp0, "                 Other direction of axis is not supported.\n");
	fprintf(fp0, "   -dx:VALUE ... Change PixelSize in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change PixelSize in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, " Output Type :\n");
	fprintf(fp0, "   -sn ......... Also print the Area of pixels for each angle.\n");
	fprintf(fp0, "                 Area is count propotional to contribution in PixelSize Unit.\n");
	fprintf(fp0, "                 For Curved Camera, area is reduced to a spherical surface.\n");
	fprintf(fp0, "   -sr ......... Same as \"-sn\", but use area with no correction\n");
	fprintf(fp0, "                 for Curved Camera. Valid only for Curved Camera.\n");
	fprintf(fp0, "   -sN ......... Also print the Number of pixels for each angle.\n");
	fprintf(fp0, "                 Even if contribution is very small area, count as 1.\n");
	fprintf(fp0, "   -sh ......... Suppress Output Header. If absent, some informations about\n");
	fprintf(fp0, "                 Original File and conversion are print before profile.\n");
	fprintf(fp0, "   -sH ......... Addtion to above, suppress output of title line.\n");
	fprintf(fp0, "   -tm ......... Show about Date, Operator, and Exposure Time in Header.\n");
	fprintf(fp0, " Output Intensity :\n");
	fprintf(fp0, "   -s .......... Print result without averaging. Simple sum of the intensity\n");
	fprintf(fp0, "                 from contributing pixel due to area of interest.\n");
	fprintf(fp0, "   -av ......... Print result after averaging. This is default setting.\n");
	fprintf(fp0, "                 Average only for azimuthal direction, and each intensity has\n");
	fprintf(fp0, "                 proper angular (or q-) width. Azimuthal width is 1.0[mm].\n");
	fprintf(fp0, "                 For Curved Camera, reduced area to sphere is used.\n");
	fprintf(fp0, "   -a .......... Print result after averaging about pixel.\n");
	fprintf(fp0, "                 Intensities are devided by the area of contributing pixel.\n");
	fprintf(fp0, "                 Angle or q for each pixel changes, so this does not make\n");
	fprintf(fp0, "                 correct profile. On this point, different from \"-av\".\n");
	fprintf(fp0, "                 For Curved Camera, reduced area to sphere is used.\n");
	fprintf(fp0, "   -an ......... Same as \"-a\", but use area with no correction for\n");
	fprintf(fp0, "                 Curved Camera. Valid only for Curved Camera.\n");
	fprintf(fp0, "   -sl ......... Same as \"-s\", but add intensities for imperfect Debye-Ring\n");
	fprintf(fp0, "                 regions as the average intensity for the angle is observed.\n");
	fprintf(fp0, "                 This gives a better result for integrated intensity.\n");
	fprintf(fp0, "                 Valid only with \"-mx\" option.\n");
	fprintf(fp0, "   -sv ......... Print result after averaging about solid angle.\n");
	fprintf(fp0, "                 Averaging unit is 1.0[sr]. With Curved Camera, this shows\n");
	fprintf(fp0, "                 simple multiplication of the result with \"-a\" options.\n");
	fprintf(fp0, "   -lc ......... Print Lorentz-correctioned intensities.\n");
	fprintf(fp0, "                 Assume isotropic specimen.\n");
	fprintf(fp0, "   -xx ......... Print 1st column using the centre of the step.\n");
	fprintf(fp0, "                 If absent, use the minimum of the step.\n");
	fprintf(fp0, " Data Region :\n");
	fprintf(fp0, "   -mx ......... Calculate using all data. If absent, calculate\n");
	fprintf(fp0, "                 within the nearest edge not to generate ambiguous data.\n");
	fprintf(fp0, "   -dt:VALUE ... Set 2theta step for 1-D Profile to \"VALUE\" [deg.].\n");
	fprintf(fp0, "                 This must be larger than 2theta-width of a pixel\n");
	fprintf(fp0, "                 around centre. If absent, use the value above.\n");
	fprintf(fp0, "   -dq:VALUE ... Set q-step for 1-D Profile to \"VALUE\" [1/A].\n");
	fprintf(fp0, "                 This must be larger than q-width of a pixel\n");
	fprintf(fp0, "                 around centre. If absent, use the value above.\n");
	fprintf(fp0, "                 Now, q = 2sin(theta)/lambda.\n");
	fprintf(fp0, "                 With this option, Wavelength must be specified.\n");
	fprintf(fp0, "               One of above 2 options must be set.\n");
	fprintf(fp0, "   -lm:VALUE ... Set wavelength to \"VALUE\". If absent, use value stored in\n");
	fprintf(fp0, "                 the R-AXIS File. This is valid for q-profile.\n");
	fprintf(fp0, "   -rg:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Calculate using pixels whose direction from the centre are\n");
	fprintf(fp0, "                 between \"VALUE1\" and \"VALUE2\" [deg.] from X-direction.\n");
	fprintf(fp0, "                 Both \"VALUE\" must be in a range from -360 deg. to +360deg.\n");
	fprintf(fp0, "                 If absent, data in whole direction is used.\n");
	fprintf(fp0, "                 When Curved Camera is used, this option is not supported.\n");
	fprintf(fp0, "   -rs ......... Also include the region symmetrical to the centre,\n");
	fprintf(fp0, "                 when the region pointed by above mentioned way is\n");
	fprintf(fp0, "                 less than 180 deg.\n");
#ifdef OMMODE
	fprintf(fp0, "   -xo:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Exclude region from \"VALUE1\" to \"VALUE2\"-1 in X-direction.\n");
	fprintf(fp0, "                 This is valid to avoid some specific region.\n");
	fprintf(fp0, "                 \"VALUE1\" and \"VALUE2\" must be integer and within IP width.\n");
	fprintf(fp0, "   -zo:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Exclude region from \"VALUE1\" to \"VALUE2\"-1 in Z-direction.\n");
	fprintf(fp0, "                 This is valid to avoid some specific region.\n");
	fprintf(fp0, "                 \"VALUE1\" and \"VALUE2\" must be integer and within IP height.\n");
#endif
	fprintf(fp0, "   The region selected below must contain the centre position.\n");
	fprintf(fp0, "   -xs:VALUE ... Set Smallest Coodinate to use in X-direction to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, start with 0 in X.\n");
	fprintf(fp0, "   -zs:VALUE ... Set Smallest Coodinate to use in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, start with 0 in Z.\n");
	fprintf(fp0, "   -xf:VALUE ... Set Largest Coodinate to use in X-direction to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, use largest coodinate in X.\n");
	fprintf(fp0, "   -zf:VALUE ... Set Largest Coodinate to use in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, use largest coodinate in Z.\n");
	fprintf(fp0, "   -xw:VALUE ... Set Width in X-direction arround centre to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, no restriction in X.\n");
	fprintf(fp0, "                 This is exclusive with the above \"-xs\" and \"-xf\".\n");
	fprintf(fp0, "   -zw:VALUE ... Set Width in Z-direction arround centre to \"VALUE\".\n");
	fprintf(fp0, "                 VALUE must be integer. If absent, no restriction in Z.\n");
	fprintf(fp0, "                 This is exclusive with the above \"-zs\" and \"-zf\".\n");
	fprintf(fp0, "   file2 ....... Output Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-\", Output to Standard Output.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}

#ifdef _OLDHP
double dArea(r, delta)
	double r, delta;
#else
double dArea(double r, double delta)
#endif
{
/*	if (fabs(delta/r/2.0) > 1.0) {
		printf("r= %f, delta = %f . exit.\n", r, delta);
		exit(EXIT_FAILURE);
	}*/
	return (r*r*xAsin(delta/r/2.0) - delta*xSqrt(4.0*r*r - delta*delta)/4.0);
}

#ifdef _OLDHP
char pixel2Theta(jInt)
	long int jInt;
#else
char pixel2Theta(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2;
	char modeFlag;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaDL, mThetaUL, mThetaDR, mThetaUR);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDL - mThetaUR == 2) {
		r1 = datPtrDblR[mThetaDR];
		r2 = datPtrDblR[mThetaDL];
		xU = sqrt(r1*r1 - zUP*zUP) - xRT;
		zR = sqrt(r1*r1 - xRT*xRT) - zUP;
		tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area; /* mThetaUR */
		xD = xLT - sqrt(r2*r2 - zDN*zDN);
		zL = zDN - sqrt(r2*r2 - xLT*xLT);
		tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area; /* mThetaUR + 2 */
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
		datPtrDblArea[mThetaUR+1] += 1.0 - tmpArea0 - tmpArea2;
		datPtrPxlNum[mThetaUR+1]++;
		datPtrDblInt[mThetaUR+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
		datPtrDblArea[mThetaUR+2] += tmpArea2;
		datPtrPxlNum[mThetaUR+2]++;
		return (0);
	}
	if (mThetaDL - mThetaUR == 1) {
		r1 = datPtrDblR[mThetaDL];
		modeFlag = (mThetaDL - mThetaDR) ? 1 : 0; /* low line : right line */
		if (mThetaUL - mThetaUR) modeFlag += 2; /* high line ; if left line , +0 */
		switch (modeFlag) {
			case 0: /* right, left */
				zL = sqrt(r1*r1 - xLT*xLT) - zUP;
				zR = sqrt(r1*r1 - xRT*xRT) - zUP;
				tmpArea0 = (xsiz*(zL+zR)/2.0 + dArea(r1, dist(xsiz, zL-zR))) / area; /* mThetaUR */
				break;
			case 1: /* low, left */
				xD = xLT - sqrt(r1*r1 - zDN*zDN);
				zL = zDN - sqrt(r1*r1 - xLT*xLT);
				tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area; /* mThetaUR */
				break;
			case 2: /* right, high */
				xU = sqrt(r1*r1 - zUP*zUP) - xRT;
				zR = sqrt(r1*r1 - xRT*xRT) - zUP;
				tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area; /* mThetaUR */
				break;
			case 3: /* low high */
				xU = sqrt(r1*r1 - zUP*zUP) - xRT;
				xD = sqrt(r1*r1 - zDN*zDN) - xRT;
				tmpArea0 = (zsiz*(xD+xU)/2.0 + dArea(r1, dist(zsiz, xD-xU))) / area; /* mThetaUR */
				break;
		}
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
		datPtrDblArea[mThetaUR+1] += 1.0 - tmpArea0;
		datPtrPxlNum[mThetaUR+1]++;
		return (0);
	}
	if (!(mThetaDL - mThetaUR)) {
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
		datPtrDblArea[mThetaUR] += 1.0;
		datPtrPxlNum[mThetaUR]++;
		return (0);
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaV(jInt)
	long int jInt;
#else
char pixel2ThetaV(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2;
	char devNum;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaDR, mThetaDL, mThetaDC, mThetaUR, mThetaUL, mThetaUC);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaUL - mThetaUC) {
		r1 = datPtrDblR[mThetaUL];
		xU = xSqrt(r1*r1 - zUP*zUP);
		tmpArea0 = dArea(r1, 2.0*xU) / area; /* mThetaUC */
		if (mThetaDL- mThetaDC) {
			r2 = datPtrDblR[mThetaDL];
			xD = xLT - xSqrt(r2*r2 - zDN*zDN);
			zL = zDN - xSqrt(r2*r2 - xLT*xLT);
			tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area;
			xD += -2.0*xLT + xsiz;
			zR = zDN - xSqrt(r2*r2 - xRT*xRT);
			tmpArea2 += (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else if (mThetaDR - mThetaDL) {
			r2 = datPtrDblR[mThetaDR];
			xD = xRT - xSqrt(r2*r2 - zDN*zDN);
			zR = zDN - xSqrt(r2*r2 - xRT*xRT);
			tmpArea2 = (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaUR - mThetaUL) {
		r1 = datPtrDblR[mThetaUR];
		xU = xSqrt(r1*r1 - zUP*zUP) + xLT;
		zL = xSqrt(r1*r1 - xLT*xLT) - zUP;
		tmpArea0 = (xU*zL/2.0 + dArea(r1, dist(xU, zL))) / area;
		if (mThetaDR - mThetaDL) {
			r2 = datPtrDblR[mThetaDR];
			xD = xRT - xSqrt(r2*r2 - zDN*zDN);
			zR = zDN - xSqrt(r2*r2 - xRT*xRT);
			tmpArea2 = (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaDC - mThetaUR) {
		r1 = datPtrDblR[mThetaDC];
		zL = xSqrt(r1*r1 - xLT*xLT) - zUP;
		zR = xSqrt(r1*r1 - xRT*xRT) - zUP;
		tmpArea0 = (xsiz*(zL+zR)/2.0 + dArea(r1, dist(xsiz, zL-zR))) / area; /* mThetaUR */
		devNum = 1;
	}
	else if (mThetaDL - mThetaDC) {
		r1 = datPtrDblR[mThetaDL];
		xD = xLT - xSqrt(r1*r1 - zDN*zDN);
		zL = zDN - xSqrt(r1*r1 - xLT*xLT);
		tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
		xD += -2.0*xLT + xsiz;
		zR = zDN - xSqrt(r1*r1 - xRT*xRT);
		tmpArea0 -= (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
		devNum = 1;
	}
	else if (mThetaDR - mThetaDL) {
		r1 = datPtrDblR[mThetaDR];
		xD = xRT - xSqrt(r1*r1 - zDN*zDN);
		zR = zDN - xSqrt(r1*r1 - xRT*xRT);
		tmpArea0 = 1.0 - (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
		devNum = 1;
	}
#ifdef DEBUG
	else if (mThetaDR == mThetaDL && mThetaDL == mThetaDC && mThetaDC == mThetaUR
	      && mThetaUR == mThetaUL && mThetaUL == mThetaUC) devNum = 0;
	else devNum = 3;
#else
	else devNum = 0;
#endif
	switch (devNum) {
		case 2:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaUC] += tmpArea0;
			datPtrPxlNum[mThetaUC]++;
			datPtrDblInt[mThetaUC+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
			datPtrDblArea[mThetaUC+1] += 1.0 - tmpArea0 - tmpArea2;
			datPtrPxlNum[mThetaUC+1]++;
			datPtrDblInt[mThetaUC+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
			datPtrDblArea[mThetaUC+2] += tmpArea2;
			datPtrPxlNum[mThetaUC+2]++;
			return (0);
			break;
		case 1:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaUC] += tmpArea0;
			datPtrPxlNum[mThetaUC]++;
			datPtrDblInt[mThetaUC+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
			datPtrDblArea[mThetaUC+1] += 1.0 - tmpArea0;
			datPtrPxlNum[mThetaUC+1]++;
			return (0);
			break;
		case 0:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
			datPtrDblArea[mThetaUC] += 1.0;
			datPtrPxlNum[mThetaUC]++;
			return (0);
			break;
		default:
			break;
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaH(jInt)
	long int jInt;
#else
char pixel2ThetaH(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2;
	char devNum;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaUL, mThetaDL, mThetaCL, mThetaUR, mThetaDR, mThetaCR);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDR - mThetaCR) {
		r1 = datPtrDblR[mThetaDR];
		zR = xSqrt(r1*r1 - xRT*xRT);
		tmpArea0 = dArea(r1, 2.0*zR) / area;
		if (mThetaDL- mThetaCL) {
			r2 = datPtrDblR[mThetaDL];
			xD = xLT - xSqrt(r2*r2 - zDN*zDN);
			zL = zDN - xSqrt(r2*r2 - xLT*xLT);
			tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area;
			xU = xLT - xSqrt(r2*r2 - zUP*zUP);
			zL += -2.0*zDN + zsiz;
			tmpArea2 += (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else if (mThetaUL - mThetaDL) {
			r2 = datPtrDblR[mThetaUL];
			xU = xLT - xSqrt(r2*r2 - zUP*zUP);
			zL = zUP - xSqrt(r2*r2 - xLT*xLT);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaUR - mThetaDR) {
		r1 = datPtrDblR[mThetaUR];
		xD = xSqrt(r1*r1 - zDN*zDN) - xRT;
		zR = xSqrt(r1*r1 - xRT*xRT) + zDN;
		tmpArea0 = (xD*zR/2.0 + dArea(r1, dist(xD, zR))) / area;
		if (mThetaUL - mThetaDL) {
			r2 = datPtrDblR[mThetaUL];
			xU = xLT - xSqrt(r2*r2 - zUP*zUP);
			zL = zUP - xSqrt(r2*r2 - xLT*xLT);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaCL - mThetaUR) {
		r1 = datPtrDblR[mThetaCL];
		xU = xSqrt(r1*r1 - zUP*zUP) - xRT;
		xD = xSqrt(r1*r1 - zDN*zDN) - xRT;
		tmpArea0 = (zsiz*(xU+xD)/2.0 + dArea(r1, dist(zsiz, xU-xD))) / area;
		devNum = 1;
	}
	else if (mThetaDL - mThetaCL) {
		r1 = datPtrDblR[mThetaDL];
		xD = xLT - xSqrt(r1*r1 - zDN*zDN);
		zL = zDN - xSqrt(r1*r1 - xLT*xLT);
		tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
		xU = xLT - xSqrt(r1*r1 - zUP*zUP);
		zL = -2.0*zDN + zsiz;
		tmpArea0 -= (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
	else if (mThetaUL - mThetaDL) {
		r1 = datPtrDblR[mThetaUL];
		xU = xLT - xSqrt(r1*r1 - zUP*zUP);
		zL = zUP - xSqrt(r1*r1 - xLT*xLT);
		tmpArea0 = 1.0 - (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
#ifdef DEBUG
	else if (mThetaUL == mThetaDL && mThetaDL == mThetaCL && mThetaCL == mThetaUR
	      && mThetaUR == mThetaDR && mThetaDR == mThetaCR) devNum = 0;
	else devNum = 3;
#else
	else devNum = 0;
#endif
	switch (devNum) {
		case 2:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
			datPtrDblArea[mThetaCR+1] += 1.0 - tmpArea0 - tmpArea2;
			datPtrPxlNum[mThetaCR+1]++;
			datPtrDblInt[mThetaCR+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
			datPtrDblArea[mThetaCR+2] += tmpArea2;
			datPtrPxlNum[mThetaCR+2]++;
			return (0);
			break;
		case 1:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
			datPtrDblArea[mThetaCR+1] += 1.0 - tmpArea0;
			datPtrPxlNum[mThetaCR+1]++;
			return (0);
			break;
		case 0:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
			datPtrDblArea[mThetaCR] += 1.0;
			datPtrPxlNum[mThetaCR]++;
			return (0);
			break;
		default:
			break;
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaC(jInt)
	long int jInt;
#else
char pixel2ThetaC(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2, numer, cosTheta, cosX, cosTheta2, cosX2;
	char modeFlag;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaDL, mThetaUL, mThetaDR, mThetaUR);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDL - mThetaUR == 2) {
		cosTheta = getcosT(mThetaDR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) - xRT;
		zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
		cosX = cos((xRT + xU / 2.0) / camr); /* Approximation for curvature */
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2); /* IF cosTheta==0.0, FPE might be caused !! */
		tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area; /* mThetaUR */
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
		zL = zDN - camr * sqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r2 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area; /* mThetaUR + 2 */
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrDblR[mThetaUR] += tmpArea0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
		datPtrDblArea[mThetaUR+1] += 1.0 - tmpArea0 - tmpArea2;
		datPtrDblR[mThetaUR+1] += (1.0 - tmpArea0 - tmpArea2) / areaFactor;
		datPtrPxlNum[mThetaUR+1]++;
		datPtrDblInt[mThetaUR+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
		datPtrDblArea[mThetaUR+2] += tmpArea2;
		datPtrDblR[mThetaUR+2] += tmpArea2 / areaFactor;
		datPtrPxlNum[mThetaUR+2]++;
		return (0);
	}
	if (mThetaDL - mThetaUR == 1) {
		modeFlag = (mThetaDL - mThetaDR) ? 1 : 0; /* low line : right line */
		if (mThetaUL - mThetaUR) modeFlag += 2; /* high line ; if left line , +0 */
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		switch (modeFlag) {
			case 0: /* right, left */
				cosX = cos(xLT / camr);
				zL = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos(xRT / camr);
				zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos((xRT + xsiz / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = (xsiz*(zL+zR)/2.0 + dArea(r1, dist(xsiz, zL-zR))) / area;
				break;
			case 1: /* low, left */
				cosX = cos(xLT / camr);
				xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
				zL = zDN - camr * sqrt(cosX*cosX/cosTheta2 - 1.0);
				cosX = cos((xLT - xD / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
				break;
			case 2: /* right, high */
				cosX = cos(xRT / camr);
				xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) - xRT;
				zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos((xRT + xU / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area;
				break;
			case 3: /* low high */
				if (cosTheta < COS_MINIMUM) tmpArea0 = zsiz * (camr * M_PI_2 - xRT) / area;
				else {
					xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) - xRT;
					xD = camr * acos(dist(1.0, zDN/camr) * cosTheta) - xRT;
					cosX = cos((xRT + (xD + xU) / 2.0) / camr);
					cosX2 = cosX * cosX;
					numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
					r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
					tmpArea0 = (zsiz*(xD+xU)/2.0 + dArea(r1, dist(zsiz, xD-xU))) / area;
				}
				break;
		}
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrDblR[mThetaUR] += tmpArea0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
		datPtrDblArea[mThetaUR+1] += 1.0 - tmpArea0;
		datPtrDblR[mThetaUR+1] += (1.0 - tmpArea0) / areaFactor;
		datPtrPxlNum[mThetaUR+1]++;
		return (0);
	}
	if (!(mThetaDL - mThetaUR)) {
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
		datPtrDblArea[mThetaUR] += 1.0;
		datPtrDblR[mThetaUR] += 1.0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		return (0);
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaVC(jInt)
	long int jInt;
#else
char pixel2ThetaVC(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2, numer, cosTheta, cosX, cosTheta2, cosX2;
	char devNum;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaDR, mThetaDL, mThetaDC, mThetaUR, mThetaUL, mThetaUC);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaUL - mThetaUC) {
		cosTheta = getcosT(mThetaUL * step);
		xU = camr * acos(dist(1.0, zUP/camr) * cosTheta);
		r1 = camr * cosTheta * xSqrt(1.0 - cosTheta*cosTheta);
		tmpArea0 = dArea(r1, 2.0*xU) / area; /* mThetaUC */
		if (mThetaDL- mThetaDC) {
			cosTheta = getcosT(mThetaDL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
			zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area;
			cosX = cos(xRT / camr);
			xD += -2.0*xLT + xsiz;
			zR = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xRT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 += (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else if (mThetaDR - mThetaDL) {
			cosTheta = getcosT(mThetaDR * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xRT / camr);
			xD = xRT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
			zR = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xRT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaUR - mThetaUL) {
		cosTheta = getcosT(mThetaUR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) + xLT;
		zL = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
		cosX = cos((-xLT + xU / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = (xU*zL/2.0 + dArea(r1, dist(xU, zL))) / area;
		if (mThetaDR - mThetaDL) {
			cosTheta = getcosT(mThetaDR * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xRT / camr);
			xD = xRT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
			zR = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xRT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xD*zR/2.0 - dArea(r2, dist(xD, zR))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaDC - mThetaUR) {
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		zL = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
		cosX = cos(xRT / camr);
		zR = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
		cosX = cos((xRT - xsiz / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = (xsiz*(zL+zR)/2.0 + dArea(r1, dist(xsiz, zL-zR))) / area;
		devNum = 1;
	}
	else if (mThetaDL - mThetaDC) {
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
		zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
		cosX = cos(xRT / camr);
		xD += -2.0*xLT + xsiz;
		zR = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xRT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 -= (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
		devNum = 1;
	}
	else if (mThetaDR - mThetaDL) {
		cosTheta = getcosT(mThetaDR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		xD = xRT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
		zR = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xRT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea2 = (xD*zR/2.0 - dArea(r1, dist(xD, zR))) / area;
		devNum = 1;
	}
#ifdef DEBUG
	else if (mThetaDR == mThetaDL && mThetaDL == mThetaDC && mThetaDC == mThetaUR
	 && mThetaUR == mThetaUL && mThetaUL == mThetaUC) devNum = 0;
	else devNum = 3;
#else
	else devNum = 0;
#endif
	switch (devNum) {
		case 2:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaUC] += tmpArea0;
			datPtrDblR[mThetaUC] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaUC]++;
			datPtrDblInt[mThetaUC+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
			datPtrDblArea[mThetaUC+1] += 1.0 - tmpArea0 - tmpArea2;
			datPtrDblR[mThetaUC+1] += (1.0 - tmpArea0 - tmpArea2) / areaFactor;
			datPtrPxlNum[mThetaUC+1]++;
			datPtrDblInt[mThetaUC+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
			datPtrDblArea[mThetaUC+2] += tmpArea2;
			datPtrDblR[mThetaUC+2] += tmpArea2 / areaFactor;
			datPtrPxlNum[mThetaUC+2]++;
			return (0);
			break;
		case 1:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaUC] += tmpArea0;
			datPtrDblR[mThetaUC] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaUC]++;
			datPtrDblInt[mThetaUC+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
			datPtrDblArea[mThetaUC+1] += 1.0 - tmpArea0;
			datPtrDblR[mThetaUC+1] += (1.0 - tmpArea0) / areaFactor;
			datPtrPxlNum[mThetaUC+1]++;
			return (0);
			break;
		case 0:
			datPtrDblInt[mThetaUC] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
			datPtrDblArea[mThetaUC] += 1.0;
			datPtrDblR[mThetaUC] += 1.0 / areaFactor;
			datPtrPxlNum[mThetaUC]++;
			return (0);
			break;
		default:
			break;
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaHC(jInt)
	long int jInt;
#else
char pixel2ThetaHC(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2, numer, cosTheta, cosX, cosTheta2, cosX2;
	char devNum;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaUL, mThetaDL, mThetaCL, mThetaUR, mThetaDR, mThetaCR);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDR - mThetaCR) {
		cosTheta =getcosT(mThetaDR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		zR = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		r1 = camr * xSqrt(1.0 - cosTheta2) / cosTheta;
		tmpArea0 = dArea(r1, 2.0*zR) / area;
		if (mThetaDL- mThetaCL) {
			cosTheta = getcosT(mThetaDL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
			zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area;
			xU = xLT - camr * acos(dist(1.0, zUP/camr) * cosTheta);
			zL += -2.0*zDN + zsiz;
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 += (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else if (mThetaUL - mThetaDL) {
			cosTheta = getcosT(mThetaUL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xU = xLT - camr * acos(dist(1.0, zUP/camr) * cosTheta);
			zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaUR - mThetaDR) {
		cosTheta = getcosT(mThetaUR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		xD = camr * acos(dist(1.0, zDN/camr) * cosTheta) - xRT;
		zR = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0) + zDN;
		cosX = cos((xRT + xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = (xD*zR/2.0 + dArea(r1, dist(xD, zR))) / area;
		if (mThetaUL - mThetaDL) {
			cosTheta = getcosT(mThetaUL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xU = xLT - camr * acos(dist(1.0, zUP/camr) * cosTheta);
			zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaCL - mThetaUR) {
		cosTheta = getcosT(mThetaDL * step);
		if (cosTheta < COS_MINIMUM) tmpArea0 = zsiz * (camr * M_PI_2 - xRT) / area;
		else {
			cosTheta2 = cosTheta * cosTheta;
			xU = camr * acos(dist(1.0, zUP/camr) * cosTheta) - xRT;
			xD = camr * acos(dist(1.0, zDN/camr) * cosTheta) - xRT;
			cosX = cos((xRT + (xD + xU) / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea0 = (zsiz*(xD+xU)/2.0 + dArea(r1, dist(zsiz, xD-xU))) / area;
		}
		devNum = 1;
	}
	else if (mThetaDL - mThetaCL) {
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xD = xLT - camr * acos(dist(1.0, zDN/camr) * cosTheta);
		zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
		xU = xLT - camr * acos(dist(1.0, zUP/camr) * cosTheta);
		zL += -2.0*zDN + zsiz;
		cosX = cos((xLT - xU / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 -= (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
	else if (mThetaUL - mThetaDL) {
		cosTheta = getcosT(mThetaUL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xU = xLT - camr * acos(dist(1.0, zUP/camr) * cosTheta);
		zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xU / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = 1.0 - (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
#ifdef DEBUG
	else if (mThetaUL == mThetaDL && mThetaDL == mThetaCL && mThetaCL == mThetaUR
	      && mThetaUR == mThetaDR && mThetaDR == mThetaCR) devNum = 0;
	else devNum = 3;
#else
	else devNum = 0;
#endif
	switch (devNum) {
		case 2:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrDblR[mThetaCR] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
			datPtrDblArea[mThetaCR+1] += 1.0 - tmpArea0 - tmpArea2;
			datPtrDblR[mThetaCR+1] += (1.0 - tmpArea0 - tmpArea2) / areaFactor;
			datPtrPxlNum[mThetaCR+1]++;
			datPtrDblInt[mThetaCR+2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
			datPtrDblArea[mThetaCR+2] += tmpArea2;
			datPtrDblR[mThetaCR+2] += tmpArea2 / areaFactor;
			datPtrPxlNum[mThetaCR+2]++;
			return (0);
			break;
		case 1:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrDblR[mThetaCR] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR+1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
			datPtrDblArea[mThetaCR+1] += 1.0 - tmpArea0;
			datPtrDblR[mThetaCR+1] += (1.0 - tmpArea0) / areaFactor;
			datPtrPxlNum[mThetaCR+1]++;
			return (0);
			break;
		case 0:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
			datPtrDblArea[mThetaCR] += 1.0;
			datPtrDblR[mThetaCR] += 1.0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			return (0);
			break;
		default:
			break;
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaCB(jInt)
	long int jInt;
#else
char pixel2ThetaCB(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2, numer, cosTheta, cosX, cosTheta2, cosX2;
	char modeFlag;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaUR, mThetaUL, mThetaDR, mThetaDL);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDL - mThetaUR == -2) {
		cosTheta = getcosT(mThetaUR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		xU = camr * acos(- dist(1.0, zUP/camr) * cosTheta) - xRT;
		zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
		cosX = cos((xRT + xU / 2.0) / camr); /* Approximation for curvature */
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area; /* mThetaUR */
		cosTheta = getcosT(mThetaDR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xD = xLT - camr * acos(- dist(1.0, zDN/camr) * cosTheta);
		zL = zDN - camr * sqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r2 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area; /* mThetaUR - 2 */
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrDblR[mThetaUR] += tmpArea0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR-1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
		datPtrDblArea[mThetaUR-1] += 1.0 - tmpArea0 - tmpArea2;
		datPtrDblR[mThetaUR-1] += (1.0 - tmpArea0 - tmpArea2) / areaFactor;
		datPtrPxlNum[mThetaUR-1]++;
		datPtrDblInt[mThetaUR-2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
		datPtrDblArea[mThetaUR-2] += tmpArea2;
		datPtrDblR[mThetaUR-2] += tmpArea2 / areaFactor;
		datPtrPxlNum[mThetaUR-2]++;
		return (0);
	}
	if (mThetaDL - mThetaUR == -1) {
		modeFlag = (mThetaDL - mThetaDR) ? 1 : 0; /* low line : right line */
		if (mThetaUL - mThetaUR) modeFlag += 2; /* high line ; if left line , +0 */
		cosTheta = getcosT(mThetaUR * step);
		cosTheta2 = cosTheta * cosTheta;
		switch (modeFlag) {
			case 0: /* right, left */
				cosX = cos(xLT / camr);
				zL = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos(xRT / camr);
				zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos((xRT + xsiz / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = (xsiz*(zL+zR)/2.0 + dArea(r1, dist(xsiz, zL-zR))) / area;
				break;
			case 1: /* low, left */
				cosX = cos(xLT / camr);
				xD = xLT - camr * acos(- dist(1.0, zDN/camr) * cosTheta);
				zL = zDN - camr * sqrt(cosX*cosX/cosTheta2 - 1.0);
				cosX = cos((xLT - xD / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
				break;
			case 2: /* right, high */
				cosX = cos(xRT / camr);
				xU = camr * acos(- dist(1.0, zUP/camr) * cosTheta) - xRT;
				zR = camr * sqrt(cosX*cosX/cosTheta2 - 1.0) - zUP;
				cosX = cos((xRT + xU / 2.0) / camr);
				cosX2 = cosX * cosX;
				numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
				r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
				tmpArea0 = (xU*zR/2.0 + dArea(r1, dist(xU, zR))) / area;
				break;
			case 3: /* low high */
				if (- cosTheta < COS_MINIMUM) tmpArea0 = zsiz * (camr * M_PI_2 - xRT) / area;
				else {
					xU = camr * acos(- dist(1.0, zUP/camr) * cosTheta) - xRT;
					xD = camr * acos(- dist(1.0, zDN/camr) * cosTheta) - xRT;
					cosX = cos((xRT + (xD + xU) / 2.0) / camr);
					cosX2 = cosX * cosX;
					numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
					r1 = camr * numer * sqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
					tmpArea0 = (zsiz*(xD+xU)/2.0 + dArea(r1, dist(zsiz, xD-xU))) / area;
				}
				break;
		}
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
		datPtrDblArea[mThetaUR] += tmpArea0;
		datPtrDblR[mThetaUR] += tmpArea0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		datPtrDblInt[mThetaUR-1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
		datPtrDblArea[mThetaUR-1] += 1.0 - tmpArea0;
		datPtrDblR[mThetaUR-1] += (1.0 - tmpArea0) / areaFactor;
		datPtrPxlNum[mThetaUR-1]++;
		return (0);
	}
	if (!(mThetaDL - mThetaUR)) {
		datPtrDblInt[mThetaUR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
		datPtrDblArea[mThetaUR] += 1.0;
		datPtrDblR[mThetaUR] += 1.0 / areaFactor;
		datPtrPxlNum[mThetaUR]++;
		return (0);
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
char pixel2ThetaHCB(jInt)
	long int jInt;
#else
char pixel2ThetaHCB(long int jInt)
#endif
{
	double r1, r2, xU, xD, zL, zR, tmpArea0, tmpArea2, numer, cosTheta, cosX, cosTheta2, cosX2;
	char devNum;
#ifdef OMMODE
	if (omitFlags(jInt)) return (0);
#endif
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : %4ld, %4ld, %4ld, %4ld, %4ld, %4ld\n", iLines, jInt, mThetaUL, mThetaDL, mThetaCL, mThetaUR, mThetaDR, mThetaCR);
#endif
	tmpArea0 = tmpArea2 = 0.0;
	if (mThetaDR - mThetaCR) {
		cosTheta = getcosT(mThetaCR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		zR = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		r1 = - camr * xSqrt(1.0 - cosTheta2) / cosTheta;
		tmpArea0 = dArea(r1, 2.0*zR) / area;
		if (mThetaDL- mThetaCL) {
			cosTheta = getcosT(mThetaCL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xD = xLT - camr * acos(- dist(1.0, zDN/camr) * cosTheta);
			zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xD / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xD*zL/2.0 - dArea(r2, dist(xD, zL))) / area;
			xU = xLT - camr * acos(- dist(1.0, zUP/camr) * cosTheta);
			zL += -2.0*zDN + zsiz;
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 += (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else if (mThetaUL - mThetaDL) {
			cosTheta = getcosT(mThetaDL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xU = xLT - camr * acos(- dist(1.0, zUP/camr) * cosTheta);
			zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaUR - mThetaDR) {
		cosTheta = getcosT(mThetaDR * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xRT / camr);
		xD = camr * acos(- dist(1.0, zDN/camr) * cosTheta) - xRT;
		zR = camr * xSqrt(cosX*cosX/cosTheta2 - 1.0) + zDN;
		cosX = cos((xRT + xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = (xD*zR/2.0 + dArea(r1, dist(xD, zR))) / area;
		if (mThetaUL - mThetaDL) {
			cosTheta = getcosT(mThetaDL * step);
			cosTheta2 = cosTheta * cosTheta;
			cosX = cos(xLT / camr);
			xU = xLT - camr * acos(- dist(1.0, zUP/camr) * cosTheta);
			zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
			cosX = cos((xLT - xU / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r2 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea2 = (xU*zL/2.0 - dArea(r2, dist(xU, zL))) / area;
			devNum = 2;
		}
		else devNum = 1;
	}
	else if (mThetaCL - mThetaUR) {
		cosTheta = getcosT(mThetaUR * step);
		if (- cosTheta < COS_MINIMUM) tmpArea0 = zsiz * (camr * M_PI_2 - xRT) / area;
		else {
			cosTheta2 = cosTheta * cosTheta;
			xU = camr * acos(- dist(1.0, zUP/camr) * cosTheta) - xRT;
			xD = camr * acos(- dist(1.0, zDN/camr) * cosTheta) - xRT;
			cosX = cos((xRT + (xD + xU) / 2.0) / camr);
			cosX2 = cosX * cosX;
			numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
			r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
			tmpArea0 = (zsiz*(xD+xU)/2.0 + dArea(r1, dist(zsiz, xD-xU))) / area;
		}
		devNum = 1;
	}
	else if (mThetaDL - mThetaCL) {
		cosTheta = getcosT(mThetaCL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xD = xLT - camr * acos(- dist(1.0, zDN/camr) * cosTheta);
		zL = zDN - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xD / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = 1.0 - (xD*zL/2.0 - dArea(r1, dist(xD, zL))) / area;
		xU = xLT - camr * acos(- dist(1.0, zUP/camr) * cosTheta);
		zL += -2.0*zDN + zsiz;
		cosX = cos((xLT - xU / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 -= (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
	else if (mThetaUL - mThetaDL) {
		cosTheta = getcosT(mThetaDL * step);
		cosTheta2 = cosTheta * cosTheta;
		cosX = cos(xLT / camr);
		xU = xLT - camr * acos(- dist(1.0, zUP/camr) * cosTheta);
		zL = zUP - camr * xSqrt(cosX*cosX/cosTheta2 - 1.0);
		cosX = cos((xLT - xU / 2.0) / camr);
		cosX2 = cosX * cosX;
		numer = (1.0 - cosX2) * cosX2 + (cosX2 - cosTheta2) * cosTheta2;
		r1 = camr * numer * xSqrt(numer) / cosTheta2 / (cosX2 * cosX2 + (1.0 - 2.0 * cosX2) * cosTheta2);
		tmpArea0 = 1.0 - (xU*zL/2.0 - dArea(r1, dist(xU, zL))) / area;
		devNum = 1;
	}
#ifdef DEBUG
	else if (mThetaUL == mThetaDL && mThetaDL == mThetaCL && mThetaCL == mThetaUR
	      && mThetaUR == mThetaDR && mThetaDR == mThetaCR) devNum = 0;
	else devNum = 3;
#else
	else devNum = 0;
#endif
	switch (devNum) {
		case 2:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrDblR[mThetaCR] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR-1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0 - tmpArea2);
			datPtrDblArea[mThetaCR-1] += 1.0 - tmpArea0 - tmpArea2;
			datPtrDblR[mThetaCR-1] += (1.0 - tmpArea0 - tmpArea2) / areaFactor;
			datPtrPxlNum[mThetaCR-1]++;
			datPtrDblInt[mThetaCR-2] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea2;
			datPtrDblArea[mThetaCR-2] += tmpArea2;
			datPtrDblR[mThetaCR-2] += tmpArea2 / areaFactor;
			datPtrPxlNum[mThetaCR-2]++;
			return (0);
			break;
		case 1:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * tmpArea0;
			datPtrDblArea[mThetaCR] += tmpArea0;
			datPtrDblR[mThetaCR] += tmpArea0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			datPtrDblInt[mThetaCR-1] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag) * (1.0 - tmpArea0);
			datPtrDblArea[mThetaCR-1] += 1.0 - tmpArea0;
			datPtrDblR[mThetaCR-1] += (1.0 - tmpArea0) / areaFactor;
			datPtrPxlNum[mThetaCR-1]++;
			return (0);
			break;
		case 0:
			datPtrDblInt[mThetaCR] += (*readDat)(&datPtr[jInt], &rato, &shiftBit, &ipDataSwapFlag);
			datPtrDblArea[mThetaCR] += 1.0;
			datPtrDblR[mThetaCR] += 1.0 / areaFactor;
			datPtrPxlNum[mThetaCR]++;
			return (0);
			break;
		default:
			break;
	}
#ifdef DEBUG
	fprintf(stdout, "%4ld, %4ld : return 1\n", iLines, jInt);
#endif
	return (1);
}

#ifdef _OLDHP
long int getThetaF(x, z)
	double x, z;
#else
long int getThetaF(double x, double z)
#endif
{
	return ((long)floor(atan(dist(x, z) / camr) / step));
}

#ifdef _OLDHP
long int getQF(x, z)
	double x, z;
#else
long int getQF(double x, double z)
#endif
{
	return ((long)floor(sqrt(2.0 * (1.0 - camr / sqrt(camr*camr+x*x+z*z))) / lambda / step));
}

#ifdef _OLDHP
long int getThetaFHiPic(x, z)
	double x, z;
#else
long int getThetaFHiPic(double x, double z)
#endif
{
	return ((long)floor(atan(dist(x, z) * (1.0-calib*(x*x+z*z)) / camr) / step));
}

#ifdef _OLDHP
long int getQFHiPic(x, z)
	double x, z;
#else
long int getQFHiPic(double x, double z)
#endif
{
	return ((long)floor(sqrt(2.0 * (1.0 - camr / sqrt(camr*camr+(x*x+z*z)*(1.0-calib*(x*x+z*z))*(1.0-calib*(x*x+z*z))))) / lambda / step));
}

#ifdef _OLDHP
long int getThetaC(x, z)
	double x, z;
#else
long int getThetaC(double x, double z)
#endif
{
	return ((long)floor(acos(cos(x/camr) / dist(1.0, z/camr)) / step));
}

#ifdef _OLDHP
long int getQC(x, z)
	double x, z;
#else
long int getQC(double x, double z)
#endif
{
	return ((long)floor(sqrt(2.0 * (1.0 - cos(x/camr) / dist(1.0, z/camr))) / lambda / step));
}

#ifdef _OLDHP
long int getThetaCB(x, z)
	double x, z;
#else
long int getThetaCB(double x, double z)
#endif
{
	return (stepNumPi - 1 - (long)floor((acos(cos(x/camr) / dist(1.0, z/camr)) - dstep) / step));
}

#ifdef _OLDHP
long int getQCB(x, z)
	double x, z;
#else
long int getQCB(double x, double z)
#endif
{
	return ((long)floor(sqrt(2.0 * (1.0 + cos(x/camr) / dist(1.0, z/camr))) / lambda / step));
}

#ifdef _OLDHP
double getcosTt(x)
	double x;
#else
double getcosTt(double x)
#endif
{
	return (cos(x));
}

#ifdef _OLDHP
double getcosTq(x)
	double x;
#else
double getcosTq(double x)
#endif
{
	return (1.0 - lam2*x*x);
}

#ifdef OMMODE
#ifdef _OLDHP
char omitFlags(j)
	long int j;
#else
char omitFlags(long int j)
#endif
{
	if (j >= omX0 && j < omX1 && iLines >= omZ0 && iLines < omZ1) return (1);
	return (0);
}
#endif

#ifdef _OLDHP
double newton(r, r0, delta)
	double r, r0, delta;
#else
double newton(double r, double r0, double delta)
#endif
{
	double prev, out;
	long int loop;
	
	out = r0;
	for (loop=0; loop<MAXLOOP; loop++) {
		prev = out;
		out = (2.0*calib*out*out*out-r) / (3.0*calib*out*out-1.0);
		if (fabs(prev-out) < delta) return (out);
	}
	return (-1.0);
}

#ifdef _OLDHP
double xSqrt(r)
	double r;
#else
double xSqrt(double r)
#endif
{
	if (r < 0.0 && r>-SQRT_IGNORE) return (0.0);
	return (sqrt(r));
}

#ifdef _OLDHP
double xAsin(r)
	double r;
#else
double xAsin(double r)
#endif
{
	if (r >  1.0 &&  r-1.0<ASIN_IGNORE) return (M_PI_2);
	if (r < -1.0 && -r-1.0<ASIN_IGNORE) return (-M_PI_2);
	return (asin(r));
}

#ifdef _OLDHP
double xAcos(r)
	double r;
#else
double xAcos(double r)
#endif
{
	if (r >  1.0 &&  r-1.0<ACOS_IGNORE) return (0.0);
	if (r < -1.0 && -r-1.0<ACOS_IGNORE) return (M_PI);
	return (acos(r));
}
